/* Driver for Realtek card reader
 * Header file
 *
 * Copyright(c) 2012 Realtek Semiconductor Corp. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *   edwin.rong(edwin_rong@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 *
 *   Tony.Nie(tony_nie@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 */

#ifndef RTS_CR_CARD_READER_H
#define RTS_CR_CARD_READER_H

#include <linux/usb.h>
#include <linux/usb_usual.h>
#include <linux/blkdev.h>
#include <linux/completion.h>
#include <linux/mutex.h>
#include <linux/pci.h>
#include <scsi/scsi_host.h>
#include <scsi/scsi_cmnd.h>

#include "trace.h"
#include "option.h"
#include "cardmanager.h"
#include "cards.h"
#include "transport.h"
#include "trans_pcie.h"

struct cr_capability;
typedef void (*rts_callback)(int);

#define USING_PPBUF

/* Function return code */
#ifndef STATUS_SUCCESS
#define STATUS_SUCCESS		0
#endif
#define STATUS_FAIL		1
#define STATUS_READ_FAIL	2
#define STATUS_WRITE_FAIL	3
#define STATUS_TIMEDOUT		4
#define STATUS_NOMEM		5
#define STATUS_TRANS_SHORT	6
#define STATUS_TRANS_LONG	7
#define STATUS_STALLED		8
#define STATUS_ERROR		10
#define STATUS_FATAL_ERROR	11

#define CD_EXIST		STATUS_SUCCESS /* 0 */
#define CD_NOT_EXIST		STATUS_FAIL /* 1 */

#define IDLE_MAX_COUNT		15
#define POLLING_WAIT_CNT	1
#define POLLING_INTERVAL	50	/* 50ms */
#define DELINK_DELAY		100
#define LED_TOGGLE_INTERVAL	6
#define LED_GPIO		0

#define WAIT_TIME 2000

/* package */
#define QFN24			0
#define LQFP48			1

#define USB_11			0
#define USB_20			1

/* Single LUN, support SD/MS */
#define DEFAULT_SINGLE		0
#define SD_LUN			1
#define MS_LUN			2

/* Bit Operation */
#define SET_BIT(data, idx)	((data) |= 1 << (idx))
#define CLR_BIT(data, idx)	((data) &= ~(1 << (idx)))
#define CHK_BIT(data, idx)	((data) & (1 << (idx)))

/* For MS Card */
#define    MAX_DEFECTIVE_BLOCK     10

#define CR_GET_VID(cr)		((cr)->vendor_id)
#define CR_GET_PID(cr)		((cr)->product_id)
#define CHECK_PID(cr, pid)	(CR_GET_PID(cr) == (pid))
#define CHECK_PKG(cr, pkg)	((cr)->package == (pkg))

#define SDIO_EXIST		0x01
#define SDIO_IGNORED		0x02
#define CHK_SDIO_EXIST(cr)	((cr)->sdio_func_exist & SDIO_EXIST)
#define SET_SDIO_EXIST(cr)	((cr)->sdio_func_exist |= SDIO_EXIST)
#define CLR_SDIO_EXIST(cr)	((cr)->sdio_func_exist &= ~SDIO_EXIST)
#define CHK_SDIO_IGNORED(cr)	((cr)->sdio_func_exist & SDIO_IGNORED)
#define SET_SDIO_IGNORED(cr)	((cr)->sdio_func_exist |= SDIO_IGNORED)
#define CLR_SDIO_IGNORED(cr)	((cr)->sdio_func_exist &= ~SDIO_IGNORED)

#define READ_REG_CMD		0
#define WRITE_REG_CMD		1
#define CHECK_REG_CMD		2

#define CR_WR_REG(cr, addr, mask, data)	\
do {							\
	int _retval = cr_wr_reg((cr),	\
			(addr), (mask), (data));	\
	if (_retval != STATUS_SUCCESS) {		\
		TRACE_RET((cr), _retval);		\
	}						\
} while (0)

#define CR_RD_REG(cr, addr, data)		\
do {							\
	int _retval = cr_rd_reg((cr),	\
			(addr), (data));		\
	if (_retval != STATUS_SUCCESS) {		\
		TRACE_RET((cr), _retval);		\
	}						\
} while (0)

/* auto_delink_en == 0: disable auto delink (may refer to hw setting)
 * auto_delink_en == 1: enable auto delink (may refer to hw setting)
 * auto_delink_en == 2: disable auto delink (don't refer to hardware setting)
 */
#define CHK_AUTODELINK_EN(cr)  ((1 == cr->option.auto_delink_en))

enum RTSX_STAT {
	RTSX_STAT_INIT = 0,
	RTSX_STAT_IDLE,
	RTSX_STAT_RUN,
	RTSX_STAT_SS,
	RTSX_STAT_DELINK,
	RTSX_STAT_SUSPEND,
	RTSX_STAT_ABORT,
	RTSX_STAT_DISCONNECT
};

#define rtsx_get_stat(cr)	((cr)->rtsx_stat)
#define rtsx_chk_stat(cr, stat)	((cr)->rtsx_stat == (stat))
void cr_set_stat(struct card_reader *cr, enum RTSX_STAT stat);

#define RTSX_SET_DELINK(cr)	((cr)->rtsx_flag |= 0x01)
#define RTSX_CLR_DELINK(cr)	((cr)->rtsx_flag &= 0xFE)
#define RTSX_TST_DELINK(cr)	((cr)->rtsx_flag & 0x01)

enum IC_VER {
	IC_VER_A,
	IC_VER_B,
	IC_VER_C = 2,
	IC_VER_D = 3,
	IC_VER_E = 4
};

#define __CHECK_IC_VER(cr, ver)	((cr)->ic_version == (ver))
#define CHECK_VERSION(cr, pid, ver)				\
	(CHECK_PID((cr), (pid)) && __CHECK_IC_VER((cr), (ver)))
#define CHECK_NOT_VERSION(cr, pid, ver)				\
	(CHECK_PID((cr), (pid)) && !__CHECK_IC_VER((cr), (ver)))

#define CHECK_VERSION_HIGHER_THAN(cr, pid, ver) \
	(CHECK_PID((cr), (pid)) && ((cr)->ic_version >= (ver)))
#define CHECK_VERSION_LOWER_THAN(cr, pid, ver) \
	(CHECK_PID((cr), (pid)) && ((cr)->ic_version <= (ver)))

struct auto_load_map {
	u16 cfg_addr;
	u8 func;
	u8 mode;
	u8 data;
};


/* Supported Clock -- card reader */
enum cr_card_clock {
	CR_CLK_20 = 1,
	CR_CLK_30,
	CR_CLK_40,
	CR_CLK_50,
	CR_CLK_60,
	CR_CLK_80,
	CR_CLK_100,
	CR_CLK_120,
	CR_CLK_150,
	CR_CLK_200
};

enum cr_QFN {
	QFN48 = 0,
	QFN64 = 1,
	QFN_INVALID,
};

struct ssc_clock_N {
	int clk;
	u8 N;
	u8 min_N;
	u8 max_N;
};

struct scsi_cmnd;
enum CARD_ID;
struct cr_ops {
	/* Callbacks */
	void (*init_options)(struct card_reader *cr);
	int (*init)(struct card_reader *cr);
	void (*cd_debounce)(struct card_reader *cr);
	void (*auto_delink)(struct card_reader *cr);
	void (*process_ocp)(struct card_reader *cr);

	/* TODO: int card will be replaced with enum CARD_ID id in struct
	 * card */
	/* NOTE: currently, the return values of implementation to detect_card
	 * in rts5139 & rts5227 ICs, are not of the same expression, but of the
	 * same value, which will be modified */
	int (*detect_card)(struct card_reader *cr, enum CARD_ID id);
	int (*select_card)(struct card_reader *cr,  struct card *card);
	int (*flush_rb)(struct card_reader *cr);

	int (*rd_reg)(struct card_reader *cr, u16 addr, u8 *data);
	int (*wr_reg)(struct card_reader *cr, u16 addr, u8 mask, u8 data);
	int (*rd_reg_ep0)(struct card_reader *cr, u16 addr, u8 *data);
	int (*wr_reg_ep0)(struct card_reader *cr, u16 addr, u8 mask, u8 data);
	int (*rd_phy8)(struct card_reader *cr, u8 addr, u8 *val);
	int (*wr_phy8)(struct card_reader *cr, u8 addr, u8 val);
	int (*rd_phy16)(struct card_reader *cr, u8 addr, u16 *val);
	int (*wr_phy16)(struct card_reader *cr, u8 addr, u16 val);
	int (*wr_reg_seq)(struct card_reader *cr, u16 addr, u16 len, u8 *data);
	int (*rd_reg_seq)(struct card_reader *cr, u16 addr, u16 len, u8 *data);

	/* pcie specific */
	int (*rd_cfg32)(struct card_reader *cr, u16 addr, u32 *val);
	int (*wr_cfg32)(struct card_reader *cr, u16 addr, u32 mask, u32 val);
	int (*rd_cfg_seq)(struct card_reader *cr, u16 addr, u8 *buf, int len);
	int (*wr_cfg_seq)(struct card_reader *cr, u16 addr, u8 *buf, int len);

	int (*get_card_status)(struct card_reader *cr, u16 *status);
	int (*get_rsp)(struct card_reader *cr, int rsp_len, int timeout);
	int (*rd_ppbuf)(struct card_reader *cr, u8 *buf, int buf_len);
	int (*wr_ppbuf)(struct card_reader *cr, u8 *buf, int buf_len);
	void (*prepare_run)(struct card_reader *cr);
	void (*pp_status)(struct card_reader *cr, struct card *card,
			u8 lun, u8 *status, u8 status_len);
	void (*read_status)(struct card_reader *cr, struct card *card,
			u8 lun, u8 *status, u8 status_len);
	int (*transfer_data)(struct card_reader *cr, enum dir_t dir,
			void *buf, u32 len, int use_sg, unsigned int *act_len,
			int timeout, u8 stage_flag);
	int (*transfer_data_03)(struct card_reader *cr, enum dir_t dir,
			void *buf, void **ptr, unsigned int *offset,
			unsigned int len, int use_sg, unsigned int *act_len,
			int timeout);

	void (*clr_card_err)(struct card_reader *cr, enum CARD_ID id);
	int (*poweron_card)(struct card_reader *cr, enum CARD_ID id);
	int (*poweroff_card)(struct card_reader *cr, enum CARD_ID id);
	void (*enable_dma)(struct card_reader *cr,
		u32 byte_cnt, u8 pack_size, enum dma_data_direction dir);
	int (*switch_clk)(struct card_reader *cr, struct card *c, int clk);
	int (*enable_card_clk)(struct card_reader *cr, enum CARD_ID id);
	int (*disable_card_clk)(struct card_reader *cr, enum CARD_ID id);
	void (*clr_hw_err)(struct card_reader *cr);
	void (*switch_to_sdio)(struct card_reader *cr);

	/* for ocp */
	int (*get_ocpstat)(struct card_reader *cr, u8 *val);
	int (*clear_ocpstat)(struct card_reader *cr);
	int (*enable_ocp)(struct card_reader *cr);
	int (*disable_ocp)(struct card_reader *cr);
	int (*init_ocp)(struct card_reader *cr);

	u32 (*cd_deglitch)(struct card_reader *cr);

	/* for LED */
	void (*turn_on_led)(struct card_reader *cr);
	void (*turn_off_led)(struct card_reader *cr);
	void (*enable_auto_blink)(struct card_reader *cr);
	void (*disable_auto_blink)(struct card_reader *cr);

	/* calc ssc clock N */
	int (*calc_ssc_clock_N)(struct card_reader *cr, struct card *card,
					struct ssc_clock_N *ssc);
	void (*deal_extra_action)(struct card_reader *cr);
	int (*read_host_sd40_iospace)(struct card_reader *cr,
					u16 addr, u8 *data, int len);
	int (*write_host_sd40_iospace)(struct card_reader *cr,
					u16 addr, u8 *data, int len);
	int (*enable_oobs_polling)(struct card_reader *cr);
	int (*disable_oobs_polling)(struct card_reader *cr);
	int (*patch_dma)(struct card_reader *cr);
};

struct card;
struct sd_command;

struct cr_sd_ops {
	int (*send_cmd_get_rsp)(struct card_reader *cr, struct card *c, u8 cmd,
				u32 arg, u8 rsp_type, u8 *rsp, int rsp_len);
	int (*send_ccmd)(struct card_reader *cr, struct card *c,
					struct sd_command *cmd);
	int (*volt_switch)(struct card_reader *cr, struct card *c);
	int (*tuning)(struct card_reader *cr);

	int (*prepare_init)(struct card_reader *cr, struct card *c);
	int (*prepare_reset)(struct card_reader *cr, struct card *c);
	int (*set_byte_count_per_blk)(struct card_reader *cr, int bcpb);
	int (*set_blk_count)(struct card_reader *cr, int count);
	int (*set_init_para)(struct card_reader *cr, struct card *c);
	int (*chg_bank_volt)(struct card_reader *cr, u8 volt);
	int (*set_clk_div)(struct card_reader *cr, u8 divider);
	int (*get_crc7)(struct card_reader *cr, u8 *buf);
	int (*read_data_normal)(struct card_reader *cr, struct card *c,
			u8 *cmd, int cmd_len, u16 byte_cnt, u16 blk_cnt,
			enum sd_bus_width bus_width,
			u8 *buf, int buf_len, int timeout);
	int (*read_data_auto_tuning)(struct card_reader *cr,
			struct card *c,	u8 *cmd, int cmd_len, u16 byte_cnt,
			u16 blk_cnt, enum sd_bus_width bus_width,
			u8 *buf, int buf_len, int timeout);
	void (*clr_err)(struct card_reader *cr);
	int (*current_over_limit400_handler)(struct card_reader *cr);
	int (*check_crc16_err)(struct card_reader *cr);
	int (*set_clk_pol_ddr_mod)(struct card_reader *cr,
						enum sd_comm_polarity_sol sol);
	int (*set_sample_push_timing)(struct card_reader *cr, struct card *c);
	int (*init_power)(struct card_reader *cr);
	int (*sd30_drive_sel)(struct card_reader *cr, u8 select);
	int (*ddr_pre_tuning_tx)(struct card_reader *cr, struct card *c);
	int (*change_phase)(struct card_reader *cr, struct card *c,
						u8 sample_point, u8 tune_dir);
	int (*wait_data_idle)(struct card_reader *cr);
	int (*is_mechanical_wp)(struct card_reader *cr);
	int (*mmc_switch_timing_bus)(struct card_reader *cr,
					struct card *c, int switch_ddr);
	int (*dummy_clk)(struct card_reader *cr);
	int (*auto_tune_clk)(struct card_reader *cr, struct card *card);
	int (*pull_ctl_enable)(struct card_reader *cr);
	int (*pull_ctl_disable)(struct card_reader *cr);
	int (*disable)(struct card_reader *cr, struct card *c);
	int (*cleanup)(struct card_reader *cr, struct card *c);
	int (*release)(struct card_reader *cr, struct card *c);
	int (*update_status)(struct card_reader *cr, struct card *c);
	int (*rw)(struct card_reader *cr, struct card *c, struct scsi_cmnd *srb,
					u32 start_sector, u16 sector_cnt);
	int (*reset_sd)(struct card_reader *cr, struct card *c);
	int (*reset_mmc)(struct card_reader *cr, struct card *c);
	/* maybe make nonsense, tobe verified */
	int (*enable)(struct card_reader *cr, struct card *c);

	/* for SD40 */
	int (*init_sd40_hw)(struct card_reader *cr, struct card *card);
	int (*init_sd40_hw_m)(struct card_reader *cr, struct card *card);
	int (*uninit_sd40_hw)(struct card_reader *cr, struct card *card);
	int (*process_dormannt_state)(struct card_reader *cr,
			struct card *card, int dir, int dor_mode);
	int (*set_power_control_mode)(struct card *card, u8 mode);
	int (*stop_sequential)(struct card *card);
	int (*patch_special_card)(struct card_reader *cr, struct card *card);
	int (*take_sd40_init)(struct card *card);
	int (*set_sd40_n_fcu)(struct card *c, u8 n_fcu);
};

struct cr_ms_ops {
	int (*prepare_init)(struct card_reader *cr, struct card *c);
	int (*prepare_reset)(struct card_reader *cr, struct card *c);
	int (*transfer_tpc)(struct card_reader *cr, struct card *c,
				u8 __mod, u8 tpc, u8 cnt,
						u8 __cfg);
	int (*switch_bus_width_4)(struct card_reader *cr);
	int (*set_push_timming_odd)(struct card_reader *cr);
	int (*write_bytes)(struct card_reader *cr,
		struct card *c, u8 tpc, u8 cnt, u8 __cfg,
						u8 *data, int data_len);
	int (*read_bytes)(struct card_reader *cr,
		struct card *c, u8 tpc, u8 cnt, u8 __cfg,
						u8 *data, int data_len);
	int (*transfer_data)(struct card_reader *cr, struct card *c,
			u8 __mod, u8 tpc, u16 sec_cnt,
			u8 __cfg, int mode_2k, int use_sg,
						void *buf, int buf_len);
	void (*clr_err)(struct card_reader *cr);
	int (*read_regs_to_buf)(struct card_reader *cr,
						u8 *buf, int start, int count);
	int (*set_sample_time_falling)(struct card_reader *cr);
	int (*set_init_para)(struct card_reader *cr, struct card *c);
	int (*get_int_status)(struct card_reader *cr, u8 *val);
	/* read_sysinfo/read_block_contents are required for reset_ms() */
	int (*read_sysinfo)(struct card_reader *cr, u8 *sysinfo);
	int (*read_block_contents)(struct card_reader *cr, u8 **__ptr);
	int (*switch_intf_mode_4bit)(struct card_reader *cr,
							struct card *c);
	int (*auto_tune_clk)(struct card_reader *cr, struct card *c);
	int (*read_page_data)(struct card_reader *cr, struct card *c,
			struct scsi_cmnd *srb, u8 trans_cfg,
			u8 *buf, unsigned int **index, unsigned int *offset);
	int (*write_page_data)(struct card_reader *cr, struct card *c,
			struct scsi_cmnd *srb, u8 *buf, unsigned int **index,
							unsigned int *offset);
	void (*polling_format_progress)(struct card_reader *cr,
							struct card *c);
	int (*disable)(struct card_reader *cr, struct card *c);
	int (*cleanup)(struct card_reader *cr, struct card *c);
	int (*release)(struct card_reader *cr, struct card *c);
	int (*reset)(struct card_reader *cr, struct card *c);
	int (*update_status)(struct card_reader *cr, struct card *c);
	/* maybe make nonsense, tobe verified */
	int (*enable)(struct card_reader *cr, struct card *c);
	int (*pull_ctl_enable)(struct card_reader *cr);
	int (*pull_ctl_disable)(struct card_reader *cr);
	int (*rw)(struct card_reader *cr, struct card *c, struct scsi_cmnd *srb,
					u32 start_sector, u16 sector_cnt);
};

struct cr_xd_ops {
	int (*prepare_init)(struct card_reader *cr);
	int (*prepare_reset)(struct card_reader *cr, struct card *c);
	int (*set_init_para)(struct card_reader *cr, struct card *c);
	int (*read_id)(struct card_reader *cr, struct card *c, u8 id_cmd,
						u8 *id_buf, u8 buf_len);
	int (*assign_phy_addr)(struct card_reader *cr,
					struct card *c, u32 addr, u8 mode);
	int (*read_redundant)(struct card_reader *cr, struct card *c,
					u32 page_addr, u8 *buf, int buf_len);
	int (*read_cis)(struct card_reader *cr, struct card *c,
				u32 page_addr, u8 *buf, int buf_len);
	int (*search_cis)(struct card_reader *cr, struct card *c);
	int (*tune_timing)(struct card_reader *cr, struct card *c,
								 int step);
	int (*check_data_blank)(u8 *redunt);
	u16 (*load_log_block_addr)(u8 *redunt);
	int (*mark_bad_block)(struct card_reader *cr, struct card *c,
								u32 phy_blk);
	int (*init_page)(struct card_reader *cr, struct card *c, u32 phy_blk,
				u16 logoff, u8 start_page, u8 end_page);
	int (*copy_page)(struct card_reader *cr, struct card *c,
			u32 old_blk, u32 new_blk, u8 start_page, u8 end_page);
	int (*auto_copy_page)(struct card_reader *cr, struct card *c,
		     u32 old_blk, u32 new_blk, u8 start_page, u8 end_page);
	int (*reset_cmd)(struct card_reader *cr, struct card *c);
	int (*send_cmd)(struct card_reader *cr, struct card *c, u8 cmd);
	int (*erase_block)(struct card_reader *cr, struct card *c,
							u32 phy_blk);
	int (*read_multiple_pages)(struct card_reader *cr, struct card *c,
		struct scsi_cmnd *srb,
		u32 phy_blk, u32 log_blk, u8 start_page, u8 end_page,
		u8 *buf, void **ptr, unsigned int *offset);
	int (*write_multiple_pages)(struct card_reader *cr, struct card *c,
		struct scsi_cmnd *srb,
		u32 old_blk, u32 new_blk, u32 log_blk, u8 start_page,
		u8 end_page, u8 *buf, void **ptr, unsigned int *offset);
	int (*disable)(struct card_reader *cr, struct card *c);
	int (*cleanup)(struct card_reader *cr, struct card *c);
	int (*release)(struct card_reader *cr, struct card *c);
	int (*update_status)(struct card_reader *cr, struct card *c);
	/* maybe make nonsense, tobe verified */
	int (*enable)(struct card_reader *cr, struct card *c);
};

struct cr_specific_cmd_handlers {
	int (*vendor_rd_mem)(struct card_reader *cr, struct scsi_cmnd *srb);
	int (*vendor_wr_mem)(struct card_reader *cr, struct scsi_cmnd *srb);
	int (*vendor_rd_sts)(struct card_reader *cr, struct scsi_cmnd *srb);
	int (*vendor_get_card_buswidth)(struct card_reader *cr,
							struct scsi_cmnd *srb);
	int (*vendor_get_sd_csd)(struct card_reader *cr, struct scsi_cmnd *srb);

	int (*app_rd_wr)(struct card_reader *cr, struct scsi_cmnd *srb);
	int (*app_suit_cmd)(struct card_reader *cr, struct scsi_cmnd *srb);
	int (*app_rd_phy_reg)(struct card_reader *cr, struct scsi_cmnd *srb);
	int (*app_wr_phy_reg)(struct card_reader *cr, struct scsi_cmnd *srb);
	int (*app_get_dev_sts)(struct card_reader *cr, struct scsi_cmnd *srb);
	int (*app_rd_host_reg)(struct card_reader *cr, struct scsi_cmnd *srb);
	int (*app_wr_host_reg)(struct card_reader *cr, struct scsi_cmnd *srb);
	int (*app_get_var)(struct card_reader *cr, struct scsi_cmnd *srb);
	int (*app_set_var)(struct card_reader *cr, struct scsi_cmnd *srb);
	int (*app_dma_rb)(struct card_reader *cr, struct scsi_cmnd *srb);
	int (*app_rd_cfg_byte)(struct card_reader *cr, struct scsi_cmnd *srb);
	int (*app_wr_cfg_byte)(struct card_reader *cr, struct scsi_cmnd *srb);
	int (*app_set_chip_mode)(struct card_reader *cr, struct scsi_cmnd *srb);
	int (*app_sd40_rw_partial_bytes)(struct card_reader *cr,
							struct scsi_cmnd *srb);

	int (*sd_pass_thru_mode)(struct card_reader *cr, struct card *c,
							struct scsi_cmnd *srb);
	int (*sd_execute_no_data)(struct card_reader *cr, struct card *c,
							struct scsi_cmnd *srb);
	int (*sd_execute_read_data)(struct card_reader *cr, struct card *c,
							struct scsi_cmnd *srb);
	int (*sd_execute_write_data)(struct card_reader *cr, struct card *c,
							struct scsi_cmnd *srb);
	int (*sd_get_cmd_rsp)(struct card_reader *cr, struct card *c,
							struct scsi_cmnd *srb);
	int (*sd_hw_rst)(struct card_reader *cr, struct card *c,
							struct scsi_cmnd *srb);

	int (*ms_format_cmnd)(struct card_reader *cr, struct card *c,
		struct scsi_cmnd *srb, int short_data_len, int quick_format);
	int (*get_ms_information)(struct card_reader *cr, struct card *c,
							struct scsi_cmnd *srb);
	int (*mg_get_local_EKB)(struct card_reader *cr, struct card *c,
							struct scsi_cmnd *srb);
	int (*mg_get_rsp_chg)(struct card_reader *cr, struct card *c,
							struct scsi_cmnd *srb);
	int (*mg_get_ICV)(struct card_reader *cr, struct card *c,
							struct scsi_cmnd *srb);
	int (*mg_set_leaf_id)(struct card_reader *cr, struct card *c,
							struct scsi_cmnd *srb);
	int (*mg_chg)(struct card_reader *cr, struct card *c,
							struct scsi_cmnd *srb);
	int (*mg_rsp)(struct card_reader *cr, struct card *c,
							struct scsi_cmnd *srb);
	int (*mg_set_ICV)(struct card_reader *cr, struct card *c,
							struct scsi_cmnd *srb);
};

struct cr_callback {
	struct cr_specific_cmd_handlers *specific_cmd_handlers;
	struct cr_ops *ops;
	struct cr_sd_ops *sd_ops_prot;
	struct cr_ms_ops *ms_ops_prot;
	struct cr_xd_ops *xd_ops_prot;
	struct cr_pm_ops *pm_ops;
};

struct hw_param {
	u32 interrupt_en; /* indicate which interrutp enable */

	u16 phy0x00_val;
	u16 phy0x02_val;
	u16 phy0x01_val;
	u16 phy0x03_val;
	u16 phy0x05_val;
	u16 phy0x08_val;
	u16 phy0x19_val;
	u16 phy0x1a_val;
	u16 phy0x1d_val;
	u16 phy0x1e_val;
	u16 phy0x1e_val_l1ss_off;
	u16 phy0x82_val;
	u16 phy0x85_val;
	u16 phy0x87_val;
	u16 phy0x88_val;
	u16 phy0x90_val;
	u16 phy0x92_val;
	u16 phy0x82_val_patch;

	u8 ocp_glitch; /* ocp glitch time */
	u8 av12s; /* bit0~2: U_REG_TUNEA12S */
	u8 stb_lh_time;
	u8 eidle_num;
};

struct card_reader {
	struct transport *tr; /* currently specified USB transport */
	struct card_manager *cm;
	struct rts_scsi *scsi;
	struct cr_option option;
	struct task_struct *polling_thread; /* the polling thread */
	struct completion polling_exit;	/* polling thread exit */
	char *name;
	u16 vendor_id;
	u16 product_id;

	u16 ssvid;
	u16 ssdid;

	u8 max_lun; /* supported num of lun */
	enum LUN_MOD lun_mod;

#if 0
	#define MAX_PHASE	31 /* rts5227 */
	#define MAX_PHASE		15 /* rts5139 */
#endif
	/* max continuous timing phase path for struct timing_phase_path array
	 * which MUST be initialised in the definition of card reader reset
	 * funciton */
	u8 sd_max_phase;

	/* new added */
	/* host_sd_spec is used to define SD protocol version supported by Host
	 * and is set in card reader configuration parameters */
	enum SD_SPEC_VER host_sd_spec;
	/* host_sd_capcity is used to define SD capcity type supported by Host,
	    defaultly is set SDHC | SDXC. */
	enum SD_CAPACITY host_sd_capcity;
	/* xpc_conf is Host's setting for xpc(28) bit of ACMD41 argument */
	int xpc_conf;
	u32 ocr_avail;

	unsigned long reset_map;
	unsigned long release_map;
	unsigned long exist_map;

	int led_toggle_counter;
	int ss_counter;
	int idle_counter;
	int resume_from_scsi;
	spinlock_t pm_lock;
	int cur_clk;		/* current card clock */
	int cur_card;		/* Current card module */
	u8 fake_card_ready;
	/* flag to indicate whether to answer MediaChange */
	unsigned long lun_mc;
	/* ASIC or FPGA */
	int asic_code;
	/* QFN24 or LQFP48 */
	int package;
	u8 ic_version;
	/* Command buffer */
	u8 *cmd_buf;
	/* Response buffer */
	u8 *rsp_buf;
	u16 card_status;
#define cr_set_card_status(cr, status) ((cr)->card_status |= (status))
#define cr_clr_card_status(cr, status) ((cr)->card_status &= ~(status))
#define cr_chk_card_status(cr, status) ((cr)->card_status & (status))
#define CARD_TYPE_UHSII			(1 << 0)
#define CARD_THERMAL_JUMP_TO_SDR50	(1 << 8)

	/* edwin: moved from struct rts5227_info */
	u32 int_reg; /* record value in Bus interrupt pending register */
	unsigned long		need_release; /* need release bit map */
	unsigned long		need_reset; /* need reset bit map */

	/* Flag to indicate that this card is just resumed from SS state,
	 * and need released before being resetted */
	unsigned long		need_reinit;

	u64			auto_delink_timer;

	int			aspm_enabled;
	u8			aspm_level[2];
	u8			host_aspm_val;
	int			driver_first_load;
	int			sdio_func_exist;
	int			sdio_func_init;
	int			hw_bypass_sd;

	u32			ocp_int;
	u8			ocp_stat;

	enum RTSX_STAT		rtsx_stat;
	u16			pm_flags;
	u8			rtsx_flag;

	u8 rcc_read_response;
	int reset_need_retry;
	struct trace_msg_t trace_msg[TRACE_ITEM_CNT];
	int msg_idx;

	void *extra;
	void *obj;
	int host_pci_cap_offset;
	int dev_pci_cap_offset;
	u8 host_pci_speed_cap;

	wait_queue_head_t wait_q;

	struct cr_specific_cmd_handlers *specific_cmd_handlers;
	struct cr_sd_ops *sd_ops_prot;
	struct cr_ms_ops *ms_ops_prot;
	struct cr_xd_ops *xd_ops_prot;
	struct cr_ops *ops;
	struct cr_pm_ops *pm_ops;

	struct card_conf cc_hw;

	struct hw_param hw_param;

	/* for RTD3 */
	u8 d3cold_supported;

#define HOST_ASPM_ENABLED		0
#define ENTERED_RTD3			1
#define LED_TEST_MODE			2
#define DELINK_STEP1_DONE		3
#define DELINK_STEP2_DONE		4
#define DELINK_STEP3_DONE		5
	unsigned long status_flag;
	int polling_interval;

	struct completion *done_resume;
};

#define cr_to_rts_pcie(cr)	((struct rts_pcie *)((cr)->tr->extra))
#define cr_to_pci_dev(cr)	(cr_to_rts_pcie(cr)->pci_dev)
#define cr_to_pci_device(cr)	(&cr_to_pci_dev(cr)->dev)

#define SET_STATUS_FLAG(cr, flag) \
	__set_bit(flag, &cr->status_flag)
#define CLEAR_STATUS_FLAG(cr, flag) \
	__clear_bit((unsigned)flag, &cr->status_flag)
#define CHECK_STATUS_FLAG(cr, flag) \
	test_bit(flag, &cr->status_flag)

struct zone_entry {
	u16 *l2p_table;
	u16 *free_table;
	u16 defect_list[MAX_DEFECTIVE_BLOCK];	/* For MS card only */
	int set_index;
	int get_index;
	int unused_blk_cnt;
	int disable_count;
	/* To indicate whether the L2P table of this zone has been built. */
	int build_flag;
};


static inline u8 double_depth(u8 depth)
{
	return ((depth > 1) ? (depth - 1) : depth);
};

int cr_rd_reg(struct card_reader *cr, u16 addr, u8 *data);
int cr_wr_reg(struct card_reader *cr, u16 addr, u8 mask, u8 data);
int cr_wr_reg_ep0(struct card_reader *cr, u16 addr, u8 mask, u8 data);
int cr_rd_reg_ep0(struct card_reader *cr, u16 addr, u8 *data);
int cr_rd_phy8(struct card_reader *cr, u8 addr, u8 *val);
int cr_wr_phy8(struct card_reader *cr, u8 addr, u8 val);
int cr_rd_phy16(struct card_reader *cr, u8 addr, u16 *val);
int cr_wr_phy16(struct card_reader *cr, u8 addr, u16 val);
int cr_rd_reg_seq(struct card_reader *cr, u16 addr, u16 len, u8 *data);
int cr_wr_reg_seq(struct card_reader *cr, u16 addr, u16 len, u8 *data);
int cr_rd_cfg32(struct card_reader *cr, u16 addr, u32 *val);
int cr_wr_cfg32(struct card_reader *cr, u16 addr, u32 mask, u32 val);
int cr_rd_cfg_seq(struct card_reader *cr, u16 addr, u8 *buf, int len);
int cr_wr_cfg_seq(struct card_reader *cr, u16 addr, u8 *buf, int len);

/* dump register in range [start, end] */
int cr_dump_register(struct card_reader *cr, u16 start, u16 end);
int cr_dump_phy(struct card_reader *cr, u8 start, u8 end);

int cr_get_card_status(struct card_reader *cr, u16 *status);
void cr_init_cmd(struct card_reader *cr);
void cr_add_cmd(struct card_reader *cr, u8 type, u16 addr,
		u8 mask, u8 data);
int cr_send_cmd(struct card_reader *cr, u8 flag, int timeout);
void cr_send_cmd_no_wait(struct card_reader *cr);
void cr_stop_cmd(struct card_reader *cr, int card);
u8 *cr_get_cmd_data(struct card_reader *cr);
int cr_get_rsp(struct card_reader *cr, int rsp_len, int timeout);
void cr_read_rsp_buf(struct card_reader *cr, int offset,
				       u8 *buf, int buf_len);
u8 *cr_get_rsp_data(struct card_reader *cr);
int cr_rd_ppbuf(struct card_reader *cr, u8 *buf, int buf_len);
int cr_wr_ppbuf(struct card_reader *cr, u8 *buf, int buf_len);
void cr_prepare_run(struct card_reader *cr);
void cr_pp_status(struct card_reader *cr, struct card *card,
		u8 lun, u8 *status, u8 status_len);
void cr_read_status(struct card_reader *cr, struct card *card,
		u8 lun, u8 *status, u8 status_len);
int cr_transfer_data(struct card_reader *cr, u8 card, void *buf,
	size_t len, int use_sg, int dir, int timeout);
int cr_transfer_data_partial(struct card_reader *cr, u8 card,
	void *buf, size_t len, int use_sg, unsigned int *index,
	unsigned int *offset, int dir, int timeout);
int cr_transfer_data_03(struct card_reader *cr,
				enum dir_t dir, void *buf, void **ptr,
				unsigned int *offset, unsigned int len,
				int use_sg, unsigned int *act_len, int timeout);

void cr_clr_card_err(struct card_reader *cr, enum CARD_ID id);
int cr_select_card(struct card_reader *cr, struct card *card);
int cr_poweron_card(struct card_reader *cr, enum CARD_ID id);
int cr_poweroff_card(struct card_reader *cr, enum CARD_ID id);
void cr_enable_dma(struct card_reader *cr, u32 byte_cnt, u8 pack_size,
		enum dma_data_direction dir);
int cr_switch_clk(struct card_reader *cr, struct card *c, int clk);
int cr_enable_card_clk(struct card_reader *cr, enum CARD_ID id);
int cr_disable_card_clk(struct card_reader *cr, enum CARD_ID id);
void cr_clr_hw_err(struct card_reader *cr);
int cr_flush_rb(struct card_reader *cr);
int cr_detect_card(struct card_reader *cr, enum CARD_ID id);
struct card_reader *cr_alloc(int preserve_size);
void cr_release(struct card_reader *cr);

int cr_init_hw(struct card_reader *cr);
void cr_init_option(struct card_reader *cr);
int cr_init(struct card_reader *cr, struct cr_capability *cr_capa,
		struct transport *tr);
void cr_process_ocp(struct card_reader *cr);
void cr_wakeup_polling_thread(struct card_reader *cr);
int start_polling_thread(struct card_reader *cr);
int cr_polling_func(struct card_reader *cr);
void cr_cd_debounce(struct card_reader *cr);
void cr_enable_host_aspm(struct card_reader *cr);
void cr_disable_host_aspm(struct card_reader *cr);
void cr_set_host_aspm(struct card_reader *cr, u8 val);
void cr_get_host_aspm(struct card_reader *cr, u8 *val);
void cr_notify_upper(struct card_reader *cr, struct card *c, int val);

u32 cr_ioread32(struct card_reader *cr, u32 reg);
void cr_iowrite32(struct card_reader *cr, u32 reg, u32 value);
u16 cr_ioread16(struct card_reader *cr, u32 reg);
void cr_iowrite16(struct card_reader *cr, u32 reg, u16 value);
u8 cr_ioread8(struct card_reader *cr, u32 reg);
void cr_iowrite8(struct card_reader *cr, u32 reg, u8 value);

int cr_read_config8(struct card_reader *cr, int reg,  u8 *value);
int cr_read_config16(struct card_reader *cr, int reg,  u16 *value);
int cr_read_config32(struct card_reader *cr, int reg,  u32 *value);
int cr_write_config8(struct card_reader *cr, int reg, u8 value);
int cr_write_config16(struct card_reader *cr, int reg, u16 value);
int cr_write_config32(struct card_reader *cr, int reg, u32 value);

void cr_print_option(struct card_reader *cr);
int cr_pci_rd_cfg_byte(struct card_reader *cr, u8 bus, u8 dev, u8 func,
		u8 offset, u8 *val);
int cr_enable_msi(struct card_reader *cr);
void cr_synchronize_irq(struct card_reader *cr);
void cr_clear_interrupt_flag(struct card_reader *cr, u32 flag);


u8 cr_get_card_spt_map_hw(struct card_reader *cr);
u8 cr_get_card_spt_map_sw(struct card_reader *cr);
u32 cr_get_card_lun_map_hw(struct card_reader *cr);
u32 cr_get_card_lun_map_sw(struct card_reader *cr);
void cr_set_card_spt_map_sw(struct card_reader *cr, u8 map);
void cr_set_card_lun_map_sw(struct card_reader *cr, u32 map);
u32 cr_get_sd_capability_hw(struct card_reader *cr);
u32 cr_get_sd_capability_sw(struct card_reader *cr);
void cr_set_sd_capability_sw(struct card_reader *cr, u32 capability);
void cr_switch_to_sdio(struct card_reader *cr);
void cr_enable_bus_int(struct card_reader *cr, u32 interrupt);
void cr_disable_bus_int(struct card_reader *cr);
void cr_exit_L1(struct card_reader *cr);
int cr_process_card_interrupt(struct card_reader *cr);
int cr_process_transfer_interrupt(struct card_reader *cr);
int cr_process_ocp_interrupt(struct card_reader *cr);
int cr_pre_handle_interrupt(struct card_reader *cr);
int cr_request_irq(struct card_reader *cr);
void cr_free_irq(struct card_reader *cr);
int cr_init_ocp(struct card_reader *cr);
int cr_disable_ocp(struct card_reader *cr);
int cr_enable_ocp(struct card_reader *cr);
int cr_clear_ocpstat(struct card_reader *cr);
int cr_get_ocpstat(struct card_reader *cr, u8 *val);
void cr_init_generic_options(struct card_reader *cr);
void cr_init_generic_sw_param(struct cr_option *option);
void cr_init_generic_hw_param(struct hw_param *hw_param);
void cr_auto_delink(struct card_reader *cr);
void cr_undo_delink(struct card_reader *cr);
int cr_init_generic_options_from_hw(struct card_reader *cr);
void cr_clear_first_install_mark(struct card_reader *cr);
void cr_deal_generic_extra_action(struct card_reader *cr);
u64 cr_get_tick(void);
void cr_inform_reset(struct card_reader *cr);
int cr_save_cfg_to_epcore(struct card_reader *cr, u16 reg, u16 cfg, u8 func);
int cr_save_reg_to_autoload(struct card_reader *cr,
		u16 reg, u8 mask, u8 data, u16 autoload);
int cr_read_host_sd40_iospace(struct card_reader *cr,
				u16 addr, u8 *data, int len);
int cr_write_host_sd40_iospace(struct card_reader *cr,
				u16 addr, u8 *data, int len);
int cr_check_card_in_init(struct card_reader *cr);
int cr_find_pci_capability(struct card_reader *cr, int cap);
void cr_patch_hw_setting(struct card_reader *cr);
int cr_swap_sdclk_polarity(struct card_reader *cr);
int cr_rd_cfg8(struct card_reader *cr, u16 addr, u8 *val);
int cr_wr_cfg8(struct card_reader *cr, u16 addr, u8 val);
int cr_find_host_pci_capability(struct card_reader *cr, int cap);
void cr_read_host_config8(struct transport *tr, u16 addr, u8 *val);
void cr_write_host_config8(struct transport *tr, u16 addr, u8 val);
void cr_patch_rtd3(struct card_reader *cr);
void cr_disable_sd_dat3_poll(struct card_reader *cr);
void cr_enable_sd_dat3_poll(struct card_reader *cr);
int cr_patch_dma(struct card_reader *cr);
void cr_get_host_devctl(struct card_reader *cr, u8 *val);
void cr_get_dev_devctl(struct card_reader *cr, u8 *val);
void cr_set_dev_devctl(struct card_reader *cr, u8 val);


static inline int cr_update_phy16(struct card_reader *cr, u8 addr, u16 mask,
		u16 append)
{
	u16 val;
	int err;

	err = cr_rd_phy16(cr, addr, &val);
	if (err)
		return err;

	err = cr_wr_phy16(cr, addr, (val & mask) | append);
	return err;
}

static inline int cr_cfg_save(struct card_reader *cr, int cfg, u16 reg)
{
	int err = 0;
	u8 val = 0;

	err = cr_read_config8(cr, cfg, &val);
	if (err < 0)
		return err;

	err = cr_wr_reg(cr, reg, 0xFF, val);
	if (err < 0)
		return err;

	return 0;
}

static inline int cr_cfg_restore(struct card_reader *cr, int cfg, u16 reg)
{
	int err = 0;
	u8 val = 0;

	err = cr_rd_reg(cr, reg, &val);
	if (err < 0)
		return err;

	err = cr_write_config8(cr, cfg, val);
	if (err < 0)
		return err;

	return 0;
}

static inline u16 cr_get_le16(u8 *beg)
{
	u16 val = *beg++;
	val |= *beg++ << 8;
	return val;
}

static inline u32 cr_get_le32(u8 *beg)
{
	u32 val = *beg++;
	val |= *beg++ << 8;
	val |= *beg++ << 16;
	val |= *beg++ << 24;
	return val;
}

static inline u16 cr_get_be16(u8 *beg)
{
	u16 val = *beg++;
	val = (val << 8) | *beg++;
	return val;
}

static inline u32 cr_get_be32(u8 *beg)
{
	u32 val = *beg++;
	val = (val << 8) | *beg++;
	val = (val << 8) | *beg++;
	val = (val << 8) | *beg++;
	return val;
}

#endif /* RTS_CR_CARD_READER_H */
