#ifndef __CARDS_H__
#define __CARDS_H__

#define SD_NR		2
#define MS_NR		3
#define XD_NR		4

#define SD_CARD		(1 << SD_NR)
#define MS_CARD		(1 << MS_NR)
#define XD_CARD		(1 << XD_NR)

enum SD_CAPACITY {
	SDSC = 0x0001,
	SDHC = 0x0002,
	SDXC = 0x0004,
};

enum sd_bus_width {
	SD_BUS_WIDTH_1 = 0x00,
	SD_BUS_WIDTH_4 = 0x01,
	SD_BUS_WIDTH_8 = 0x02,
};

enum sd_comm_polarity_sol {
	SD_COMM_POL_SOL_ANG = 0,	/* analog solution */
	SD_COMM_POL_SOL_DTL,		/* digital solution */
	SD_COMM_POL_SOL_DRV,		/* driver solution */
};

enum SD_SPEC_VER {
	SD10X = 0,
	SD110,
	SD200,
	SD30X,
};

enum card_clock {
	CLK_20 = 1,
	CLK_30,
	CLK_40,
	CLK_50,
	CLK_60,
	CLK_80,
	CLK_100,
	CLK_120,
	CLK_150,
	CLK_200
};

/* Define card id logically, which will be assigned to car_id member of struct
 * card */
enum CARD_ID {
	INVAL_ID = -1,
	SD_ID = 0,
	MS_ID,
	XD_ID,
	SD40_ID,
	/* ... add more here */
	REALTEK_MAX_CARD_ID,
};

struct card {
	char *name;
	enum CARD_ID id;		/* Logically defined card ID */
	unsigned int channel;	/* tobe set with info from scsi_dev */
	unsigned int target;	/* tobe set with info from scsi_dev */
	u8 lun;
	u32 capacity;
	u8  bus_width;
	u32 debounce_cnt;	/* for debounce operation. */
	u16 card_flags;
#define EXIST		0x0001
#define READY		0x0002 /* currently, this means the card is initialized
				  by reset() */
#define FAIL		0x0004
#define EJECTED		0x0008
#define WP		0x0010
#define QUIESCENCE	0x0020 /* most of SCSI commands not allowed */
#define ERASING		0x0040 /* only SD card has this feature */
#define LOCKED		0x0080 /* only SD card has this feature */
#define MC		0x0100 /* media changed */
#define FAIL_RESET	0x0200 /* set if it's still failed for processing reset
				  after retry for card->retry count */
#define FAIL_RELEASE	0x0400 /* set if it's still failed for processing
				  release after retry for card->retry count */
#define RPM_READY	0x2000 /* card is present and ready even released */
#define FAIL_RW		0x4000 /* rw always failed after first failed */
#define OPTIONS_ALL		(READY | MC | ERASING | LOCKED | WP)
#define OPTIONS_TEST_UNIT_READY	(READY | MC | ERASING | LOCKED)
#define OPTIONS_READ_CPCTY	(READY | MC)
#define OPTIONS_READ_FMT_CPCTY	(READY | MC)

	u16 supported_features; /* defines the card supported features,
				   such as LOCK, ERASE, WP, etc., will be
				   setup in the initialization */
	u16 format_progress; /* some card never updates this field */
	int max_reset_retry; /* retry count for reset/release operation, default
			      * 5 */
	int reset_retry; /* retry number */

	int cd_debounce_begin;
	u64 cd_timestamp;

	int rw_need_retry;
	int rw_retry_cnt;

#define GO_SD20_DIRECT		(1 << 0)
#define GO_SD_LEGACY_DIRECT	(1 << 1)
#define CONCRETE_MMC_CARD	(1 << 2)
#define CONCRETE_SD_CARD	(1 << 3)
#define RESET_MEET_OCP		(1 << 4)
#define RESET_UHSII_NOW		(1 << 5)
#define RESET_E_MMC45_NOW	(1 << 6)
#define FORBIT_USHII_HD_MODE	(1 << 7)
#define PATCH_SPECIAL_SD40_PHY	(1 << 8)
#define CONTINUE_RESETTING_SD	(1 << 9)
#define UHSII_TO_BYPASS_CURRENT	(1 << 10)
#define UHSII_TO_RANGE_A	(1 << 11)
#define UHSI_FIX_TX_PHASE		(1 << 12)
#define UHSI_1V9_RERTY		(1 << 13)

	u32 reset_flags;

#define FORBIT_UHSII_UNSPECIFIED_LEN_MODE	(1 << 0)
#define PATCH_TOSHIBA_UHSII			(1 << 1)
	u32 runtime_flags;

	int uhsii_hd_rw_fail;
	int uhsii_fd_rw_fail;

	struct list_head link;
	int (*reset_card)(struct card *card);
	int (*detect_card)(struct card *card);
	int (*rw)(struct scsi_cmnd *srb, struct card *card,
			u32 start_sector, u16 sector_cnt);
	int (*cleanup)(struct card *card);
	int (*release)(struct card *card);
	int (*enable_card)(struct card *card);
	int (*disable_card)(struct card *card);
	int (*power_off_card)(struct card *card);
	int (*power_on_card)(struct card *card);
	int (*monitor_card)(struct card *card);
	int (*switch_clock)(struct card *card);
	int (*set_card_flags)(struct card *card, u8 flags);
	int (*clear_card_flags)(struct card *card, u8 flags);
	int (*update_report_status)(struct card *card);
	struct card_manager *cm;
	void *extra;
};


#endif
