/* Driver for Realtek card reader
 * Header file
 *
 * Copyright(c) 2012 Realtek Semiconductor Corp. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *   edwin.rong(edwin_rong@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 *
 *   Tony.Nie(tony_nie@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 */

#ifndef __MS_H__
#define __MS_H__

#include "cardmanager.h"
#include "cardreader.h"
#include "cards.h"

#define SUPPORT_MAGIC_GATE
#define	MS_COMM_MAX_RETRY_COUNT	3

#define	MS_COMM_EXTRA_SIZE		0x9

#define	MS_COMM_WRT_PRTCT		0x01

#define MS_COMM_SEQ_START_CRITERIA	0x20

#define MS_COMM_IDX_LEN 17

/* Error Code */
#define	MS_NO_ERROR				0x00
#define	MS_CRC16_ERROR				0x80
#define	MS_TO_ERROR				0x40
#define	MS_NO_CARD				0x20
#define	MS_NO_MEMORY				0x10
#define	MS_CMD_NK				0x08
#define	MS_FLASH_READ_ERROR			0x04
#define	MS_FLASH_WRITE_ERROR			0x02
#define	MS_BREQ_ERROR				0x01
#define	MS_NOT_FOUND				0x03

/* INT flags obtained by accessing IC-specific register
 * RTS5139/REG_MS_TRANS_CFG */
#define	MS_COMM_CRC16_ERR			0x20
#define	MS_COMM_RDY_TIMEOUT			0x10
#define	MS_COMM_INT_CMDNK			0x08
#define	MS_COMM_INT_BREQ			0x04
#define	MS_COMM_INT_ERR				0x02
#define	MS_COMM_INT_CED				0x01

/* Transfer Protocol Command */
#define READ_PAGE_DATA				0x02
#define READ_REG				0x04
#define	GET_INT					0x07
#define WRITE_PAGE_DATA				0x0D
#define WRITE_REG				0x0B
#define SET_RW_REG_ADRS				0x08
#define SET_CMD					0x0E

#define	PRO_READ_LONG_DATA			0x02
#define	PRO_READ_SHORT_DATA			0x03
#define PRO_READ_REG				0x04
#define	PRO_READ_QUAD_DATA			0x05
#define PRO_GET_INT				0x07
#define	PRO_WRITE_LONG_DATA			0x0D
#define	PRO_WRITE_SHORT_DATA			0x0C
#define	PRO_WRITE_QUAD_DATA			0x0A
#define PRO_WRITE_REG				0x0B
#define PRO_SET_RW_REG_ADRS			0x08
#define PRO_SET_CMD				0x0E
#define PRO_EX_SET_CMD				0x09

/* #ifdef SUPPORT_MAGIC_GATE */
#define MG_GET_ID		0x40
#define MG_SET_LID		0x41
#define MG_GET_LEKB		0x42
#define MG_SET_RD		0x43
#define MG_MAKE_RMS		0x44
#define MG_MAKE_KSE		0x45
#define MG_SET_IBD		0x46
#define MG_GET_IBD		0x47
/* #endif */

/* #ifdef XC_POWERCLASS */
#define XC_CHG_POWER		0x16
/* #endif */

/* ++ CMD over Memory Stick */
/*  Flash CMD */
#define BLOCK_READ		0xAA
#define	BLOCK_WRITE		0x55
#define BLOCK_END		0x33
#define BLOCK_ERASE		0x99
#define FLASH_STOP		0xCC

/*  Function CMD */
#define SLEEP			0x5A
#define CLEAR_BUF		0xC3
#define MS_RESET		0x3C
/* -- CMD over Memory Stick */

/* ++ CMD over Memory Stick Pro */
/*  Flash CMD */
#define PRO_READ_DATA		0x20
#define	PRO_WRITE_DATA		0x21
#define PRO_READ_ATRB		0x24
#define PRO_STOP		0x25
#define PRO_ERASE		0x26
#define	PRO_READ_2K_DATA	0x27
#define	PRO_WRITE_2K_DATA	0x28

/*  Function CMD */
#define PRO_FORMAT		0x10
#define PRO_SLEEP		0x11
/* -- CMD over Memory Stick Pro */

/*  register inside memory stick */
#define	IntReg			0x01
#define StatusReg0		0x02
#define StatusReg1		0x03

#define SystemParm		0x10
#define BlockAdrs		0x11
#define CMDParm			0x14
#define PageAdrs		0x15

#define OverwriteFlag		0x16
#define ManagemenFlag		0x17
#define LogicalAdrs		0x18
#define ReserveArea		0x1A

/*  register inside memory pro */
#define	Pro_IntReg		0x01
#define Pro_StatusReg		0x02
#define Pro_TypeReg		0x04
#define	Pro_IFModeReg		0x05
#define Pro_CatagoryReg		0x06
#define Pro_ClassReg		0x07

#define Pro_SystemParm		0x10
#define Pro_DataCount1		0x11
#define Pro_DataCount0		0x12
#define Pro_DataAddr3		0x13
#define Pro_DataAddr2		0x14
#define Pro_DataAddr1		0x15
#define Pro_DataAddr0		0x16

#define Pro_TPCParm		0x17
#define Pro_CMDParm		0x18

/*  define for INT Register
 *
 * the content of register is obatined through cr_ms_read_bytes(cr, c, GET_INT,
 * 1,), which is built up with the following flags
 */
#define	INT_REG_CED		0x80
#define	INT_REG_ERR		0x40
#define	INT_REG_BREQ		0x20
#define	INT_REG_CMDNK		0x01

/*  INT signal */
#define	INT_CED			0x01
#define	INT_ERR			0x02
#define	INT_BREQ		0x04
#define	INT_CMDNK		0x08

/*  define for OverwriteFlag Register */
#define	BLOCK_BOOT		0xC0
#define	BLOCK_OK		0x80
#define	PAGE_OK			0x60
#define	DATA_COMPL		0x10

/*  define for ManagemenFlag Register */
#define	NOT_BOOT_BLOCK		0x4
#define	NOT_TRANSLATION_TABLE	0x8

#define setPS_NG	1
#define setPS_Error	0

/*  define for Pro_SystemParm Register */
#define	PARALLEL_8BIT_IF	0x40
#define	PARALLEL_4BIT_IF	0x00
#define	SERIAL_IF		0x80

/*  define for StatusReg0 Register */
#define BUF_FULL	0x10
#define BUF_EMPTY	0x20

/*  define for StatusReg1 Register */
#define	MEDIA_BUSY	0x80
#define	FLASH_BUSY	0x40
#define	DATA_ERROR	0x20
#define	STS_UCDT	0x10
#define	EXTRA_ERROR	0x08
#define	STS_UCEX	0x04
#define	FLAG_ERROR	0x02
#define	STS_UCFG	0x01

#define MS_SHORT_DATA_LEN	32

#define FORMAT_SUCCESS		0
#define FORMAT_FAIL		1
#define FORMAT_IN_PROGRESS	2

#define	MS_SET_BAD_BLOCK_FLG(ms_card)	((ms_card)->multi_flag |= 0x80)
#define MS_CLR_BAD_BLOCK_FLG(ms_card)	((ms_card)->multi_flag &= 0x7F)
#define MS_TST_BAD_BLOCK_FLG(ms_card)	((ms_card)->multi_flag & 0x80)

#define CHECK_MS_TRANS_FAIL(cr, retval)	\
	(((retval) != STATUS_SUCCESS) || \
	(cr->rsp_buf[0] & MS_TRANSFER_ERR))

#define MODE_512_SEQ		0x01
#define MODE_2K_SEQ		0x02

#define MS_FREE_TABLE_CNT	512

#define TYPE_MS			0x0000
#define TYPE_MSPRO		0x0001

#define MS_4BIT			0x0100
#define MS_8BIT			0x0200
#define MS_HG			0x0400
#define MS_XC			0x0800

#define HG8BIT			(MS_HG | MS_8BIT)

#define CHK_MSPRO(ms_card)	\
	(((ms_card)->ms_type & 0xFF) == TYPE_MSPRO)
#define CHK_HG8BIT(ms_card)	\
	(CHK_MSPRO(ms_card) && (((ms_card)->ms_type & HG8BIT) == HG8BIT))
#define CHK_MSXC(ms_card)	\
	(CHK_MSPRO(ms_card) && ((ms_card)->ms_type & MS_XC))
#define CHK_MSHG(ms_card)	\
	(CHK_MSPRO(ms_card) && ((ms_card)->ms_type & MS_HG))

#define CHK_MS8BIT(ms_card)	(((ms_card)->ms_type & MS_8BIT))
#define CHK_MS4BIT(ms_card)	(((ms_card)->ms_type & MS_4BIT))

#define PRO_UNDER_FORMATTING(ms_card)		\
	((ms_card)->pro_under_formatting)
#define SET_FORMAT_STATUS(ms_card, status)	\
	((ms_card)->format_status = (status))
#define CHK_FORMAT_STATUS(ms_card, status)	\
	((ms_card)->format_status == (status))

/* Header */
#define	HEADER_ID0	(PPBUF_BASE2)			/* 0 */
#define	HEADER_ID1	(PPBUF_BASE2 + 1)		/* 1 */
/* System Entry */
#define	DISABLED_BLOCK0	(PPBUF_BASE2 + 0x170 + 4)	/* 2 */
#define	DISABLED_BLOCK1	(PPBUF_BASE2 + 0x170 + 5)	/* 3 */
#define	DISABLED_BLOCK2	(PPBUF_BASE2 + 0x170 + 6)	/* 4 */
#define	DISABLED_BLOCK3	(PPBUF_BASE2 + 0x170 + 7)	/* 5 */
/* Boot & Attribute Information */
#define	BLOCK_SIZE_0	(PPBUF_BASE2 + 0x1a0 + 2)	/* 6 */
#define	BLOCK_SIZE_1	(PPBUF_BASE2 + 0x1a0 + 3)	/* 7 */
#define	BLOCK_COUNT_0	(PPBUF_BASE2 + 0x1a0 + 4)	/* 8 */
#define	BLOCK_COUNT_1	(PPBUF_BASE2 + 0x1a0 + 5)	/* 9 */
#define	EBLOCK_COUNT_0	(PPBUF_BASE2 + 0x1a0 + 6)	/* 10 */
#define	EBLOCK_COUNT_1	(PPBUF_BASE2 + 0x1a0 + 7)	/* 11 */
#define	PAGE_SIZE_0	(PPBUF_BASE2 + 0x1a0 + 8)	/* 12 */
#define	PAGE_SIZE_1	(PPBUF_BASE2 + 0x1a0 + 9)	/* 13 */

/*joey 2004-08-07 for MS check Procedure */
#define MS_Device_Type	(PPBUF_BASE2 + 0x1D8)		/* 14 */
#define	MS_4bit_Support	(PPBUF_BASE2 + 0x1D3)		/* 15 */
/* end */


struct ms_delay_write_tag {
	u16 old_phyblock;
	u16 new_phyblock;
	u16 logblock;
	u8 pageoff;
	u8 delay_write_flag;
};

struct ms_info {
	u16 ms_type;
	u8 block_shift;
	u8 page_off;
	u16 total_block;
	u16 boot_block;
	/* u32 capacity; */

	u8 check_ms_flow;
	u8 switch_8bit_fail;
	u8 err_code;

	struct zone_entry *segment;
	int segment_cnt;

	int pro_under_formatting;
	int format_status;
	u16 progress;
	u8 raw_sys_info[96];
	u8 raw_ms_id[16];
/* #ifdef SUPPORT_PCGL_1P18 */
	u8 raw_model_name[48];
/* #endif */

	u8 multi_flag;

	/* Sequential RW */
	u8 seq_mode;
	enum dma_data_direction prev_dir;
	u32 prev_sec_addr;
	u16 prev_sec_cnt;
	u32 total_sec_cnt;
	u8 last_rw_int;

	struct ms_delay_write_tag delay_write;

	int cleanup_counter;
	int counter;

	int ms_clock;

/* #ifdef SUPPORT_MAGIC_GATE */
	u8 magic_gate_id[16];
	u8 mg_entry_num;
	int mg_auth;    /* flag to indicate authentication process */
/* #endif */
};

struct card;
struct card_reader;


static inline void ms_set_error_code(struct card_reader *cr,
						struct card *c, u8 err_code)
{
	struct ms_info *ms_card = c->extra;

	ms_card->err_code = err_code;
}

static inline int ms_check_error_code(struct card_reader *cr,
						struct card *c, u8 err_code)
{
	struct ms_info *ms_card = c->extra;

	return (ms_card->err_code == err_code);
}

static inline int ms_parse_error_code(struct card_reader *cr)
{
	TRACE_RET(cr, STATUS_FAIL);
}


int ms_send_cmd(struct card_reader *cr, struct card *card, u8 cmd, u8 cfg);
int ms_card_reset(struct card *c);
int ms_card_disable(struct card *c);
int ms_card_rw(struct scsi_cmnd *srb, struct card *c, u32 lba, u16 num);
int ms_card_cleanup(struct card *c);
int ms_card_release(struct card *c);
int ms_card_update_status(struct card *c);

int ms_comm_initialize_wrap(struct card *c);


int ms_read_sysinfo(struct card_reader *cr, u8 *sysinfo);
int ms_read_block_contents(struct card_reader *cr, u8 **__ptr);
int ms_switch_intf_mode_4bit(struct card_reader *cr, struct card *c);
int ms_read_page_data(struct card_reader *cr, struct card *c,
			struct scsi_cmnd *srb, u8 trans_cfg,
			u8 *buf, unsigned int **index, unsigned int *offset);
int ms_write_page_data(struct card_reader *cr, struct card *c,
			struct scsi_cmnd *srb, u8 *buf, unsigned int **index,
							unsigned int *offset);

int ms_prepare_init(struct card_reader *cr, struct card *c);
int ms_prepare_reset(struct card_reader *cr, struct card *c);
int ms_set_init_para(struct card_reader *cr, struct card *c);

int ms_transfer_tpc(struct card_reader *cr, struct card *c,
		u8 __mod, u8 tpc, u8 cnt, u8 __cfg);
void ms_clear_error(struct card_reader *cr);
int ms_write_bytes(struct card_reader *cr, struct card *c, u8 tpc,
		u8 cnt, u8 __cfg, u8 *data, int data_len);
int ms_read_bytes(struct card_reader *cr, struct card *c, u8 tpc,
		u8 cnt, u8 __cfg, u8 *data, int data_len);
int ms_transfer_data(struct card_reader *cr, struct card *c,
			u8 __mod, u8 tpc, u16 sec_cnt,
			u8 __cfg, int mode_2k, int use_sg,
						void *buf, int buf_len);
int ms_switch_bus_width_4(struct card_reader *cr);
int ms_set_push_timming_odd(struct card_reader *cr);
int ms_read_regs_to_buf(struct card_reader *cr, u8 *buf, int offset, int count);
int ms_set_sample_time_falling(struct card_reader *cr);
int ms_get_int_status(struct card_reader *cr, u8 *val);
int ms_read_sysinfo(struct card_reader *cr, u8 *sysinfo);
int ms_read_block_contents(struct card_reader *cr, u8 **__ptr);
int ms_switch_intf_mode_4bit(struct card_reader *cr, struct card *c);
int ms_auto_tune_clock(struct card_reader *cr, struct card *c);
int ms_write_page_data(struct card_reader *cr, struct card *c,
			struct scsi_cmnd *srb, u8 *buf, unsigned int **index,
							unsigned int *offset);
int ms_read_page_data(struct card_reader *cr, struct card *c,
			struct scsi_cmnd *srb, u8 trans_cfg,
			u8 *buf, unsigned int **index, unsigned int *offset);
void ms_polling_format_progress(struct card_reader *cr, struct card *c);
int mspro_read_format_progress(struct card_reader *cr,
				struct card *card, const int short_data_len);
int ms_reset(struct card_reader *cr, struct card *c);
int ms_disable(struct card_reader *cr, struct card *c);
int ms_cleanup(struct card_reader *cr, struct card *c);
int ms_rw(struct card_reader *cr, struct card *c,
			struct scsi_cmnd *srb, u32 lba, u16 num);
int ms_release(struct card_reader *cr, struct card *c);
int ms_update_status(struct card_reader *cr, struct card *c);
int ms_read_status_reg(struct card_reader *cr, struct card *c);
int ms_power_off_card3v3(struct card_reader *cr);
int ms_set_rw_reg_addr(struct card_reader *cr, struct card *c,
		u8 read_start, u8 read_cnt, u8 write_start, u8 write_cnt);
int ms_switch_clock(struct card_reader *cr, struct card *card);
int ms_identify_media_type(struct card_reader *cr,
					struct card *c, int switch_8bit_bus);
int ms_mspro_set_rw_cmd(struct card_reader *cr,
			struct card *c,	u32 start_sec, u16 sec_cnt, u8 cmd);
int ms_pull_ctl_enable(struct card_reader *cr);
void ms_free_l2p_tbl(struct card_reader *cr, struct card *card);
#endif /*__MS_H__*/
