/* Driver for Realtek PCI-Express card reader
 *
 * Copyright(c) 2009 Realtek Semiconductor Corp. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *   wwang (wei_wang@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 *
 *   edwin.rong(edwin_rong@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 *
 *   Tony.Nie(tony_nie@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 */

#include <linux/blkdev.h>
#include <linux/kthread.h>
#include <linux/sched.h>


#include "ms.h"
#include "cardreader.h"
#include "trace.h"
#include "rts_cr.h"
#include "rts_scsi.h"
#include "scsi_cmd.h"
#include "cr_regs.h"
#include "scsi_xfer.h"
#include "ms_mg.h"

int ms_format(struct card_reader *cr, struct card *card,
		struct scsi_cmnd *srb, int short_data_len, int quick_format)
{
	struct ms_info *ms_card = card->extra;
	int retval, i;
	u8 buf[8], tmp;
	u16 para;

	rts_trace(TRACE_MS, "<--\n");

	retval = ms_switch_clock(cr, card);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, STATUS_FAIL);
	retval = ms_set_rw_reg_addr(cr, card, 0x00, 0x00,
						Pro_TPCParm, 0x01);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, STATUS_FAIL);
	memset(buf, 0, 2);
	switch (short_data_len) {
	case 32:
		buf[0] = 0;
		break;
	case 64:
		buf[0] = 1;
		break;
	case 128:
		buf[0] = 2;
		break;
	case 256:
	default:
		buf[0] = 3;
		break;
	}

	for (i = 0; i < MS_MAX_RETRY_COUNT; i++) {
		retval = ms_write_bytes(cr, card,
						PRO_WRITE_REG, 1,
						NO_WAIT_INT, buf, 2);
		if (retval == STATUS_SUCCESS)
			break;
	}
	if (i == MS_MAX_RETRY_COUNT)
		TRACE_RET(cr, STATUS_FAIL);
	/* Format command */
	if (quick_format)
		para = 0x0000;
	else
		para = 0x0001;
	retval = ms_mspro_set_rw_cmd(cr, card, 0, para, PRO_FORMAT);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, STATUS_FAIL);
	/* Check INT */
	cr_rd_reg(cr, REG_MS_TRANS_CFG, &tmp);
	if (tmp & (MS_INT_CMDNK | MS_INT_ERR))
		TRACE_RET(cr, STATUS_FAIL);
	if ((tmp & (MS_INT_BREQ | MS_INT_CED)) ==
		MS_INT_BREQ) {
		ms_card->pro_under_formatting = 1;
		ms_card->progress = 0;
		card->format_progress = 0;
		ms_card->format_status = FORMAT_IN_PROGRESS;
		return STATUS_SUCCESS;
	}
	if (tmp & MS_INT_CED) {
		ms_card->pro_under_formatting = 0;
		ms_card->progress = 0;
		card->format_progress = 0;
		ms_card->format_status = FORMAT_SUCCESS;
		set_sense_type(cr, SCSI_LUN(srb), SENSE_TYPE_NO_SENSE);
		return STATUS_SUCCESS;
	}

	TRACE_RET(cr, STATUS_FAIL);
}

#ifdef SUPPORT_MAGIC_GATE

#ifdef READ_BYTES_WAIT_INT
static int ms_poll_int(struct card_reader *cr)
{
	int retval;
	u8 val;

	cr_init_cmd(cr);
	cr_add_cmd(cr, CHECK_REG_CMD, REG_MS_TRANS_CFG,
				MS_INT_CED, MS_INT_CED);
	retval = cr_send_cmd(cr, MS_CARD, 5000);
	if (retval != STATUS_SUCCESS) {
		TRACE_RET(cr, STATUS_FAIL);
	}
	val = *cr_get_cmd_data(cr);
	if (val & MS_INT_ERR)
		TRACE_RET(cr, STATUS_FAIL);

	return STATUS_SUCCESS;
}
#endif

#ifdef MS_SAMPLE_INT_ERR
/* FW double check error status here.
 * MGRWTest.exe Test Fail with SanDisk PRO-Duo 2GB,
 * because HW may miss ERR signal when sampling INT status. */
static int ms_check_ms_err(struct card_reader *cr)
{
	int retval;
	u8 val;

	retval = cr_rd_reg(cr, REG_MS_TRANSFER, &val);
	if (retval != STATUS_SUCCESS)
		return 1;
	if (val & REG_MS_TRANSFER_ERR)
		return 1;
	retval = cr_rd_reg(cr, REG_MS_TRANS_CFG, &val);
	if (retval != STATUS_SUCCESS)
		return 1;
	if (val & (MS_INT_ERR | MS_INT_CMDNK))
		return 1;

	return 0;
}
#else
static int ms_check_ms_err(struct card_reader *cr)
{
	int retval;
	u8 val;

	retval = cr_rd_reg(cr, REG_MS_TRANSFER, &val);
	if (retval != STATUS_SUCCESS)
		return 1;
	if (val & REG_MS_TRANSFER_ERR)
		return 1;

	return 0;
}
#endif

static int ms_mg_send_ex_cmd(struct card_reader *cr, struct card *card,
							u8 cmd, u8 entry_num)
{
	int retval, i;
	u8 data[8];

	data[0] = cmd;
	data[1] = 0;
	data[2] = 0;
	data[3] = 0;
	data[4] = 0;
	data[5] = 0;
	data[6] = entry_num;
	data[7] = 0;
	for (i = 0; i < MS_MAX_RETRY_COUNT; i++) {
		retval = ms_write_bytes(cr, card,
						PRO_EX_SET_CMD, 7,
						WAIT_INT, data, 8);
		if (retval == STATUS_SUCCESS)
			break;
	}
	if (i == MS_MAX_RETRY_COUNT)
		TRACE_RET(cr, STATUS_FAIL);
	/* Check
	 * REG_MS_TRANSFER_ERR (including CRC Error, INT_ERR and INT_CMDNK)
	 * MS_TransferTPC will not check INT_ERR and INT_CMDNK signal
	 * with WAIT_INT mode. */
	if (ms_check_ms_err(cr)) {
		ms_clear_error(cr);
		TRACE_RET(cr, STATUS_FAIL);
	}

	return STATUS_SUCCESS;
}

/*
 * Get MagciGate ID and set Leaf ID to medium.
 *
 * After receiving this SCSI command, adapter shall fulfill 2 tasks
 * below in order:
 * 1. send GET_ID TPC command to get MagicGate ID and
 * hold it till Response&challenge CMD.
 * 2. send SET_ID TPC command to medium with Leaf ID released
 * by host in this SCSI CMD.
 */

int ms_mg_set_leaf_id(struct card_reader *cr, struct card *card,
							struct scsi_cmnd *srb)
{
	int retval;
	int i;
	u8 lun = SCSI_LUN(srb);
	u8 buf1[32], buf2[12];

	rts_trace(TRACE_MS, "<--\n");

	if (scsi_bufflen(srb) < 12) {
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_INVALID_CMD_FIELD);
		TRACE_RET(cr, STATUS_FAIL);
	}
	ms_cleanup(cr, card);
	retval = ms_switch_clock(cr, card);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, STATUS_FAIL);
	/* Set SET_ID (41h) to the CMD Register */
	retval = ms_mg_send_ex_cmd(cr, card, MG_SET_LID, 0);
	if (retval != STATUS_SUCCESS) {
		set_sense_type(cr, lun, SENSE_TYPE_MG_KEY_FAIL_NOT_ESTAB);
		TRACE_RET(cr, STATUS_FAIL);
	}
	/* Write Leaf ID to medium */
	memset(buf1, 0, 32);
	scsi_stor_get_xfer_buf(buf2, min(12, (int)scsi_bufflen(srb)), srb);
	for (i = 0; i < 8; i++)
		buf1[8+i] = buf2[4+i];
	retval = ms_write_bytes(cr, card, PRO_WRITE_SHORT_DATA,
					32, WAIT_INT, buf1, 32);
	if (retval != STATUS_SUCCESS) {
		set_sense_type(cr, lun, SENSE_TYPE_MG_KEY_FAIL_NOT_ESTAB);
		TRACE_RET(cr, STATUS_FAIL);
	}
	if (ms_check_ms_err(cr)) {
		set_sense_type(cr, lun, SENSE_TYPE_MG_KEY_FAIL_NOT_ESTAB);
		ms_clear_error(cr);
		TRACE_RET(cr, STATUS_FAIL);
	}

	return STATUS_SUCCESS;
}

/*
 * Send Local EKB to host.
 * After receiving this SCSI command, adapter shall
 * read the divided data(1536 bytes totally)
 * from medium by using READ_LONG_DATA TPC for 3 times,
 * and report data to host with
 * data-length is 1052 bytes.
 */
int ms_mg_get_local_EKB(struct card_reader *cr, struct card *card,
							struct scsi_cmnd *srb)
{
	int retval = STATUS_FAIL;
	int bufflen;
	u8 lun = SCSI_LUN(srb);
	u8 *buf = NULL;

	rts_trace(TRACE_MS, "<--\n");

	ms_cleanup(cr, card);
	retval = ms_switch_clock(cr, card);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, STATUS_FAIL);
	buf = (u8 *)cr_alloc_dma_buf(cr, 1540, GFP_KERNEL);
	if (!buf)
		TRACE_RET(cr, STATUS_ERROR);
	/* Push 4-bytes data in advance: Report Key Data Length(41Ah) */
	buf[0] = 0x04;
	buf[1] = 0x1A;
	buf[2] = 0x00;
	buf[3] = 0x00;
	/* Send GET_LEKB (42h) command to the CMD Register */
	retval = ms_mg_send_ex_cmd(cr, card, MG_GET_LEKB, 0);
	if (retval != STATUS_SUCCESS) {
		set_sense_type(cr, lun, SENSE_TYPE_MG_KEY_FAIL_NOT_AUTHEN);
		TRACE_GOTO(cr, GetEKBFinish);
	}
	retval = ms_transfer_data(cr, card, MS_TM_AUTO_READ,
					PRO_READ_LONG_DATA, 3,
					WAIT_INT, 0, 0, buf + 4, 1536);
	if (retval != STATUS_SUCCESS) {
		set_sense_type(cr, lun, SENSE_TYPE_MG_KEY_FAIL_NOT_AUTHEN);
		ms_clear_error(cr);
		TRACE_GOTO(cr, GetEKBFinish);
	}
	if (ms_check_ms_err(cr)) {
		set_sense_type(cr, lun, SENSE_TYPE_MG_KEY_FAIL_NOT_AUTHEN);
		ms_clear_error(cr);
		TRACE_RET(cr, STATUS_FAIL);
	}
	bufflen = min(1052, (int)scsi_bufflen(srb));
	scsi_stor_set_xfer_buf(buf, bufflen, srb);
	/*scsi_set_resid(srb, scsi_bufflen(srb) - bufflen); */
GetEKBFinish:
	if (buf)
		cr_free_dma_buf(cr, buf);

	return retval;
}

/*
 * Send challenge(host) to medium.
 * After receiving this SCSI command, adapter shall
 * sequentially issues TPC commands to the medium for
 * writing 8-bytes data as challenge by host within a short data packet.
 */
int ms_mg_chg(struct card_reader *cr, struct card *card,
							struct scsi_cmnd *srb)
{
	struct ms_info *ms_card = card->extra;
	int retval;
	int bufflen;
	int i;
	u8 lun = SCSI_LUN(srb);
	u8 buf[32];

	rts_trace(TRACE_MS, "<--\n");

	ms_cleanup(cr, card);
	retval = ms_switch_clock(cr, card);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, STATUS_FAIL);
	/* Send GET_ID (40h) command to the CMD Register */
	retval = ms_mg_send_ex_cmd(cr, card, MG_GET_ID, 0);
	if (retval != STATUS_SUCCESS) {
		set_sense_type(cr, lun, SENSE_TYPE_MG_INCOMPATIBLE_MEDIUM);
		TRACE_RET(cr, STATUS_FAIL);
	}
	/* Read data from medium */
	retval = ms_read_bytes(cr, card, PRO_READ_SHORT_DATA,
					32, WAIT_INT, buf, 32);
	if (retval != STATUS_SUCCESS) {
		set_sense_type(cr, lun, SENSE_TYPE_MG_INCOMPATIBLE_MEDIUM);
		TRACE_RET(cr, STATUS_FAIL);
	}
	if (ms_check_ms_err(cr)) {
		set_sense_type(cr, lun, SENSE_TYPE_MG_INCOMPATIBLE_MEDIUM);
		ms_clear_error(cr);
		TRACE_RET(cr, STATUS_FAIL);
	}
	/* Save MagicGate ID (16) */
	memcpy(ms_card->magic_gate_id, buf, 16);
#ifdef READ_BYTES_WAIT_INT
	retval = ms_poll_int(cr);
	if (retval != STATUS_SUCCESS) {
		set_sense_type(cr, lun, SENSE_TYPE_MG_INCOMPATIBLE_MEDIUM);
		TRACE_RET(cr, STATUS_FAIL);
	}
#endif
	/* Send SET_RD (43h) command to the CMD Register */
	retval = ms_mg_send_ex_cmd(cr, card, MG_SET_RD, 0);
	if (retval != STATUS_SUCCESS) {
		set_sense_type(cr, lun, SENSE_TYPE_MG_INCOMPATIBLE_MEDIUM);
		TRACE_RET(cr, STATUS_FAIL);
	}
	bufflen = min(12, (int)scsi_bufflen(srb));
	scsi_stor_get_xfer_buf(buf, bufflen, srb);
	for (i = 0; i < 8; i++)
		buf[i] = buf[4+i];
	for (i = 0; i < 24; i++)
		buf[8+i] = 0;
	retval = ms_write_bytes(cr, card, PRO_WRITE_SHORT_DATA,
						32, WAIT_INT, buf, 32);
	if (retval != STATUS_SUCCESS) {
		set_sense_type(cr, lun, SENSE_TYPE_MG_INCOMPATIBLE_MEDIUM);
		TRACE_RET(cr, STATUS_FAIL);
	}
	if (ms_check_ms_err(cr)) {
		set_sense_type(cr, lun, SENSE_TYPE_MG_INCOMPATIBLE_MEDIUM);
		ms_clear_error(cr);
		TRACE_RET(cr, STATUS_FAIL);
	}
	/* The authentication process is re-started. */
	ms_card->mg_auth = 0;

	return STATUS_SUCCESS;
}

/*
 * Send Response and Challenge data  to host.
 * After receiving this SCSI command, adapter shall
 * communicates with the medium, get parameters(HRd, Rms, MagicGateID)
 * by using READ_SHORT_DATA TPC and send the data to host according
 * to certain format required by MG-R specification.
 * The paremeter MagicGateID is the one that adapter
 * has obtained from the medium by TPC
 * commands in Set Leaf ID command phase previously.
 */
int ms_mg_get_rsp_chg(struct card_reader *cr, struct card *card,
							struct scsi_cmnd *srb)
{
	struct ms_info *ms_card = card->extra;
	int retval;
	int bufflen;
	u8 lun = SCSI_LUN(srb);
	u8 buf1[32], buf2[36];

	rts_trace(TRACE_MS, "<--\n");

	ms_cleanup(cr, card);
	retval = ms_switch_clock(cr, card);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, STATUS_FAIL);
	/* Send MAKE_RMS(44h) command */
	retval = ms_mg_send_ex_cmd(cr, card, MG_MAKE_RMS, 0);
	if (retval != STATUS_SUCCESS) {
		set_sense_type(cr, lun, SENSE_TYPE_MG_KEY_FAIL_NOT_AUTHEN);
		TRACE_RET(cr, STATUS_FAIL);
	}
	/* Read data from medium */
	retval = ms_read_bytes(cr, card, PRO_READ_SHORT_DATA,
					32, WAIT_INT, buf1, 32);
	if (retval != STATUS_SUCCESS) {
		set_sense_type(cr, lun, SENSE_TYPE_MG_KEY_FAIL_NOT_AUTHEN);
		TRACE_RET(cr, STATUS_FAIL);
	}
	if (ms_check_ms_err(cr)) {
		set_sense_type(cr, lun, SENSE_TYPE_MG_KEY_FAIL_NOT_AUTHEN);
		ms_clear_error(cr);
		TRACE_RET(cr, STATUS_FAIL);
	}
	buf2[0] = 0x00;
	buf2[1] = 0x22;
	buf2[2] = 0x00;
	buf2[3] = 0x00;
	memcpy(buf2 + 4, ms_card->magic_gate_id, 16);
	memcpy(buf2 + 20, buf1, 16);
	bufflen = min(36, (int)scsi_bufflen(srb));
	scsi_stor_set_xfer_buf(buf2, bufflen, srb);
	/* scsi_set_resid(srb, (int)scsi_bufflen(srb) - bufflen); */
#ifdef READ_BYTES_WAIT_INT
	retval = ms_poll_int(cr);
	if (retval != STATUS_SUCCESS) {
		set_sense_type(cr, lun, SENSE_TYPE_MG_KEY_FAIL_NOT_AUTHEN);
		TRACE_RET(cr, STATUS_FAIL);
	}
#endif

	return STATUS_SUCCESS;
}

/*
 * Send response(host) to medium.
 * After receiving this SCSI command, adapter shall sequentially issues
 * TPC commands to the medium for writing 8-bytes data as challenge
 * by host within a short data packet.
 */
int ms_mg_rsp(struct card_reader *cr, struct card *card,
							struct scsi_cmnd *srb)
{
	struct ms_info *ms_card = card->extra;
	int retval;
	int i;
	int bufflen;
	u8 lun = SCSI_LUN(srb);
	u8 buf[32];

	rts_trace(TRACE_MS, "<--\n");

	ms_cleanup(cr, card);
	retval = ms_switch_clock(cr, card);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, STATUS_FAIL);
	/* Send MAKE_KSE(45h) command to the CMD Register */
	retval = ms_mg_send_ex_cmd(cr, card, MG_MAKE_KSE, 0);
	if (retval != STATUS_SUCCESS) {
		set_sense_type(cr, lun, SENSE_TYPE_MG_KEY_FAIL_NOT_AUTHEN);
		TRACE_RET(cr, STATUS_FAIL);
	}
	bufflen = min(12, (int)scsi_bufflen(srb));
	scsi_stor_get_xfer_buf(buf, bufflen, srb);
	for (i = 0; i < 8; i++)
		buf[i] = buf[4+i];
	for (i = 0; i < 24; i++)
		buf[8+i] = 0;
	retval = ms_write_bytes(cr, card, PRO_WRITE_SHORT_DATA,
					32, WAIT_INT, buf, 32);
	if (retval != STATUS_SUCCESS) {
		set_sense_type(cr, lun, SENSE_TYPE_MG_KEY_FAIL_NOT_AUTHEN);
		TRACE_RET(cr, STATUS_FAIL);
	}
	if (ms_check_ms_err(cr)) {
		set_sense_type(cr, lun, SENSE_TYPE_MG_KEY_FAIL_NOT_AUTHEN);
		ms_clear_error(cr);
		TRACE_RET(cr, STATUS_FAIL);
	}

	/* Indicates that the authentication process is terminated. */
	ms_card->mg_auth = 1;

	return STATUS_SUCCESS;
}

/*
 * Send ICV data to host.
 * After receiving this SCSI command, adapter shall read the divided data
 * (1024 bytes totally) from medium by using READ_LONG_DATA TPC for 2 times,
 * and report data to host with data-length is 1028 bytes.
 * Since the extra 4 bytes data is just only a prefix to original data that
 * read from medium, so that the 4-byte data pushed into Ring buffer precedes
 * data tramsinssion from medium to Ring buffer by DMA mechanisim in order
 * to get maximum performance and minimum code size simultaneously.
 */
int ms_mg_get_ICV(struct card_reader *cr, struct card *card,
							struct scsi_cmnd *srb)
{
	struct ms_info *ms_card = card->extra;
	int retval;
	int bufflen;
	u8 lun = SCSI_LUN(srb);
	u8 *buf = NULL;

	rts_trace(TRACE_MS, "<--\n");

	ms_cleanup(cr, card);
	retval = ms_switch_clock(cr, card);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, STATUS_FAIL);
	buf = (u8 *)cr_alloc_dma_buf(cr, 1028, GFP_KERNEL);
	if (!buf)
		TRACE_RET(cr, STATUS_ERROR);
	/* Push 4-bytes data in advance: Report Key Data Length(402h) */
	buf[0] = 0x04;
	buf[1] = 0x02;
	buf[2] = 0x00;
	buf[3] = 0x00;
	/* Send GET_IBD (47h) command */
	retval = ms_mg_send_ex_cmd(cr, card, MG_GET_IBD,
						ms_card->mg_entry_num);
	if (retval != STATUS_SUCCESS) {
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_UNRECOVER_READ_ERR);
		TRACE_GOTO(cr, GetICVFinish);
	}
	retval = ms_transfer_data(cr, card, MS_TM_AUTO_READ,
					PRO_READ_LONG_DATA, 2,
					WAIT_INT, 0, 0, buf + 4, 1024);
	if (retval != STATUS_SUCCESS) {
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_UNRECOVER_READ_ERR);
		ms_clear_error(cr);
		TRACE_GOTO(cr, GetICVFinish);
	}
	if (ms_check_ms_err(cr)) {
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_UNRECOVER_READ_ERR);
		ms_clear_error(cr);
		TRACE_RET(cr, STATUS_FAIL);
	}

	bufflen = min(1028, (int)scsi_bufflen(srb));
	scsi_stor_set_xfer_buf(buf, bufflen, srb);
	/* scsi_set_resid(srb, scsi_bufflen(srb) - bufflen); */
GetICVFinish:
	if (buf)
		cr_free_dma_buf(cr, buf);

	return retval;
}

/*
 * Send ICV data to medium.
 * After receiving this SCSI command, adapter shall receive 1028 bytes and
 * write the later 1024 bytes to medium by WRITE_LONG_DATA TPC consecutively.
 * Since the first 4-bytes data is just only a prefix to original data
 * that sent by host, and it should be skipped by shifting DMA pointer
 * before writing 1024 bytes to medium.
 */
int ms_mg_set_ICV(struct card_reader *cr, struct card *card,
							struct scsi_cmnd *srb)
{
	struct ms_info *ms_card = card->extra;
	int retval;
	int bufflen;
#ifdef MG_SET_ICV_SLOW
	int i;
#endif
	u8 lun = SCSI_LUN(srb);
	u8 *buf = NULL;

	rts_trace(TRACE_MS, "<--\n");

	ms_cleanup(cr, card);
	retval = ms_switch_clock(cr, card);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, STATUS_FAIL);
	buf = (u8 *)cr_alloc_dma_buf(cr, 1028, GFP_KERNEL);
	if (!buf)
		TRACE_RET(cr, STATUS_ERROR);
	bufflen = min(1028, (int)scsi_bufflen(srb));
	scsi_stor_get_xfer_buf(buf, bufflen, srb);
	/* Send SET_IBD (46h) command */
	retval = ms_mg_send_ex_cmd(cr, card, MG_SET_IBD,
						ms_card->mg_entry_num);
	if (retval != STATUS_SUCCESS) {
		if (ms_card->mg_auth == 0) {
			if ((buf[5] & 0xC0) != 0)
				set_sense_type(cr, lun,
					SENSE_TYPE_MG_KEY_FAIL_NOT_ESTAB);
			else
				set_sense_type(cr, lun,
					SENSE_TYPE_MG_WRITE_ERR);
		} else {
			set_sense_type(cr, lun, SENSE_TYPE_MG_WRITE_ERR);
		}
		TRACE_GOTO(cr, SetICVFinish);
	}
	/* Write data to medium */
#ifdef MG_SET_ICV_SLOW
	for (i = 0; i < 2; i++) {
		udelay(50);
		cr_init_cmd(cr);
		cr_add_cmd(cr, WRITE_REG_CMD, REG_MS_TPC,
					0xFF, PRO_WRITE_LONG_DATA);
		cr_add_cmd(cr, WRITE_REG_CMD, REG_MS_TRANS_CFG,
					0xFF, WAIT_INT);
		cr_add_cmd(cr, WRITE_REG_CMD,
						REG_CARD_DATA_SOURCE, 0x01,
						RING_BUFFER);
		cr_enable_dma(cr, 512, DMA_512, DMA_TO_DEVICE);
		cr_add_cmd(cr, WRITE_REG_CMD, REG_MS_TRANSFER,
					0xFF, REG_MS_TRANSFER_START |
					MS_TM_NORMAL_WRITE);
		cr_add_cmd(cr, CHECK_REG_CMD, REG_MS_TRANSFER,
						REG_MS_TRANSFER_END,
						REG_MS_TRANSFER_END);
		cr_send_cmd_no_wait(cr);
		retval = cr_transfer_data(cr, MS_CARD,
				buf + 4 + i*512, 512, 0, DMA_TO_DEVICE, 3000);
		if ((retval < 0) || ms_check_ms_err(cr)) {
			ms_clear_error(cr);
			if (ms_card->mg_auth == 0) {
				if ((buf[5] & 0xC0) != 0)
					set_sense_type(cr, lun,
						SENSE_TYPE_MG_KEY_FAIL_NOT_ESTAB);
				else
					set_sense_type(cr, lun,
						SENSE_TYPE_MG_WRITE_ERR);
			} else {
				set_sense_type(cr, lun, SENSE_TYPE_MG_WRITE_ERR);
			}
			retval = STATUS_FAIL;
			TRACE_GOTO(cr, SetICVFinish);
		}
	}
#else
	retval = ms_transfer_data(cr, card, MS_TM_AUTO_WRITE,
					PRO_WRITE_LONG_DATA, 2,
					WAIT_INT, 0, 0, buf + 4, 1024);
	if ((retval != STATUS_SUCCESS) || ms_check_ms_err(cr)) {
		ms_clear_error(cr);
		if (ms_card->mg_auth == 0) {
			if ((buf[5] & 0xC0) != 0)
				set_sense_type(cr, lun,
					SENSE_TYPE_MG_KEY_FAIL_NOT_ESTAB);
			else
				set_sense_type(cr, lun,
					SENSE_TYPE_MG_WRITE_ERR);
		} else {
			set_sense_type(cr, lun, SENSE_TYPE_MG_WRITE_ERR);
		}
		TRACE_GOTO(cr, SetICVFinish);
	}
#endif
SetICVFinish:
	if (buf)
		cr_free_dma_buf(cr, buf);
	return retval;
}

#endif /* SUPPORT_MAGIC_GATE */


