/* Driver for Realtek PCI-Express card reader
 *
 * Copyright(c) 2009 Realtek Semiconductor Corp. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *   wwang (wei_wang@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 *
 *   Tony.Nie(tony_nie@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 */

#include <linux/blkdev.h>
#include <linux/kthread.h>
#include <linux/sched.h>
#include <linux/vmalloc.h>
#include "option.h"
#include "cardreader.h"
#include "cr_regs.h"
#include "trace.h"
#include "rts_scsi.h"
#include "cards.h"
#include "scsi_cmd.h"
#include "scsi_xfer.h"
#include "sd.h"

int sd_do_send_cmd_legacy(struct card_reader *cr, struct card *card,
		struct sd_command *cmd);
int sd_process_cmd_error_4_legacy(struct card_reader *cr,
				struct card *card, struct sd_command *cmd);
int sd_retry_by_responce(struct card_reader *cr, struct sd_command *cmd);

int sd_cprm_send_cmd(struct card_reader *cr, struct card *card,
		struct sd_command *cmd);
int sd_ge_check_media_type(struct card_reader *cr, struct card *card,
		struct scsi_cmnd *srb);
int sd_ge_media_spec_function(struct card_reader *cr, struct card *card,
		struct scsi_cmnd *srb);
int sd_ge_get_sd_info(struct card_reader *cr, struct card *card,
		struct scsi_cmnd *srb);
int sd_ge_speed_class_control(struct card_reader *cr, struct card *card,
		struct scsi_cmnd *srb);
int sd_ge_change_bus_if_mode(struct card_reader *cr, struct card *card,
		struct scsi_cmnd *srb);
int sd_ge_security_transfer(struct card_reader *cr, struct card *card,
		struct scsi_cmnd *srb);

static int sd_soft_reset(struct card_reader *cr, struct card *card)
{
	return sd_reset_sd(cr, card);
}

static int sd_cprm_get_uhsii_rsp(struct card_reader *cr, struct card *card,
						int len, u8 *rsp, u8 rsp_type)
{
	cr_init_cmd(cr);

	cr_add_cmd(cr, READ_REG_CMD, REG_SD40_RX_ARG1, 0xFF, 0);

	cr_add_cmd(cr, READ_REG_CMD,  REG_SD40_RXPLD_DW0_3, 0xFF, 0);
	cr_add_cmd(cr, READ_REG_CMD,  REG_SD40_RXPLD_DW0_2, 0xFF, 0);
	cr_add_cmd(cr, READ_REG_CMD,  REG_SD40_RXPLD_DW0_1, 0xFF, 0);
	cr_add_cmd(cr, READ_REG_CMD,  REG_SD40_RXPLD_DW0_0, 0xFF, 0);

	cr_add_cmd(cr, READ_REG_CMD,  REG_SD40_RXPLD_DW1_3, 0xFF, 0);
	cr_add_cmd(cr, READ_REG_CMD,  REG_SD40_RXPLD_DW1_2, 0xFF, 0);
	cr_add_cmd(cr, READ_REG_CMD,  REG_SD40_RXPLD_DW1_1, 0xFF, 0);
	cr_add_cmd(cr, READ_REG_CMD,  REG_SD40_RXPLD_DW1_0, 0xFF, 0);

	cr_add_cmd(cr, READ_REG_CMD,  REG_SD40_RXPLD_DW2_3, 0xFF, 0);
	cr_add_cmd(cr, READ_REG_CMD,  REG_SD40_RXPLD_DW2_2, 0xFF, 0);
	cr_add_cmd(cr, READ_REG_CMD,  REG_SD40_RXPLD_DW2_1, 0xFF, 0);
	cr_add_cmd(cr, READ_REG_CMD,  REG_SD40_RXPLD_DW2_0, 0xFF, 0);

	cr_add_cmd(cr, READ_REG_CMD,  REG_SD40_RXPLD_DW3_3, 0xFF, 0);
	cr_add_cmd(cr, READ_REG_CMD,  REG_SD40_RXPLD_DW3_2, 0xFF, 0);
	cr_add_cmd(cr, READ_REG_CMD,  REG_SD40_RXPLD_DW3_1, 0xFF, 0);
	cr_add_cmd(cr, READ_REG_CMD,  REG_SD40_RXPLD_DW3_0, 0xFF, 0);

	cr_add_cmd(cr, READ_REG_CMD, REG_SD_CMD5, 0xFF, 0);
	if (cr_send_cmd(cr, SD_CARD, 100))
		TRACE_RET(cr, STATUS_FAIL);
	memcpy(rsp, cr_get_cmd_data(cr), len);

	return 0;
}

static int sd_cprm_get_legacy_rsp(struct card_reader *cr, struct card *card,
						int len, u8 *rsp, u8 rsp_type)
{
	int retval, rsp_len;
	u16 reg_addr;

	if (rsp_type == SD_RSP_TYPE_R0)
		return STATUS_SUCCESS;
	cr_init_cmd(cr);
	if (rsp_type == SD_RSP_TYPE_R2) {
		/*
		 * Read data from ping-pong buffer
		 */
		for (reg_addr = PPBUF_BASE2;
			reg_addr < PPBUF_BASE2 + 16; reg_addr++) {
			cr_add_cmd(cr, READ_REG_CMD,
					reg_addr, 0xFF, 0);
		}
		rsp_len = 17;
	} else if (rsp_type != SD_RSP_TYPE_R0) {
		/*
		 * Read data from SD_CMDx registers
		 */
		for (reg_addr = REG_SD_CMD0;
			reg_addr <= REG_SD_CMD4; reg_addr++) {
			cr_add_cmd(cr, READ_REG_CMD,
					reg_addr, 0xFF, 0);
		}
		rsp_len = 6;
	}
	cr_add_cmd(cr, READ_REG_CMD, REG_SD_CMD5, 0xFF, 0);
	retval = cr_send_cmd(cr, SD_CARD, 100);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, STATUS_FAIL);
	if (rsp) {
		int min_len = (rsp_len < len) ? rsp_len : len;
		memcpy(rsp, cr_get_cmd_data(cr), min_len);
		rts_trace(TRACE_SD, "min_len = %d\n", min_len);
		rts_trace(TRACE_SD,
			"Response in cmd buf: 0x%x 0x%x 0x%x 0x%x\n",
			rsp[0], rsp[1], rsp[2], rsp[3]);
	}

	return STATUS_SUCCESS;
}

static int sd_cprm_get_rsp(struct card_reader *cr, struct card *card,
						int len, u8 *rsp, u8 rsp_type)
{
	struct sd_info *sd_card = card->extra;
	int ret = 0;

	if (CHK_SD_UHSII(sd_card))
		ret = sd_cprm_get_uhsii_rsp(cr, card, len, rsp, rsp_type);
	else
		ret = sd_cprm_get_legacy_rsp(cr, card, len, rsp, rsp_type);

	return ret;
}

int sd_cprm_pass_thru_mode(struct card_reader *cr,
				struct card *card, struct scsi_cmnd *srb)
{
	struct sd_info *sd_card = card->extra;
	u8 lun = SCSI_LUN(srb);
	int len;
	u8 buf[18] = {
		0x00,
		0x00,
		0x00,
		0x0E,
		0x00,	/* Version Number */
		0x00,	/* WP | Media Type */
		0x00,	/* RCA (Low byte) */
		0x00,	/* RCA (High byte) */
		0x53,	/* 'S' */
		0x44,	/* 'D' */
		0x20,	/* ' ' */
		0x43,	/* 'C' */
		0x61,	/* 'a' */
		0x72,	/* 'r' */
		0x64,	/* 'd' */
		0x00,	/* Max LUN Number */
		0x00,
		0x00,
	};

	sd_card->pre_cmd_err = 0;

	if (check_dev_flag(cr, CHECK_MEDIA_CHANGE)
			&& (!check_card_flag(card, MC))) {
		set_card_flags(card, MC);
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_CHANGE);
		TRACE_RET(cr, TRANSPORT_FAILED);
	}

	if ((0x53 != srb->cmnd[2]) || (0x44 != srb->cmnd[3]) ||
			(0x20 != srb->cmnd[4]) || (0x43 != srb->cmnd[5]) ||
			(0x61 != srb->cmnd[6]) || (0x72 != srb->cmnd[7]) ||
			(0x64 != srb->cmnd[8])) {
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_INVALID_CMD_FIELD);
		TRACE_RET(cr, TRANSPORT_FAILED);
	}
	switch (srb->cmnd[1] & 0x0F) {
	case 0:
		sd_card->sd_pass_thru_en = 0;
		break;

	case 1:
		sd_card->sd_pass_thru_en = 1;
		break;

	default:
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_INVALID_CMD_FIELD);
		TRACE_RET(cr, TRANSPORT_FAILED);
	}
	/* 0x01:SD Memory Card; 0x02:Other Media; 0x03:Illegal Media; */
	buf[5] = (1 == CHK_SD(sd_card)) ?  0x01 : 0x02;
	if (check_card_flag(card, WP))
		buf[5] |= 0x80;
	buf[6] = (u8)(sd_card->sd_addr >> 16);
	buf[7] = (u8)(sd_card->sd_addr >> 24);
	buf[15] = cr->max_lun;
	len = min(18, (int)scsi_bufflen(srb));
	scsi_stor_set_xfer_buf(buf, len, srb);

	return TRANSPORT_GOOD;
}

static inline int sd_cprm_get_rsp_type(struct scsi_cmnd *srb, u8 *rsp_type,
								int *rsp_len)
{
	if (!rsp_type || !rsp_len)
		return STATUS_FAIL;

	switch (srb->cmnd[10]) {
	case 0x03:
		*rsp_type = SD_RSP_TYPE_R0; /*no response */
		*rsp_len = 0;
		break;

	case 0x04:
		*rsp_type = SD_RSP_TYPE_R1; /* R1,R6(,R4,R5) */
		*rsp_len = 6;
		break;

	case 0x05:
		*rsp_type = SD_RSP_TYPE_R1b; /* R1b */
		*rsp_len = 6;
		break;

	case 0x06:
		*rsp_type = SD_RSP_TYPE_R2; /* R2 */
		*rsp_len = 17;
		break;

	case 0x07:
		*rsp_type = SD_RSP_TYPE_R3; /* R3 */
		*rsp_len = 6;
		break;

	default:
		return STATUS_FAIL;
	}

	return STATUS_SUCCESS;
}

static u32 get_erase_timeot_value(struct card *card)
{
	struct card_reader *cr = card->cm->cr;
	struct sd_info *sd_card = card->extra;
	u8 au;
	u32 erase_block;
	u32 au_block;
	u32 erase_au;
	u32 timeout;

	if ((sd_card->sd_erase_size != 0) && (sd_card->sd_erase_timeout != 0)) {
		if (CHK_SD_UHS(sd_card) || CHK_SD_UHSII(sd_card))
			au = sd_card->sd_uhs_au_size;
		else
			au = sd_card->sd_au_size;
		switch (au) {
		case 0x01:
			au_block = 32;
			break;
		case 0x02:
			au_block = 64;
			break;
		case 0x03:
			au_block = 128;
			break;
		case 0x04:
			au_block = 256;
			break;
		case 0x05:
			au_block = 512;
			break;
		case 0x06:
			au_block = 1024;
			break;
		case 0x07:
			au_block = 2048;
			break;
		case 0x08:
			au_block = 4096;
			break;
		case 0x09:
			au_block = 8192;
			break;
		case 0x0A:
			au_block = 16384;
			break;
		case 0x0B:
			au_block = 24576;
			break;
		case 0x0C:
			au_block = 32768;
			break;
		case 0x0D:
			au_block = 49152;
			break;
		case 0x0E:
			au_block = 65536;
			break;
		case 0x0F:
			au_block = 131072;
			break;
		default:
			au_block = 0;
			rts_trace(TRACE_SD, "Au size not defined!");
			break;
		}

		if (CHK_SD_HCXC(sd_card) && au_block) {
			erase_block = sd_card->sd_erase_end_block
				- sd_card->sd_erase_start_block;
			erase_au = (erase_block+au_block-1) / au_block;
			rts_trace(TRACE_SD, "erase_au = 0x%08x\n", erase_au);

			timeout = (sd_card->sd_erase_timeout * 1000
				+ sd_card->sd_erase_size - 1) / sd_card->sd_erase_size;
			timeout = timeout * erase_au
				+ sd_card->sd_erase_offset * 1000;
			rts_trace(TRACE_SD, "timeout_calc = 0x%08x\n", timeout);

			timeout = (timeout > cr->option.cprm_cmd_timeout) ?
				timeout : cr->option.cprm_cmd_timeout;
			rts_trace(TRACE_SD, "timeout_final = 0x%08x\n",
				timeout);
		} else
			timeout = cr->option.cprm_cmd_timeout;
	} else
		timeout = cr->option.cprm_cmd_timeout;

	return timeout;
}


int sd_cprm_no_data(struct card_reader *cr, struct card *card,
						struct scsi_cmnd *srb)
{
	struct sd_info *sd_card = card->extra;
	struct sd_command cmd;
	u8 lun = SCSI_LUN(srb);
	int retval, rsp_len;
	u8 cmd_idx, rsp_type;
	u8 standby = 0, acmd = 0;
	u32 arg;

	if (!sd_card->sd_pass_thru_en) {
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_INVALID_CMD_FIELD);
		TRACE_RET(cr, TRANSPORT_FAILED);
	}

	retval = sd_switch_clock(cr, card);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, TRANSPORT_FAILED);
	if (sd_card->pre_cmd_err) {
		sd_card->pre_cmd_err = 0;
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_CHANGE);
		TRACE_RET(cr, TRANSPORT_FAILED);
	}
	cmd_idx = srb->cmnd[2] & 0x3F;
	if (srb->cmnd[1] & 0x02)
		standby = 1;
	if (srb->cmnd[1] & 0x01)
		acmd = 1;
	arg = ((u32)srb->cmnd[3] << 24) | ((u32)srb->cmnd[4] << 16) |
				((u32)srb->cmnd[5] << 8) | srb->cmnd[6];
	/* get response type */
	retval = sd_cprm_get_rsp_type(srb, &rsp_type, &rsp_len);
	if (retval != STATUS_SUCCESS) {
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_INVALID_CMD_FIELD);
		TRACE_RET(cr, TRANSPORT_FAILED);
	}
	sd_card->last_rsp_type = rsp_type;
	retval = sd_switch_clock(cr, card);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, TRANSPORT_FAILED);
	/* Set H/W SD/MMC Bus Width */
#ifdef SUPPORT_SD_LOCK
	if ((sd_card->sd_lock_status & SD_LOCK_1BIT_MODE) == 0) {
		if (CHK_MMC_8BIT(sd_card)) {
			retval = cr_wr_reg(cr, REG_SD_CFG1, 0x03,
						SD_BUS_WIDTH_8);
			if (retval != STATUS_SUCCESS)
				TRACE_RET(cr, TRANSPORT_FAILED);
		} else if (CHK_SD(sd_card) || CHK_MMC_4BIT(sd_card)) {
			retval = cr_wr_reg(cr, REG_SD_CFG1, 0x03,
						SD_BUS_WIDTH_4);
			if (retval != STATUS_SUCCESS)
				TRACE_RET(cr, TRANSPORT_FAILED);
		}
	}
#else
	retval = cr_wr_reg(cr, REG_SD_CFG1, 0x03,
						SD_BUS_WIDTH_4);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, TRANSPORT_FAILED);
#endif
	if (standby) {
		retval = sd_select_card(cr, card, 0);
		if (retval != STATUS_SUCCESS)
			TRACE_GOTO(cr, SD_Execute_Cmd_Failed);
	}

	if (cmd_idx == ERASE_WR_BLK_START)
		sd_card->sd_erase_start_block = arg;
	else if (cmd_idx == ERASE_WR_BLK_END)
		sd_card->sd_erase_end_block = arg;

	if (acmd)
		INIT_SD_CMD(cmd, cmd_idx, arg, SD_APP_CMD, rsp_type, rsp_len);
	else
		INIT_SD_CMD(cmd, cmd_idx, arg, 0, rsp_type, rsp_len);

	if ((acmd == 0) && (cmd_idx == ERASE_CMD))
		cmd.timeout = get_erase_timeot_value(card);
	else
		cmd.timeout = cr->option.cprm_cmd_timeout;

	retval = sd_cprm_send_cmd(cr, card, &cmd);
	if (retval != STATUS_SUCCESS) {
		TRACE_GOTO(cr, SD_Execute_Cmd_Failed);
	} else {
		struct sd_info *sd_card = card->extra;
		int len = min(sizeof(sd_card->rsp), sizeof(cmd.rsp));
		memcpy(sd_card->rsp, cmd.rsp, len);
	}

	if (standby) {
		retval = sd_select_card(cr, card, 1);
		if (retval != STATUS_SUCCESS)
			TRACE_GOTO(cr, SD_Execute_Cmd_Failed);
	}
#ifdef SUPPORT_SD_LOCK
	/* Get SD lock status */
	retval = sd_update_lock_status(cr, card);
	if (retval != STATUS_SUCCESS)
		TRACE_GOTO(cr, SD_Execute_Cmd_Failed);
#endif
	scsi_set_resid(srb, 0);
	return TRANSPORT_GOOD;
SD_Execute_Cmd_Failed:
	sd_card->pre_cmd_err = 1;
	set_sense_type(cr, lun, SENSE_TYPE_NO_SENSE);
	sd_release(cr, card);
	clr_card_flags(card, READY);
	rts_trace(TRACE_SD, "Reset SD for CPMR CMD fail!\n");
	process_card_reset(card);
	if (!check_card_flag(card, READY))
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_NOT_PRESENT);

	TRACE_RET(cr, TRANSPORT_FAILED);
}

static int sd_cprm_rw_uhsii_multi_blocks(struct card_reader *cr,
				struct card *card, struct scsi_cmnd *srb)
{
	struct sd_info *sd_card = card->extra;
	struct sd40_tlp tlp;
	u8 cmd_code = srb->cmnd[2] & 0x3F;
	u8 tmode = 0;

	memset(&tlp, 0, sizeof(tlp));

	tlp.header[0] = HD_DCMD | APP_PACKET | sd_card->sd40_did;
	tlp.header[1] = sd_card->sd40_sid_tid;

	if ((READ_MULTIPLE_BLOCK == cmd_code)
			|| (WRITE_MULTIPLE_BLOCK == cmd_code)) {
		/* Use LM_LEN_UNSPECIF mode for ACMD18 and ACMD25 */
		if (srb->cmnd[1] & 0x01) {
			tmode = (DAM_INC_ADDR | TLUM_BLOCK | LM_LEN_UNSPECIF | 0);
		} else {
			tmode = DAM_INC_ADDR | TLUM_BLOCK;
			tmode |=  LM_LEN_SPECIF | sd_card->lan_mode;
		}
	} else {
		tmode = (DAM_INC_ADDR | TLUM_BLOCK | LM_LEN_UNSPECIF | 0);
	}

	tlp.arg[0] = tmode;
	if (DMA_TO_DEVICE == srb->sc_data_direction)
		tlp.arg[0] |= ARG_DIR_W;
	else
		tlp.arg[0] |= ARG_DIR_R;

	tlp.arg[1] = cmd_code & 0x3F;
	if (srb->cmnd[1] & 0x01)
		tlp.arg[1] |= (1 << 6);

	tlp.payload[0] = srb->cmnd[3];
	tlp.payload[1] = srb->cmnd[4];
	tlp.payload[2] = srb->cmnd[5];
	tlp.payload[3] = srb->cmnd[6];

	tlp.payload[4] = 0;
	tlp.payload[5] = srb->cmnd[7];
	tlp.payload[6] = srb->cmnd[8];
	tlp.payload[7] = srb->cmnd[9];

	tlp.scatter_gather = 1;
	tlp.sg = scsi_sglist(srb);
	tlp.timeout = cr->option.cprm_cmd_timeout;

	return sd40_send_tlp(card, &tlp);
}

static int sd_cprm_read_legacy_multi_blocks(struct card_reader *cr,
				struct card *card, struct scsi_cmnd *srb)
{
	struct sd_info *sd_card = card->extra;
	int ret = 0, rsp_len;
	u8 cmd_idx, bus_width, acmd = 0, rsp_type = 0;
	u32 data_len;

	if (srb->cmnd[1] & 0x01)
		acmd = 1;

	if (acmd) {
		struct sd_command app_cmd;
		INIT_SD_CMD(app_cmd, APP_CMD, sd_card->sd_addr,
						0, SD_RSP_TYPE_R1, 0);
		ret = sd_cprm_send_cmd(cr, card, &app_cmd);
		if (ret)
			return ret;
	}

	cmd_idx = srb->cmnd[2] & 0x3F;
	data_len = ((u32)srb->cmnd[7] << 16) |
		((u32)srb->cmnd[8] << 8) | srb->cmnd[9];

	sd_cprm_get_rsp_type(srb, &rsp_type, &rsp_len);

	if ((sd_card->sd_lock_status & SD_LOCK_1BIT_MODE) == 0) {
		if (CHK_MMC_8BIT(sd_card))
			bus_width = SD_BUS_WIDTH_8;
		else if (CHK_SD(sd_card) || CHK_MMC_4BIT(sd_card))
			bus_width = SD_BUS_WIDTH_4;
		else
			bus_width = SD_BUS_WIDTH_1;
	} else {
		bus_width = SD_BUS_WIDTH_4;
	}

	rts_trace(TRACE_SD, "bus_width = %d\n", bus_width);

	cr_init_cmd(cr);
	cr_enable_dma(cr, data_len, DMA_512, DMA_FROM_DEVICE);
	cr_add_cmd(cr, WRITE_REG_CMD, REG_SD_BYTE_CNT_H, 0xFF, 0x02);
	cr_add_cmd(cr, WRITE_REG_CMD, REG_SD_BYTE_CNT_L, 0xFF, 0x00);
	cr_add_cmd(cr, WRITE_REG_CMD, REG_SD_BLOCK_CNT_H, 0xFF,
			(srb->cmnd[7] & 0xFE) >> 1);
	cr_add_cmd(cr, WRITE_REG_CMD, REG_SD_BLOCK_CNT_L, 0xFF,
			(u8)((data_len & 0x0001FE00) >> 9));
	cr_add_cmd(cr, WRITE_REG_CMD, REG_SD_CMD0, 0xFF, 0x40 | cmd_idx);
	cr_add_cmd(cr, WRITE_REG_CMD, REG_SD_CMD1, 0xFF, srb->cmnd[3]);
	cr_add_cmd(cr, WRITE_REG_CMD, REG_SD_CMD2, 0xFF, srb->cmnd[4]);
	cr_add_cmd(cr, WRITE_REG_CMD, REG_SD_CMD3, 0xFF, srb->cmnd[5]);
	cr_add_cmd(cr, WRITE_REG_CMD, REG_SD_CMD4, 0xFF, srb->cmnd[6]);
	cr_add_cmd(cr, WRITE_REG_CMD, REG_SD_CFG1, 0x03, bus_width);
	cr_add_cmd(cr, WRITE_REG_CMD, REG_SD_CFG2, 0xFF, rsp_type);
	cr_add_cmd(cr, WRITE_REG_CMD, REG_SD_TRANSFER,
			0xFF, SD_TM_AUTO_READ_2 | SD_TRANSFER_START);
	cr_add_cmd(cr, CHECK_REG_CMD, REG_SD_TRANSFER,
			SD_TRANSFER_END, SD_TRANSFER_END);
	cr_send_cmd_no_wait(cr);
	ret = cr_transfer_data(cr, SD_CARD,
			scsi_sglist(srb), scsi_bufflen(srb),
			scsi_sg_count(srb), DMA_FROM_DEVICE, 10000);
	if (ret < 0) {
		sd_clear_sd_error(cr);
		TRACE_RET(cr, STATUS_FAIL);
	}

	return 0;
}

static int sd_cprm_read_multi_blocks(struct card_reader *cr,
				struct card *card, struct scsi_cmnd *srb)
{
	struct sd_info *sd_card = card->extra;

	if (CHK_SD_UHSII(sd_card))
		return sd_cprm_rw_uhsii_multi_blocks(cr, card, srb);
	else
		return sd_cprm_read_legacy_multi_blocks(cr, card, srb);
}

 /* (g_bySDReadPartialBlock) Single Block Read */
static int sd_cprm_read_single_block(struct card_reader *cr,
				struct card *card, struct scsi_cmnd *srb)
{
	struct sd_command cmd;
	struct sd_info *sd_card = card->extra;
	u8 *buf;
	u8 cmd_idx, bus_width, acmd = 0;
	u32 arg, data_len, min_len;
	u16 byte_cnt, blk_cnt;

	if (srb->cmnd[1] & 0x01)
		acmd = 1;
	cmd_idx = srb->cmnd[2] & 0x3F;
	data_len = ((u32)srb->cmnd[7] << 16) |
		((u32)srb->cmnd[8] << 8) | srb->cmnd[9];

	byte_cnt = ((u16)(srb->cmnd[8] & 0x03) << 8) | srb->cmnd[9];
	blk_cnt = 1;

	if ((sd_card->sd_lock_status & SD_LOCK_1BIT_MODE) == 0) {
		if (CHK_MMC_8BIT(sd_card))
			bus_width = SD_BUS_WIDTH_8;
		else if (CHK_SD(sd_card) || CHK_MMC_4BIT(sd_card))
			bus_width = SD_BUS_WIDTH_4;
		else
			bus_width = SD_BUS_WIDTH_1;
	} else {
		bus_width = SD_BUS_WIDTH_4;
	}
	rts_trace(TRACE_SD, "bus_width = %d\n", bus_width);

	arg = (u32) (srb->cmnd[3] << 24);
	arg |= (u32) (srb->cmnd[4] << 16);
	arg |= (u32) (srb->cmnd[5] << 8);
	arg |= (u32) (srb->cmnd[6]);

	buf = (u8 *)kmalloc(data_len, GFP_KERNEL);
	if (buf == NULL)
		TRACE_RET(cr, TRANSPORT_ERROR);

	if (acmd)
		INIT_SD_CMD(cmd, 0x40 | cmd_idx, arg, SD_APP_CMD, 0, 0);
	else
		INIT_SD_CMD(cmd, 0x40 | cmd_idx, arg, 0, 0, 0);

	init_sd_dcmd_data(cmd, buf, byte_cnt, byte_cnt, blk_cnt,
			bus_width, SD_TM_NORMAL_READ, FROM_SD, 2000);

	if (sd_send_cmd(cr, card, &cmd)) {
		kfree(buf);
		sd_clear_sd_error(cr);
		TRACE_RET(cr, STATUS_FAIL);
	}

	min_len = min(data_len, scsi_bufflen(srb));
	scsi_stor_set_xfer_buf(buf, min_len, srb);
	kfree(buf);

	return 0;
}

int sd_cprm_read_data(struct card_reader *cr,
				struct card *card, struct scsi_cmnd *srb)
{
	struct sd_info *sd_card = card->extra;
	struct sd_command cmd;
	u8 lun = SCSI_LUN(srb);
	int retval, rsp_len, i;
	int read_err = 0;
	u8 rsp_type, send_cmd12 = 0, standby = 0;
	u32 data_len;

	if (!sd_card->sd_pass_thru_en) {
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_INVALID_CMD_FIELD);
		TRACE_RET(cr, TRANSPORT_FAILED);
	}

	if (sd_card->pre_cmd_err) {
		sd_card->pre_cmd_err = 0;
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_CHANGE);
		TRACE_RET(cr, TRANSPORT_FAILED);
	}

	retval = sd_switch_clock(cr, card);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, TRANSPORT_FAILED);
	if (srb->cmnd[1] & 0x04)
		send_cmd12 = 1;
	if (srb->cmnd[1] & 0x02)
		standby = 1;
	data_len = ((u32)srb->cmnd[7] << 16) | ((u32)srb->cmnd[8] << 8) |
								srb->cmnd[9];
	/* get response type */
	retval = sd_cprm_get_rsp_type(srb, &rsp_type, &rsp_len);
	if (retval != STATUS_SUCCESS) {
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_INVALID_CMD_FIELD);
		TRACE_RET(cr, TRANSPORT_FAILED);
	}
	sd_card->last_rsp_type = rsp_type;
	retval = sd_switch_clock(cr, card);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, TRANSPORT_FAILED);

	if ((data_len < 512) && !(CHK_SD_UHSII(sd_card))) {
		INIT_SD_CMD(cmd, SET_BLOCKLEN, data_len, 0, SD_RSP_TYPE_R1, 0);
		retval = sd_cprm_send_cmd(cr, card, &cmd);
		if (retval != STATUS_SUCCESS)
			TRACE_GOTO(cr, SD_Execute_Read_Cmd_Failed);
	}

	if (standby) {
		retval = sd_select_card(cr, card, 0);
		if (retval != STATUS_SUCCESS)
			TRACE_GOTO(cr, SD_Execute_Read_Cmd_Failed);
	}

	if (data_len <= 512) {
		if (sd_cprm_read_single_block(cr, card, srb)) {
			read_err = 1;
			TRACE_GOTO(cr, SD_Execute_Read_Cmd_Failed);
		}
	} else if (!(data_len & 0x1FF)) {
		if (sd_cprm_read_multi_blocks(cr, card, srb)) {
			read_err = 1;
			TRACE_GOTO(cr, SD_Execute_Read_Cmd_Failed);
		}
	} else {
		TRACE_GOTO(cr, SD_Execute_Read_Cmd_Failed);
	}

	retval = sd_cprm_get_rsp(cr, card, rsp_len,
					sd_card->rsp, rsp_type);
	if (retval != STATUS_SUCCESS)
		TRACE_GOTO(cr, SD_Execute_Read_Cmd_Failed);

	if (standby) {
		retval = sd_select_card(cr, card, 1);
		if (retval != STATUS_SUCCESS)
			TRACE_GOTO(cr, SD_Execute_Read_Cmd_Failed);
	}

	if (send_cmd12) {
		INIT_SD_CMD(cmd, STOP_TRANSMISSION, 0, 0, SD_RSP_TYPE_R1b, 0);
		cmd.timeout = 3000;
		retval = sd_cprm_send_cmd(cr, card, &cmd);
		if (retval != STATUS_SUCCESS)
			TRACE_GOTO(cr, SD_Execute_Read_Cmd_Failed);
	}

	if ((data_len < 512) && !(CHK_SD_UHSII(sd_card))) {
		INIT_SD_CMD(cmd, SET_BLOCKLEN, 0x200, 0, SD_RSP_TYPE_R1, 0);
		retval = sd_cprm_send_cmd(cr, card, &cmd);
		if (retval != STATUS_SUCCESS)
			TRACE_GOTO(cr, SD_Execute_Read_Cmd_Failed);
		retval = cr_wr_reg(cr, REG_SD_BYTE_CNT_H, 0xFF, 0x02);
		if (retval != STATUS_SUCCESS)
			TRACE_GOTO(cr, SD_Execute_Read_Cmd_Failed);
		retval = cr_wr_reg(cr, REG_SD_BYTE_CNT_L, 0xFF, 0x00);
		if (retval != STATUS_SUCCESS)
			TRACE_GOTO(cr, SD_Execute_Read_Cmd_Failed);
	}

	for (i = 0; i < 3; i++) {
		int flag = 0;

		/* There is CMD7 or CMD12 sent before CMD13 */
		if ((srb->cmnd[1] & 0x02) || (srb->cmnd[1] & 0x04))
			flag = SD_SKIP_OVER_RNAGE_ERR;
		INIT_SD_CMD(cmd, SEND_STATUS, sd_card->sd_addr,
				flag, SD_RSP_TYPE_R1, 0);
		retval = sd_cprm_send_cmd(cr, card, &cmd);
		if (retval == STATUS_SUCCESS)
			break;
	}

	if (retval != STATUS_SUCCESS)
		TRACE_GOTO(cr, SD_Execute_Read_Cmd_Failed);

	scsi_set_resid(srb, 0);

	return TRANSPORT_GOOD;

SD_Execute_Read_Cmd_Failed:
	sd_card->pre_cmd_err = 1;
	set_sense_type(cr, lun, SENSE_TYPE_NO_SENSE);
	if (read_err)
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_UNRECOVER_READ_ERR);
	sd_release(cr, card);
	clr_card_flags(card, READY);
	rts_trace(TRACE_SD, "Reset SD for CPMR CMD fail!\n");
	process_card_reset(card);
	if (!check_card_flag(card, READY))
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_NOT_PRESENT);

	TRACE_RET(cr, TRANSPORT_FAILED);
}


static int sd_cprm_write_legacy_multil_blocks(struct card_reader *cr,
				struct card *card, struct scsi_cmnd *srb)
{
	struct sd_command cmd;
	struct sd_info *sd_card = card->extra;
	u8 cmd_idx, bus_width, acmd = 0, rsp_type = 0;
	u32 arg, data_len;
	u16 blk_cnt;
	int rsp_len = 0;

	sd_cprm_get_rsp_type(srb, &rsp_type, &rsp_len);

	if (srb->cmnd[1] & 0x01)
		acmd = 1;
	cmd_idx = srb->cmnd[2] & 0x3F;
	data_len = ((u32)srb->cmnd[7] << 16) |
		((u32)srb->cmnd[8] << 8) | srb->cmnd[9];
	blk_cnt = 1;

	if ((sd_card->sd_lock_status & SD_LOCK_1BIT_MODE) == 0) {
		if (CHK_MMC_8BIT(sd_card))
			bus_width = SD_BUS_WIDTH_8;
		else if (CHK_SD(sd_card) || CHK_MMC_4BIT(sd_card))
			bus_width = SD_BUS_WIDTH_4;
		else
			bus_width = SD_BUS_WIDTH_1;
	} else {
		bus_width = SD_BUS_WIDTH_4;
	}
	rts_trace(TRACE_SD, "bus_width = %d\n", bus_width);

	arg = (u32) (srb->cmnd[3] << 24);
	arg |= (u32) (srb->cmnd[4] << 16);
	arg |= (u32) (srb->cmnd[5] << 8);
	arg |= (u32) (srb->cmnd[6]);

	if (acmd)
		INIT_SD_CMD(cmd, 0x40 | cmd_idx, arg, SD_APP_CMD, rsp_type, rsp_len);
	else
		INIT_SD_CMD(cmd, 0x40 | cmd_idx, arg, 0, rsp_type, rsp_len);

	if (sd_cprm_send_cmd(cr, card, &cmd)) {
		sd_clear_sd_error(cr);
		TRACE_RET(cr, STATUS_FAIL);
	}

	cr_init_cmd(cr);
	cr_enable_dma(cr, data_len, DMA_512, DMA_TO_DEVICE);
	cr_add_cmd(cr, WRITE_REG_CMD, REG_SD_BYTE_CNT_H, 0xFF, 0x02);
	cr_add_cmd(cr, WRITE_REG_CMD, REG_SD_BYTE_CNT_L, 0xFF, 0x00);
	cr_add_cmd(cr, WRITE_REG_CMD, REG_SD_BLOCK_CNT_H, 0xFF,
					(srb->cmnd[7] & 0xFE) >> 1);
	cr_add_cmd(cr, WRITE_REG_CMD,
		REG_SD_BLOCK_CNT_L, 0xFF, (u8)((data_len & 0x0001FE00) >> 9));
	cr_add_cmd(cr, WRITE_REG_CMD,
		REG_SD_TRANSFER, 0xFF, SD_TM_AUTO_WRITE_3 | SD_TRANSFER_START);
	cr_add_cmd(cr, CHECK_REG_CMD,
		REG_SD_TRANSFER, SD_TRANSFER_END, SD_TRANSFER_END);
	cr_send_cmd_no_wait(cr);
	return  cr_transfer_data(cr, SD_CARD,
			scsi_sglist(srb), scsi_bufflen(srb),
			scsi_sg_count(srb), DMA_TO_DEVICE, 10000);
}

static int sd_cprm_write_multil_blocks(struct card_reader *cr,
				struct card *card, struct scsi_cmnd *srb)
{
	struct sd_info *sd_card = card->extra;

	if (CHK_SD_UHSII(sd_card))
		return sd_cprm_rw_uhsii_multi_blocks(cr, card, srb);
	else
		return sd_cprm_write_legacy_multil_blocks(cr, card, srb);
}

 /* (g_bySDReadPartialBlock) Single Block Write */
static int sd_cprm_write_single_block(struct card_reader *cr,
				struct card *card, struct scsi_cmnd *srb)
{
	struct sd_command cmd;
	struct sd_info *sd_card = card->extra;
	u8 *buf;
	u8 cmd_idx, bus_width, acmd = 0, rsp_type = 0;
	u32 arg, data_len;
	u16 byte_cnt, blk_cnt;
	int rsp_len = 0;

	sd_cprm_get_rsp_type(srb, &rsp_type, &rsp_len);

	if (srb->cmnd[1] & 0x01)
		acmd = 1;
	cmd_idx = srb->cmnd[2] & 0x3F;
	data_len = ((u32)srb->cmnd[7] << 16) |
		((u32)srb->cmnd[8] << 8) | srb->cmnd[9];

	byte_cnt = ((u16)(srb->cmnd[8] & 0x03) << 8) | srb->cmnd[9];
	blk_cnt = 1;

	if ((sd_card->sd_lock_status & SD_LOCK_1BIT_MODE) == 0) {
		if (CHK_MMC_8BIT(sd_card))
			bus_width = SD_BUS_WIDTH_8;
		else if (CHK_SD(sd_card) || CHK_MMC_4BIT(sd_card))
			bus_width = SD_BUS_WIDTH_4;
		else
			bus_width = SD_BUS_WIDTH_1;
	} else {
		bus_width = SD_BUS_WIDTH_4;
	}
	rts_trace(TRACE_SD, "bus_width = %d\n", bus_width);

	arg = (u32) (srb->cmnd[3] << 24);
	arg |= (u32) (srb->cmnd[4] << 16);
	arg |= (u32) (srb->cmnd[5] << 8);
	arg |= (u32) (srb->cmnd[6]);

	buf = (u8 *)kmalloc(data_len, GFP_KERNEL);
	if (buf == NULL)
		TRACE_RET(cr, TRANSPORT_ERROR);
	scsi_stor_get_xfer_buf(buf, data_len, srb);

	if (acmd)
		INIT_SD_CMD(cmd, 0x40 | cmd_idx, arg, SD_APP_CMD, rsp_type,
			rsp_len);
	else
		INIT_SD_CMD(cmd, 0x40 | cmd_idx, arg, 0, rsp_type, rsp_len);

	init_sd_dcmd_data(cmd, buf, byte_cnt, byte_cnt, blk_cnt,
			bus_width, SD_TM_AUTO_WRITE_2, TO_SD, 2000);

	if (sd_send_cmd(cr, card, &cmd)) {
		kfree(buf);
		sd_clear_sd_error(cr);
		TRACE_RET(cr, STATUS_FAIL);
	}

	kfree(buf);

	return 0;
}

int sd_cprm_write_data(struct card_reader *cr,
				struct card *card, struct scsi_cmnd *srb)
{
	struct sd_info *sd_card = card->extra;
	u8 lun = SCSI_LUN(srb);
	int retval, rsp_len, i;
	int write_err = 0;
	u8 cmd_idx, rsp_type;
	u8 send_cmd12 = 0, standby = 0;
	u32 data_len;
	struct sd_command cmd;
	int lock_cmd_fail = 0;
	u8 sd_lock_state = 0;
	u8 lock_cmd_type = 0;

	if (!sd_card->sd_pass_thru_en) {
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_INVALID_CMD_FIELD);
		TRACE_RET(cr, TRANSPORT_FAILED);
	}
	if (sd_card->pre_cmd_err) {
		sd_card->pre_cmd_err = 0;
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_CHANGE);
		TRACE_RET(cr, TRANSPORT_FAILED);
	}
	retval = sd_switch_clock(cr, card);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, TRANSPORT_FAILED);
	cmd_idx = srb->cmnd[2] & 0x3F;
	if (srb->cmnd[1] & 0x04)
		send_cmd12 = 1;
	if (srb->cmnd[1] & 0x02)
		standby = 1;
	data_len = ((u32)srb->cmnd[7] << 16) |
				((u32)srb->cmnd[8] << 8) | srb->cmnd[9];
#ifdef SUPPORT_SD_LOCK
	if (cmd_idx == LOCK_UNLOCK) {
		sd_lock_state = sd_card->sd_lock_status;
		sd_lock_state &= SD_LOCKED;  /* Only SD_LOCKED bit is cared */
	}
#endif
	/* get response type */
	retval = sd_cprm_get_rsp_type(srb, &rsp_type, &rsp_len);
	if (retval != STATUS_SUCCESS) {
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_INVALID_CMD_FIELD);
		TRACE_RET(cr, TRANSPORT_FAILED);
	}
	sd_card->last_rsp_type = rsp_type;
	retval = sd_switch_clock(cr, card);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, TRANSPORT_FAILED);
	/* Set H/W SD/MMC Bus Width */
#ifdef SUPPORT_SD_LOCK
	if ((sd_card->sd_lock_status & SD_LOCK_1BIT_MODE) == 0) {
		if (CHK_MMC_8BIT(sd_card)) {
			retval = cr_wr_reg(cr, REG_SD_CFG1, 0x03,
						SD_BUS_WIDTH_8);
			if (retval != STATUS_SUCCESS)
				TRACE_RET(cr, TRANSPORT_FAILED);
		} else if (CHK_SD(sd_card) || CHK_MMC_4BIT(sd_card)) {
			retval = cr_wr_reg(cr, REG_SD_CFG1, 0x03,
						SD_BUS_WIDTH_4);
			if (retval != STATUS_SUCCESS)
				TRACE_RET(cr, TRANSPORT_FAILED);
		}
	}
#else
	retval = cr_wr_reg(cr, REG_SD_CFG1, 0x03, SD_BUS_WIDTH_4);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, TRANSPORT_FAILED);
#endif
	if ((data_len < 512) && !(CHK_SD_UHSII(sd_card))) {
		INIT_SD_CMD(cmd, SET_BLOCKLEN, data_len, 0, SD_RSP_TYPE_R1, 5);
		retval = sd_cprm_send_cmd(cr, card, &cmd);
		if (retval != STATUS_SUCCESS)
			TRACE_GOTO(cr, SD_Execute_Write_Cmd_Failed);
	}

	if (standby) {
		retval = sd_select_card(cr, card, 0);
		if (retval != STATUS_SUCCESS)
			TRACE_GOTO(cr, SD_Execute_Write_Cmd_Failed);
	}

	if (data_len <= 512) {
		if (cmd_idx == LOCK_UNLOCK) {
			u8 *buf = kmalloc(data_len, GFP_KERNEL);

			if (buf == NULL)
				TRACE_RET(cr, TRANSPORT_ERROR);
			scsi_stor_get_xfer_buf(buf, data_len, srb);
			lock_cmd_type = buf[0] & 0x0F;
			scsi_stor_set_xfer_buf(buf, data_len, srb);
			kfree(buf);
		}

		retval = sd_cprm_write_single_block(cr, card, srb);
	} else if (!(data_len & 0x1FF)) {
		retval = sd_cprm_write_multil_blocks(cr, card, srb);
	} else {
		TRACE_GOTO(cr, SD_Execute_Write_Cmd_Failed);
	}

	if (retval < 0) {
		write_err = 1;
		sd_clear_sd_error(cr);
		TRACE_GOTO(cr, SD_Execute_Write_Cmd_Failed);
	}

	retval = sd_cprm_get_rsp(cr, card, rsp_len,
					sd_card->rsp, rsp_type);
	if (retval != STATUS_SUCCESS)
		TRACE_GOTO(cr, SD_Execute_Write_Cmd_Failed);

#ifdef SUPPORT_SD_LOCK
	if (cmd_idx == LOCK_UNLOCK) {
		/*
		 * The totally erase time depend on the capacity of
		 * the erased card,it may be takes several seconds or
		 * even up to more than 1 minute.
		 */
		if (lock_cmd_type == SD_ERASE) {
			/*
			 * report SD_ERASE successfully.
			 */
			sd_card->sd_erase_status = SD_UNDER_ERASING;
			scsi_set_resid(srb, 0);
			return TRANSPORT_GOOD;
		}
		/*
		 * CMD42 for some card (Panasonic SDHC 4GB) may last for
		 * a very long time(~200ms), We should wait CMD42 finished,
		 * otherwise following CMD may timeout.
		 */
		cr_init_cmd(cr);
		cr_add_cmd(cr, CHECK_REG_CMD, REG_SD_BUS_STAT,
						SD_DAT0_STATUS,
						SD_DAT0_STATUS);
		/* Don't care return value */
		cr_send_cmd(cr, SD_CARD, 250);
		/*
		 * Get SD lock status: CMD13(SEND_STATUS) should be issued
		 * immediately after CMD42
		 */
		retval = sd_update_lock_status(cr, card);
		if (retval != STATUS_SUCCESS) {
			rts_trace(TRACE_SD, "Lock command fail!\n");
			lock_cmd_fail = 1;
		}
	}
#endif /* SUPPORT_SD_LOCK */

	if (standby) {
		retval = sd_select_card(cr, card, 1);
		if (retval != STATUS_SUCCESS)
			TRACE_GOTO(cr, SD_Execute_Write_Cmd_Failed);
	}

	if (send_cmd12) {
		INIT_SD_CMD(cmd, STOP_TRANSMISSION, 0, 0, SD_RSP_TYPE_R1b, 0);
		cmd.timeout = 3000;
		retval = sd_cprm_send_cmd(cr, card, &cmd);
		if (retval != STATUS_SUCCESS)
			TRACE_GOTO(cr, SD_Execute_Write_Cmd_Failed);
	}

	if ((data_len < 512) && !(CHK_SD_UHSII(sd_card))) {
		INIT_SD_CMD(cmd, SET_BLOCKLEN, 0x200, 0, SD_RSP_TYPE_R1, 0);
		retval = sd_cprm_send_cmd(cr, card, &cmd);
		if (retval != STATUS_SUCCESS)
			TRACE_GOTO(cr, SD_Execute_Write_Cmd_Failed);
		retval = cr_wr_reg(cr, REG_SD_BYTE_CNT_H, 0xFF, 0x02);
		if (retval != STATUS_SUCCESS)
			TRACE_GOTO(cr, SD_Execute_Write_Cmd_Failed);
		cr_wr_reg(cr, REG_SD_BYTE_CNT_L, 0xFF, 0x00);
		if (retval != STATUS_SUCCESS)
			TRACE_GOTO(cr, SD_Execute_Write_Cmd_Failed);
	}


	for (i = 0; i < 3; i++) {
		int flag = 0;

		/* There is CMD7 or CMD12 sent before CMD13 */
		if ((srb->cmnd[1] & 0x02) || (srb->cmnd[1] & 0x04))
			flag = SD_SKIP_OVER_RNAGE_ERR;
		INIT_SD_CMD(cmd, SEND_STATUS, sd_card->sd_addr,
				flag, SD_RSP_TYPE_R1, 0);
		retval = sd_cprm_send_cmd(cr, card, &cmd);
		if (retval == STATUS_SUCCESS)
			break;
	}
	if (retval != STATUS_SUCCESS)
		TRACE_GOTO(cr, SD_Execute_Write_Cmd_Failed);

#ifdef SUPPORT_SD_LOCK
	if (cmd_idx == LOCK_UNLOCK) {
		if (!lock_cmd_fail) {
			rts_trace(TRACE_SD, "lock_cmd_type = 0x%x\n",
							lock_cmd_type);
			if (lock_cmd_type & SD_CLR_PWD)
				sd_card->sd_lock_status &= ~SD_PWD_EXIST;
			if (lock_cmd_type & SD_SET_PWD)
				sd_card->sd_lock_status |= SD_PWD_EXIST;
		}
		rts_trace(TRACE_SD, "sd_lock_state = 0x%x,"
				"sd_card->sd_lock_status = 0x%x\n",
				sd_lock_state, sd_card->sd_lock_status);
		if (sd_lock_state ^ (sd_card->sd_lock_status & SD_LOCKED)) {
			/*
			 * locked state has been changed!
			 */
			sd_card->sd_lock_notify = 1;
			if (sd_lock_state) {
				/*
				 * Locked -> Open
				 */
				if (sd_card->sd_lock_status &
						SD_LOCK_1BIT_MODE) {
					/*
					 * Reset SD card with SDR mode.
					 */
					sd_card->sd_lock_status |=
						(SD_UNLOCK_POW_ON | SD_SDR_RST);
					/*
					 * Need to reset SD card. Password is
					 * preserved in Ring buffer.
					 * Reset SD or MMC Card
					 */
					if (CHK_SD(sd_card)) {
						retval = sd_reset_sd(cr, card);
						if (retval != STATUS_SUCCESS) {
							sd_card->sd_lock_status &=
								~(SD_UNLOCK_POW_ON | SD_SDR_RST);
							TRACE_GOTO(cr, SD_Execute_Write_Cmd_Failed);
						}
					}
					sd_card->sd_lock_status &=
						~(SD_UNLOCK_POW_ON | SD_SDR_RST);
				}
			}
		}
	}
	if (lock_cmd_fail) {
		scsi_set_resid(srb, 0);
		set_sense_type(cr, lun, SENSE_TYPE_NO_SENSE);
		TRACE_RET(cr, TRANSPORT_FAILED);
	}
#endif  /* SUPPORT_SD_LOCK */
	scsi_set_resid(srb, 0);

	return TRANSPORT_GOOD;
SD_Execute_Write_Cmd_Failed:
	sd_card->pre_cmd_err = 1;
	set_sense_type(cr, lun, SENSE_TYPE_NO_SENSE);
	if (write_err)
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_WRITE_ERR);
	sd_release(cr, card);
	clr_card_flags(card, READY);
	rts_trace(TRACE_SD, "Reset SD for CPMR CMD fail!\n");
	process_card_reset(card);
	if (!check_card_flag(card, READY))
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_NOT_PRESENT);

	TRACE_RET(cr, TRANSPORT_FAILED);
}

int sd_cprm_get_cmd_rsp(struct card_reader *cr, struct card *card,
							struct scsi_cmnd *srb)
{
	struct sd_info *sd_card = card->extra;
	u8 lun = SCSI_LUN(srb);
	int count;
	u16 data_len;

	if (!sd_card->sd_pass_thru_en) {
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_INVALID_CMD_FIELD);
		TRACE_RET(cr, TRANSPORT_FAILED);
	}
	if (sd_card->pre_cmd_err) {
		sd_card->pre_cmd_err = 0;
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_CHANGE);
		TRACE_RET(cr, TRANSPORT_FAILED);
	}
	data_len = ((u16)srb->cmnd[7] << 8) | srb->cmnd[8];
	if (sd_card->last_rsp_type == SD_RSP_TYPE_R0) {
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_INVALID_CMD_FIELD);
		TRACE_RET(cr, TRANSPORT_FAILED);
	} else if (sd_card->last_rsp_type == SD_RSP_TYPE_R2) {
		count = (data_len < 17) ? data_len : 17;
	} else {
		count = (data_len < 6) ? data_len : 6;
	}

	scsi_stor_set_xfer_buf(sd_card->rsp, count, srb);
	rts_trace(TRACE_SD, "Response length: %d\n", data_len);
	rts_trace(TRACE_SD, "Response:  0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x\n",
					sd_card->rsp[0], sd_card->rsp[1],
					sd_card->rsp[2], sd_card->rsp[3],
					sd_card->rsp[4], sd_card->rsp[5],
					sd_card->rsp[6], sd_card->rsp[7],
					sd_card->rsp[8], sd_card->rsp[9],
					sd_card->rsp[10], sd_card->rsp[11],
					sd_card->rsp[12], sd_card->rsp[13],
					sd_card->rsp[14], sd_card->rsp[15]
	);

	scsi_set_resid(srb, 0);

	return TRANSPORT_GOOD;
}

int sd_cprm_hw_rst(struct card_reader *cr, struct card *card,
						struct scsi_cmnd *srb)
{
	struct sd_info *sd_card = card->extra;
	u8 lun = SCSI_LUN(srb);
	int retval;

	if (!sd_card->sd_pass_thru_en) {
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_INVALID_CMD_FIELD);
		TRACE_RET(cr, TRANSPORT_FAILED);
	}
	if (sd_card->pre_cmd_err) {
		sd_card->pre_cmd_err = 0;
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_CHANGE);
		TRACE_RET(cr, TRANSPORT_FAILED);
	}
	if ((0x53 != srb->cmnd[2]) || (0x44 != srb->cmnd[3]) ||
			(0x20 != srb->cmnd[4]) || (0x43 != srb->cmnd[5]) ||
			(0x61 != srb->cmnd[6]) || (0x72 != srb->cmnd[7]) ||
			(0x64 != srb->cmnd[8])) {
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_INVALID_CMD_FIELD);
		TRACE_RET(cr, TRANSPORT_FAILED);
	}
	switch (srb->cmnd[1] & 0x0F) {
	case 0:
		/*
		 * SD Card Power Off -> ON and Initialization
		 */
#ifdef SUPPORT_SD_LOCK
		if (0x64 == srb->cmnd[9])
			/*
			 * Command Mode
			 */
			sd_card->sd_lock_status |= SD_SDR_RST;
#endif /* SUPPORT_SD_LOCK */
		retval = sd_reset_card(cr, card);
		if (retval != STATUS_SUCCESS) {
#ifdef SUPPORT_SD_LOCK
			sd_card->sd_lock_status &= ~SD_SDR_RST;
#endif
			set_sense_type(cr, lun, SENSE_TYPE_MEDIA_NOT_PRESENT);
			sd_card->pre_cmd_err = 1;
			TRACE_RET(cr, TRANSPORT_FAILED);
		}
#ifdef SUPPORT_SD_LOCK
		sd_card->sd_lock_status &= ~SD_SDR_RST;
#endif
		break;

	case 1:
		/* reset CMD(GO_IDLE_STATE) and Initialization
		 * (without SD Card Power Off -> ON) */
		retval = sd_soft_reset(cr, card);
		if (retval != STATUS_SUCCESS) {
			set_sense_type(cr, lun, SENSE_TYPE_MEDIA_NOT_PRESENT);
			sd_card->pre_cmd_err = 1;
			TRACE_RET(cr, TRANSPORT_FAILED);
		}
		break;

	default:
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_INVALID_CMD_FIELD);
		TRACE_RET(cr, TRANSPORT_FAILED);
	}

	scsi_set_resid(srb, 0);

	return TRANSPORT_GOOD;
}

static int sd_cprm_check_card_status(struct card_reader *cr,
				struct card *card, struct sd_command *cmd)
{
	u8 *rsp = (u8 *) cmd->rsp;

	/* Check Status */
	if ((SELECT_CARD == cmd->opcode) || (APP_CMD == cmd->opcode)
			|| (SEND_STATUS == cmd->opcode)
			|| (STOP_TRANSMISSION == cmd->opcode)) {
		if (cmd->opcode != STOP_TRANSMISSION
				&& !(cmd->flags & SD_SKIP_OVER_RNAGE_ERR)) {
			if (rsp[1] & 0x80)
				TRACE_RET(cr, STATUS_FAIL);
		}

		/* exclude bit25 CARD_IS_LOCKED */
		if (rsp[1] & 0x7D) {
			rts_trace(TRACE_SD, "rsp[1]: 0x%02x\n", rsp[1]);
			TRACE_RET(cr, STATUS_FAIL);
		}

		if (rsp[2] & 0xF8) {
			rts_trace(TRACE_SD, "rsp[2]: 0x%02x\n", rsp[2]);
			TRACE_RET(cr, STATUS_FAIL);
		}

		if (SELECT_CARD == cmd->opcode) {
			u8 rsp_type = cmd->rsp_type;
			if (SD_RSP_TYPE_R2 == rsp_type) {
				if ((rsp[3] & 0x1E) != 0x04)
					TRACE_RET(cr, STATUS_FAIL);
			} else if (SD_RSP_TYPE_R2 == rsp_type) {
				if ((rsp[3] & 0x1E) != 0x03)
					TRACE_RET(cr, STATUS_FAIL);
			}

		}
	}

	return 0;
}

static int sd_cprm_check_responce(struct card_reader *cr,
				struct card *card, struct sd_command *cmd)
{
	u8 rsp_type = cmd->rsp_type;
	u8 *rsp = (u8 *) cmd->rsp;
	u8 sd_status1;

	cr_rd_reg(cr, REG_SD_STAT1, &sd_status1);

	/* Check (Start,Transmission) bit of Response */
	if ((rsp[0] & 0xC0) != 0) {
		sd_set_error_code(cr, card, SD_STS_ERR);
		TRACE_RET(cr, STATUS_FAIL);
	}

	/* Check CRC7 */
	if (!(rsp_type & SD_NO_CHECK_CRC7)) {
		if (sd_status1 & SD_CRC7_ERR) {
			sd_set_error_code(cr, card, SD_CRC_ERR);
			TRACE_RET(cr, STATUS_FAIL);
		}
	}


	return sd_cprm_check_card_status(cr, card, cmd);
}

static int sd_cprm_send_legacy_ccmd(struct card_reader *cr, struct card *card,
		struct sd_command *cmd)
{
	int ret;
	unsigned int rty_cnt = 0;

	sd_clear_error_code(cr, card);

	/* for ACMD, send CMD55 first */
	if (cmd->flags & SD_APP_CMD) {
		struct sd_command app_cmd;
		struct sd_info *sd_card = card->extra;

		INIT_SD_CMD(app_cmd, APP_CMD, sd_card->sd_addr,
						0, SD_RSP_TYPE_R1, 0);
		ret = sd_cprm_send_legacy_ccmd(cr, card, &app_cmd);
		if (ret)
			return ret;
	}

	do {
		ret = sd_do_send_cmd_legacy(cr, card, cmd);
		if (ret) {
			sd_process_cmd_error_4_legacy(cr, card, cmd);
			break;
		}

		if (cmd->rsp_type == SD_RSP_TYPE_R0)
			break;

		ret = sd_cprm_check_responce(cr, card, cmd);
		if (ret) {
			int need_retry = sd_retry_by_responce(cr, cmd);
			if (need_retry) {
				if (rty_cnt++ < cmd->max_retries) {
					msleep(20);
					continue;
				}
			}
		}

		break;
	} while (1);

	if (!ret) {
		/*struct sd_info *sd_card = card->extra;
		int len = min(sizeof(sd_card->rsp), sizeof(cmd->rsp));
		memcpy(sd_card->rsp, cmd->rsp, len);*/
	}

	return ret;
}

static int sd_cprm_send_uhsii_ccmd(struct card_reader *cr, struct card *card,
		struct sd_command *cmd)
{
	int ret = 0;

	ret = sd40_send_sdtran_ccmd(card, cmd);

	if (!ret) {
		/*struct sd_info *sd_card = card->extra;
		int len = min(sizeof(sd_card->rsp), sizeof(cmd->rsp));
		memcpy(sd_card->rsp, cmd->rsp, len);*/

		return sd_cprm_check_card_status(cr, card, cmd);
	}

	return ret;
}

static int sd_cprm_send_ccmd(struct card_reader *cr, struct card *card,
		struct sd_command *cmd)
{
	struct sd_info *sd_card = card->extra;

	if (CHK_SD_UHSII(sd_card))
		return sd_cprm_send_uhsii_ccmd(cr, card, cmd);
	else
		return sd_cprm_send_legacy_ccmd(cr, card, cmd);
}

int sd_cprm_send_cmd(struct card_reader *cr, struct card *card,
		struct sd_command *cmd)
{
	rts_trace(TRACE_SD, "SD/MMC CMD %d, arg = 0x%08x\n",
					cmd->opcode & 0x3F, cmd->arg);

	if (cmd->flags & SD_CMD_WITH_DATA) {
		/*TODO: */
		return -1;
	} else {
		return sd_cprm_send_ccmd(cr, card, cmd);
	}
}


/* For SeeQVault */
static unsigned char security_protocol_list[] = {
	/* reserved */
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	/* supported security protocol list length */
	0x00, 0x02,
	/* supported security protocol list */
	0x00, 0xFE
}; /* Security Protocol List */

static unsigned char certificate_data[] =
{
	/* reserved */
	0x00, 0x00,
	/* certificate length */
	0x00, 0x00,
	/* certificate data */
	0x00
};

int sd_ge_check_media_type(struct card_reader *cr, struct card *card,
		struct scsi_cmnd *srb)
{
	struct sd_info *sd_card = card->extra;
	u8 gesf = sd_card->ge.ge_card ? 1 : 0;
	u8 lun = SCSI_LUN(srb);
	u8 info = 0x02 | gesf;
	u16 alloc_len = cr_get_be16(&srb->cmnd[4]);

	if (srb->cmnd[0] != 0xEC || alloc_len != 1) {
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_INVALID_CMD_FIELD);
		TRACE_RET(cr, TRANSPORT_FAILED);
	}

	scsi_check_xfer_len(srb, alloc_len);
	scsi_stor_set_xfer_buf(&info, alloc_len, srb);
	scsi_set_resid(srb, 0);
	return TRANSPORT_GOOD;
}

int sd_ge_media_spec_function(struct card_reader *cr, struct card *card,
		struct scsi_cmnd *srb)
{
	u8 lun = SCSI_LUN(srb);
	int err = 0;

	if (srb->cmnd[0] != 0xED) {
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_INVALID_CMD_FIELD);
		TRACE_RET(cr, TRANSPORT_FAILED);
	}

	switch(srb->cmnd[1]) {
	case MSF_GET_SD_INFO:
		err = sd_ge_get_sd_info(cr, card, srb);
		break;
	case MSF_SPEED_CLASS_CONTROL:
		err = sd_ge_speed_class_control(cr, card, srb);
		break;
	case MSF_CHANGE_BUS_IF_MODE:
		err = sd_ge_change_bus_if_mode(cr, card, srb);
		break;
	default:
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_INVALID_CMD_FIELD);
		TRACE_RET(cr, TRANSPORT_FAILED);
		break;
	}
	return err;
}

int sd_ge_get_sd_info(struct card_reader *cr, struct card *card,
		struct scsi_cmnd *srb)
{
	u8 *info;
	struct sd_info *sd_card = card->extra;
	u8 *ssr;
	u8 scsb = sd_card->ge.ge_card ? 1 : 0;
	u8 lun = SCSI_LUN(srb);
	struct sd_command cmd;
	u16 alloc_len = cr_get_be16(&srb->cmnd[10]);

	if (srb->cmnd[0] != MEDIA_SPEC_FUNCTION ||
		srb->cmnd[1] != MSF_GET_SD_INFO ||
		alloc_len != 0x0080) {
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_INVALID_CMD_FIELD);
		TRACE_RET(cr, TRANSPORT_FAILED);
	}

	info = kzalloc(alloc_len, GFP_KERNEL);
	info[0] = 0;			/* status: success */

	/* 1-3: reserved */

	memcpy(&info[4], sd_card->raw_csd, 16);
	memcpy(&info[20], sd_card->raw_scr, 8);
	ssr = &info[28];
	INIT_SD_CMD(cmd, 0x40 | SD_STATUS, 0, SD_APP_CMD, 0, 0);
	init_sd_dcmd_data(cmd, ssr, 64, 64, 1, SD_BUS_WIDTH_4,
			SD_TM_NORMAL_READ, FROM_SD, 100);
	if (sd_send_cmd(cr, card, &cmd)) {
		sd_wait_data_idle(cr);
		sd_clear_sd_error(cr);
		TRACE_RET(cr, TRANSPORT_ERROR);
	}

	info[92] = scsb;

	if (scsb) {
		if (CHK_SD_UHSII(sd_card)) {
			switch (sd_card->setting.host.speed_range) {
			case SPEED_SEL_RANGE_A:
				info[93] = 0x0 << 4;
				break;
			case SPEED_SEL_RANGE_B:
				info[93] = 0x1 << 4;
				break;
			default:
				info[93] = 0xF << 4;
				rts_trace(TRACE_SD, "unknown speed range\n");
				break;
			}
		} else {
			info[93] = 0xF << 4;
		}
		if (CHK_SD_UHSII_HDX30(sd_card))
			info[93] |= 0x4;
		else if (CHK_SD_UHSII_FDX15(sd_card))
			info[93] |= 0x3;
		else if (CHK_SD30_SPEED(sd_card))
			info[93] |= 0x2;
		else if (CHK_SD_HS(sd_card))
			info[93] |= 0x1;
		else
			info[93] |= 0x0;
	} else {
		info[93] = 0xFF;
	}

	if (CHK_SD_UHSII(sd_card)) {
		switch (sd_card->setting.host.speed_range) {
		case SPEED_SEL_RANGE_A:
			info[94] = 26;
			break;
		case SPEED_SEL_RANGE_B:
			info[94] = 52;
			break;
		default:
			rts_trace(TRACE_SD, "unknown speed range\n");
			break;
		}
	} else {
		info[94] = (u8)(sd_card->sd_clock);
	}

	info[95] = (u8)(sd_card->n_fcu);
	if (info[93] == 0xFF || (info[93] & 0xF) < 0x3)
		info[95] = 0;

	/* 96-127: reserved */

	scsi_check_xfer_len(srb, alloc_len);
	scsi_stor_set_xfer_buf(info, alloc_len, srb);
	scsi_set_resid(srb, 0);

	vfree(info);
	return TRANSPORT_GOOD;
}

int sd_ge_speed_class_control(struct card_reader *cr, struct card *card,
		struct scsi_cmnd *srb)
{
	u8 status = 0;
	u32 arg = cr_get_be32(&srb->cmnd[4]);
	u16 alloc_len = (u16)(cr_get_be16(&srb->cmnd[10]));
	u8 lun = SCSI_LUN(srb);
	int err = 0;

	if (srb->cmnd[0] != MEDIA_SPEC_FUNCTION ||
		srb->cmnd[1] != MSF_SPEED_CLASS_CONTROL ||
		alloc_len != 0x0001) {
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_INVALID_CMD_FIELD);
		TRACE_RET(cr, TRANSPORT_FAILED);
	}

	err = sd_speed_class_control(cr, card, arg);
	if (err) {
		status = 1;	/* an error occurred*/
		scsi_check_xfer_len(srb, alloc_len);
		scsi_stor_set_xfer_buf(&status, alloc_len, srb);
		TRACE_RET(cr, TRANSPORT_ERROR);
	}
	scsi_check_xfer_len(srb, alloc_len);
	scsi_stor_set_xfer_buf(&status, alloc_len, srb);
	scsi_set_resid(srb, 0);
	return err;
}

int sd_ge_change_bus_if_mode(struct card_reader *cr, struct card *card,
		struct scsi_cmnd *srb)
{
	u8 status = 0, lun = SCSI_LUN(srb);
	u16 alloc_len = cr_get_be16(&srb->cmnd[10]);

	if (srb->cmnd[0] != MEDIA_SPEC_FUNCTION ||
		srb->cmnd[1] != MSF_CHANGE_BUS_IF_MODE ||
		alloc_len != 0x0001) {
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_INVALID_CMD_FIELD);
		TRACE_RET(cr, TRANSPORT_FAILED);
	}

	/* TODO: change bus if mode */
	status = 0x01;			/* dont support change bus if mode */

	scsi_check_xfer_len(srb, alloc_len);
	scsi_stor_set_xfer_buf(&status, alloc_len, srb);
	scsi_set_resid(srb, 0);
	return 0;
}

int sd_ge_check_status(u8 status, int out)
{
	int res = 0;
	if (out) {
		if (SD_GE_STATUS_RESPONSE == status
			|| SD_GE_STATUS_SEND == status) {
			rts_trace(TRACE_SCSI, "Invalid OUT during SATUS_SEND or STATUS_RESPONSE\n");
			res = -1;
		}
	} else if (SD_GE_STATUS_RECEIVE == status
			|| SD_GE_STATUS_READY == status) {
			rts_trace(TRACE_SCSI, "Invalid IN during SATUS_READY or STATUS_RECEIVE\n");
			res = -1;
	}
	return res;
}

#define SD_GE_MAX_WAIT_CHECK_CNT 5
int sd_ge_wait_check_status(struct card_reader *cr, struct card *card,
		u8 status)
{
	u8 loop = 0;
	u8 stat = 0;
	int res = -1;
	for (loop = 0; loop < SD_GE_MAX_WAIT_CHECK_CNT; loop++) {
		stat = sd_ge_extr(cr, card, SD_GE_ADDR_STATUS);
		if (stat == status) {
			res = 0;
			break;
		}
		msleep(2);
	}
	return res;
}

int sd_ge_security_protocol_out(struct card_reader *cr, struct card *card,
		struct scsi_cmnd *srb)
{
	struct sd_info *sd_card = card->extra;
	struct sd_ge *ge = &sd_card->ge;
	int err = 0;
	u8 lun = SCSI_LUN(srb);
	u32 arg = 0;
	u8 *buf = NULL;
	int header_len = 512;
	u32 alloc_len = cr_get_be32(&srb->cmnd[6]) * 512;
	u32 block_count = alloc_len / 512;
	u32 bus = block_count > 512 ? 1 : 0;
	u32 buc = bus ? block_count / 64 : block_count;
	rts_trace(TRACE_SD, "OUT and alloc_len (%#x)\n", alloc_len);
	/* OUT header first */
	/* check and wait status is READY */
	if (sd_ge_wait_check_status(cr, card, SD_GE_STATUS_READY)) {
		rts_trace(TRACE_SD, "wait READY failed\n");
		set_sense_type(cr, lun, SENSE_TYPE_OVERLAPPED_CMD_ATTEMPTED);
		sd_ge_go_to_ready(cr, card);
		TRACE_RET(cr, TRANSPORT_FAILED);
	}
	buf = kzalloc(512, GFP_KERNEL);
	if (!buf)
		TRACE_RET(cr, TRANSPORT_FAILED);
	if (alloc_len < 512)
		header_len = alloc_len;
	scsi_stor_get_xfer_buf(buf, header_len, srb);
	arg = sd_extr_arg_init_single(0, ge->fno, 0,
			ge->start_addr + SD_GE_ADDR_CMD_PORT,
			0);
	err = sd_write_extr_single(cr, card, arg, buf);
	kfree(buf);
	buf = NULL;
	if (err) {
		rts_trace(TRACE_SD, "OUT header failed\n");
		sd_ge_go_to_ready(cr, card);
		TRACE_RET(cr, TRANSPORT_FAILED);
	}
	if (alloc_len > 512) {
		/* OUT payload */
		rts_trace(TRACE_SD, "OUT payload and length=(%#x)\n", alloc_len-512);
		/* check and wait status is RECEIVE */
		if (sd_ge_wait_check_status(cr, card, SD_GE_STATUS_RECEIVE)) {
			rts_trace(TRACE_SD, "wait RECEIVE failed\n");
			set_sense_type(cr, lun, SENSE_TYPE_MEDIA_WRITE_ERR);
			sd_ge_go_to_ready(cr, card);
			TRACE_RET(cr, TRANSPORT_FAILED);
		}
		buf = cr_alloc_dma_buf(cr, alloc_len-512, GFP_KERNEL);
		if (NULL == buf) {
			rts_trace(TRACE_SD, "Not enough memory\n");
			TRACE_RET(cr, TRANSPORT_FAILED);
		}
		scsi_stor_get_xfer_buf(buf, alloc_len-512, srb);
		arg = sd_extr_arg_init_multi(0, ge->fno, bus,
				ge->start_addr + SD_GE_ADDR_CMD_PLD_PORT, buc-1);
		err = sd_write_extr_multi(cr, card, arg, buf);
		cr_free_dma_buf(cr, buf);
		buf = NULL;
		if (err) {
			rts_trace(TRACE_SD, "OUT payload failed\n");
			sd_ge_go_to_ready(cr, card);
			TRACE_RET(cr, TRANSPORT_FAILED);
		}
		/* need or not ? */
		/*sd_stop_sequential(card);*/
	}
	scsi_set_resid(srb, 0);
	return err;
}

int sd_ge_security_protocol_in(struct card_reader *cr, struct card *card,
		struct scsi_cmnd *srb)
{
	struct sd_info *sd_card = card->extra;
	struct sd_ge *ge = &sd_card->ge;
	int err = 0;
	u8 lun = SCSI_LUN(srb);
	u32 arg = 0;
	u8 *buf = NULL;
	int resp_len = 512;
	u32 alloc_len = cr_get_be32(&srb->cmnd[6]) * 512;
	u32 block_count = alloc_len / 512;
	u32 bus = block_count > 512 ? 1 : 0;
	u32 buc = bus ? block_count / 64 : block_count;
	u8 skip_payload = 0;
	if (alloc_len > 512) {
		/* IN payload first*/
		rts_trace(TRACE_SD, "In payload and length = (%#x)\n", alloc_len-512);
		/* check and wait status is RECEIVE */
		if (sd_ge_wait_check_status(cr, card, SD_GE_STATUS_SEND)) {
			rts_trace(TRACE_SD, "wait RECEIVE failed, skip payload\n");
			skip_payload = 1;
		}
		buf = cr_alloc_dma_buf(cr, alloc_len-512, GFP_KERNEL);
		if (NULL==buf) {
			rts_trace(TRACE_SD, "Not enough memory\n");
			TRACE_RET(cr, TRANSPORT_FAILED);
		}
		memset(buf, 0, alloc_len - 512);
		if (!skip_payload) {
			arg = sd_extr_arg_init_multi(0, ge->fno, bus,
				ge->start_addr + SD_GE_ADDR_RESP_PLD_PORT,
				buc-1);
			err = sd_read_extr_multi(cr, card, arg, buf);
		}
		if (0 == err)
			scsi_stor_set_xfer_buf(buf, alloc_len-512, srb);
		cr_free_dma_buf(cr, buf);
		buf = NULL;
		if (err) {
			rts_trace(TRACE_SD, "In payload failed\n");
			sd_ge_go_to_ready(cr, card);
			TRACE_RET(cr, TRANSPORT_FAILED);
		}
		/* need or not ? */
		/*sd_stop_sequential(card);*/
	}

	/* IN response */
	/* check and wait status is RESPONSE */
	if (sd_ge_wait_check_status(cr, card, SD_GE_STATUS_RESPONSE)) {
		rts_trace(TRACE_SD, "wait RESPONSE failed\n");
		set_sense_type(cr, lun, SENSE_TYPE_OVERLAPPED_CMD_ATTEMPTED);
		sd_ge_go_to_ready(cr, card);
		TRACE_RET(cr, TRANSPORT_FAILED);
	}
	rts_trace(TRACE_SD, "IN response\n");
	buf = kzalloc(512, GFP_KERNEL);
	arg = sd_extr_arg_init_single(0, ge->fno, 0,
			ge->start_addr + SD_GE_ADDR_RESP_PORT,
			0);
	err = sd_read_extr_single(cr, card, arg, buf);
	if (0 == err)
		scsi_stor_set_xfer_buf(buf, resp_len, srb);
	kfree(buf);
	if (err) {
		rts_trace(TRACE_SD, "In response failed\n");
		sd_ge_go_to_ready(cr, card);
		TRACE_RET(cr, TRANSPORT_FAILED);
	}
	scsi_set_resid(srb, 0);
	return err;
}


/**
 * sd_ge_security_transfer - SECURITY PROTOCOL IN/OUT
 *
 * ref:
 * Gamma/EMID USB Command Set For USB Mass Storage Class, Page: 11-15
 *
 *
 */
int sd_ge_security_transfer(struct card_reader *cr, struct card *card,
		struct scsi_cmnd *srb)
{
	struct sd_info *sd_card = card->extra;
	int out = -1, err = 0;
	u8 lun = SCSI_LUN(srb);
	u8 sp = srb->cmnd[1];		/* security protocol */
	u16 sp_spec = (srb->cmnd[2] << 8) | srb->cmnd[3];
	u32 alloc_len = cr_get_be32(&srb->cmnd[6]) * 512;

	if(sd_card->ge.ge_card == 0) {
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_INVALID_CMD_FIELD);
		TRACE_RET(cr, TRANSPORT_FAILED);
	}

	if (srb->cmnd[0] == SECURITY_PROTOCOL_IN)
		out = 0;
	else if (srb->cmnd[0] == SECURITY_PROTOCOL_OUT)
		out = 1;

	if (alloc_len != scsi_bufflen(srb)) {
		rts_trace(TRACE_SD, "alloc_len[%d] != scsi_bufflen[%d]\n",
			alloc_len, scsi_bufflen(srb));
	}

	/* sp: SECURITY PROTOCOL
	 * 0x00h	- security protocol information
	 * 0xFEh	- SECURITY PROTOCOL IN/OUT
	 */
	rts_trace(TRACE_SD, "SECURITY PROTOCOL IN/OUT check operation\n");
	if (out < 0 || (sp != 0xFE && sp != 0) || 0 == alloc_len
		|| (srb->cmnd[4] & 0x80) != 0x80) { /* invalid INC_512 */
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_INVALID_CMD_FIELD);
		TRACE_RET(cr, TRANSPORT_FAILED);
	}

	/* sp_spec: SECURITY PROTOCOL SPECIFIC
	 * 0 - supported security protocol list
	 * 1 - certificate data
	 */
	rts_trace(TRACE_SD, "sp_spec check operation\n");
	if (sp == 0 && sp_spec == 0) {	/* supported security protocol list */
		u32 sendbytes = 0;
		if (alloc_len < sizeof(security_protocol_list))
			sendbytes = alloc_len;
		else
			sendbytes = sizeof(security_protocol_list);
		scsi_check_xfer_len(srb, alloc_len);
		scsi_stor_set_xfer_buf(security_protocol_list, sendbytes, srb);
		scsi_set_resid(srb, 0);
		return 0;

	} else if (sp == 0 && sp_spec == 1) { /* certificate data */
		u32 sendbytes = 0;
		if (alloc_len < sizeof(certificate_data))
			sendbytes = alloc_len;
		else
			sendbytes = sizeof(certificate_data);
		scsi_check_xfer_len(srb, alloc_len);
		scsi_stor_set_xfer_buf(certificate_data, sendbytes, srb);
		scsi_set_resid(srb, 0);
		return 0;

	} else if (sp == 0) {		/* unknown sp_spec */
		rts_trace(TRACE_SD, "SECURITY PROTOCOL SPECIFIC: %d\n",
			sp_spec);
		set_sense_type(cr, lun, SENSE_TYPE_MEDIA_INVALID_CMD_FIELD);
		TRACE_RET(cr, TRANSPORT_FAILED);
	}

	if (out)
		err = sd_ge_security_protocol_out(cr, card, srb);
	else
		err = sd_ge_security_protocol_in(cr, card, srb);
	return err;

}
