/* Driver for Realtek card reader
 * Header file
 *
 * Copyright(c) 2012 Realtek Semiconductor Corp. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *   edwin.rong(edwin_rong@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 *
 *   Tony.Nie(tony_nie@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 */

#ifndef __SD_H__
#define __SD_H__

#include"cards.h"
#include"sd40.h"

struct card_reader;
struct card;

#define SUPPORT_SD_LOCK

/* define capability map for SD */
#define SD_CAP_MMC_1BIT			(1 << 0)
#define SD_CAP_MMC_4BIT			(1 << 1)
#define SD_CAP_MMC_8BIT			(1 << 2)

#define SD_CAP_HS			(1 << 8)
#define SD_CAP_SDR50			(1 << 9)
#define SD_CAP_DDR50			(1 << 10)
#define SD_CAP_SDR104			(1 << 11)

#define SD_CAP_UHSII			(1 << 15)

#define SD_CAP_MMC_SDR200		(1 << 25)

#define SD_CAP_UHSI	\
	(SD_CAP_SDR50 | SD_CAP_DDR50 | SD_CAP_SDR104)

#define SD_CAP_MMC	\
	(SD_CAP_MMC_1BIT | SD_CAP_MMC_4BIT | SD_CAP_MMC_8BIT)
#define SD_CAP_SD	\
	(SD_CAP_HS | SD_CAP_UHSI | SD_CAP_UHSII)


#define WAIT_DATA_READY_RTY_CNT	255
#define MAX_PHASE	31

/* SD bank voltage */
#define SD_IO_3V3		0
#define SD_IO_1V8		1
#define SD_IO_1V9		2

/* sd_ctl bit map */
/* SD push point control, bit 0, 1 */
#define REG_SD_PUSH_POINT_CTL_MASK		0x03
#define SD_PUSH_POINT_DELAY		0x01
#define SD_PUSH_POINT_AUTO		0x02
/* SD sample point control, bit 2, 3 */
#define REG_SD_SAMPLE_POINT_CTL_MASK	0x0C
#define SD_SAMPLE_POINT_DELAY		0x04
#define SD_SAMPLE_POINT_AUTO		0x08
/* SD DDR Tx phase set by user, bit 4 */
#define SD_DDR_TX_PHASE_SET_BY_USER	0x10
/* MMC DDR Tx phase set by user, bit 5 */
#define MMC_DDR_TX_PHASE_SET_BY_USER	0x20
/* Support MMC DDR mode, bit 6 */
#define SUPPORT_MMC_DDR_MODE		0x40
/* Reset MMC at first */
#define RESET_MMC_FIRST			0x80
#define SD_FORBID_RETRY_OF_RW		0x100
#define EMMC45_DONT_SWITCH_POWER	0x200


/* sd40_ctl*/
#define FIX_CMD12_ISSUE			0x40
#define PATCH_SPECIAL_UHSII		0x80
#define DO_NOT_SD40_INIT		0x100
#define SKIP_SD40_INIT_CMD		0x200
#define SD40_LOW_PWR_MODE		0x400  /*Fast mode or low power mode*/
#define SD40_PHY_TEST_MODE		0x800
#define SD40_HD_MODE			0x1000
#define SD40_BYTE_MODE			0x2000
#define SD40_SMALL_CAP			0x4000
#define SD40_RW_CM_TRAN			0x8000
#define SD40_RW_NO_LEN			0x10000
#define SD40_GO_LEGACY_WAY		0x20000
#define SD40_NO_ERR_HANDLING		0x40000   /* return any error */
#define SD40_NO_MMC_FLOW		0x80000   /* return any error */
#define SD40_PHISON_TEST		0x100000
#define SD40_RS_FPGA_CARD		0x200000
#define SD40_SCARM_DIS			0x400000
#define SD40_RANGE_B_EN			0x800000
#define SD40_PRE_READ_EN		0x1000000
#define SD40_EN_DORMANT			0x2000000
#define SD40_EN_EBNUM			0x4000000
#define SD40_SSC_EN			0x8000000
#define SD40_SUPPORT_HIBERNATE		0x10000000
#define SD40_IGNORE_ASPM		0x20000000
#define SD40_USE_EXT_CLOCK		0x40000000
#define SD40_PRE_WRITE_EN		0x80000000

#define SUPPORT_VOLTAGE	0x003C0000

/* Error Code */
#define	SD_NO_ERROR		0x0
#define	SD_CRC_ERR		0x80
#define	SD_TO_ERR		0x40
#define	SD_NO_CARD		0x20
#define SD_BUSY			0x10
#define	SD_STS_ERR		0x08
#define SD_RSP_TIMEOUT		0x04
#define SD_IO_ERR		0x02
#define SD_CMD12_ERR		0x01

/* MMC/SD Command Index */
/* Basic command (class 0) */
#define GO_IDLE_STATE		0
#define	SEND_OP_COND		1 /* reserved for SD */
#define	ALL_SEND_CID		2
#define	SET_RELATIVE_ADDR	3
#define	SEND_RELATIVE_ADDR	3
#define	SET_DSR			4
#define IO_SEND_OP_COND		5
#define	SWITCH			6
#define	SELECT_CARD		7
#define	DESELECT_CARD		7
/* CMD8 is "SEND_EXT_CSD" for MMC4.x Spec
 * while is "SEND_IF_COND" for SD 2.0 */
#define	SEND_EXT_CSD		8
#define	SEND_IF_COND		8
/* end  */
#define	SEND_CSD		9
#define	SEND_CID		10
#define	VOLTAGE_SWITCH		11
#define	READ_DAT_UTIL_STOP	11 /* reserved for SD */
#define	STOP_TRANSMISSION	12
#define	SEND_STATUS		13
#define	GO_INACTIVE_STATE	15

/* Block oriented read commands (class 2) */
#define	SET_BLOCKLEN		16
#define	READ_SINGLE_BLOCK	17
#define	READ_MULTIPLE_BLOCK	18
#define	SEND_TUNING_PATTERN	19
#define SPEED_CLASS_CONTROL	20
#define SEND_TUNING_BLOCK	21
#define SET_BLOCK_COUNT		23

/* Bus Width Test */
#define	BUSTEST_R		14
#define	BUSTEST_W		19
/* end */

/* Block oriented write commands (class 4) */
#define	WRITE_BLOCK		24
#define	WRITE_MULTIPLE_BLOCK	25
#define	PROGRAM_CSD		27
#define SET_WRITE_PROT		28
#define CLR_WRITE_PROT		29

/* Erase commands */
#define	ERASE_WR_BLK_START	32
#define	ERASE_WR_BLK_END	33
#define	ERASE_CMD		38

/* Block Oriented Write Protection Commands */
#define LOCK_UNLOCK		42

#define READ_EXTR_SINGLE	48
#define WRITE_EXTR_SINGLE	49

#define	IO_RW_DIRECT		52

/* Application specific commands (class 8) */
#define	APP_CMD			55
#define	GEN_CMD			56

#define READ_EXTR_MULTI		58
#define WRITE_EXTR_MULTI	59

/* EXT register read/write argument */
#define SD_EXTR_ARG_MIO_SHIFT		31
#define SD_EXTR_ARG_MIO_MASK		0x1
#define SD_EXTR_ARG_IOFNO_SHIFT		28
#define SD_EXTR_ARG_IOFNO_MASK		0x7
#define SD_EXTR_ARG_MEMFNO_SHIFT	27
#define SD_EXTR_ARG_MEMFNO_MASK		0xF
#define SD_EXTR_ARG_MW_SHIFT		26
#define SD_EXTR_ARG_MW_MASK		0x1
#define SD_EXTR_ARG_ADDR_SHIFT		9
#define SD_EXTR_ARG_ADDR_MASK		0x1FFFF
#define SD_EXTR_ARG_LEN_MASK		0x1FF
#define SD_EXTR_ARG_MASK_MASK		0xFF
#define SD_EXTR_ARG_BUS_SHIFT		SD_EXTR_ARG_MW_SHIFT
#define SD_EXTR_ARG_BUS_MASK		SD_EXTR_ARG_MW_MASK
#define SD_EXTR_ARG_BUC_MASK		SD_EXTR_ARG_LEN_MASK
u32 sd_extr_arg_init_single(u32 mio, u32 fno, u32 mw, u32 addr, u32 len);
u32 sd_extr_arg_init_multi(u32 mio, u32 fno, u32 bus, u32 addr, u32 buc);
u32 sd_extr_arg_single_len(u32 arg);

/* EXT General Info */
#define SD_EXTR_EXT_NUM_MAX		4
#define SD_EXTR_SET_NUM_MAX		4
#define SD_EXTR_SET_IOFNO_SHIFT		18
#define SD_EXTR_SET_IOFNO_MASK		0x7
#define SD_EXTR_SET_MEMFNO_SHIFT	18
#define SD_EXTR_SET_MEMFNO_MASK		0xF
#define SD_EXTR_SET_ADDR_MASK		0x1FFFF

/* Gamma/EMID Extension Register */
#define SD_GE_ADDR_VERSION		0x00
#define SD_GE_ADDR_STATUS		0x04
#define   SD_GE_STATUS_READY		0x00
#define   SD_GE_STATUS_RECEIVE		0x01
#define   SD_GE_STATUS_SEND		0x02
#define   SD_GE_STATUS_RESPONSE		0x03
#define SD_GE_ADDR_ERROR		0x05
#define   SD_GE_ERROR_VTM		0x80
#define SD_GE_ADDR_OPERATION		0x08
#define   SD_GE_OP_GOTO_READY		0x01
#define SD_GE_ADDR_CMD_PORT		0x0C
#define SD_GE_ADDR_CMD_PLD_PORT		0x0D
#define SD_GE_ADDR_RESP_PLD_PORT	0x0E
#define SD_GE_ADDR_RESP_PORT		0x0F
u8 sd_ge_extr(struct card_reader *cr, struct card *card, u8 addr);
int sd_ge_go_to_ready(struct card_reader *cr, struct card *card);

/* SD Application command Index */
#define	SET_BUS_WIDTH			6
#define	SD_STATUS			13
#define	SEND_NUM_WR_BLOCKS		22
#define	SET_WR_BLK_ERASE_COUNT		23
#define	SD_APP_OP_COND			41
#define	SET_CLR_CARD_DETECT		42
#define	SEND_SCR			51

/* SD TIMEOUT function return error */
#define	SD_READ_COMPLETE	0x00
#define	SD_READ_TO		0x01
#define	SD_READ_ADVENCE		0x02

/* SD v1.1 CMD6 SWITCH function */
#define	SD_CHECK_MODE		0x00
#define	SD_SWITCH_MODE		0x80
#define	SD_FUNC_GROUP_1		0x01
#define	SD_FUNC_GROUP_2		0x02
#define	SD_FUNC_GROUP_3		0x03
#define	SD_FUNC_GROUP_4		0x04
#define	SD_CHECK_SPEC_V1_1	0xFF

/* SD Command Argument */
#define	NO_ARGUMENT	                        0x00
#define	CHECK_PATTERN				0x000000AA
#define	VOLTAGE_SUPPLY_RANGE			0x00000100 /* 2.7~3.6V */
#define	SUPPORT_HIGH_AND_EXTENDED_CAPACITY	0x40000000
#define	SUPPORT_MAX_POWER_PERMANCE	        0x10000000
#define	SUPPORT_1V8	                        0x01000000

/* Switch Command Error Code */
#define	SWTICH_NO_ERR	  0x00
#define	CARD_NOT_EXIST	  0x01
#define	SPEC_NOT_SUPPORT  0x02
#define	CHECK_MODE_ERR	  0x03
#define	CHECK_NOT_READY	  0x04
#define	SWITCH_CRC_ERR	  0x05
#define	SWITCH_MODE_ERR	  0x06
#define	SWITCH_PASS	  0x07

/* #ifdef SUPPORT_SD_LOCK */
/* CMD42 Parameter */
#define SD_ERASE		0x08
#define SD_LOCK			0x04
#define SD_UNLOCK		0x00
#define SD_CLR_PWD		0x02
#define SD_SET_PWD		0x01

#define SD_PWD_LEN		0x10

/* SD lock unlock Status */
#define SD_LOCKED		0x80	/* Global lock status */
#define SD_LOCK_1BIT_MODE	0x40 /**/
#define SD_PWD_EXIST		0x20
#define SD_UNLOCK_POW_ON	0x01 /**/
#define SD_SDR_RST		0x02 /* Reset SD30 card with current DDR mode to
				      * SDR mode. */
/* g_bySDEraseStatus */
#define SD_NOT_ERASE		0x00
#define SD_UNDER_ERASING	0x01
#define SD_COMPLETE_ERASE	0x02
/* SD_RW FAIL status */
#define SD_RW_FORBIDDEN		0x0F	/* read/write is forbidden (SD card)  */
/* #endif */

/* Function Group Definition */
/* Function Group 1 */
#define	HS_SUPPORT			0x01
#define	SDR50_SUPPORT			0x02
#define	SDR104_SUPPORT			0x03
#define	DDR50_SUPPORT			0x04
#define	HS_SUPPORT_MASK			0x02
#define	SDR50_SUPPORT_MASK		0x04
#define	SDR104_SUPPORT_MASK		0x08
#define	DDR50_SUPPORT_MASK		0x10
#define	HS_QUERY_SWITCH_OK		0x01
#define	SDR50_QUERY_SWITCH_OK		0x02
#define	SDR104_QUERY_SWITCH_OK		0x03
#define	DDR50_QUERY_SWITCH_OK		0x04
#define	HS_SWITCH_BUSY			0x02
#define	SDR50_SWITCH_BUSY		0x04
#define	SDR104_SWITCH_BUSY		0x08
#define	DDR50_SWITCH_BUSY		0x10
#define	FUNCTION_GROUP1_SUPPORT_OFFSET       0x0D
#define FUNCTION_GROUP1_QUERY_SWITCH_OFFSET  0x10
#define FUNCTION_GROUP1_CHECK_BUSY_OFFSET    0x1D
/* Function Group 3 */
#define	DRIVING_TYPE_A	        0x01
#define	DRIVING_TYPE_B		    0x00
#define	DRIVING_TYPE_C		    0x02
#define	DRIVING_TYPE_D	        0x03
#define	DRIVING_TYPE_A_MASK	    0x02
#define	DRIVING_TYPE_B_MASK	    0x01
#define	DRIVING_TYPE_C_MASK	    0x04
#define	DRIVING_TYPE_D_MASK	    0x08
#define	TYPE_A_QUERY_SWITCH_OK	0x01
#define	TYPE_B_QUERY_SWITCH_OK	0x00
#define	TYPE_C_QUERY_SWITCH_OK  0x02
#define	TYPE_D_QUERY_SWITCH_OK  0x03
#define	TYPE_A_SWITCH_BUSY	    0x02
#define	TYPE_B_SWITCH_BUSY	    0x01
#define	TYPE_C_SWITCH_BUSY      0x04
#define	TYPE_D_SWITCH_BUSY      0x08
#define	FUNCTION_GROUP3_SUPPORT_OFFSET       0x09
#define FUNCTION_GROUP3_QUERY_SWITCH_OFFSET  0x0F
#define FUNCTION_GROUP3_CHECK_BUSY_OFFSET    0x19

/* Function Group 4 */
#define	CURRENT_LIMIT_200	    0x00
#define	CURRENT_LIMIT_400	    0x01
#define	CURRENT_LIMIT_600	    0x02
#define	CURRENT_LIMIT_800	    0x03
#define	CURRENT_LIMIT_200_MASK	0x01
#define	CURRENT_LIMIT_400_MASK	0x02
#define	CURRENT_LIMIT_600_MASK	0x04
#define	CURRENT_LIMIT_800_MASK	0x08
#define	CURRENT_LIMIT_200_QUERY_SWITCH_OK    0x00
#define	CURRENT_LIMIT_400_QUERY_SWITCH_OK    0x01
#define	CURRENT_LIMIT_600_QUERY_SWITCH_OK    0x02
#define	CURRENT_LIMIT_800_QUERY_SWITCH_OK    0x03
#define	CURRENT_LIMIT_200_SWITCH_BUSY        0x01
#define	CURRENT_LIMIT_400_SWITCH_BUSY	     0x02
#define	CURRENT_LIMIT_600_SWITCH_BUSY        0x04
#define	CURRENT_LIMIT_800_SWITCH_BUSY        0x08

#define POWER_LIMIT_0_72_W		     0x00
#define POWER_LIMIT_1_44_W		     0x01
#define POWER_LIMIT_2_16_W		     0x02
#define POWER_LIMIT_2_88_W		     0x03
#define POWER_LIMIT_1_80_W		     0x04
#define POWER_LIMIT_0_72_W_MASK		     (1 << 0)
#define POWER_LIMIT_1_44_W_MASK		     (1 << 1)
#define POWER_LIMIT_2_16_W_MASK		     (1 << 2)
#define POWER_LIMIT_2_88_W_MASK		     (1 << 3)
#define POWER_LIMIT_1_80_W_MASK		     (1 << 4)


#define	FUNCTION_GROUP4_SUPPORT_OFFSET       0x07
#define FUNCTION_GROUP4_QUERY_SWITCH_OFFSET  0x0F
#define FUNCTION_GROUP4_CHECK_BUSY_OFFSET    0x17
/* Switch Function Status Offset */
#define	DATA_STRUCTURE_VER_OFFSET   0x11 /* The high offset */

/* #define      TOTAL_READ_PHASE    0x20 */
/* #define      TOTAL_WRITE_PHASE    0x20 */
/* MMC v4.0 */
/* #define MMC_52MHZ_SPEED                       0x0001 */
/* #define MMC_26MHZ_SPEED                       0x0002 */
#define MMC_8BIT_BUS			0x0010
#define MMC_4BIT_BUS			0x0020
/* #define MMC_SECTOR_MODE                       0x0100 */
#define MMC_SWITCH_ERR			0x80

/* Tuning direction RX or TX */
#define TUNE_TX    0x00
#define TUNE_RX	   0x01
/* For Change_DCM_FreqMode Function */
#define CHANGE_TX  0x00
#define CHANGE_RX  0x01
#define DCM_HIGH_FREQUENCY_MODE  0x00
#define DCM_LOW_FREQUENCY_MODE   0x01
#define DCM_HIGH_FREQUENCY_MODE_SET  0x0C
#define DCM_Low_FREQUENCY_MODE_SET   0x00

#define TYPE_SD			0x0000
#define TYPE_MMC		0x0001

/* TYPE_SD */
#define SD_HS			0x0100
#define SD_SDR50		0x0200
#define SD_DDR50		0x0400
#define SD_SDR104		0x0800
#define SD_UHSII_BIM_MASK	0x001F	/* Bus Interface Mode */
#define   SD_UHSII_FDX15	0x0022
#define   SD_UHSII_FDX30	0x0024
#define   SD_UHSII_HDX15	0x0028
#define   SD_UHSII_HDX30	0x0030
#define SD_UHSII		0x0020

#define SD_SC			(1 << 0)
#define SD_HC			(1 << 1)
#define SD_XC			(1 << 2)
#define SD_HCXC			(SD_HC | SD_XC)

#define CLR_SD_HCXC(sd_card)		((sd_card)->capacity_type &= ~SD_HCXC)
#define SET_SD_HCXC(sd_card)		((sd_card)->capacity_type = SD_HCXC)
#define CHK_SD_HCXC(sd_card)	\
	(CHK_SD(sd_card) && ((sd_card)->capacity_type & SD_HCXC))

#define SET_SD_SC(sd_card)		((sd_card)->capacity_type = SD_SC)
#define CLR_SD_SC(sd_card)		((sd_card)->capacity_type &= ~SD_SC)
#define CHK_SD_SC(sd_card)	\
	(CHK_SD(sd_card) && ((sd_card)->capacity_type & SD_SC))

#define CHK_SD_HC(sd_card) \
	(CHK_SD_HCXC(sd_card) && ((sd_card)->capacity <= 0x4000000))
#define CHK_SD_XC(sd_card) \
	(CHK_SD_HCXC(sd_card) && ((sd_card)->capacity > 0x4000000))

/* TYPE_MMC */
#define MMC_26M			0x0100
#define MMC_52M			0x0200
#define MMC_4BIT		0x0400
#define MMC_8BIT		0x0800
#define MMC_SECTOR_MODE		0x1000
#define MMC_DDR52		0x2000
#define MMC_SDR200		0x4000

/* SD card */
#define CHK_SD(sd_card)			(((sd_card)->sd_type & 0x01) == TYPE_SD)
#define CHK_SD_HS(sd_card)	\
	(CHK_SD(sd_card) && ((sd_card)->sd_type & SD_HS))
#define CHK_SD_SDR50(sd_card)		\
	(CHK_SD(sd_card) && ((sd_card)->sd_type & SD_SDR50))
#define CHK_SD_DDR50(sd_card)	\
	(CHK_SD(sd_card) && ((sd_card)->sd_type & SD_DDR50))
#define CHK_SD_SDR104(sd_card)	\
	(CHK_SD(sd_card) && ((sd_card)->sd_type & SD_SDR104))
#define CHK_SD30_SPEED(sd_card)	\
	(CHK_SD_SDR50(sd_card) || CHK_SD_DDR50(sd_card) ||\
	 CHK_SD_SDR104(sd_card))
#define CHK_SD_UHSII(sd_card) \
	(CHK_SD(sd_card) && ((sd_card)->sd_type & SD_UHSII))

#define CHK_SD_UHSII_BIM(sd_card, bim) \
	((sd_card)->sd_type & SD_UHSII_BIM_MASK & (bim))

#define CHK_SD_UHSII_FDX15(sd_card) \
	(CHK_SD_UHSII(sd_card) && CHK_SD_UHSII_BIM((sd_card), SD_UHSII_FDX15))
#define CHK_SD_UHSII_FDX30(sd_card) \
	(CHK_SD_UHSII(sd_card) && CHK_SD_UHSII_BIM((sd_card), SD_UHSII_FDX30))
#define CHK_SD_UHSII_HDX15(sd_card) \
	(CHK_SD_UHSII(sd_card) && CHK_SD_UHSII_BIM((sd_card), SD_UHSII_HDX15))
#define CHK_SD_UHSII_HDX30(sd_card) \
	(CHK_SD_UHSII(sd_card) && CHK_SD_UHSII_BIM((sd_card), SD_UHSII_HDX30))


#define CHK_SD_UHS(sd_card) \
	((CHK_SD_UHSII(sd_card) || CHK_SD30_SPEED(sd_card)))


#define SET_SD(sd_card)			((sd_card)->sd_type = TYPE_SD)
#define SET_SD_HS(sd_card)		((sd_card)->sd_type |= SD_HS)
#define SET_SD_SDR50(sd_card)		((sd_card)->sd_type |= SD_SDR50)
#define SET_SD_DDR50(sd_card)		((sd_card)->sd_type |= SD_DDR50)
#define SET_SD_SDR104(sd_card)		((sd_card)->sd_type |= SD_SDR104)

#define SET_SD_UHSII_FDX15(sd_card)	((sd_card)->sd_type |= SD_UHSII_FDX15)
#define SET_SD_UHSII_FDX30(sd_card)	((sd_card)->sd_type |= SD_UHSII_FDX30)
#define SET_SD_UHSII_HDX15(sd_card)	((sd_card)->sd_type |= SD_UHSII_HDX15)
#define SET_SD_UHSII_HDX30(sd_card)	((sd_card)->sd_type |= SD_UHSII_HDX30)


#define CLR_SD_HS(sd_card)		((sd_card)->sd_type &= ~SD_HS)
#define CLR_SD_SDR50(sd_card)		((sd_card)->sd_type &= ~SD_SDR50)
#define CLR_SD_DDR50(sd_card)		((sd_card)->sd_type &= ~SD_DDR50)
#define CLR_SD_SDR104(sd_card)		((sd_card)->sd_type &= ~SD_SDR104)
#define CLR_SD30_SPEED(sd_card)	\
	((sd_card)->sd_type &= ~(SD_SDR50|SD_DDR50|SD_SDR104))

#define CLR_SD_UHSII_FDX15(sd_card)	((sd_card)->sd_type &= ~SD_UHSII_FDX15)
#define CLR_SD_UHSII_FDX30(sd_card)	((sd_card)->sd_type &= ~SD_UHSII_FDX30)
#define CLR_SD_UHSII_HDX15(sd_card)	((sd_card)->sd_type &= ~SD_UHSII_HDX15)
#define CLR_SD_UHSII_HDX30(sd_card)	((sd_card)->sd_type &= ~SD_UHSII_HDX30)


/* MMC card */
#define CHK_MMC(sd_card)	\
	(((sd_card)->sd_type & 0x01) == TYPE_MMC)
#define CHK_MMC_26M(sd_card)	\
	(CHK_MMC(sd_card) && ((sd_card)->sd_type & MMC_26M))
#define CHK_MMC_52M(sd_card)	\
	(CHK_MMC(sd_card) && ((sd_card)->sd_type & MMC_52M))
#define CHK_MMC_4BIT(sd_card)	\
	(CHK_MMC(sd_card) && ((sd_card)->sd_type & MMC_4BIT))
#define CHK_MMC_8BIT(sd_card)	\
	(CHK_MMC(sd_card) && ((sd_card)->sd_type & MMC_8BIT))
#define CHK_MMC_SECTOR_MODE(sd_card)\
	(CHK_MMC(sd_card) && ((sd_card)->sd_type & MMC_SECTOR_MODE))
#define CHK_MMC_DDR52(sd_card)	\
	(CHK_MMC(sd_card) && ((sd_card)->sd_type & MMC_DDR52))
#define CHK_MMC_SDR200(sd_card)	\
	(CHK_MMC(sd_card) && ((sd_card)->sd_type & MMC_SDR200))

#define SET_MMC(sd_card)		((sd_card)->sd_type = TYPE_MMC)
#define SET_MMC_26M(sd_card)		((sd_card)->sd_type |= MMC_26M)
#define SET_MMC_52M(sd_card)		((sd_card)->sd_type |= MMC_52M)
#define SET_MMC_4BIT(sd_card)		((sd_card)->sd_type |= MMC_4BIT)
#define SET_MMC_8BIT(sd_card)		((sd_card)->sd_type |= MMC_8BIT)
#define SET_MMC_SECTOR_MODE(sd_card)	((sd_card)->sd_type |= MMC_SECTOR_MODE)
#define SET_MMC_DDR52(sd_card)		((sd_card)->sd_type |= MMC_DDR52)
#define SET_MMC_SDR200(sd_card)		((sd_card)->sd_type |= MMC_SDR200)

#define CLR_MMC_26M(sd_card)		((sd_card)->sd_type &= ~MMC_26M)
#define CLR_MMC_52M(sd_card)		((sd_card)->sd_type &= ~MMC_52M)
#define CLR_MMC_4BIT(sd_card)		((sd_card)->sd_type &= ~MMC_4BIT)
#define CLR_MMC_8BIT(sd_card)		((sd_card)->sd_type &= ~MMC_8BIT)
#define CLR_MMC_SECTOR_MODE(sd_card)	((sd_card)->sd_type &= ~MMC_SECTOR_MODE)
#define CLR_MMC_DDR52(sd_card)		((sd_card)->sd_type &= ~MMC_DDR52)
#define CLR_MMC_SDR200(sd_card)		((sd_card)->sd_type &= ~MMC_SDR200)

#define CHK_MMC_HS(sd_card)	\
	(CHK_MMC_52M(sd_card) || CHK_MMC_26M(sd_card))

#define CLR_MMC_HS(sd_card)			\
do {						\
	CLR_MMC_DDR52(sd_card);			\
	CLR_MMC_52M(sd_card);			\
	CLR_MMC_26M(sd_card);			\
} while (0)

#define SD_SUPPORT_CLASS_TEN		0x01
#define SD_SUPPORT_1V8			0x02

#define SD_SET_CLASS_TEN(sd_card)	\
	((sd_card)->sd_setting |= SD_SUPPORT_CLASS_TEN)
#define SD_CHK_CLASS_TEN(sd_card)	\
	((sd_card)->sd_setting & SD_SUPPORT_CLASS_TEN)
#define SD_CLR_CLASS_TEN(sd_card)	\
	((sd_card)->sd_setting &= ~SD_SUPPORT_CLASS_TEN)
#define SD_SET_1V8(sd_card)		\
	((sd_card)->sd_setting |= SD_SUPPORT_1V8)
#define SD_CHK_1V8(sd_card)		\
	((sd_card)->sd_setting & SD_SUPPORT_1V8)
#define SD_CLR_1V8(sd_card)		\
	((sd_card)->sd_setting &= ~SD_SUPPORT_1V8)

/* Hardcoded times for host to request card to send RCA */
#define SD_SEND_RCA_MAX	100

#define R0 0x00 /* no response */
#define R1 0x01
#define R1B 0x02
#define R2 0x03
#define R3 0x04
#define R4 0x05
#define R5 0x06
#define R6 0x07
#define R7 0x08

/******************************Basic command set ***********************/
/* GO_IDLE_STATE: Reset cards to idle state */
#define CMD0 0
#define CMD0_R R0
#define CMD1 1 /* SEND_OP_COND, reserved for SD */
#define CMD1_R R3
/* ALL_SEND_CID: Asks any card to send the CID num-bers on the CMD line
(any card that is connected to the host will respond) */
#define CMD2 2
#define CMD2_R R2
/* SEND_RELATIVE_ADDR: Ask the card to publish a new relative address (RCA) */
#define CMD3 3
#define CMD3_R R6
/* SET_DSR: Programs the DSR of all cards */
#define CMD4 4
#define CMD4_R R0
/* SELECT/DESELECT_CARD: Command toggles a card between the stand-by and
transfer states or between the programming and dis-connect states.  */
#define CMD7 7
#define CMD7_R R1B
/* SEND_IF_COND: Sends SD Memory Card interface condition, which includes host
supply voltage information and asks the card whether card supports voltage*/
#define CMD8 8
#define CMD8_R R7
/* SEND_CSD: Addressed card sends its card-specific data (CSD) on the CMD
 * line. */
#define CMD9 9
#define CMD9_R R2
/* SEND_CID: Addressed card sends its card identi-fication (CID) on CMD the
 * line. */
#define CMD10 10
#define CMD10_R R2
/* VOLTAGE_SWITCH: Switch to 1.8V bus signaling level. */
#define CMD11 11
#define CMD11_R R1

/* STOP_TRANSMISSION: Forces the card to stop transmission */
#define CMD12 12
#define CMD12_R R1B
/* SEND_STATUS: Addressed card sends its status register. */
#define CMD13 13
#define CMD13_R R1
/* GO_INACTIVE_STATE: Sends an addressed card into the Inactive State. */
#define CMD15 15
#define CMD15_R R0

/*************************** Block read commands **********************/
/* SET_BLOCKLEN: Set the block length */
#define CMD16 16
#define CMD16_R R1
/* READ_SINGLE_BLOCK: Read a single block */
#define CMD17 17
#define CMD17_R R1
/* READ_MULTIPLE_BLOCK: Read multiple blocks until a CMD12 */
#define CMD18 18
#define CMD18_R R1
/* SEND_TUNING_BLOCK: 64 bytes tuning pattern is sent for SDR50 and SDR104*/
#define CMD19 19
#define CMD19_R R1
/* SPEED_CLASS_CONTROL: Speed Class control command. Refer to Section
 * 4.13.2.8.*/
#define CMD20 20
#define CMD20_R R1B
/* SET_BLOCK_COUNT: Specify block count for CMD18 and CMD25.*/
#define CMD23 23
#define CMD23_R R1

/*************************** Block write commands ***********************/
/* WRITE_BLOCK: Write a block of the size selected with CMD16 */
#define CMD24 24
#define CMD24_R R1
/* WRITE_MULTIPLE_BLOCK: Multiple block write until a CMD12 */
#define CMD25 25
#define CMD25_R R1
/* PROGRAM_CSD: Program the programmable bits of the CSD */
#define CMD27 27
#define CMD27_R R1
/*************************** Write protection ***************************/
/* SET_WRITE_PROT: Set the write protection bit of the addressed group */
#define CMD28 28
#define CMD28_R R1B
/* CLR_WRITE_PROT: Clear the write protection bit of the addressed group */
#define CMD29 29
#define CMD29_R R1B
/* SEND_WRITE_PROT: Ask the card for the status of the write protection bits */
#define CMD30 30
#define CMD30_R R1
/*************************** Erase commands **************************/
/* ERASE_WR_BLK_START: Set the address of the first write block to be erased */
#define CMD32 32
#define CMD32_R R1
/* ERASE_WR_BLK_END: Set the address of the last write block to be erased */
#define CMD33 33
#define CMD33_R R1
/* ERASE: Erases all previously selected write blocks.  */
#define CMD38 38
#define CMD38_R R1B
/*************************** Lock Card commands ***********************/
/* LOCK_UNLOCK: Used to set/reset the password or lock/unlock the card. */
#define CMD42 42
#define CMD42_R R1

/* Commands from 42 to 54, not defined here */
/*************************** Application-specific commands ***************/
/* APP_CMD: Flag that the next command is application-specific */
#define CMD55 55
#define CMD55_R R1
/* GEN_CMD: General purpose I/O for application-specific commands */
#define CMD56 56
#define CMD56_R R1
/* Read the OCR (SPI mode only) */
#define CMD58 58
#define CMD58_R R3

/*************************** Application-specific commands **************/
/* SET_BUS_WIDTH: Defines the data bus width ('00'=1bit or '10'=4 bits bus) to
be used for data transfer. The allowed data bus widths are given in SCR
register.*/
#define ACMD6 6
#define ACMD6_R R1
/* SD_STATUS : Send the SD Status. The status fields are given in Table
 * 4-40.  */
#define ACMD13 13
#define ACMD13_R R1
/* SEND_NUM_WR_BLOCKS:Send the number of the written (with-out errors) write
blocks.  Responds with 32bit+CRC data block.  */
#define ACMD22 22
#define ACMD22_R R1
/* SET_WR_BLK_ERASE_COUNT: Set the number of write blocks to be pre-erased
before writing (to be used for faster Multiple Block WR com-mand). "1"=default
(one wr block) */
#define ACMD23 23
#define ACMD23_R R1
/* SD_SEND_OP_COND: Get the card's OCR (SD mode) */
#define ACMD41 41
#define ACMD41_R R3
/* SET_CLR_CARD_DETECT: Connect or disconnect the 50kOhm internal pull-up on
CD/DAT[3] */
#define ACMD42 42
#define ACMD42_R R1
/* SEND_SCR: Reads the SD Configuration Register (SCR).  */
#define ACMD51 51
#define ACMD51_R R1

/*************************** Switch function commands **************/
/* SWITCH_FUNC */
#define CMD6 6
#define CMD6_R R1

/****************************************************************/
#define ACMD41_MAX_REPEAT_CNTS 255

/* SD bank voltage */
#define SD_COMM_IO_3V3		0
#define SD_COMM_IO_1V8		1

#define ENTRY_DORMANT 1
#define EXIT_DORMANT 0
#define DOR_HIBERNATE_MODE 0x80
#define DOR_NORMAL_MODE 0

enum dormant_state {
	DOR_DORMANT_STATE = 0,
	DOR_HIBERNATE_STATE = 1,
	DOR_ACTIVE_STATE = 2,
};

#define PLD0_BASE	10
#define PLD1_BASE	14

#define SD40_OFF_NORMAL	0x01
#define SD40_OFF_FORCE	0x02

enum {
	SUCCESS_RESET = 0,
	ERR_PARTIAL_RESET = -1,
	ERR_FULL_RESET = -2,
};


enum SD_CCC {
	CCC0 = 1 << 0,
	CCC1 = 1 << 1,
	CCC2 = 1 << 2,
	CCC3 = 1 << 3,
	CCC4 = 1 << 4,
	CCC5 = 1 << 5,
	CCC6 = 1 << 6,
	CCC7 = 1 << 7,
	CCC8 = 1 << 8,
	CCC9 = 1 << 9,
	CCC10 = 1 << 10,
	CCC11 = 1 << 11,
};


enum ACMD41_ARG {
	SPT_VOLT = 0x3c << 16,
	S18 = 0x01 << 24,
	XPC = 0x01 << 28,
	FB = 0x01 << 29,
	HCS = 0x01 << 30,
	BSY = 0x01 << 31,
};


enum sd_byte_count_per_blk {
	BCPB_512B = 512,
	BCPB_1024B = 1024,
};


/* SD Tuning Data Structure */
/* Record continuous timing phase path */
struct timing_phase_path {
	int start;
	int end;
	int mid;
	int len;
};

/* sd_ctl bit map */
/* SD push point control, bit 0, 1 */
#define SD_COMM_PUSH_POINT_CTL_MASK		0x03
#define SD_COMM_PUSH_POINT_DELAY		0x01
#define SD_COMM_PUSH_POINT_AUTO			0x02
/* SD sample point control, bit 2, 3 */
#define SD_COMM_SAMPLE_POINT_CTL_MASK		0x0C
#define SD_COMM_SAMPLE_POINT_DELAY		0x04
#define SD_COMM_SAMPLE_POINT_AUTO		0x08
/* SD DDR Tx phase set by user, bit 4 */
#define SD_COMM_DDR_TX_PHASE_SET_BY_USER	0x10
/* MMC DDR Tx phase set by user, bit 5 */
#define SD_COMM_MMC_DDR_TX_PHASE_SET_BY_USER	0x20
/* Support MMC DDR mode, bit 6 */
/*#define SUPPORT_MMC_DDR_MODE          0x40 */
#define SD_COMM_SUPPORT_UHS50_MMC44		0x40
/* Reset MMC at first */
#define SD_COMM_RESET_MMC_FIRST			0x80

/* SD Configure 2 Register */
#define	SD_CALCULATE_CRC7		0x00
#define	SD_NO_CALCULATE_CRC7		0x80
#define	SD_CHECK_CRC16			0x00
#define	SD_NO_CHECK_CRC16		0x40
#define SD_NO_CHECK_WAIT_CRC_TO		0x20
#define	SD_WAIT_BUSY_END		0x08
#define	SD_NO_WAIT_BUSY_END		0x00
#define	SD_CHECK_CRC7			0x00
#define	SD_NO_CHECK_CRC7		0x04
#define	SD_RSP_LEN_0			0x00
#define	SD_RSP_LEN_6			0x01
#define	SD_RSP_LEN_17			0x02
/* SD/MMC Response Type Definition */
/* SD_CALCULATE_CRC7, SD_CHECK_CRC16, SD_NO_WAIT_BUSY_END,
 * SD_NO_CHECK_CRC7, SD_RSP_LEN_0 */
#define	SD_RSP_TYPE_R0		0x04
/* SD_CALCULATE_CRC7, SD_CHECK_CRC16, SD_NO_WAIT_BUSY_END,
 * SD_CHECK_CRC7, SD_RSP_LEN_6 */
#define	SD_RSP_TYPE_R1		0x01
/* SD_CALCULATE_CRC7, SD_CHECK_CRC16, SD_WAIT_BUSY_END,
 * SD_CHECK_CRC7, SD_RSP_LEN_6 */
#define	SD_RSP_TYPE_R1b		0x09
/* SD_CALCULATE_CRC7, SD_CHECK_CRC16, SD_NO_WAIT_BUSY_END,
 * SD_CHECK_CRC7, SD_RSP_LEN_17 */
#define	SD_RSP_TYPE_R2		0x02
/* SD_CALCULATE_CRC7, SD_CHECK_CRC16, SD_NO_WAIT_BUSY_END,
 *SD_NO_CHECK_CRC7, SD_RSP_LEN_6 */
#define	SD_RSP_TYPE_R3		0x05
/* SD_CALCULATE_CRC7, SD_CHECK_CRC16, SD_NO_WAIT_BUSY_END,
 * SD_NO_CHECK_CRC7, SD_RSP_LEN_6 */
#define	SD_RSP_TYPE_R4		0x05
/* SD_CALCULATE_CRC7, SD_CHECK_CRC16, SD_NO_WAIT_BUSY_END,
 * SD_CHECK_CRC7, SD_RSP_LEN_6 */
#define	SD_RSP_TYPE_R5		0x01
/* SD_CALCULATE_CRC7, SD_CHECK_CRC16, SD_NO_WAIT_BUSY_END,
 * SD_CHECK_CRC7, SD_RSP_LEN_6 */
#define	SD_RSP_TYPE_R6		0x01
/* SD_CALCULATE_CRC7, SD_CHECK_CRC16, SD_NO_WAIT_BUSY_END,
 * SD_CHECK_CRC7, SD_RSP_LEN_6 */
#define	SD_RSP_TYPE_R7		0x01


#define SD_MAX_RETRY_COUNT	3
#define SD_CMD_TIMEOUT		(200)

struct sd_command {
	u32			opcode;
	u32			arg;
#define SD_APP_CMD		(1 << 0)
#define SD_CMD_WITH_DATA	(1 << 1)
#define SD_SKIP_OVER_RNAGE_ERR	(1 << 2)
	unsigned int		flags;		/* expected response type */
	unsigned int		max_retries;	/* max number of retries */
	unsigned int		error;		/* command error */
	unsigned int		timeout;	/* in milliseconds */

	u8			rsp[64];
	unsigned int		rsp_len;
	u8			rsp_type; /* Maybe it was not needed */

	struct {
		u32 bytes;
		u16 blocks;
		u8 bus_width;
		u8 trans_mode;
		enum {
			TO_SD,
			FROM_SD,
		} trans_dir;
		u8 *buf;
		int buf_len;
	} data;
};

#define INIT_SD_CMD(cmd, code, argument, flag, type, len) \
	do { \
		memset(&cmd, 0, sizeof(cmd)); \
		cmd.opcode = code; \
		cmd.arg = argument;	\
		cmd.flags = flag;	\
		cmd.rsp_type = type; \
		cmd.rsp_len = len; \
		cmd.max_retries = SD_MAX_RETRY_COUNT; \
		cmd.timeout = SD_CMD_TIMEOUT; \
	} while (0)


#define init_sd_dcmd_data(cmd, buff, buff_len, byte, block, width, mode, dir, tm) \
	do { \
		cmd.data.buf = buff;	\
		cmd.data.buf_len = buff_len;	\
		cmd.data.bytes = byte;	\
		cmd.data.blocks	= block;	\
		cmd.data.bus_width = width;	\
		cmd.data.trans_mode = mode;	\
		cmd.data.trans_dir = dir;	\
		cmd.timeout = tm; \
		cmd.flags |= SD_CMD_WITH_DATA;	\
	} while (0)

/* SD_CONFIURE3 */
#define	SD_RSP_80CLK_TIMEOUT_EN	    0x01
struct sd_cid {
	unsigned int		manfid;
	char			prod_name[8];
	unsigned int		serial;
	unsigned short		oemid;
	unsigned short		year;
	unsigned char		hwrev;
	unsigned char		fwrev;
	unsigned char		month;
};

struct sd_csd {
	unsigned char		structure;
	unsigned char		mmca_vsn;
	unsigned short		cmdclass;
	unsigned short		tacc_clks;
	unsigned int		tacc_ns;
	unsigned int		r2w_factor;
	unsigned int		max_dtr;
	unsigned int		erase_size;		/* In sectors */
	unsigned int		read_blkbits;
	unsigned int		write_blkbits;
	unsigned int		capacity;
	unsigned int		read_partial:1,
				read_misalign:1,
				write_partial:1,
				write_misalign:1;
};

struct sd_ext_csd {
	u8			rev;
	u8			erase_group_def;
	u8			sec_feature_support;
	unsigned int		sa_timeout;		/* Units: 100ns */
	unsigned int		hs_max_dtr;
	unsigned int		sectors;
	unsigned int		card_type;
	unsigned int		hc_erase_size;		/* In sectors */
	unsigned int		hc_erase_timeout;	/* In milliseconds */
	unsigned int		sec_trim_mult;	/* Secure trim multiplier  */
	unsigned int		sec_erase_mult;	/* Secure erase multiplier */
	unsigned int		trim_timeout;		/* In milliseconds */
	int			enhanced_area_en;	/* enable bit */
	unsigned long long	enhanced_area_offset;	/* Units: Byte */
	unsigned int		enhanced_area_size;	/* Units: KB */
};

struct sd_scr {
	unsigned char		sda_vsn;
	enum sd_bus_width	bus_widths;
};

struct sd_extr_header {
	u16 version;
	u16 len;
	u8 num;
};

struct sd_extr_reg {
	u16 sfc;
	u16 fcc;
	u16 fmc;
	u8 fmn[16];
	u16 pfc;
	u8 fn[16];
	u16 next_addr;
	u8 reg_set_num;
	u32 reg_set[SD_EXTR_SET_NUM_MAX];
};

struct sd_ge {
	u32		start_addr;
	u8		fno;
	int		ge_card;	/* if Gamma-Emid card */
};

struct sd_info {
	u16 sd_type;
	u8 err_code;
	u8 sd_data_buf_ready;
	u32 sd_addr;
	u32 capacity;
	u32 capacity_type;

	u8 raw_csd[16];
	u8 raw_cid[16];
	u8 raw_scr[8];

	/* RW info */
	struct {
		enum {
			NORMAL_RW = 0,
			SEQ_RW,
			PRE_READ,
			PRE_WRITE
		} rw_mode;
		enum dma_data_direction prev_dir;
		enum dma_data_direction cur_dir;
		u32 prev_sec_addr;
		u32 prev_sec_cnt;
		u32 read_sectors;
		u32 write_sectors;
		u32 seq_num;
		u32 sd40_seq_sectors;
	};

	int counter;

	int sd_clock;

	int mmc_dont_switch_bus;

	int sd_pass_thru_en;
	int pre_cmd_err;
	u8 last_rsp_type;
	u8 rsp[24];

	u8 func_group1_mask;
	u8 func_group2_mask;
	u8 func_group3_mask;
	u8 func_group4_mask;

	u8 current_limit;
	u8 sd_switch_fail;
	u8 sd_read_phase;


	u8 sd_reset_fail;	/* sangdy2010-07-01 */
	u8 sd_send_status_en;

	/* #ifdef SUPPORT_SD_LOCK */
	u8 sd_lock_status;
	u8 sd_erase_status;
	u8 sd_lock_notify;
	/* #endif */

	/* Default Tx/Rx phase */
	u8 sd_default_tx_phase;
	u8 sd_default_rx_phase;

	int non_sd40_flow;
	int sd40_2L_HD;
	int hibernate_mode_en;

	/* for UHSII card */
	struct {
		u32 dormant;
		u8 sd40_did; /* Destination ID*/
		u8 sd40_sid_tid; /* Source ID and Transaction ID */
		u8 sd40_config;
		u8 lan_mode;
		u8 n_fcu;
		u8 tmode;
		/* u8 tmode_byte; */
		int dcmd_tlen;
		int trans_abort;
		int write_at_LM_1;
		struct {
			struct sd40_capabilities dev;
			struct sd40_capabilities host;
		} cap;

		struct {
			struct sd40_capabilities dev;
			struct sd40_capabilities host;
		} setting;
	};

	int need_retune;

	struct reset_info {
		int s18a;
		int sd_dont_switch;
		enum  sd_bus_width switch_bus_width;
	} r;

	/*for cprm erase card*/
	u8 sd_cprm_sp;
	u8 sd_au_size;
	u8 sd_uhs_au_size;
	u16 sd_erase_size;
	u8 sd_erase_timeout;
	u8 sd_erase_offset;
	u32 sd_erase_start_block;
	u32 sd_erase_end_block;
	struct sd_ge ge;

	u32 protected_area_size;
	u8 cprm_security_sp;
};

void sd_comm_set_err_code(struct card_reader *cr, struct card *c,
								u8 err_code);
void sd_comm_clr_err_code(struct card_reader *cr, struct card *c);
int sd_comm_check_err_code(struct card_reader *cr, struct card *c,
								u8 err_code);
int sd_comm_query_switch_result(struct card_reader *cr,
					u8 func_group, u8 func_to_switch,
							u8 *buf, int buf_len);
int sd_comm_check_switch(struct card_reader *cr, struct card *c,
					u8 func_group, u8 func_to_switch,
					enum sd_bus_width bus_width);
int sd_comm_ddr_tuning(struct card_reader *cr, struct card *c);
int sd_comm_mmc_ddr_tuning(struct card_reader *cr,
							struct card *c);
int sd_comm_sdr_tuning(struct card_reader *cr, struct card *c);
int sd_comm_mmc_ddr_tuning(struct card_reader *cr,
							struct card *c);
int sd_comm_wait_state_data_ready(struct card_reader *cr,
		struct card *c, u8 state, u8 data_ready, int polling_cnt);
int sd_comm_check_wp_state(struct card_reader *cr, struct card *c);
int sd_comm_select_card(struct card_reader *cr, struct card *c,
								int select);
int sd_comm_update_lock_status(struct card_reader *cr,
							struct card *c);
void sd_clr_err(struct card_reader *cr);
int sd_voltage_switch(struct card_reader *cr, struct card *c);
int sd_set_clock_divider(struct card_reader *cr, u8 divider);
int sd_mmc_switch_timing_bus(struct card_reader *cr,
					struct card *c, int switch_ddr);
int sd_dummy_clock(struct card_reader *cr);
void sd_set_error_code(struct card_reader *cr, struct card *c,
		u8 err_code);
void sd_clear_error_code(struct card_reader *cr, struct card *c);
int sd_check_error_code(struct card_reader *cr, struct card *c,
								u8 err_code);

struct card;
struct card_reader;
int sd_card_reset(struct card *c);
int sd_card_disable(struct card *c);
int sd_card_rw(struct scsi_cmnd *srb, struct card *c, u32 lba, u16 num);
int sd_card_cleanup(struct card *c);
int sd_card_release(struct card *c);
int sd_card_update_status(struct card *c);

int sd_comm_initialize_wrap(struct card *c);
void sd_choose_proper_clock(struct card_reader *cr, struct card *card);
int sd_switch_clock(struct card_reader *cr, struct card *card);
void sd_clear_sd_error(struct card_reader *cr);
void sd_print_register(struct card_reader *cr);
int sd_check_csd(struct card_reader *cr, struct card *c, char check_wp);
int sd_set_sample_push_timing(struct card_reader *cr, struct card *c);
int sd_prepare_init(struct card_reader *cr, struct card *c);
int sd_prepare_reset(struct card_reader *cr, struct card *c);
int sd_set_byte_count_per_blk(struct card_reader *cr, int bcpb);
int sd_set_blk_count(struct card_reader *cr, int count);
int sd_change_bank_voltage(struct card_reader *cr, u8 volt);
int sd_get_crc7(struct card_reader *cr, u8 *buf);
int sd_init_power(struct card_reader *cr);
int sd_change_phase(struct card_reader *cr, struct card *c,
				u8 sample_point, u8 tune_dir);
int sd_wait_data_idle(struct card_reader *cr);
int sd_set_init_para(struct card_reader *cr, struct card *c);
int sd_mmc_switch_timing_bus(struct card_reader *cr,
			struct card *c, int switch_ddr);
int sd_auto_tune_clock(struct card_reader *cr, struct card *c);
int check_sd_speed_prior(u32 sd_speed_prior);
int check_sd_current_prior(u32 sd_current_prior);
int sd_select_card(struct card_reader *cr, struct card *c, int select);
int sd__update_lock_status(struct card_reader *cr, struct card *c);
int sd_check_spec(struct card_reader *cr, struct card *c);
int sd_update_lock_status(struct card_reader *cr, struct card *c);
int sd_check_data0_status(struct card_reader *cr, struct card *card);
int sd_reset_card(struct card_reader *cr, struct card *c);
int sd_reset_sd(struct card_reader *cr, struct card *c);
int sd_cleanup_work(struct card_reader *cr, struct card *c);
int sd_read_data(struct card_reader *cr,
	struct card *card, u8 trans_mode, u8 *cmd, int cmd_len,
	u16 byte_cnt, u16 blk_cnt, u8 bus_width,
				u8 *buf, int buf_len, int timeout);
int sd_pull_ctl_disable(struct card_reader *cr);
int sd_pull_ctl_enable(struct card_reader *cr);
int sd_release(struct card_reader *cr, struct card *c);
void sd_get_appcmd_responce_type(u32 opcode, u8 *type, unsigned int *len);
void sd_get_cmd_responce_type(u32 opcode, u8 *type, unsigned int *len);
int sd_send_cmd(struct card_reader *cr, struct card *card,
		struct sd_command *cmd);
int sd_power_off_card3v3(struct card_reader *cr);
int sd_sd30_drive_sel(struct card_reader *cr, u8 voltage);
int sd_check_sdio_card_exist(struct card *card);
int sd_stop_transmission(struct card *card);
int sd_send_status(struct card *card, u8 *rsp);
void sd_stop_seq_mode(struct card_reader *cr, struct card *card);
int sd_stop_sequential(struct card *card);
int sd_check_seq(struct card *card, struct scsi_cmnd *srb);
int sd_send_cid(struct card *card, u8 *cid);
int sd_clear_status(struct card *card);
int sd_cmd_support(struct card *card, int opcode);
int sd_speed_class_control(struct card_reader *cr, struct card *card, u32 arg);
int sd_read_extr_single(struct card_reader *cr, struct card *card, u32 arg,
		u8 *__buf);
int sd_write_extr_single(struct card_reader *cr, struct card *card, u32 arg,
		u8 *__buf);
int sd_read_extr_multi(struct card_reader *cr, struct card *card, u32 arg,
		u8 *buf);
int sd_write_extr_multi(struct card_reader *cr, struct card *card, u32 arg,
		u8 *buf);
static inline int sd_ge_status_ready(struct card_reader *cr, struct card *card)
{
	return sd_ge_extr(cr, card, SD_GE_ADDR_STATUS) == SD_GE_STATUS_READY;
}
static inline int sd_ge_status_resp(struct card_reader *cr, struct card *card)
{
	return sd_ge_extr(cr, card, SD_GE_ADDR_STATUS) == SD_GE_STATUS_RESPONSE;
}
static inline int sd_ge_status_send(struct card_reader *cr, struct card *card)
{
	return sd_ge_extr(cr, card, SD_GE_ADDR_STATUS) == SD_GE_STATUS_SEND;
}
static inline int sd_ge_status_recv(struct card_reader *cr, struct card *card)
{
	return sd_ge_extr(cr, card, SD_GE_ADDR_STATUS) == SD_GE_STATUS_RECEIVE;
}
int sd_init_sd_cfg_reg(struct card_reader *cr, struct card *c);


#endif /* SD_COMM_H */
