#ifndef __SD40_H__
#define __SD40_H__

#define MAX_SD40_PAYLOAD_LEN			16
#define NATIVE_PACKET				0x80
#define APP_PACKET				0x00
#define HD_TYPE_MASK				(0x07<<4)
#define HD_CCMD					(0x00<<4)
#define HD_DCMD					(0x01<<4)
#define HD_RES					(0x02<<4)
#define HD_DATA					(0x03<<4)
#define HD_DID_MASK				0x0F

#define ARG_DIR					0x80
#define ARG_DIR_W				0x80
#define ARG_DIR_R				0x00
#define ARG_PLEN_MASK				0x30
#define PLEN_0B					0x00 /* 0 byte */
#define PLEN_4B					0x10 /* 4 bytes */
#define PLEN_8B					0x20 /* 8 bytes */
#define PLEN_16B				0x30 /* 16 bytes */
#define ARG_IOADR_H_MASK			0x0F

/* IOADR of Generic Capabilities Register (DW) */
#define SD40_IOADR_GEN_CAP_L			0x00
#define SD40_IOADR_GEN_CAP_H			0x01

/* IOADR of PHY Capabilities Register (DW)  */
#define SD40_IOADR_PHY_CAP_L			0x02
#define SD40_IOADR_PHY_CAP_H			0x03

/* IOADR LINK/TRAN Capabilities Register (DW) */
#define SD40_IOADR_LINK_CAP_L			0x04
#define SD40_IOADR_LINK_CAP_H			0x05

#define SD40_IOADR_GEN_SET_L			0x08
#define SD40_IOADR_GEN_SET_H			0x09

#define SD40_IOADR_PHY_SET_L			0x0A
#define SD40_IOADR_PHY_SET_H			0x0B

#define SD40_IOADR_LINK_SET_L			0x0C
#define SD40_IOADR_LINK_SET_H			0x0D

/* Node ID (First or Last)	ENUMERATE */
#define SD40_ID_F_MASK				0xF0
#define SD40_ID_L_MASK				0x0F

/* SD40 I/O Address space offset */
#define SD40_IOADR_CFG_BASE			0x00  /*000h : 0FFh*/
#define SD40_IOADR_INT_BASE			0x01  /*100h : 17Fh*/
#define SD40_IOADR_ST_BASE			0x01  /*180h : 1FFh*/
#define SD40_IOADR_CMD_BASE			0x02  /*200h : 2FFh*/
#define SD40_IOADR_VENDOR_BASE			0x0F  /*F00h : FFFh*/


/* Command Register (CMD_REG).  DW, Base on IOADR_CMD_BASE */
#define SD40_FULL_RESET				0x00
#define SD40_GO_DORMANT_STATE			0x01
#define SD40_DEVICE_INIT			0x02
#define SD40_ENUMERATE				0x03
#define SD40_TRANS_ABORT			0x04


/* Group  Descriptor */
#define SD40_GD_0				0x00
#define SD40_GD_1				0x10
#define SD40_GD_2				0x20
#define SD40_GD_3				0x30
#define SD40_GD_4				0x40
#define SD40_GD_5				0x50

/* Group  Allocated  Power   DEVICE_INIT */
#define SD40_GAP_MASK				0x0F
#define SD40_GAP_0				0x00
#define SD40_GAP_100mA				0x01
#define SD40_GAP_200mA				0x02
#define SD40_GAP_300mA				0x03
#define SD40_GAP_400mA				0x04
#define SD40_GAP_500mA				0x05
#define SD40_GAP_600mA				0x06
#define SD40_GAP_700mA				0x07
#define SD40_GAP_800mA				0x08
#define SD40_GAP_900mA				0x09
#define SD40_GAP_1000mA				0x0A
#define SD40_GAP_1100mA				0x0B
#define SD40_GAP_1200mA				0x0C
#define SD40_GAP_1300mA				0x0D
#define SD40_GAP_1400mA				0x0E

#define SD40_CF					0x08

/* Device  Allocated  Power   DEVICE_INIT */
#define SD40_DAP_Default			0x00
#define SD40_DAP_100mA				0x10
#define SD40_DAP_200mA				0x20
#define SD40_DAP_300mA				0x30
#define SD40_DAP_400mA				0x40
#define SD40_DAP_500mA				0x50
#define SD40_DAP_600mA				0x60
#define SD40_DAP_700mA				0x70
#define SD40_DAP_800mA				0x80
#define SD40_DAP_900mA				0x90
#define SD40_DAP_1000mA				0xA0
#define SD40_DAP_1100mA				0xB0
#define SD40_DAP_1200mA				0xC0
#define SD40_DAP_1300mA				0xD0
#define SD40_DAP_1400mA				0xE0

#define SD40_LANE_MODE_MASK			0x3F
#define SD40_LANE_MODE_2L_HD			0x01
#define SD40_LANE_MODE_2D1U_FD			0x02
#define SD40_LANE_MODE_1D2U_FD			0x04
#define SD40_LANE_MODE_2D2U_FD			0x08

#define SD40_APP_TYPE_MASK			0x07
#define SD40_APP_TYPE_SD_MEMORY			0x01
#define SD40_APP_TYPE_SDIO			0x02
#define SD40_APP_TYPE_EMBEDDED			0x04


#define SD40_DEV_PHY_HIBERNATE_SUPPORT		0x80
#define SD40_DEV_IN_DORMANT			0x10



#define LOW_PWR_MODE				0x01
#define FAST_PWR_MODE				0x00

#define SD40_CONFIG_COMPLETE			0x80
#define SD40_CONFIG_UNCOMPLETE			0x00

#define SPEED_SEL_MASK				0xC0
#define SPEED_SEL_RANGE_A			(0x00 << 6)
#define SPEED_SEL_RANGE_B			(0x01 << 6)

#define DEVICE_INIT_RETRY_CNT			30

#define init_sd40_ccmd_header(header, np, type, did, sid, tid)	\
	do { \
		header = (np | type | did) << 8;	\
		heade |= (sid << 4) | (tdi & 0x7);	\
	} while (0);

#define init_sd40_ccmd_arg(arg, dir, plen, base_addr, addr)	\
	do { \
		arg = (dir | plen | base_addr) << 8;	\
		arg |= addr;	\
	} while (0);

struct sd40_capabilities {
	struct {
		u8 lan_mode;
		u8 lan_2L_HD;
	}; /* for generic */

	struct {
		u8 hibernate;
		u8 n_lss;
		u8 speed_range;
	}; /* phy */

	struct {
		u8 gap;
		u8 n_fcu;
	}; /* link_tran */

	u8 raw_generic[16];
	u8 raw_phy[16];
	u8 raw_link[16];

};

struct sd40_tlp {
	u8 header[2];
	u8 arg[2];
	union {
		u8 payload[16];
		struct {
			union {
				u32 sd_arg;
				u32 addr;
			};
			u32 trans_len;
		} ext_arg;
	};

	u8 response[20];

	int scatter_gather;
	union {
		u8 *data;
		struct scatterlist *sg;
	};

	u32 timeout;
};

struct sd_command;
int sd40_send_tlp(struct card *card, struct sd40_tlp *tlp);
int sd40_send_sdtran_dcmd(struct card *card, struct sd_command *cmd);
int sd40_send_sdtran_ccmd(struct card *card, struct sd_command *cmd);
int sd40_send_device_init(struct card *card, u16 payload_h, u16 *payload_d);
int sd40_enumerate_device(struct card *card, u8 node_id_h, u8 *node_id_d);
int sd40_go_dormant_state(struct card *card, int dor_mode);
int sd40_trans_abort(struct card *card);
int sd40_read_cfg_reg(struct card *card, u8 addr, u8 *cfg, int len);
int sd40_write_cfg_reg(struct card *card, u8 addr, u8 *cfg, int len);
int sd40_send_native_ccmd(struct card *card, u16 header, u16 arg, u8 *payload);
int sd40_initialization(struct card_reader *cr, struct card *c);
int sd_prepare_uhsii_rw(struct card *card, struct scsi_cmnd *srb);
int sd40_process_transfer_error(struct card_reader *cr, struct card *c);

/* callback defined by HOST */
int sd40_init_hw(struct card_reader *cr, struct card *card);
int sd40_init_hw_m(struct card_reader *cr, struct card *card);
int sd40_uninit_hw(struct card_reader *cr, struct card *card);
int sd40_process_domant_state(struct card_reader *cr, struct card *c,
		int dir, int dor_mode);
void dump_all_sd40_register(struct card_reader *cr);
int sd40_take_dev_init_process(struct card *card);
int sd40_common_take_dev_init_process(struct card *card);
#endif
