/* Driver for Realtek PCIe card reader
 * Header file
 *
 * Copyright(c) 2009 Realtek Semiconductor Corp. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *   wwang (wei_wang@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 *
 *   edwin.rong(edwin_rong@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 *
 *   Tony.Nie(tony_nie@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 */

#ifndef __CR_REGS_H__
#define __CR_REGS_H__

#ifdef USING_PPBUF
#define PPBUF_LEN			512
#else
#define PPBUF_LEN			64
#endif

/* PCI Operation Register Address */
#define HCBAR		0x00
#define HCBCTLR		0x04
#define HDBAR		0x08
#define HDBCTLR		0x0C
#define HAIMR		0x10
#define BIPR		0x14
#define BIER		0x18

/* Host command buffer control register */
#define STOP_CMD		(0x01 << 28)

/* Host data buffer control register */
#define SDMA_MODE		0x00
#define ADMA_MODE		(0x02 << 26)
#define STOP_DMA		(0x01 << 28)
#define TRIG_DMA		(0x01 << 31)

/* Bus interrupt pending register */
#define CMD_DONE_INT		(1 << 31)
#define DATA_DONE_INT		(1 << 30)
#define TRANS_OK_INT		(1 << 29)
#define TRANS_FAIL_INT		(1 << 28)
#define SD_DAT3_INT		(1 << 27)
#define MS_INT			(1 << 26)
#define SD_INT			(1 << 25)
#define GPIO0_INT		(1 << 24)
#define OC_INT			(1 << 23)
#define SD_WRITE_PROTECT	(1 << 19)
#define MS_EXIST		(1 << 17)
#define SD_EXIST		(1 << 16)
#define XD_EXIST		(1 << 18)
#define DELINK_INT		GPIO0_INT
#define SD_OC_INT		(1 << 22)

#define CARD_INT		(MS_INT | SD_INT | SD_DAT3_INT)
#define NEED_COMPLETE_INT	(DATA_DONE_INT | \
						TRANS_OK_INT | \
						TRANS_FAIL_INT)
#define RTSX_INT		(CMD_DONE_INT | \
						NEED_COMPLETE_INT | \
						CARD_INT | \
						GPIO0_INT | \
						OC_INT)

#define CARD_EXIST		(MS_EXIST | SD_EXIST)

/* Bus interrupt enable register */
#define CMD_DONE_INT_EN		(1 << 31)
#define DATA_DONE_INT_EN	(1 << 30)
#define TRANS_OK_INT_EN		(1 << 29)
#define TRANS_FAIL_INT_EN	(1 << 28)
#define MS_INT_EN		(1 << 26)
#define SD_INT_EN		(1 << 25)
#define XD_INT_EN		(1 << 27)
#define GPIO0_INT_EN		(1 << 24)
#define OC_INT_EN		(1 << 23)
#define DELINK_INT_EN		GPIO0_INT_EN
#define SD_OC_INT_EN		(1 << 22)


#define READ_REG_CMD		0
#define WRITE_REG_CMD		1
#define CHECK_REG_CMD		2

#define HOST_TO_DEVICE		0
#define DEVICE_TO_HOST		1

#define REG_SD_CFG1			0xFDA0
#define REG_SD_CFG2			0xFDA1
#define REG_SD_CFG3			0xFDA2
#define   SD30_CLK_END_EN		0x10

#define REG_SD_STAT1		0xFDA3
#define REG_SD_STAT2		0xFDA4
#define REG_SD_BUS_STAT		0xFDA5
#define REG_SD_PAD_CTL		0xFDA6
#define REG_SD_SAMPLE_POINT_CTL	0xFDA7
#define REG_SD_PUSH_POINT_CTL	0xFDA8
#define SD_CLK_POLARITY_SWAP    (1 << 0)
#define REG_SD_CMD0			0xFDA9
#define REG_SD_CMD1			0xFDAA
#define REG_SD_CMD2			0xFDAB
#define REG_SD_CMD3			0xFDAC
#define REG_SD_CMD4			0xFDAD
#define REG_SD_CMD5			0xFDAE
#define REG_SD_BYTE_CNT_L		0xFDAF
#define REG_SD_BYTE_CNT_H		0xFDB0
#define REG_SD_BLOCK_CNT_L		0xFDB1
#define REG_SD_BLOCK_CNT_H		0xFDB2
#define REG_SD_TRANSFER		0xFDB3
#define REG_SD_CMD_STATE		0xFDB5
#define REG_SD_DATA_STATE		0xFDB6
#define REG_SD_STOP_SDCLK_CFG		0xFDB8
#define   SD30_CLK_STOP_CFG_EN		0x04
#define   SD30_CLK_STOP_CFG1		0x02
#define   SD30_CLK_STOP_CFG0		0x01


#define REG_SD_DUTY_CTL			0xFDBC

#define	REG_DCM_DRP_CTL		0xFC23
#define	REG_DCM_DRP_TRIG		0xFC24
#define	REG_DCM_DRP_CFG		0xFC25
#define	REG_DCM_DRP_WR_DATA_L	0xFC26
#define	REG_DCM_DRP_WR_DATA_H	0xFC27
#define	REG_DCM_DRP_RD_DATA_L	0xFC28
#define	REG_DCM_DRP_RD_DATA_H	0xFC29
#define REG_SD_VPCLK0_CTL		0xFC2A
#define REG_SD_VPCLK1_CTL		0xFC2B
#define REG_SD_DCMPS0_CTL		0xFC2C
#define REG_SD_DCMPS1_CTL		0xFC2D
#define REG_SD_VPTX_CTL		REG_SD_VPCLK0_CTL
#define REG_SD_VPRX_CTL		REG_SD_VPCLK1_CTL
#define REG_SD_DCMPS_TX_CTL		REG_SD_DCMPS0_CTL
#define REG_SD_DCMPS_RX_CTL		REG_SD_DCMPS1_CTL

#define REG_CARD_CLK_SOURCE			0xFC2E

#define REG_CARD_PWR_CTL			0xFD50
#define REG_PUPD_CFG			0x40
#define BPP_POWER_OFF				0x0F
#define BPP_POWER_5_PERCENT_ON			0x0E
#define BPP_POWER_10_PERCENT_ON			0x0C
#define BPP_POWER_15_PERCENT_ON			0x08
#define BPP_POWER_ON				0x00
#define BPP_POWER_MASK				0x0F


#define REG_CARD_CLK_SWITCH		0xFD51
#define REG_CARD_SHARE_MODE		0xFD52
#define REG_CARD_DRIVE_SEL		0xFD53
#define REG_CARD_STOP			0xFD54
#define REG_CARD_OE			0xFD55

#define REG_OLT_LED_CTL		0xFC1E
#define REG_GPIO_CTL		0xFC1F

#define REG_CARD_DATA_SOURCE	0xFD5B
#define REG_CARD_SELECT		0xFD5C
#define REG_SD30_DRIVE_SEL		0xFD5E
#define SD30_CMD_DRIVE_SEL		0xFD5E
#define SD30_CLK_DRIVE_SEL		0xFD5A
#define SD30_DAT_DRIVE_SEL		0xFD5F

#define REG_CARD_CLK_EN		0xFD69

#define REG_FPDCTL			0xFC00
#define REG_PDINFO			0xFC01

#define REG_CLK_CTL			0xFC02
#define REG_CLK_DIV			0xFC03
#define   CRC_CLK_SEL_SHIFT		0x04
#define   CRC_CLK_SEL_MASK		(0x07 << CRC_CLK_SEL_SHIFT)
#define   CRC_CLK_31D25			(0x00 << CRC_CLK_SEL_SHIFT)
#define   CRC_CLK_SSC			(0x01 << CRC_CLK_SEL_SHIFT)
#define   CRC_CLK_SSC_DIV_2		(0x02 << CRC_CLK_SEL_SHIFT)
#define   CRC_CLK_SCC_DIV_4		(0x03 << CRC_CLK_SEL_SHIFT)
#define   CRC_CLK_SSC_DIV_8		(0x04 << CRC_CLK_SEL_SHIFT)
#define   MCU_CLK_DIV_MASK		0x0F
#define REG_CLK_SEL			0xFC04

#define REG_SSC_DIV_N_0		0xFC0F
#define   SSC_DIV_N_0_MASK	0xFF
#define REG_SSC_DIV_N_1		0xFC10
#define   SSC_DIV_N_1_MASK	0x03

#define REG_RCCTL			0xFC14

#define REG_FPGA_PULL_CTL		0xFC1D
#define SD40_EXTCLK_SEL_MASK		(1 << 2)
#define SD40_EXTCLK_SEL			(1 << 2)
#define SD40_EXTCLK_UNSEL		(0 << 2)

#define REG_DBG_OE			0xFD57

#define REG_CARD_PULL_CTL1		0xFD60
#define REG_CARD_PULL_CTL2		0xFD61
#define REG_CARD_PULL_CTL3		0xFD62
#define REG_CARD_PULL_CTL4		0xFD63
#define REG_CARD_PULL_CTL5		0xFD64
#define REG_CARD_PULL_CTL6		0xFD65
#define REG_CARD_EXIST			0xFD66

/* PCI Express Related Registers */
#define REG_IRQEN0			0xFE20
#define REG_IRQSTAT0			0xFE21
#define REG_IRQEN1			0xFE22
#define REG_IRQSTAT1			0xFE23
#define REG_TLPRIEN			0xFE24
#define REG_TLPRISTAT			0xFE25
#define REG_TLPTIEN			0xFE26
#define REG_TLPTISTAT			0xFE27
#define REG_DMATC0			0xFE28
#define REG_DMATC1			0xFE29
#define REG_DMATC2			0xFE2A
#define REG_DMATC3			0xFE2B
#define REG_DMACTL			0xFE2C
#define REG_BCTL			0xFE2D
#define REG_RBBC0			0xFE2E
#define REG_RBBC1			0xFE2F
#define REG_RBDAT			0xFE30

#define REG_RBCTL			0xFE34
#define U_AUTO_DMA_EN			(1 << 5)
#define U_SD40_WRITE_SPEED_UP		(1 << 4)

#define REG_CFGADDR0			0xFE35
#define REG_CFGADDR1			0xFE36
#define REG_CFGDATA0			0xFE37
#define REG_CFGDATA1			0xFE38
#define REG_CFGDATA2			0xFE39
#define REG_CFGDATA3			0xFE3A
#define REG_CFGRWCTL			0xFE3B
#define REG_PHYRWCTL			0xFE3C
#define REG_PHYDATA0			0xFE3D
#define REG_PHYDATA1			0xFE3E
#define REG_PHYADDR			0xFE3F
#define REG_MSGRXDATA0			0xFE40
#define REG_MSGRXDATA1			0xFE41
#define REG_MSGRXDATA2			0xFE42
#define REG_MSGRXDATA3			0xFE43
#define REG_MSGTXDATA0			0xFE44
#define REG_MSGTXDATA1			0xFE45
#define REG_MSGTXDATA2			0xFE46
#define REG_MSGTXDATA3			0xFE47
#define REG_MSGTXCTL			0xFE48

#define REG_PETXCFG			0xFF03 /* 0xFE49 not used */
#define LTR_CTL				0xFE4A
#define LTR_LATENCY_MODE_MASK		(1 << 6)
#define LTR_LATENCY_MODE_HW		(0 << 6)
#define LTR_LATENCY_MODE_SW		(1 << 6)

#define REG_OBFF_CFG			0xFE4C


#define REG_CDRESUMECTL			0xFE52
#define REG_WAKE_SEL_CTL		0xFE54
#define REG_PCLK_CTL			0xFE55
#define WRITE_BOUNDARY_BUG_FIX		(1 << 4)
#define CFG_DMA_LEN_EXT			(1 << 6)
#define PCLK_MODE_SEL			(1 << 5)

#define REG_PME_FORCE_CTL		0xFE56
#define   FORCE_PME_CONTROL		(1 << 4)
#define   SEND_PME_FOR_CARD		(1 << 0)
#define RTS522A_REG_PME_FORCE_CTL	0xFF78

#define REG_ASPM_FORCE_CTL		0xFE57
#define FORCE_ASPM_CTL0			(1 << 4)
#define FORCE_ASPM_VAL_MASK		0x03

#define REG_PM_CLK_FORCE_CTL		0xFE58
#define CLK_PM_EN					(1 << 0)
#define REG_FUNC1_FORCE_CTL		0xFE59
#define REG_PERST_GLITCH_WIDTH		0xFE5C
#define REG_CHANGE_LINK_STATE		0xFE5B
#define REG_RESET_LOAD_REG		0xFE5E
#define REG_HOST_SLEEP_STATE		0xFE60

#define REG_PM_EVENT_DBG		0xFE71
#define   REG_PM_EVENT_DBG0		(1 << 3)

#define REG_NFTS_TX_CTRL		0xFE72

#define REG_PWR_GATE_CTRL		0xFE75
#define REG_PWD_SUSPEND_EN		0xFE76
#define U_REG_SUSPEND_EN		(1 << 3)


#define REG_LDO_SEL			0xFE78

#define REG_MS_CFG			0xFD40
#define REG_MS_TPC			0xFD41
#define REG_MS_TRANS_CFG		0xFD42
#define REG_MS_TRANSFER			0xFD43
#define MS_INT_REG			0xFD44
#define REG_MS_BYTE_CNT			0xFD45
#define REG_MS_SECTOR_CNT_L		0xFD46
#define REG_MS_SECTOR_CNT_H		0xFD47
#define REG_MS_DBUS_H			0xFD48

#define REG_SSC_CTL1			0xFC11
#define REG_SSC_CTL2			0xFC12


/* rts5227 OCP-related registers */
#define REG_OCPCTL			0xFD6A
#define REG_OCPSTAT			0xFD6E
#define REG_OCPGLITCH			0xFD6C
#define REG_OCPPARA1			0xFD6B
#define REG_OCPPARA2			0xFD6D

#define REG_SDIO_MUX_CTRL		0xFD6F
#define SDIO_BUS_CTRL			0x01
#define SDIO_CD_CTRL			0x02

#define SDDAT3_POLL_CFG		0xFD88
#define   SDDAT3_POLL_MASK	0x80
#define   SDDAT3_POLL_EN	0x80

#define REG_L1SUB_CFG0			0xFE8C
#define REG_L1SUB_CFG1			0xFE8D
#define REG_L1SUB_CFG2			0xFE8E
#define REG_L1SUB_CFG3			0xFE8F

#define REG_CFG_OOBS_OFF_TIMER		0xFEA6
#define REG_CFG_OOBS_ON_TIMER		0xFEA7
#define REG_CFG_VCM_ON_TIMER		0xFEA8
#define REG_CFG_OOBS_POLLING		0xFEA9

#define AUX_CLK_ACTIVE_SEL_MASK		(1 << 0)
#define MAC_CKSW_DONE			(0)

/* Memory mapping */
#define SRAM_BASE			0xE600
#define RBUF_BASE			0xF400
#define PPBUF_BASE1			0xF800
/* #ifdef USING_PPBUF */
#define PPBUF_BASE2			0xFA00
/* #else */
/* #define PPBUF_BASE2			0xF800 */
/* #endif */
#define IMAGE_FLAG_ADDR0		0xCE80
#define IMAGE_FLAG_ADDR1		0xCE81

/* Configure space registers */
/*#define LCTLR		0x80*/

/* Power down control */
#define SSC_PDCTL		0x01
#define OC_PDCTL		0x02

/* FPDCTL */
#define SSC_POWER_DOWN		0x01
#define SD_OC_POWER_DOWN	0x02
#define ALL_POWER_DOWN		0x03
#define OC_POWER_DOWN		0x02
#define DISABLE_AUTOPD_OCP	(1 << 3)

/* Card Power Control Register */
#define PMOS_STRG_MASK		0x10
#define PMOS_STRG_800mA		0x10
#define PMOS_STRG_400mA		0x00

#define POWER_OFF		0x03
#define PARTIAL_POWER_ON	0x01
#define POWER_ON		0x00

#define MS_POWER_OFF		0x0C
#define MS_PARTIAL_POWER_ON	0x04
#define MS_POWER_ON		0x00
#define MS_POWER_MASK		0x0C

#define SD_POWER_OFF		0x03
#define SD_PARTIAL_POWER_ON	0x01
#define SD_POWER_ON		0x00
#define SD_POWER_MASK		0x03

/* Card Output Enable Register */
#define XD_OUTPUT_EN		0x02
#define SD_OUTPUT_EN		0x04
#define MS_OUTPUT_EN		0x08

/* System Clock Control Register */
#define CLK_LOW_FREQ		0x01

/* System Clock Divider Register */
#define REG_CLK_DIV_1		0x01
#define REG_CLK_DIV_2		0x02
#define REG_CLK_DIV_4		0x03
#define REG_CLK_DIV_8		0x04

/* System Clock Select Register */
#define SSC_80			0
#define SSC_100			1
#define SSC_120			2
#define SSC_150			3
#define SSC_200			4

/* Card Clock Enable Register */
#define SD_CLK_EN		0x04
#define MS_CLK_EN		0x08
#define XD_CLK_EN		0x02
#define SD40_CLK_EN		0x10

/* Card Select Register */
#define XD_MOD_SEL		0x01
#define SD_MOD_SEL		0x02
#define MS_MOD_SEL		0x03
#define SD40_MOD_SEL		0x05
#define CARD_MOD_MASK		0x07

/* Card Clock Ctl */
#define CHANGE_CLK		0x01

/* SD Card Register */

/* SD_STAT1 */
#define	SD_CRC7_ERR			0x80
#define	SD_CRC16_ERR			0x40
#define	SD_CRC_WRITE_ERR		0x20
#define	SD_CRC_WRITE_ERR_MASK		0x1C
#define	GET_CRC_TIME_OUT		0x02

#define	SD_TUNING_COMPARE_ERR		0x01

/* SD_STAT2 */
#define	SD_RSP_80CLK_TIMEOUT		0x01

/* SD_BUS_STAT */
#define	SD_CLK_TOGGLE_EN		0x80
#define	SD_CLK_FORCE_STOP	        0x40
#define	SD_DAT3_STATUS		        0x10
#define	SD_DAT2_STATUS		        0x08
#define	SD_DAT1_STATUS		        0x04
#define	SD_DAT0_STATUS		        0x02
#define	SD_CMD_STATUS			0x01

/* SD_PAD_CTL */
#define	SD_IO_USING_1V8		        0x80
#define	SD_IO_USING_3V3		        0x7F
#define	TYPE_A_DRIVING		        0x00
#define	TYPE_B_DRIVING			0x01
#define	TYPE_C_DRIVING			0x02
#define	TYPE_D_DRIVING		        0x03

/* SD_SAMPLE_POINT_CTL */
#define	DDR_FIX_RX_DAT			0x00
#define	DDR_VAR_RX_DAT			0x80
#define	DDR_FIX_RX_DAT_EDGE		0x00
#define	DDR_FIX_RX_DAT_14_DELAY		0x40
#define	DDR_FIX_RX_CMD			0x00
#define	DDR_VAR_RX_CMD			0x20
#define	DDR_FIX_RX_CMD_POS_EDGE		0x00
#define	DDR_FIX_RX_CMD_14_DELAY		0x10
#define	SD20_RX_POS_EDGE		0x00
#define	SD20_RX_14_DELAY		0x08
#define SD20_RX_SEL_MASK		0x08

/* SD_PUSH_POINT_CTL */
#define	DDR_FIX_TX_CMD_DAT		0x00
#define	DDR_VAR_TX_CMD_DAT		0x80
#define	DDR_FIX_TX_DAT_14_TSU		0x00
#define	DDR_FIX_TX_DAT_12_TSU		0x40
#define	DDR_FIX_TX_CMD_NEG_EDGE		0x00
#define	DDR_FIX_TX_CMD_14_AHEAD		0x20
#define	SD20_TX_NEG_EDGE		0x00
#define	SD20_TX_14_AHEAD		0x10
#define SD20_TX_SEL_MASK		0x10
#define	DDR_VAR_SDCLK_POL_SWAP		0x01

/* SD_TRANSFER */
#define	SD_TRANSFER_START		0x80
#define	SD_TRANSFER_END			0x40
#define SD_STAT_IDLE			0x20
#define	SD_TRANSFER_ERR			0x10
/* SD Transfer Mode definition */
#define	SD_TM_NORMAL_WRITE		0x00
#define	SD_TM_AUTO_WRITE_3		0x01
#define	SD_TM_AUTO_WRITE_4		0x02
#define	SD_TM_AUTO_READ_3		0x05
#define	SD_TM_AUTO_READ_4		0x06
#define	SD_TM_CMD_RSP			0x08
#define	SD_TM_AUTO_WRITE_1		0x09
#define	SD_TM_AUTO_WRITE_2		0x0A
#define	SD_TM_NORMAL_READ		0x0C
#define	SD_TM_AUTO_READ_1		0x0D
#define	SD_TM_AUTO_READ_2		0x0E
#define	SD_TM_AUTO_TUNING		0x0F

/* SD_VPTX_CTL / SD_VPRX_CTL */
#define PHASE_CHANGE			0x80
#define PHASE_NOT_RESET			0x40

/* SD_DCMPS_TX_CTL / SD_DCMPS_RX_CTL */
#define DCMPS_CHANGE			0x80
#define DCMPS_CHANGE_DONE		0x40
#define DCMPS_ERROR			0x20
#define DCMPS_CURRENT_PHASE	0x1F

/* SD Configure 1 Register */
#define SD_CLK_DIVIDE_0			0x00
#define	SD_CLK_DIVIDE_256		0xC0
#define	SD_CLK_DIVIDE_128		0x80
#define	SD_ASYNC_FIFO_NOT_RST		0x10
#define	SD_20_MODE			0x00
#define	SD_DDR_MODE			0x04
#define	SD_30_MODE			0x08

#define SD_CLK_DIVIDE_MASK		0xC0

/* SD_CMD_STATE */
#define SD_CMD_IDLE			0x80

/* SD_DATA_STATE */
#define SD_DATA_IDLE			0x80

/* DCM_DRP_CTL */
#define DCM_RESET			0x08
#define DCM_LOCKED			0x04
#define DCM_208M			0x00
#define DCM_TX			        0x01
#define DCM_RX			        0x02

/* DCM_DRP_TRIG */
#define DRP_START			0x80
#define DRP_DONE			0x40

/* DCM_DRP_CFG */
#define DRP_WRITE			0x80
#define DRP_READ			0x00
#define DCM_WRITE_ADDRESS_50		0x50
#define DCM_WRITE_ADDRESS_51		0x51
#define DCM_READ_ADDRESS_00		0x00
#define DCM_READ_ADDRESS_51		0x51


/* MS_CFG */
#define	SAMPLE_TIME_RISING		0x00
#define	SAMPLE_TIME_FALLING		0x80
#define	PUSH_TIME_DEFAULT		0x00
#define	PUSH_TIME_ODD			0x40
#define	NO_EXTEND_TOGGLE		0x00
#define	EXTEND_TOGGLE_CHK		0x20
#define	MS_BUS_WIDTH_1			0x00
#define	MS_BUS_WIDTH_4			0x10
#define	MS_BUS_WIDTH_8			0x18
#define	MS_2K_SECTOR_MODE		0x04
#define	MS_512_SECTOR_MODE		0x00
#define	MS_TOGGLE_TIMEOUT_EN		0x00
#define	MS_TOGGLE_TIMEOUT_DISEN		0x01
#define MS_NO_CHECK_INT			0x02

/* MS_TRANS_CFG */
#define	WAIT_INT			0x80
#define	NO_WAIT_INT			0x00
#define	NO_AUTO_READ_INT_REG		0x00
#define	AUTO_READ_INT_REG		0x40
#define	MS_CRC16_ERR			0x20
#define	MS_RDY_TIMEOUT			0x10
#define	MS_INT_CMDNK			0x08
#define	MS_INT_BREQ			0x04
#define	MS_INT_ERR			0x02
#define	MS_INT_CED			0x01

/* MS_TRANSFER */
#define	REG_MS_TRANSFER_START		0x80
#define	REG_MS_TRANSFER_END			0x40
#define	REG_MS_TRANSFER_ERR			0x20
#define	MS_BS_STATE			0x10
#define	MS_TM_READ_BYTES		0x00
#define	MS_TM_NORMAL_READ		0x01
#define	MS_TM_WRITE_BYTES		0x04
#define	MS_TM_NORMAL_WRITE		0x05
#define	MS_TM_AUTO_READ			0x08
#define	MS_TM_AUTO_WRITE		0x0C

/* CARD_SHARE_MODE */
#define CARD_SHARE_MASK			0x0F
#define	CARD_SHARE_NORMAL		0x00
#define CARD_SHARE_XD			0x02
#define	CARD_SHARE_SD			0x04
#define	CARD_SHARE_MS			0x08

/* Card Transfer Reset Register */
#define SD_STOP			0x04
#define MS_STOP			0x08
#define SD_CLR_ERR		0x40
#define MS_CLR_ERR		0x80

/* CARD_CLK_SOURCE */
#define CRC_FIX_CLK		(0x00 << 0)
#define CRC_VAR_CLK0		(0x01 << 0)
#define CRC_VAR_CLK1		(0x02 << 0)
#define SD30_FIX_CLK		(0x00 << 2)
#define SD30_VAR_CLK0		(0x01 << 2)
#define SD30_VAR_CLK1		(0x02 << 2)
#define SAMPLE_FIX_CLK		(0x00 << 4)
#define SAMPLE_VAR_CLK0		(0x01 << 4)
#define SAMPLE_VAR_CLK1		(0x02 << 4)

/* Card Data Source Register */
#define PINGPONG_BUFFER		0x01
#define RING_BUFFER		0x00

/* Ring Buffer Control Register */
#define RB_FLUSH		0x80
#define RB_FULL			0x02
#define RB_EMPTY		0x01

/* PCI Express Related Registers
 * IRQEN0	--	0xFE20 */
#define DMA_DONE_INT_EN			0x80
#define SUSPEND_INT_EN			0x40
#define LINK_RDY_INT_EN			0x20
#define LINK_DOWN_INT_EN		0x10

/* IRQSTAT0 */
#define DMA_DONE_INT			0x80
#define SUSPEND_INT			0x40
#define LINK_RDY_INT			0x20
#define LINK_DOWN_INT			0x10

/* IRQEN1 */
#define MRD_ERR_INT_EN			0x40
#define MWR_ERR_INT_EN			0x20
#define SCSI_CMD_INT_EN			0x10
#define TLP_RCV_INT_EN			0x08
#define TLP_TRSMT_INT_EN		0x04
#define MRD_COMPLETE_INT_EN		0x02
#define MWR_COMPLETE_INT_EN		0x01

/* IRQSTAT1 */
#define MRD_ERR_INT			0x40
#define MWR_ERR_INT			0x20
#define SCSI_CMD_INT			0x10
#define TLP_RX_INT			0x08
#define TLP_TX_INT			0x04
#define MRD_COMPLETE_INT		0x02
#define MWR_COMPLETE_INT		0x01

/* TLPRIEN */
#define MSG_RX_INT_EN			0x08
#define MRD_RX_INT_EN			0x04
#define MWR_RX_INT_EN			0x02
#define CPLD_RX_INT_EN			0x01

/* TLPRISTAT */
#define MSG_RX_INT			0x08
#define MRD_RX_INT			0x04
#define MWR_RX_INT			0x02
#define CPLD_RX_INT			0x01

/* TLPTIEN */
#define MSG_TX_INT_EN			0x08
#define MRD_TX_INT_EN			0x04
#define MWR_TX_INT_EN			0x02
#define CPLD_TX_INT_EN			0x01

/* TLPTISTAT */
#define MSG_TX_INT			0x08
#define MRD_TX_INT			0x04
#define MWR_TX_INT			0x02
#define CPLD_TX_INT			0x01

/* DMACTL */
#define DMA_RST				0x80
#define U_SHORT_DMA_EN			0x08
#define DMA_BUSY			0x04
#define DMA_DIR_TO_CARD			0x00
#define DMA_DIR_FROM_CARD		0x02
#define DMA_EN				0x01
#define DMA_128				(0 << 4)
#define DMA_256				(1 << 4)
#define DMA_512				(2 << 4)
#define DMA_1024			(3 << 4)
#define DMA_2048			(4 << 4)
#define DMA_4096			(5 << 4)
#define DMA_8192			(6 << 4)
#define DMA_16384			(7 << 4)
#define DMA_32768			(0 << 4)
#define DMA_PACK_SIZE_MASK		0x30

/* Card Reader Controller Registers */

/* MODEDCT */
#define	RSTB_MODE_DETECT		0x80
#define	MODE_OUT_VLD			0x40
#define	MODE_OUT_0_NONE			0x00
#define	MODE_OUT_10_NONE		0x04
#define	MODE_OUT_10_47			0x05
#define	MODE_OUT_10_180			0x06
#define	MODE_OUT_10_680			0x07
#define	MODE_OUT_16_NONE		0x08
#define	MODE_OUT_16_47			0x09
#define	MODE_OUT_16_180			0x0A
#define	MODE_OUT_16_680			0x0B
#define	MODE_OUT_NONE_NONE		0x0C
#define	MODE_OUT_NONE_47		0x0D
#define	MODE_OUT_NONE_180		0x0E
#define	MODE_OUT_NONE_680		0x0F

/* OCPCTL */
#define SD_DETECT_EN			0x08
#define SD_OCP_INT_EN			0x04
#define SD_OCP_INT_CLR			0x02
#define SD_OC_CLR			0x01

#define SDVIO_DETECT_EN			(1 << 7)
#define SDVIO_OCP_INT_EN		(1 << 6)
#define SDVIO_OCP_INT_CLR		(1 << 5)
#define SDVIO_OC_CLR			(1 << 4)

/* OCPSTAT */
#define SD_OCP_DETECT			0x08
#define SD_OC_NOW			0x04
#define SD_OC_EVER			0x02

#define SDVIO_OC_NOW			(1 << 6)
#define SDVIO_OC_EVER			(1 << 5)

/* OCPGLITCH */
#define SD_OCP_GLITCH_6_4		0x00
#define SD_OCP_GLITCH_64		0x01
#define SD_OCP_GLITCH_640		0x02
#define SD_OCP_GLITCH_1000		0x03
#define SD_OCP_GLITCH_2000		0x04
#define SD_OCP_GLITCH_4000		0x05
#define SD_OCP_GLITCH_8000		0x06
#define SD_OCP_GLITCH_10000		0x07

#define SDVIO_OCP_GLITCH_MASK		0xF0
#define SDVIO_OCP_GLITCH_NONE		0x00
#define SDVIO_OCP_GLITCH_50U		0x10
#define SDVIO_OCP_GLITCH_100U		0x20
#define SDVIO_OCP_GLITCH_200U		0x30
#define SDVIO_OCP_GLITCH_600U		0x40
#define SDVIO_OCP_GLITCH_800U		0x50
#define SDVIO_OCP_GLITCH_1M		0x60
#define SDVIO_OCP_GLITCH_2M		0x70
#define SDVIO_OCP_GLITCH_3M		0x80
#define SDVIO_OCP_GLITCH_4M		0x90
#define SDVIO_OCP_GLIVCH_5M		0xA0
#define SDVIO_OCP_GLITCH_6M		0xB0
#define SDVIO_OCP_GLITCH_7M		0xC0
#define SDVIO_OCP_GLITCH_8M		0xD0
#define SDVIO_OCP_GLITCH_9M		0xE0
#define SDVIO_OCP_GLITCH_10M		0xF0

#define SD_OCP_GLITCH_MASK		0x0F
#define SD_OCP_GLITCH_NONE		0x00
#define SD_OCP_GLITCH_50U		0x01
#define SD_OCP_GLITCH_100U		0x02
#define SD_OCP_GLITCH_200U		0x03
#define SD_OCP_GLITCH_600U		0x04
#define SD_OCP_GLITCH_800U		0x05
#define SD_OCP_GLITCH_1M		0x06
#define SD_OCP_GLITCH_2M		0x07
#define SD_OCP_GLITCH_3M		0x08
#define SD_OCP_GLITCH_4M		0x09
#define SD_OCP_GLIVCH_5M		0x0A
#define SD_OCP_GLITCH_6M		0x0B
#define SD_OCP_GLITCH_7M		0x0C
#define SD_OCP_GLITCH_8M		0x0D
#define SD_OCP_GLITCH_9M		0x0E
#define SD_OCP_GLITCH_10M		0x0F

/* OCPPARA1 */
#define SDVIO_OCP_TIME_60		0x00
#define SDVIO_OCP_TIME_100		0x10
#define SDVIO_OCP_TIME_200		0x20
#define SDVIO_OCP_TIME_400		0x30
#define SDVIO_OCP_TIME_600		0x40
#define SDVIO_OCP_TIME_800		0x50
#define SDVIO_OCP_TIME_1100		0x60
#define SDVIO_OCP_TIME_MASK		0x70

#define SD_OCP_TIME_60			0x00
#define SD_OCP_TIME_100			0x01
#define SD_OCP_TIME_200			0x02
#define SD_OCP_TIME_400			0x03
#define SD_OCP_TIME_600			0x04
#define SD_OCP_TIME_800			0x05
#define SD_OCP_TIME_1100		0x06
#define SD_OCP_TIME_MASK		0x07

/* OCPPARA2 */
#define SDVIO_OCP_THD_190		0x00
#define SDVIO_OCP_THD_250		0x10
#define SDVIO_OCP_THD_320		0x20
#define SDVIO_OCP_THD_380		0x30
#define SDVIO_OCP_THD_440		0x40
#define SDVIO_OCP_THD_500		0x50
#define SDVIO_OCP_THD_570		0x60
#define SDVIO_OCP_THD_630		0x70
#define SDVIO_OCP_THD_MASK		0x70

#define SD_OCP_THD_450			0x00
#define SD_OCP_THD_550			0x01
#define SD_OCP_THD_650			0x02
#define SD_OCP_THD_750			0x03
#define SD_OCP_THD_850			0x04
#define SD_OCP_THD_950			0x05
#define SD_OCP_THD_1050			0x06
#define SD_OCP_THD_1150			0x07
#define SD_OCP_THD_MASK			0x07

/* FPGA_PULL_CTL */
#define FPGA_MS_PULL_CTL_EN		0xEF
#define FPGA_SD_PULL_CTL_EN		0xF7
#define FPGA_MS_PULL_CTL_BIT		0x10
#define FPGA_SD_PULL_CTL_BIT		0x08

/* OLT_LED_CTL */
#define LED_SHINE_EN			0x08
#define LED_SPEED_MASK			0x07

/* SSC_CTL1 */
#define SSC_RSTB		0x80
#define SSC_8X_EN		0x40
#define SSC_FIX_FRAC		0x20
#define SSC_SEL_1M		0x00
#define SSC_SEL_2M		0x08
#define SSC_SEL_4M		0x10
#define SSC_SEL_8M		0x18

/* SSC_CTL2 */
#define SSC_DEPTH_MASK		0x07
#define SSC_DEPTH_DISALBE	0x00
#define SSC_DEPTH_4M		0x01
#define SSC_DEPTH_2M		0x02
#define SSC_DEPTH_1M		0x03
#define SSC_DEPTH_512K		0x04
#define SSC_DEPTH_256K		0x05
#define SSC_DEPTH_128K		0x06
#define SSC_DEPTH_64K		0x07

/* CARD_PULL_CTL1 */
#define SD_D7_NP		0x00
#define SD_D7_PD		(0x01 << 4)
#define SD_DAT7_PU		(0x02 << 4)
#define SD_CLK_NP		0x00
#define SD_CLK_PD		(0x01 << 2)
#define SD_CLK_PU		(0x02 << 2)
#define SD_D5_NP		0x00
#define SD_D5_PD		0x01
#define SD_D5_PU		0x02

#define MS_D1_NP		0x00
#define MS_D1_PD		(0x01 << 6)
#define MS_D1_PU		(0x02 << 6)
#define MS_D2_NP		0x00
#define MS_D2_PD		(0x01 << 4)
#define MS_D2_PU		(0x02 << 4)
#define MS_CLK_NP		0x00
#define MS_CLK_PD		(0x01 << 2)
#define MS_CLK_PU		(0x02 << 2)
#define MS_D6_NP		0x00
#define MS_D6_PD		0x01
#define MS_D6_PU		0x02

/* CARD_PULL_CTL2 */
#define SD_D6_NP		0x00
#define SD_D6_PD		(0x01 << 6)
#define SD_D6_PU		(0x02 << 6)
#define SD_D0_NP		0x00
#define SD_D0_PD		(0x01 << 4)
#define SD_D0_PU		(0x02 << 4)
#define SD_D1_NP		0x00
#define SD_D1_PD		0x01
#define SD_D1_PU		0x02

#define MS_D3_NP		0x00
#define MS_D3_PD		(0x01 << 6)
#define MS_D3_PU		(0x02 << 6)
#define MS_D0_NP		0x00
#define MS_D0_PD		(0x01 << 4)
#define MS_D0_PU		(0x02 << 4)
#define MS_BS_NP		0x00
#define MS_BS_PD		(0x01 << 2)
#define MS_BS_PU		(0x02 << 2)

/* CARD_PULL_CTL3 */
#define SD_D4_NP		0x00
#define SD_D4_PD		(0x01 << 6)
#define SD_D4_PU		(0x02 << 6)

#define MS_D7_NP		0x00
#define MS_D7_PD		(0x01 << 6)
#define MS_D7_PU		(0x02 << 6)

/* CARD_PULL_CTL4 */
#define SD_D3_NP		0x00
#define SD_D3_PD		(0x01 << 4)
#define SD_D3_PU		(0x02 << 4)
#define SD_D2_NP		0x00
#define SD_D2_PD		(0x01 << 2)
#define SD_D2_PU		(0x02 << 2)

/* CARD_PULL_CTL5 */
#define MS_INS_PD		0x00
#define MS_INS_PU		(0x01 << 7)
#define SD_WP_NP		0x00
#define SD_WP_PD		(0x01 << 5)
#define SD_WP_PU		(0x02 << 5)
#define SD_CD_PD		0x00
#define SD_CD_PU		(0x01 << 4)
#define SD_CMD_NP		0x00
#define SD_CMD_PD		(0x01 << 2)
#define SD_CMD_PU		(0x02 << 2)

/* CARD_PULL_CTL6 */
#define MS_D5_NP		0x00
#define MS_D5_PD		(0x01 << 2)
#define MS_D5_PU		(0x02 << 2)
#define MS_D4_NP		0x00
#define MS_D4_PD		0x01
#define MS_D4_PU		0x02

/* PM_CLK_FORCE_CTL */
#define FORCE_PM_CLOCK		0x10
#define EN_CLOCK_PM		0x01

/* HOST_SLEEP_STATE */
#define HOST_ENTER_S3		0x02
#define HOST_ENTER_S1		0x01

/* PDINFO */
#define AUX_PWR_DETECTED	0x01

/* phy register 0x1C */
#define PHY_DEBUG_MODE		0x01

/* PWR_GATE_CTRL */
#define PWR_GATE_EN		0x01
#define LDO3318_PWR_MASK	0x06
#define LDO_ON			0x06
#define LDO_SUSPEND		0x02
#define LDO_OFF			0x00

#define LDO3318_SRC_SEL_MASK	0x03
#define LDO3318_SRC_NONE	0x00
#define LDO3318_SRC_PMOS	0x01
#define LDO3318_SRC_DV33	0x02

#define REG_PRE_RW_MODE			0xFD70
#define EN_INFINITE_MODE		(1 << 0)

/* CD_PAD_CTL */
#define REG_CD_PAD_CTL			0xFD73
#define CD_DISABLE_MASK			0x07
#define MS_CD_DISABLE			0x04
#define SD_CD_DISABLE			0x02
#define XD_CD_DISABLE			0x01
#define CD_DISABLE			0x07
#define CD_ENABLE			0x00
#define MS_CD_EN_ONLY			0x03
#define SD_CD_EN_ONLY			0x05
#define XD_CD_EN_ONLY			0x06
#define FORCE_CD_LOW_MASK		0x38
#define FORCE_CD_XD_LOW			0x08
#define FORCE_CD_SD_LOW			0x10
#define FORCE_CD_MS_LOW			0x20
#define CD_AUTO_DISABLE			0x40

#define FPGA_3V3			0x07
#define FPGA_1V8			0x01
#define ASIC_1V7			0x00
#define ASIC_1V8			0x01
#define ASIC_1V9			0x02
#define ASIC_2V0			0x03
#define ASIC_2V7			0x04
#define ASIC_2V8			0x05
#define ASIC_3V2			0x06
#define ASIC_3V3			0x07

#define REG_PERSTLB_DEGLITCH		0xFF41
#define AUTOLOAD_CFG1		0xFF44
#define AUTOLOAD_CFG2		0xFF45

#define AUTOLOAD_CFG3		0xFF46
#define U_PM_D3_EN			(1 << 4)
#define CD_RESUME_EN_MASK		0xF0

#define AUTOLOAD_CFG4		0xFF47

/* LINK/TRAN Setting Register  0xE030~0xE037 */
#define REG_SD40_LINK_SETTING_0			0xE030
#define REG_SD40_LINK_SETTING_1			0xE031
#define REG_SD40_LINK_SETTING_2			0xE032
#define REG_SD40_LINK_SETTING_3			0xE033
#define REG_SD40_LINK_SETTING_4			0xE034
#define REG_SD40_LINK_SETTING_5			0xE035
#define REG_SD40_LINK_SETTING_6			0xE036
#define REG_SD40_LINK_SETTING_7			0xE037


/* Status Register   0xE600~0xE607 */
#define REG_SD40_ST_REG_0			0xE600
#define REG_SD40_ST_REG_1			0xE601

/* SD40_ST_REG_2  ----   0xE602 */
#define REG_SD40_ST_REG_2			0xE602
#define ILLEGAL_HEADER_ERROR			0x01
#define DEVICE_SPECIFIC_ERROR			0x02
#define RETRY_EXPIRE_ERROR			0x04

#define REG_SD40_ST_REG_3			0xE603
#define REG_SD40_ST_REG_4			0xE604
#define REG_SD40_ST_REG_5			0xE605
#define REG_SD40_ST_REG_6			0xE606
#define REG_SD40_ST_REG_7			0xE607
#define REG_SD40_HDR0				0xEC00
#define REG_SD40_HDR1				0xEC01
#define REG_SD40_ARG0				0xEC02
#define REG_SD40_ARG1				0xEC03

#define REG_SD40_ARG2				0xEC04
#define ARG_TMOD_MASK				(0x0F<<3)
#define DAM_FIX_ADDR				(0x01<<3)
#define DAM_INC_ADDR				(0x00<<3)
#define TLUM_BYTE				(0x02<<3)
#define TLUM_BLOCK				(0x00<<3)
#define LM_LEN_SPECIF				(0x04<<3)
#define LM_LEN_UNSPECIF				(0x00<<3)
#define DM_2L_HD				(0x08<<3)
#define DM_FD					(0x00<<3)

#define REG_SD40_ARG3				0xEC05
#define REG_SD40_CMD
#define SD_APPCMD				(0x01<<6)
#define SD_CMD_INDEX_MASK			0x3F

#define REG_SD40_ARG4				0xEC06
#define REG_SD40_PLD_DW0_0			0xEC07
#define REG_SD40_PLD_DW0_1			0xEC08
#define REG_SD40_PLD_DW0_2			0xEC09
#define REG_SD40_PLD_DW0_3			0xEC0A
#define REG_SD40_PLD_DW1_0			0xEC0B
#define REG_SD40_PLD_DW1_1			0xEC0C
#define REG_SD40_PLD_DW1_2			0xEC0D
#define REG_SD40_PLD_DW1_3			0xEC0E
#define REG_SD40_PLD_DW2_0			0xEC0F
#define REG_SD40_PLD_DW2_1			0xEC10
#define REG_SD40_PLD_DW2_2			0xEC11
#define REG_SD40_PLD_DW2_3			0xEC12
#define REG_SD40_PLD_DW3_0			0xEC13
#define REG_SD40_PLD_DW3_1			0xEC14
#define REG_SD40_PLD_DW3_2			0xEC15
#define REG_SD40_PLD_DW3_3			0xEC16
#define REG_SD40_RX_HDR0			0xEC17
#define REG_SD40_RX_HDR1			0xEC18
#define REG_SD40_RX_ARG0			0xEC19  /*Rx pkt arg[7:0]*/

#define REG_SD40_RX_ARG1			0xEC1A  /*Rx pkt arg[15:8]*/
#define RES_NAK					0x80
#define RES_ERR_CODE_MASK			(0x07 << 4)
#define RES_COND_ERR				(0x01 << 4)
#define RES_ARG_ERR				(0x02 << 4)
#define RES_GEN_ERR				(0x03 << 4)

#define REG_SD40_RXPLD_DW0_0			0xEC1B  /*Mct_pld_dw0[7:0]*/
#define REG_SD40_RXPLD_DW0_1			0xEC1C  /*Mct_pld_dw0[15:8]*/
#define REG_SD40_RXPLD_DW0_2			0xEC1D  /*Mct_pld_dw0[23:16]*/
#define REG_SD40_RXPLD_DW0_3			0xEC1E  /*Mct_pld_dw0[31:24]*/
#define REG_SD40_RXPLD_DW1_0			0xEC1F  /*Mct_pld_dw1[7:0]*/
#define REG_SD40_RXPLD_DW1_1			0xEC20  /*Mct_pld_dw1[15:8]*/
#define REG_SD40_RXPLD_DW1_2			0xEC21  /*Mct_pld_dw1[23:16]*/
#define REG_SD40_RXPLD_DW1_3			0xEC22  /*Mct_pld_dw1[31:24]*/
#define REG_SD40_RXPLD_DW2_0			0xEC23  /*Mct_pld_dw2[7:0]*/
#define REG_SD40_RXPLD_DW2_1			0xEC24  /*Mct_pld_dw2[15:8]*/
#define REG_SD40_RXPLD_DW2_2			0xEC25  /*Mct_pld_dw2[23:16]*/
#define REG_SD40_RXPLD_DW2_3			0xEC26  /*Mct_pld_dw2[31:24]*/
#define REG_SD40_RXPLD_DW3_0			0xEC27  /*Mct_pld_dw3[7:0]*/
#define REG_SD40_RXPLD_DW3_1			0xEC28  /*Mct_pld_dw3[15:8]*/
#define REG_SD40_RXPLD_DW3_2			0xEC29  /*Mct_pld_dw3[23:16]*/
#define REG_SD40_RXPLD_DW3_3			0xEC2A  /*Mct_pld_dw3[31:24]*/

#define REG_SD40_TRAN				0xEC2B
#define SD40_TRANS_START			0x80
#define SD40_TRANS_END				0x40
#define SD40_TRANS_ERR				0x10


#define REG_SD40_CTRL			0xEC2C
#define SD40_MAC_RST				0x80
#define SD40_PHY_RST				0x40
#define SD40_PHY_ISO				0x20
#define SD40_DLSM_RST				0x10
#define SD40_TSSM_RST				0x08
#define SD40_GO_HIBERNATE			0x04
#define SD40_GO_DORMANT				0x02
#define SD40_EXIT_DORMANT			0x01

#define REG_SD40_STATUS0			0xEC2D

#define REG_SD40_STATUS1			0xEC2E
#define SD40_HEAD_ERR				0x40
#define SD40_FRAM_ERR				0x20
#define SD40_CRC_ERR				0x10
#define TSSM_STATE_MASK				0x0F

#define REG_SD40_STATUS2			0xEC2F

#define REG_SD40_STATUS3			0xEC30
#define DLSM_STATE_MASK				0xF8
#define DLSM_STATE_IDLE				(0x5 << 3)
#define SD40_CLK_RDY				0x04
#define SD40_DET				0x02
#define SD40_NORMAL_MODE			0x01

#define REG_SD40_STATUS4			0xEC31

#define REG_SD40_Dummy0				0xEC32
#define SD40_DEV_INIT_TO_EXT_EN		0x80
#define SD40_DEV_INIT_TO_EXT_DIS		0x00

#define REG_SD40_Dummy1				0xEC33

#define REG_SD40_MISC				0xEC34
#define SD40_TMOUT_ERR_CLR			0x80
#define SD40_RETRY_EN				0x20
#define SD40_RETRY_DIS				0x00
#define SD40_CMD_RSP				0x10
#define SD40_CMD_NO_RSP				0x00
#define DPHY_LINK_WIDTH_16B			0x08
#define DPHY_LINK_WIDTH_8B			0x00
#define SCRAM_DIS_EN				0x04
#define SCRAM_EN				0x00
#define TX_EDGE_NEG				0x02
#define TX_EDGE_POS				0x00
#define RX_EDGE_NEG				0x01
#define RX_EDGE_POS				0x00

#define REG_SD40_TMOUT_0			0xEC48
#define SD40_T_EIDLE_STB_OUT			0x80
#define SD40_T_ACTIVE_INIT_OUT			0x40
#define SD40_T_LIDLE_LIDLE_OUT			0x20
#define SD40_T_LIDLE_FCREQ_BOOT_OUT		0x10
#define SD40_T_DINI_CMD_OUT			0x08
#define SD40_T_CMD_RSP_OUT			0x04
#define SD40_T_FWD_BCMD_OUT			0x02
#define SD40_T_RES_FCREQ_OUT			0x01

#define REG_SD40_TMOUT_1			0xEC49
#define SD40_TIME_OUT_ERR_EN			0x80
#define SD40_T_STAT_FCREQ_OUT			0x08
#define SD40_T_FCREQ_RDY_OUT			0x04
#define SD40_T_FCRDY_EDB_OUT			0x02
#define SD40_T_EDB_STAT_OUT			0x01

#define REG_SD40_TRAN_LEN_0			0xEC4C  /* U_tran_len[7:0] */
#define REG_SD40_TRAN_LEN_1			0xEC4D
#define REG_SD40_TRAN_LEN_2			0xEC4E
#define REG_SD40_TRAN_LEN_3			0xEC4F /* U_tran_len[31:24] */

#define REG_SD40_EBSY				0xEC4B
#define SD40_HAS_EBSY                           0x80
#define SD40_NO_EBSY                            0x00
#define SD40_WAIT_EBSY                          0x40
#define SD40_NO_WAIT_EBSY                       0x00
#define SD40_EBSY_READ_RES_TMOUT		0x0
#define SD40_EBSY_WRITE_RES_TMOUT		0x1
#define SD40_STAT_EBSY_TMOUT                    0x2

#define REG_SD40_TRAN_LEN_0			0xEC4C  /* U_tran_len[7:0] */
#define REG_SD40_TRAN_LEN_1			0xEC4D
#define REG_SD40_TRAN_LEN_2			0xEC4E
#define REG_SD40_TRAN_LEN_3			0xEC4F /* U_tran_len[31:24] */

#define REG_SD40_ERR_ST				0xEC51
#define SD40_LINK_MEM_ERR			0x80
#define SD40_LINK_UNRECV_ERR			0x40
#define SD40_LINK_RECV_ERR			0x20
#define SD40_RETRY_EXP_R_ERR			0x10
#define SD40_PACKET_ERR				0x08
#define SD40_RETRY_EXP_W_ERR			0x04
#define SD40_HEADER_ERR				0x02
#define SD40_NACK_ERR				0x01

#define SD40_HD_SPEED_UP			0xEC53
#define SD40_WR_SPEED_UP_MASK			0x02
#define SD40_WR_SPEED_UP_EN			0x02
#define SD40_WR_SPEED_UP_DIS			0x00

#define SD40_STOP_CTL			0xEC5A
#define SD40_LINK_RST			0x02

#define SD40_BASE_REGISTER			0xE000
/* Generic Capabilities Register  0xE000~0xE007 */
#define REG_SD40_GEN_CAPABILITY_0		0xE000
#define REG_SD40_GEN_CAPABILITY_1		0xE001
#define REG_SD40_GEN_CAPABILITY_2		0xE002

/* PHY Capabilities Register   0xE008~0xE00F */
#define REG_SD40_PHY_CAPABILITY_0		0xE008
#define REG_SD40_PHY_CAPABILITY_1		0xE009
#define REG_SD40_PHY_CAPABILITY_2		0xE00A
#define REG_SD40_PHY_CAPABILITY_3		0xE00B
#define REG_SD40_PHY_CAPABILITY_4		0xE00C
#define SD40_N_LSS_SYNC_MASK			0x0F
#define REG_SD40_PHY_CAPABILITY_5		0xE00D
#define REG_SD40_PHY_CAPABILITY_6		0xE00E
#define REG_SD40_PHY_CAPABILITY_7		0xE00F

/* LINK/TRAN Capabilities Register   0xE010~0xE017 */
#define REG_SD40_LINK_CAPABILITY_0		0xE010
#define REG_SD40_LINK_CAPABILITY_1		0xE011
#define REG_SD40_LINK_CAPABILITY_2		0xE012
#define REG_SD40_LINK_CAPABILITY_3		0xE013
#define REG_SD40_LINK_CAPABILITY_4		0xE014
#define REG_SD40_LINK_CAPABILITY_5		0xE015
#define REG_SD40_LINK_CAPABILITY_6		0xE016
#define REG_SD40_LINK_CAPABILITY_7		0xE017

/* Generic Setting Register    0xE020~0xE027 */
#define REG_SD40_GEN_SETTING_0			0xE020

#define REG_SD40_GEN_SETTING_1			0xE021
#define REG_SD40_GEN_SETTING_2			0xE022
#define REG_SD40_GEN_SETTING_3			0xE023
#define REG_SD40_GEN_SETTING_4			0xE024
#define REG_SD40_GEN_SETTING_5			0xE025
#define REG_SD40_GEN_SETTING_6			0xE026

/* SD40_GEN_SETTING_7	----0xE027 */
#define REG_SD40_GEN_SETTING_7			0xE027
#define REG_SD40_CONFIG_RESULT			REG_SD40_GEN_SETTING_7

/* PHY Setting Register   0xE028~0xE02F */
#define REG_SD40_PHY_SETTING_0			0xE028
#define REG_SD40_PHY_SETTING_1			0xE029
#define REG_SD40_PHY_SETTING_2			0xE02A
#define REG_SD40_PHY_SETTING_3			0xE02B
#define REG_SD40_PHY_SETTING_4			0xE02C
#define REG_SD40_PHY_SETTING_5			0xE02D
#define REG_SD40_PHY_SETTING_6			0xE02E
#define REG_SD40_PHY_SETTING_7			0xE02F

#endif /* __CR_REGS_H_ */
