#ifndef __CR_SYSFS_H__
#define __CR_SYSFS_H__

#include <linux/version.h>

#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 38)
#ifndef RHEL_RELEASE
#define kstrto_function(ftype)						\
static inline int kstrto##ftype(const char *s, unsigned int base, ftype *res) \
{								\
	unsigned long tmp;					\
	int err = 0;						\
	err = strict_strtoul(s, base, &tmp);			\
	if (err)						\
		return err;					\
	*res = (ftype)tmp;					\
	return err;						\
}
kstrto_function(u8)
kstrto_function(u16)
kstrto_function(u32)
#endif	/* RHEL_RELEASE */
#endif	/* KERNEL_VERSION(2, 6, 38) */

#define to_cr_driver(driver) container_of(driver, struct cr_driver, pci_driver)

/* show sw parameter */
#define cr_show_sw_param8(field)					\
static ssize_t								\
show_##field(struct device_driver *driver, char *buf)			\
{									\
	struct cr_driver *crd = to_cr_driver(to_pci_driver(driver));	\
									\
	return sprintf(buf, "0x%02x\n", crd->sw_param.field);		\
}

#define cr_show_sw_param16(field)					\
static ssize_t								\
show_##field(struct device_driver *driver, char *buf)			\
{									\
	struct cr_driver *crd = to_cr_driver(to_pci_driver(driver));	\
									\
	return sprintf(buf, "0x%04x\n", crd->sw_param.field);		\
}

#define cr_show_sw_param32(field)					\
static ssize_t								\
show_##field(struct device_driver *driver, char *buf)			\
{									\
	struct cr_driver *crd = to_cr_driver(to_pci_driver(driver));	\
									\
	return sprintf(buf, "0x%08x\n", crd->sw_param.field);		\
}

/* store hw param */
#define cr_store_sw_param8(field)					\
static ssize_t								\
store_##field(struct device_driver *driver, const char *buf, size_t count) \
{									\
	struct cr_driver *crd = to_cr_driver(to_pci_driver(driver));	\
	u8 val = 0;							\
									\
	if (kstrtou8(buf, 0, &val) < 0)					\
		return -EINVAL;						\
									\
	crd->sw_param.field = val;					\
									\
	return count;							\
}

#define cr_store_sw_param16(field)					\
static ssize_t								\
store_##field(struct device_driver *driver, const char *buf, size_t count) \
{									\
	struct cr_driver *crd = to_cr_driver(to_pci_driver(driver));	\
	u16 val = 0;							\
									\
	if (kstrtou16(buf, 0, &val) < 0)				\
		return -EINVAL;						\
									\
	crd->sw_param.field = val;					\
									\
	return count;							\
}

#define cr_store_sw_param32(field)					\
static ssize_t								\
store_##field(struct device_driver *driver, const char *buf, size_t count) \
{									\
	struct cr_driver *crd = to_cr_driver(to_pci_driver(driver));	\
	u32 val = 0;							\
									\
	if (kstrtou32(buf, 0, &val) < 0)				\
		return -EINVAL;						\
									\
	crd->sw_param.field = val;					\
									\
	return count;							\
}

/* show hw parameter */
#define cr_show_hw_param8(field)					\
static ssize_t								\
show_##field(struct device_driver *driver, char *buf)			\
{									\
	struct cr_driver *crd = to_cr_driver(to_pci_driver(driver));	\
									\
	return sprintf(buf, "0x%02x\n", crd->hw_param.field);		\
}

#define cr_show_hw_param16(field)					\
static ssize_t								\
show_##field(struct device_driver *driver, char *buf)			\
{									\
	struct cr_driver *crd = to_cr_driver(to_pci_driver(driver));	\
									\
	return sprintf(buf, "0x%04x\n", crd->hw_param.field);		\
}

#define cr_show_hw_param32(field)					\
static ssize_t								\
show_##field(struct device_driver *driver, char *buf)			\
{									\
	struct cr_driver *crd = to_cr_driver(to_pci_driver(driver));	\
									\
	return sprintf(buf, "0x%08x\n", crd->hw_param.field);		\
}

/* store hw param */
#define cr_store_hw_param8(field)					\
static ssize_t								\
store_##field(struct device_driver *driver, const char *buf, size_t count) \
{									\
	struct cr_driver *crd = to_cr_driver(to_pci_driver(driver));	\
	u8 val = 0;							\
									\
	if (kstrtou8(buf, 0, &val) < 0)					\
		return -EINVAL;						\
									\
	crd->hw_param.field = val;					\
									\
	return count;							\
}

#define cr_store_hw_param16(field)					\
static ssize_t								\
store_##field(struct device_driver *driver, const char *buf, size_t count) \
{									\
	struct cr_driver *crd = to_cr_driver(to_pci_driver(driver));	\
	u16 val = 0;							\
									\
	if (kstrtou16(buf, 0, &val) < 0)				\
		return -EINVAL;						\
									\
	crd->hw_param.field = val;					\
									\
	return count;							\
}

#define cr_store_hw_param32(field)					\
static ssize_t								\
store_##field(struct device_driver *driver, const char *buf, size_t count) \
{									\
	struct cr_driver *crd = to_cr_driver(to_pci_driver(driver));	\
	u32 val = 0;							\
									\
	if (kstrtou32(buf, 0, &val) < 0)				\
		return -EINVAL;						\
									\
	crd->hw_param.field = val;					\
									\
	return count;							\
}

#define CR_DRIVER_ATTR(crattr)			\
	static DRIVER_ATTR(crattr, S_IRWXU, show_##crattr, store_##crattr)

extern const struct attribute_group *cr_driver_attr_groups[];

#endif /* define __CR_SYSFS_H__ */
