/* Driver for Realtek PCIe card reader
 * Header file
 *
 * Copyright(c) 2009 Realtek Semiconductor Corp. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *   wwang (wei_wang@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 *
 *   edwin.rong(edwin_rong@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 *
 *   Tony.Nie(tony_nie@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 */

#ifndef __CR_OPERATIONS_H__
#define __CR_OPERATIONS_H__

#include <asm/io.h>

#include "trace.h"
#include "cardreader.h"
#include "cardmanager.h"

int cr_common_wr_reg(struct card_reader *cr, u16 addr, u8 mask, u8 data);
int cr_common_rd_reg(struct card_reader *cr, u16 addr, u8 *data);
int cr_common_wr_cfg32(struct card_reader *cr, u16 addr, u32 mask, u32 val);
int cr_common_rd_cfg32(struct card_reader *cr, u16 addr, u32 *val);
int cr_common_wr_cfg_seq(struct card_reader *cr, u16 addr, u8 *buf, int len);
int cr_common_rd_cfg_seq(struct card_reader *cr, u16 addr, u8 *buf, int len);
int cr_common_wr_phy16(struct card_reader *cr, u8 addr, u16 val);
int cr_common_rd_phy16(struct card_reader *cr, u8 addr, u16 *val);
int cr_common_app_get_dev_sts(struct card_reader *cr, struct card *card,
							struct scsi_cmnd *srb);
int cr_common_clr_phy_reg_bit(struct card_reader *cr, u8 reg, u8 bit);
int cr_common_set_phy_reg_bit(struct card_reader *cr, u8 reg, u8 bit);

void cr_common_init_cmd(struct card_reader *cr);
void cr_common_add_cmd(struct card_reader *cr, u8 cmd_type, u16 reg_addr,
							u8 mask, u8 data);
void cr_common_send_cmd_no_wait(struct card_reader *cr);
void cr_common_stop_cmd(struct card_reader *cr, int card);
int cr_common_send_cmd(struct card_reader *cr, u8 card, int timeout);
int cr_common_read_ppbuf(struct card_reader *cr, u8 *buf, int buf_len);
int cr_common_write_ppbuf(struct card_reader *cr, u8 *buf, int buf_len);
int cr_common_detect_card(struct card_reader *cr, enum CARD_ID id);

int cr_common_vendor_rd_mem(struct card_reader *cr, struct scsi_cmnd *srb);
int cr_common_vendor_wr_mem(struct card_reader *cr, struct scsi_cmnd *srb);
int cr_common_vendor_rd_sts(struct card_reader *cr, struct card *card,
					struct scsi_cmnd *srb);
int cr_common_app_rd_phy_reg(struct card_reader *cr, struct scsi_cmnd *srb);
int cr_common_app_wr_phy_reg(struct card_reader *cr, struct scsi_cmnd *srb);
int cr_common_app_suit_rw_mem_cmd_buf(struct card_reader *cr,
							struct scsi_cmnd *srb);
int cr_common_app_dma_rb(struct card_reader *cr, struct scsi_cmnd *srb);
int cr_common_app_get_var(struct card_reader *cr, struct scsi_cmnd *srb);
int cr_common_app_set_var(struct card_reader *cr, struct scsi_cmnd *srb);
int cr_common_app_rd_cfg_byte(struct card_reader *cr, struct scsi_cmnd *srb);
int cr_common_app_wr_cfg_byte(struct card_reader *cr, struct scsi_cmnd *srb);
int cr_common_app_rd_host_reg(struct card_reader *cr, struct scsi_cmnd *srb);
int cr_common_app_wr_host_reg(struct card_reader *cr, struct scsi_cmnd *srb);
int cr_common_app_set_chip_mode(struct card_reader *cr, struct scsi_cmnd *srb);
void cr_common_enable_dma(struct card_reader *cr, u32 byte_cnt,
		u8 pack_size, enum dma_data_direction dir);
void cr_common_cd_debounce(struct card_reader *cr);
int cr_common_flush_rb(struct card_reader *cr);
int cr_common_poweron_card(struct card_reader *cr, enum CARD_ID id);
int cr_common_poweroff_card(struct card_reader *cr, enum CARD_ID id);
int cr_common_enable_card_clock(struct card_reader *cr, enum CARD_ID id);
int cr_common_disable_card_clock(struct card_reader *cr, enum CARD_ID id);
int cr_common_switch_clock(struct card_reader *cr, struct card *card, int clk);
int cr_common_select_card(struct card_reader *cr, struct card *card);
int cr_comm_read_host_sd40_iospace(struct card_reader *cr,
					u16 addr, u8 *data, int len);
int cr_comm_write_host_sd40_iospace(struct card_reader *cr,
					u16 addr, u8 *data, int len);

#endif /* RTS5227_PROTCL_H */
