/* Driver for Realtek card reader
 *
 * Copyright(c) 2012 Realtek Semiconductor Corp. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *   edwin.rong(edwin_rong@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 *
 *   Tony.Nie(tony_nie@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 */

#ifndef RTS_CR_OPTION_H
#define RTS_CR_OPTION_H

#include <linux/kernel.h>
#include <linux/device.h>

#define MS_FORMATTER_ENABLED(cr)	((cr)->option.mspro_formatter_enable)

#define PRE_READ_LEN			0x400000
#define PRE_READ_TH			64

/*unit: 512B,  >> 1M + RING_BUFFER_SIZE */
#define PRE_READ_RESERVED		0x1000

enum dev_aspm_mode {
	DEV_ASPM_DISABLE = 0,
	DEV_ASPM_DYNAMIC = 1,
	DEV_ASPM_BACKDOOR = 2,
	DEV_ASPM_STATIC	= 3,
};

struct cr_option {
	int mspro_formatter_enable;
	/* card clock expected by user for fpga platform */
	int fpga_sd_sdr104_clk;
	int fpga_sd_ddr50_clk;
	int fpga_sd_sdr50_clk;
	int fpga_sd_hs_clk;
	int fpga_mmc_52m_clk;
	int fpga_mmc_sdr200_clk;
	int fpga_ms_hg_clk;
	int fpga_ms_4bit_clk;
	int fpga_ms_1bit_clk;

	/* card clock expected by user for asic platform */
	int asic_sd_sdr104_clk;
	int asic_sd_ddr50_clk;
	int asic_sd_sdr50_clk;
	int asic_sd_hs_clk;
	int asic_mmc_52m_clk;
	int asic_mmc_sdr200_clk;
	int asic_ms_hg_clk;
	int asic_ms_4bit_clk;
	int asic_ms_1bit_clk;
	int cd_debounce_time;

	u8 ssc_depth_sd_sdr104;	/* sw */
	u8 ssc_depth_sd_ddr50;	/* sw */
	u8 ssc_depth_sd_sdr50;	/* sw */
	u8 ssc_depth_sd_hs;	/* sw */
	u8 ssc_depth_mmc_52m;	/* sw */
	u8 ssc_depth_ms_hg;	/* sw */
	u8 ssc_depth_ms_4bit;	/* sw */
	u8 ssc_depth_low_speed;	/* sw */

	/* priority of choosing sd speed funciton */
	u32 sd_speed_prior;

	/* sd card control */
	u32 sd_ctl;
	u32 sd40_ctl;

	/* Enable Selective Suspend */
	int ss_en;
	/* Interval to enter SS from IDLE state (second) */
	int ss_delay;
	int needs_remote_wakeup;
	u8 ww_enable;	/* sangdy2010-08-03:add for remote wakeup */

	int ignore_sd;
	int use_hw_setting;

	/* Enable SSC clock */
	int ssc_en;

	int auto_delink_en;

	/* sangdy2010-07-13:add FT2 fast mode */
	int FT2_fast_mode;
	/* sangdy2010-07-15:
	 * add for config delay between 1/4 PMOS and 3/4 PMOS */
	int pwr_delay;

	int xd_rw_step;		/* add to tune xd tRP */
	int D3318_off_delay;	/* add to tune D3318 off delay time */
	int delink_delay;	/* add to tune delink delay time */
	/* add for rts5129 to enable/disable D3318 off */
	u8 rts5129_D3318_off_enable;
	u8 sd20_pad_drive;	/* add to config SD20 PAD drive */
	u8 sd30_pad_drive;	/* add to config SD30 pad drive */
	/*if reset or rw fail,then set SD20 pad drive again */
	u8 reset_or_rw_fail_set_pad_drive;

	u8 sd40_initial_retry_enable;

	u8 rcc_fail_flag;	/* add to indicate whether rcc bug happen */
	u8 rcc_bug_fix_en;	/* if set,then support fixing rcc bug */
	u8 debounce_num;	/* debounce number */
	int polling_time;	/* polling delay time */
	int polling_interval_run;
	int polling_interval_idle;
	int polling_interval_ss;
	u8 led_toggle_interval;	/* used to control led toggle speed */
	int xd_rwn_step;
	u8 sd_send_status_en;
	u8 ms_errreg_fix;
	u8 reset_mmc_first;
	u8 speed_mmc;		/* when set, then try CMD55 only twice */
	u8 blink_led;
	u8 led_blink_speed;
	u8 led_always_on;	/* if set, then led always on when card exist */
	u8 dv18_voltage;	/* add to tune dv18 voltage */

	/* Data transferred in ADMA mode or not */
	int adma_mode;
	u8 lun_mode;			/* sw, hw */
	int otp_valid;

	/* ASPM value to enable L0S/L1 */
	u8 dev_aspm_val;

	int power_down_in_ss;		/* sw, hw */

	/* deprecated: sdr104_en, ddr50_en and sdr50_en had been
	 * merged into sd_capability
	 * */
	int sdr104_en;
	int ddr50_en;
	int sdr50_en;

	/* Flag to indicate whether this chip is for barossa */
	int baro_pkg;			/* QFN or LQFP, hw */

	/* ASIC or FPGA */
	int phy_debug_mode;		/* hw */
	int aux_pwr_exist;		/* hw */
	/* bit0: enable power class 1 */
	/* bit1: enable power class 2 */
	u8 ms_power_class_en;		/* hw */

	/* common: int mspro_formatter_enable;	// sw */

	/* Remote wake up enabled or not */
	int remote_wakeup_en;		/* sw */

	int ss_idle_period;		/* sw */

	/* Configure ASPM dynamically */
	int config_host_aspm;		/* sw */

	u8 card_drive_sel;		/* sw */
	u8 sd30_drive_sel_1v8;		/* sw */
	u8 sd30_drive_sel_3v3;		/* sw */

	u8 sd30_clk_drive_sel_1v8;      /* sw */
	u8 sd30_cmd_drive_sel_1v8;      /* sw */
	u8 sd30_dat_drive_sel_1v8;      /* sw */
	u8 sd30_clk_drive_sel_3v3;      /* sw */
	u8 sd30_cmd_drive_sel_3v3;      /* sw */
	u8 sd30_dat_drive_sel_3v3;      /* sw */

	u8 sd_400mA_ocp_thd;		/* sw */
	u8 sd_800mA_ocp_thd;		/* sw */
	u8 ms_ocp_thd;

	u8 efuse_ldo_voltage;

	int chk_int_en;
	int led_reverse;
	int led_period;

	/* Enable msi */
	int msi_en;			/* sw */

	int sd_timeout;		/* sw */
	int xd_timeout;
	int ms_timeout;		/* sw */
	int mspro_timeout;		/* sw */
	int sd40_timeout;		/* sw */

	/* In idle state, auto power down */
	int auto_power_down;		/* sw */


	int pmos_pwr_on_interval;	/* sw */
	int sd_voltage_switch_delay;	/* sw */
	int s3_pwr_off_delay;		/* sw */

	int force_clkreq_0;		/* sw */

	int ft2_fast_mode;		/* sw */

	int do_delink_before_power_down;/* sw */

	int polling_config;		/* sw */

	int delink_stage1_step;	/* sw */
	int delink_stage2_step;	/* sw */
	int delink_stage3_step;	/* sw */

	int auto_delink_in_L1;		/* sw */
	int safe_delink;

	int hp_watch_bios_hotplug;	/* sw */

	int support_ms_8bit;		/* sw */

	u8 max_payload;

	/* priority of choosing sd current limit funciton */
	u32 sd_current_prior;		/* sw */

	int sd_retry_cnt;
	int sdio_retry_cnt;

	int support_mmc;

	/* Relink time: */
	/* 0x000000 --> 25ms */
	/* 0x000FFF --> 40ms */
	/* 0x00FFFF --> 188ms */
	/* 0x02FFFF --> 500ms */
	/* 0x05FFFF --> 1s */
	/* 0x08FFFF --> 1.5s */
	/* 0x0FFFFF --> 2.56s */
	/* 0x1FFFFF --> 5.17s */
	u32 relink_time;		/* sw */

	int pre_read_en;		/* sw */
	/* Pre read threshold */
	int pre_read_th;		/* sw */
	int sd40_nfcu;

	/* /////////////////////////////////////////////// */
	/* for rts5209 */
	u8 sd_io; /* SD IO driver in charge or not */
	u8 sd_int; /* SD IO interrupt triggered or not */
	int auto_delink_allowed;
	int chip_insert_with_sdio;
	int sdio_aspm;
	int sdio_idle;
	int sdio_counter;
	u8 sdio_raw_data[12];
	int sdio_func_exist;
	int sdio_in_charge;

	/* Default Tx/Rx phase */
	u8 sd_sdr104_default_tx_phase; /* sw */
	u8 sd_sdr104_default_rx_phase; /* sw */
	u8 sd_sdr50_default_tx_phase;	/* sw */
	u8 sd_sdr50_default_rx_phase;	/* sw */
	u8 sd_ddr50_default_tx_phase;	/* sw */
	u8 sd_ddr50_default_rx_phase;	/* sw */

	/* SD/MMC Tx phase */
	u8 mmc_ddr_tx_phase;
	u8 mmc_sdr200_tx_phase;

	int phase_min_cnt;

	/* for rts5227 */
	/* Enable LTR */
	int ltr_en;		/* sw */
	int ltr_enabled;
	int ltr_active;
	u32 ltr_active_latency;
	u32 ltr_idle_latency;
	u32 ltr_l1off_latency;

	int obff_en;
	int obff_enabled;
	int phy_optimize;
	u8 ltr_l1off_sspwrgate;

	/* tmp: for rts5227 */
	int l1sub_fe49;
	int phy03_bit5;

	int aux_clk_freq;
	int clk_req_bypass_short;

	int ocp_en;
	int write_clear_interrupt;
	u8  ltr_l1off_snooze_sspwrgate;
	u8  reg_0xfe76;
	u8  sdr_tx_tuning_en;
	u8  force_host_aspm_val;

	u16 sd40_ebnum;


	u8 card_spt_map;
	/* u32 card_lun_map; */
	u32 sd_capability;

	enum dev_aspm_mode dev_aspm_mode;
	u8 rtd3_en;

	u32 l1_snooze_delay;

	/* bit0-3: N_LSS_DIR */
	u32 n_lss_dir;
	u8 pcie_oobs_max_value;

#define TURN_ON_LED_AT_START		(1 << 0)
#define RTD3_TEST_EN			(1 << 1)
#define ASPM_L1_1_EN			(1 << 2)
#define ASPM_L1_2_EN			(1 << 3)
#define PM_L1_1_EN			(1 << 4)
#define PM_L1_2_EN			(1 << 5)
#define HANDSHAKE_EN			(1 << 6)
#define LTR_OBFF_SETBACK_EN		(1 << 7)
#define LTR_L1SS_PWR_GATE_EN		(1 << 8)
#define FORCE_HOST_ASPM_EN		(1 << 9)
#define SD_CD_WP_REVERSE_EN		(1 << 10)
#define POLLING_OCP_EN			(1 << 11)
#define LTR_EXT_SETBACK_EN		(1 << 12)
#define	L1SS_SETBACK_EN			(1 << 13)
#define S3_CLEAR_SETBACK_EN		(1 << 14)
#define L1_SNOOZE_TEST_EN		(1 << 15)
#define CLEAR_0xFE8D_BIT0_EN		(1 << 16)
#define LTR_L1SS_PWR_GATE_CHECK_CARD_EN	(1 << 17)
#define RESET_CARD_AFTER_RESUME_EN	(1 << 18)
#define CONFIG_N_LSS_DIR		(1 << 19)
#define L1SS_SETBACK_FOR_INTEL_EN	(1 << 20)
#define EXTRA_ACTION_FOR_ASPM_EN	(1 << 21)
#define WRITE_CFG_DIFF_RTD3_S3_EN	(1 << 22)
#define RESET_CARD_MANUAL		(1 << 23)
#define CHECK_MEDIA_CHANGE		(1 << 24)
#define FREE_CLKREQ_AT_DELINK		(1 << 25)
#define CONFIG_EIDLE_DEGLITCH		(1 << 26)
#define IGNORE_HW_SETTING_SD40_CAP	(1 << 27)
#define EN_PCIE_MAX_READ_REQ_SIZE	(1 << 28)
#define POWEROFF_UHSII_AT_IDLE		(1 << 29)
#define SD40_WRITE_SPEED_UP_EN		(1 << 30)
#define CLEAR_0xFD57_EN			(1 << 31)

	u32 dev_flags;

	u32 sd_flags;

	u32 ms_flags;

	/* used seperately */
	u32 patch_flags;

	int delay_for_rw_reg;

	u32 cprm_cmd_timeout;	/* ms */

	u8 sd40_cache_rw_en;
	u8 ack_every_tlp;

	u8 thermal_sensor_en;
	u8 thermal_direct_to_step3;
	u64 thermal_time_step1;
	u64 thermal_time_step2;
	u64 thermal_time_step3;
	u64 thermal_time_recovery;
	u8 force_stop_after_rw_fail;
	u8 sd_dat3_cd_en;
	u8 dma_len_ext_en;
	u8 sd40_seq_speedup_en;
	u8 sd40_oobs_autoK_en;
	u8 polling_oobs_en;
	u8 save_oobs_autoK_en;
	u8 mem_pwr_cut_en;

	u32 idle_max_cnt;
	u32 sd40_hiber_pwr_off_delay;
	u8 sdvcc_pd;
	u8 eios_patch_en;
};

#define set_dev_flag(cr, flag) \
	((cr)->option.dev_flags |= (flag))
#define clear_dev_flag(cr, flag) \
	((cr)->option.dev_flags &= ~(flag))
#define check_dev_flag(cr, flag) \
	((cr)->option.dev_flags & (flag))

#define set_patch_flag(cr, flag)	\
	((cr)->option.patch_flags |=  (flag))
#define clear_patch_flag(cr, flag)	\
	((cr)->option.patch_flags &=  ~(flag))
#define check_patch_flag(cr, flag)	\
	((cr)->option.patch_flags & (flag))

#define set_sd_flag(cr, flag)	\
	((cr)->option.sd_ctl |=  (flag))
#define clear_sd_flag(cr, flag)	\
	((cr)->option.sd_ctl &=  ~(flag))
#define check_sd_flag(cr, flag)	\
	((cr)->option.sd_ctl & (flag))

#define set_sd40_flag(cr, flag)	\
	((cr)->option.sd40_ctl |= (flag))
#define clear_sd40_flag(cr, flag)	\
	((cr)->option.sd40_ctl &=  ~(flag))
#define check_sd40_flag(cr, flag)	\
	((cr)->option.sd40_ctl & (flag))

#endif /* RTS_CR_OPTION_H */
