/* Driver for Realtek PCIe card reader
 * Header file
 *
 * Copyright(c) 2012 Realtek Semiconductor Corp. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *   edwin.rong(edwin_rong@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 *
 *   Tony.Nie(tony_nie@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 */

#ifndef RTS_CR_PM
#define RTS_CR_PM
#include <linux/pm.h>

#include "cardreader.h"

/* PM switch for entire power management of the driver */
#define RTS_PM	1

#define PM_S1	1
#define PM_S3	3

/* Power down control */
#define SSC_PDCTL	0x01
#define OC_PDCTL	0x02

enum led_switch {
	ON = 0,
	OFF = 1,
};

enum PM_FLAG {
	PM_FULL_ON		= 0x0001,
	PM_POWR_SAVING		= 0x0002,
	PM_POWR_SAVING_MAX	= 0x0004,
};

struct cr_pm_ops {
	int (*setup)(struct card_reader *cr);
	int (*suspend)(struct device *dev, pm_message_t state);
	int (*resume)(struct device *dev);
	void (*shutdown)(struct device *dev);
	void (*update_pm)(struct card_reader *cr);
	void (*bef_pow_dwn)(struct card_reader *cr, int pm_stat);
	int (*do_resume)(struct card_reader *cr);
	void (*full_on)(struct card_reader *cr);
	void (*power_saving)(struct card_reader *cr);
	void (*power_saving_max)(struct card_reader *cr);
	int (*force_power_on)(struct card_reader *cr, u8 ctl);
	int (*force_power_down)(struct card_reader *cr, u8 ctl);
	void (*enable_aspm)(struct card_reader *cr);
	void (*disable_aspm)(struct card_reader *cr);

	/* before auto delink */
	int (*do_before_auto_delink)(struct card_reader *cr);

	int (*set_ltr_latency)(struct card_reader *cr, u32 latency);
	int (*set_l1off_sub)(struct card_reader *cr, u32 val);
	void (*set_l1off_cfg_sub_D0)(struct card_reader *cr, int active);
};

void cr_pm_set_flags(struct card_reader *cr, u16 flags);
int cr_pm_check_flags(struct card_reader *cr, u16 flags);
int cr_pm_setup(struct card_reader *cr);
void cr_pm_bef_pow_dwn(struct card_reader *cr, int pm_stat);
int cr_pm_do_resume(struct card_reader *cr);
void cr_pm_update_pm(struct card_reader *cr);
void cr_pm_update_pm_real(struct card_reader *cr);
void cr_turn_on_led(struct card_reader *cr);
void cr_turn_off_led(struct card_reader *cr);
void cr_enable_auto_blink(struct card_reader *cr);
void cr_disable_auto_blink(struct card_reader *cr);
void cr_blink_led(struct card_reader *cr);
int cr_force_power_on(struct card_reader *cr, u8 ctl);
int cr_force_power_down(struct card_reader *cr, u8 ctl);
void cr_enable_aspm(struct card_reader *cr);
void cr_disable_aspm(struct card_reader *cr);
void cr_set_l1off_sub_cfg_D0(struct card_reader *cr, int active);
void cr_set_l1off_sub_cfg_D3(struct card_reader *cr);
int cr_set_ltr_latency(struct card_reader *cr, u32 latency);
int cr_set_l1off_sub(struct card_reader *cr, u32 val);

#endif /* RTS_CR_PM */
