/* Driver for Realtek card reader
 *
 * Copyright(c) 2012 Realtek Semiconductor Corp. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *   edwin.rong(edwin_rong@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 *
 *   Tony.Nie(tony_nie@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 */

#ifndef RTS_CR_TRANSPORT_H
#define RTS_CR_TRANSPORT_H

#include <linux/mutex.h>

#define MAX_RW_REG_CNT	1024

/*
 * Transport return codes
 */
/* Transport good, command good */
#define TRANSPORT_GOOD		0
/* Transport good, command failed */
#define TRANSPORT_FAILED	1
/* Command failed, no auto-sense */
#define TRANSPORT_NO_SENSE	2
/* Transport bad (i.e. device dead) */
#define TRANSPORT_ERROR		3

enum transport_index {
	REALTEK_TRANSPORT_USB = 0,
	REALTEK_TRANSPORT_PCIE,
	/* ... add more here */
	REALTEK_MAX_TRANSPORTS,
};

enum dir_t {
	DIR_IN,
	DIR_OUT,
};

struct transport;

typedef int (*fp_init)(struct transport *tr, void *extra);

typedef int (*fp_release)(struct transport *tr);

typedef int (*fp_transfer_data)(struct transport *tr, enum dir_t dir,
			void *buf, u32 len, int use_sg, unsigned int *act_len,
			int timeout, u8 stage_flag);

typedef int (*fp_transfer_data_01)(struct transport *tr,
	unsigned int pipe, u8 request, u8 requesttype, u16 value, u16 index,
	void *data, u16 size, int timeout);

typedef int (*fp_transfer_data_02)(struct transport *tr, u16 *status);

typedef int (*fp_transfer_data_03)(struct transport *tr,
				enum dir_t dir, void *buf, void **ptr,
				unsigned int *offset, unsigned int len,
				int use_sg, unsigned int *act_len, int timeout);

/* we don't adopt function pointer array due to it's possible that
 * the parameters or the type of return value may be different.
 * if so, we can define a new type function pointer like above.
 */
struct transport_callback {
	fp_init init;
	fp_release release;
	fp_transfer_data transport_data;
	fp_transfer_data_01 transport_data_01;
	fp_transfer_data_02 transport_data_02;
	fp_transfer_data_03 transport_data_03;
};

struct transport {
	u32 catch_link_error;
	struct mutex trans_mutex;
	char *name;
	struct transport_callback cb; /* dprecated */
	void *extra;
	struct card_reader *cr;
	struct card_manager *cm;

	unsigned char *iobuf; /* I/O buffer */
	dma_addr_t iobuf_dma;
};

extern struct transport realtek_transport_sets[REALTEK_MAX_TRANSPORTS];

int tr_init(struct transport *tr, void *extra);

int tr_release(struct transport *tr);

int tr_transfer_data(struct transport *tr, u8 card, void *buf,
	size_t len, int use_sg, int dir, int timeout);
int tr_transfer_data_partial(struct transport *tr, u8 card,
	void *buf, size_t len, int use_sg, unsigned int *index,
	unsigned int *offset, int dir, int timeout);
u8 *tr_get_cmd_data(struct transport *tr);

int tr_transfer_data_01(struct transport *tr,
	unsigned int pipe, u8 request, u8 requesttype, u16 value, u16 index,
	void *data, u16 size, int timeout);

int tr_transfer_data_02(struct transport *tr, u16 *status);

int tr_transfer_data_03(struct transport *tr,
				unsigned int pipe, void *buf, void **ptr,
				unsigned int *offset, unsigned int len,
				int use_sg, unsigned int *act_len, int timeout);


struct transport *transport_alloc(int preserve_size);
int transport_init(struct transport *tr, struct transport *tr_config,
		void *extra);
void transport_release(struct transport *tr);
int tr_find_host_pci_capability(struct transport *tr, int cap);
int tr_read_host_config8(struct transport *tr, int addr, u8 *val);
int tr_write_host_config8(struct transport *tr, int addr, u8 val);
void tr_enable_host_aspm(struct transport *tr);
void tr_disable_host_aspm(struct transport *tr);
void tr_set_host_aspm(struct transport *tr, u8 val);
void tr_get_host_aspm(struct transport *tr, u8 *val);
void tr_notify_upper(struct transport *tr, void *arg, int val);
u32 tr_ioread32(struct transport *tr, u32 reg);
void tr_iowrite32(struct transport *tr, u32 reg, u32 value);
u16 tr_ioread16(struct transport *tr, u32 reg);
void tr_iowrite16(struct transport *tr, u32 reg, u16 value);
u8 tr_ioread8(struct transport *tr, u32 reg);
void tr_iowrite8(struct transport *tr, u32 reg, u8 value);

int tr_read_config8(struct transport *tr, int reg,  u8 *value);
int tr_read_config16(struct transport *tr, int reg,  u16 *value);
int tr_read_config32(struct transport *tr, int reg,  u32 *value);
int tr_write_config8(struct transport *tr, int reg, u8 value);
int tr_write_config16(struct transport *tr, int reg, u16 value);
int tr_write_config32(struct transport *tr, int reg, u32 value);
void tr_init_cmd(struct transport *tr);
void tr_add_cmd(struct transport *tr, u8 type, u16 addr, u8 mask, u8 data);
void tr_send_cmd_no_wait(struct transport *tr);
void tr_stop_cmd(struct transport *tr, int card);
int tr_send_cmd(struct transport *tr, u8 card, int timeout);
int tr_enable_msi(struct transport *tr);
void tr_synchronize_irq(struct transport *tr);
int tr_find_pci_capability(struct transport *tr, int cap);

void tr_get_host_devctl(struct transport *tr, u8 *val);

#endif /* RTS_CR_TRANSPORT_H */
