/* Driver for Realtek PCIe card reader
 * Header file
 *
 * Copyright(c) 2009 Realtek Semiconductor Corp. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *   wwang (wei_wang@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 *
 *   edwin.rong(edwin_rong@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 *
 *   Tony.Nie(tony_nie@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 */

#ifndef RTL8411_H
#define RTL8411_H

#include "cardreader.h"
#include "rts_cr.h"
#include "cardmanager.h"

#define RTL8411_PID			0x5289
#define RTL8411B_PID			0x5287
#define RTL8411_CARD_GPIO		0xFD58
#define RTL8411_CARD_AUTO_BLINK		0xFD56

/* LDO_CTL */
#define RTL8411_LDO_CTL			0xFC1E
#define BPP_LDO_POWB			0x03
#define BPP_LDO_ON			0x00
#define BPP_LDO_SUSPEND			0x02
#define BPP_LDO_OFF			0x03

#define RTL8411_LDO_SSC_CTL		0xFC1F
#define RTL8411_LDO_EN			0x08

#define RTL8411_EFUSE_CONTENT		0xFE5F

#define RTL8411_TUNED18			0x07
#define RTL8411_TUNED18_SHIFT		4

#define RTL8411_SDBUS_IS_3V3		0x04
#define RTL8411_SD30_3V3		0x04
#define RTL8411_SD30_1V8		0x00

#define RTL8411_OCPCTL			0xFC15
#define RTL8411_OCPSTAT			0xFC16
#define RTL8411_OCPGLITCH		0xFC17
#define RTL8411_OCPPARA1		0xFC18
#define RTL8411_OCPPARA2		0xFC19

#define RTL8411_PACKAGE_MODE		0xFD51

#define RTL8411_CD_DISABLE_MASK		0x06
#define RTL8411_CD_DISABLE		0x06
#define RTL8411_MS_CD_EN_ONLY		0x02
#define RTL8411_SD_CD_EN_ONLY		0x04

#define RTL8411B_DRIVER_ATTATCHED	0x74C
#define NIC_DRIVER_ATTATCHED		(1 << 24)
#define CR__DRIVER_ATTATCHED		(1 << 25)

#define RTL8411_U_CR_HW_PFM_EN		(1 << 2)
#define RTL8411_U_CR_SW_PFM_EN		(1 << 1)

#define RTL8411_LTR_CTL			0xFE92

struct rtl8411_info {
	int x;
	u8 ldo_tuned18;
	int driver_unload;
	int hw_aspm_enabled;

	/* if sdr104 really exist and PM_FULL_ON,
	 * then set sdr104_exist
	 */
	int sdr104_exist;
};

int rtl8411_init_cr(struct card_reader *cr);
void rtl8411_init_options(struct card_reader *cr);
void rtl8411_init_sw_param(struct cr_option *option);
void rtl8411_init_hw_param(struct hw_param *hw_param);
int rtl8411_init(struct card_reader *cr);
int rtl8411_poweron_card(struct card_reader *cr, enum CARD_ID id);
int rtl8411_power_off_card(struct card_reader *cr, enum CARD_ID id);
u32 rtl8411_cd_deglitch(struct card_reader *cr);
int rtl8411_ms_pull_ctl_disable(struct card_reader *cr);
int rtl8411_ms_pull_ctl_enable(struct card_reader *cr);
int rtl8411_sd_pull_ctl_disable(struct card_reader *cr);
int rtl8411_sd_pull_ctl_enable(struct card_reader *cr);
int rtl8411_sd_init_power(struct card_reader *cr);
int rtl8411_release_sd_card(struct card_reader *cr, struct card *card);
int rtl8411_sd_disable_card(struct card_reader *cr, struct card *card);
int rtl8411_sd_update_report_status(struct card_reader *cr, struct card *card);
int rtl8411_clear_ocpstat(struct card_reader *cr);
int rtl8411_enable_ocp(struct card_reader *cr);
int rtl8411_disable_ocp(struct card_reader *cr);
int rtl8411_get_ocpstat(struct card_reader *cr, u8 *val);
int rtl8411_init_ocp(struct card_reader *cr);
void rtl8411_turn_on_led(struct card_reader *cr);
void rtl8411_turn_off_led(struct card_reader *cr);
void rtl8411_enable_auto_blink(struct card_reader *cr);
void rtl8411_disable_auto_blink(struct card_reader *cr);
int rtl8411_sd_change_bank_voltage(struct card_reader *cr, u8 voltage);
int rtl8411_sd_current_over_limit400_handler(struct card_reader *cr);
int rtl8411_sd30_drive_sel(struct card_reader *cr, u8 select);
int rtl8411_calc_ssc_clock_N(struct card_reader *cr,
		struct card *card, struct ssc_clock_N *ssc);

void rtl8411_before_power_down(struct card_reader *cr, int pm_stat);
int rtl8411_do_resume(struct card_reader *cr);
void rtl8411_blink_led(struct card_reader *cr);
void rtl8411_trigger_led(struct card_reader *cr, enum led_switch s);
int rtl8411_force_power_on(struct card_reader *cr, u8 ctl);
int rtl8411_force_power_down(struct card_reader *cr, u8 ctl);
void rtl8411_enable_host_aspm(struct card_reader *cr);
void rtl8411_disable_host_aspm(struct card_reader *cr);
void rtl8411_set_host_aspm(struct card_reader *cr, u8 val);
void rtl8411_get_host_aspm(struct card_reader *cr, u8 *val);
void rtl8411_enable_aspm(struct card_reader *cr);
void rtl8411_disable_aspm(struct card_reader *cr);
void rtl8411_update_pm(struct card_reader *cr);
void rtl8411_enable_auto_blink(struct card_reader *cr);
void rtl8411_disable_auto_blink(struct card_reader *cr);
int rtl8411_set_ltr_latency(struct card_reader *cr, u32 latency);
void rtl8411_init_aspm(struct card_reader *cr);
int rtl8411_release_ms_card(struct card_reader *cr, struct card *card);
void rtl8411_deal_extra_action(struct card_reader *cr);
int rtl8411_set_ltr_latency(struct card_reader *cr, u32 latency);
int rtl8411_set_l1off_sub(struct card_reader *cr, u32 val);
void rtl8411_power_saving_max(struct card_reader *cr);

#endif /* RTL8411_H */
