/* Driver for Realtek PCIe card reader
 *
 * Copyright(c) 2012 Realtek Semiconductor Corp. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *   Tony.Nie(tony_nie@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 */

#include <linux/blkdev.h>
#include <linux/kthread.h>
#include <linux/sched.h>
#include <linux/workqueue.h>
#include <linux/dma-mapping.h>
#include <linux/pci.h>
#include <scsi/scsi.h>
#include <linux/module.h>
#include <linux/init.h>

#include "cardreader.h"
#include "cr_driver.h"

#include "cr_regs.h"
#include "pm.h"
#include "trace.h"
#include "trans_pcie.h"
#include "rts_cr.h"
#include "cards/sd.h"
#include "rtl8411.h"

extern struct cr_driver cr_pcie_driver;

struct cr_sd_ops rtl8411_sd_ops_prot = {
	.init_power = rtl8411_sd_init_power,
	.pull_ctl_enable = rtl8411_sd_pull_ctl_enable,
	.pull_ctl_disable = rtl8411_sd_pull_ctl_disable,
	.disable = rtl8411_sd_disable_card,
	.release = rtl8411_release_sd_card,
	.update_status = rtl8411_sd_update_report_status,
	.chg_bank_volt = rtl8411_sd_change_bank_voltage,
	.current_over_limit400_handler =
		rtl8411_sd_current_over_limit400_handler,
	.sd30_drive_sel = rtl8411_sd30_drive_sel,
	/* .rw = rtl8411_sd_rw, */
	/* .update_status = */
};

struct cr_ms_ops rtl8411_ms_ops_prot = {
	.pull_ctl_enable = rtl8411_ms_pull_ctl_enable,
	.pull_ctl_disable = rtl8411_ms_pull_ctl_disable,
	.release = rtl8411_release_ms_card,
};

struct cr_ops rtl8411_ops = {
	.init_options = rtl8411_init_options,
	.init = rtl8411_init,
	.poweron_card = rtl8411_poweron_card,
	.poweroff_card = rtl8411_power_off_card,
	.cd_deglitch = rtl8411_cd_deglitch,
	.get_ocpstat = rtl8411_get_ocpstat,
	.clear_ocpstat = rtl8411_clear_ocpstat,
	.enable_ocp = rtl8411_enable_ocp,
	.disable_ocp = rtl8411_disable_ocp,
	.init_ocp = rtl8411_init_ocp,

	.turn_on_led = rtl8411_turn_on_led,
	.turn_off_led = rtl8411_turn_off_led,
	.enable_auto_blink = rtl8411_enable_auto_blink,
	.disable_auto_blink = rtl8411_disable_auto_blink,
	.calc_ssc_clock_N = rtl8411_calc_ssc_clock_N,
	.deal_extra_action = rtl8411_deal_extra_action,
};

struct cr_pm_ops rtl8411_pm_ops = {
	.bef_pow_dwn = rtl8411_before_power_down,
	.do_resume = rtl8411_do_resume,
	.enable_aspm = rtl8411_enable_aspm,
	.disable_aspm = rtl8411_disable_aspm,
	.force_power_on = rtl8411_force_power_on,
	.force_power_down = rtl8411_force_power_down,
	.set_ltr_latency = rtl8411_set_ltr_latency,
	.set_l1off_sub = rtl8411_set_l1off_sub,
	.power_saving_max = rtl8411_power_saving_max,
};

struct cr_capability rtl8411b_capability = {
	.pid = 0x5287,
	.vid = 0x10EC,
	.name = "rtl8411",
	.extra_size = sizeof(struct rtl8411_info),
	.card_conf = {
		.lun_mod = SIG_LUN,
		.card_spt_map = (1 << SD_ID) | (1 << MS_ID),
		.card_lun_map = (LUN2 << 8) | (LUN1 << 4) | (LUN0),
		.sd_capability =  SD_CAP_MMC_1BIT | SD_CAP_MMC_4BIT
			| SD_CAP_MMC_8BIT | SD_CAP_HS | SD_CAP_UHSI,
	},
	.callback = {
		.specific_cmd_handlers = NULL,
		.sd_ops_prot = &rtl8411_sd_ops_prot,
		.ms_ops_prot = &rtl8411_ms_ops_prot,
		.ops = &rtl8411_ops,
		.pm_ops = &rtl8411_pm_ops,
	},
	.drv_data = &cr_pcie_driver,
};

struct cr_capability rtl8411_capability = {
	.pid = 0x5289,
	.vid = 0x10EC,
	.name = "rtl8411",
	.extra_size = sizeof(struct rtl8411_info),
	.card_conf = {
		.lun_mod = SIG_LUN,
		.card_spt_map = (1 << SD_ID) | (1 << MS_ID),
		.card_lun_map = (LUN2 << 8) | (LUN1 << 4) | (LUN0),
		.sd_capability =  SD_CAP_MMC_1BIT | SD_CAP_MMC_4BIT
			| SD_CAP_MMC_8BIT | SD_CAP_HS | SD_CAP_UHSI,
	},
	.callback = {
		.specific_cmd_handlers = NULL,
		.sd_ops_prot = &rtl8411_sd_ops_prot,
		.ms_ops_prot = &rtl8411_ms_ops_prot,
		.ops = &rtl8411_ops,
		.pm_ops = &rtl8411_pm_ops,
	},
	.tr = &realtek_transport_sets[REALTEK_TRANSPORT_PCIE],
	.drv_data = &cr_pcie_driver,
};
