/* Driver for Realtek PCIe card reader
 *
 * Copyright(c) 2009 Realtek Semiconductor Corp. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *   wwang (wei_wang@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 *
 *   edwin.rong(edwin_rong@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 *
 *   Tony.Nie(tony_nie@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 */

#include <linux/blkdev.h>
#include <linux/kthread.h>
#include <linux/sched.h>
#include <linux/workqueue.h>
#include <linux/dma-mapping.h>
#include <linux/pci.h>
#include <scsi/scsi.h>
#include <scsi/scsi_cmnd.h>
#include <linux/module.h>
#include <linux/init.h>

#include "cr_system.h"
#include "rts5227.h"
#include "cr_regs.h"
#include "pm.h"
#include "trace.h"
#include "trans_pcie.h"
#include "rts_cr.h"
#include "cards.h"
#include "cards/sd.h"
#include "cards/ms.h"

static void rts5227_init_aspm(struct card_reader *cr);

static int rts5227_init_pull_ctl(struct card_reader *cr)
{
	int retval;

	cr_init_cmd(cr);

	cr_add_cmd(cr, WRITE_REG_CMD, REG_CARD_PULL_CTL2, 0xFF, 0x55);
	cr_add_cmd(cr, WRITE_REG_CMD, REG_CARD_PULL_CTL3, 0xFF, 0xD5);

	cr_add_cmd(cr, WRITE_REG_CMD, REG_CARD_PULL_CTL5, 0xFF, 0x55);
	cr_add_cmd(cr, WRITE_REG_CMD, REG_CARD_PULL_CTL6, 0xFF, 0x15);

	retval = cr_send_cmd(cr, 0, 100);
	if (retval < 0)
		TRACE_RET(cr, STATUS_FAIL);

	return STATUS_SUCCESS;
}

static void rts5227_extra_action_4_aspm(struct card_reader *cr)
{
	struct cr_option *option = &(cr->option);

	if (DEV_ASPM_BACKDOOR == option->dev_aspm_mode) {
		int lctlr = cr->dev_pci_cap_offset + PCI_EXP_LNKCTL;
		u8 val = 0;
		cr_read_config8(cr, lctlr, &val);
		val &= 0xFC;
		val |= 2;
		cr_write_config8(cr, lctlr, val);
	}
	if ((!option->config_host_aspm)
		&& check_dev_flag(cr, FORCE_HOST_ASPM_EN))
			cr_set_host_aspm(cr, cr->host_aspm_val);
}

static int rts5227_optimize_phy(struct card_reader *cr)
{
	struct cr_option *option = &(cr->option);
	int ret;

	if (option->phy_optimize) {
		ret = cr_wr_phy16(cr, 0x00, cr->hw_param.phy0x00_val);
		if (ret != STATUS_SUCCESS)
			TRACE_RET(cr, STATUS_FAIL);

		ret = cr_wr_phy16(cr, 0x03, cr->hw_param.phy0x03_val);
		if (ret != STATUS_SUCCESS)
			TRACE_RET(cr, STATUS_FAIL);

	}

	if (option->force_clkreq_0)
		cr_wr_phy16(cr, 0x1E, cr->hw_param.phy0x1e_val_l1ss_off);
	else
		cr_wr_phy16(cr, 0x1E, cr->hw_param.phy0x1e_val);

	cr_wr_phy16(cr, 0x05, cr->hw_param.phy0x05_val);
	cr_wr_phy16(cr, 0x02, cr->hw_param.phy0x02_val);
	cr_wr_phy16(cr, 0x1D, cr->hw_param.phy0x1d_val);
	cr_wr_phy16(cr, 0x08, cr->hw_param.phy0x08_val);

	return 0;
}

int rts5227_reset_cr(struct card_reader *cr)
{
	struct cr_option *option = &(cr->option);
	struct hw_param *hw_param = &(cr->hw_param);
	struct rts_pcie *pcie = cr->tr->extra;
	int retval;
	int lctlr = cr->dev_pci_cap_offset + PCI_EXP_LNKCTL;

	cr_iowrite32(cr, HCBAR, pcie->host_cmds_addr);

	if(CHECK_PID(cr, 0x522A)) {
		cr_wr_reg(cr, REG_L1SUB_CFG0, 0x0F, 0x0C);
		cr_update_phy16(cr, 0x01, 0xFFDF, 0);
	}

	/* Disable ASPM at first, 2010-12-02, by wwang */
	/* rts5227_disable_aspm(cr); */
	rts5227_init_aspm(cr);

	if (cr->asic_code) {
		if(CHECK_PID(cr, 0x5227)) {
			if ((rts5227_optimize_phy(cr)))
				TRACE_RET(cr, STATUS_FAIL);

			retval = cr_wr_phy16(cr, 0x19, hw_param->phy0x19_val);
			if (retval != STATUS_SUCCESS)
				TRACE_RET(cr, STATUS_FAIL);
			msleep(1);
			retval = cr_wr_phy16(cr, 0x0A, 0x05C0);
			if (retval != STATUS_SUCCESS)
				TRACE_RET(cr, STATUS_FAIL);
		} else if(CHECK_PID(cr, 0x522A)) {
			if (CHECK_VERSION(cr, 0x522A, IC_VER_A)) {
				retval = cr_wr_phy16(cr, 0x03, cr->hw_param.phy0x03_val);
				if (retval != STATUS_SUCCESS)
					TRACE_RET(cr, STATUS_FAIL);

				/*For FLD*/
				cr_wr_phy16(cr, 0x02, cr->hw_param.phy0x02_val);
				cr_wr_phy16(cr, 0x1A, cr->hw_param.phy0x1a_val);
				cr_wr_phy16(cr, 0x1D, cr->hw_param.phy0x1d_val);

				/*For BER*/
				cr_wr_phy16(cr, 0x1E, cr->hw_param.phy0x1e_val);
			}
		}
	}

	if (option->eios_patch_en) {
		cr_update_phy16(cr, 0x19, 0xEFFF, 0);
		cr_update_phy16(cr, 0x1F, 0xFFFD, 0x2);
	}

	if (option->phy03_bit5) {
		u16 val;
		cr_rd_phy16(cr, 0x03, &val);
		val |= (1 << 5);
		cr_wr_phy16(cr, 0x03, val);
	}

	cr_wr_reg(cr, REG_HOST_SLEEP_STATE, 0x03, 0x00);

	/* Disable card clock */
	cr_wr_reg(cr, REG_CARD_CLK_EN, 0x1E, 0);

	cr_init_ocp(cr);
	if (cr->option.ocp_en) {
		if (CHECK_VERSION(cr, 0x5227, IC_VER_A))
			cr_clear_ocpstat(cr);
	}

	/* Enable OCP Current Limit */
	if ((CHECK_NOT_VERSION(cr, 0x5227, IC_VER_A) &&
			CHECK_NOT_VERSION(cr, 0x5227, IC_VER_B)) &&
		(CHECK_NOT_VERSION(cr, 0x522A, IC_VER_A) &&
			CHECK_NOT_VERSION(cr, 0x522A, IC_VER_B))) {
		u16 val;
		cr_rd_phy16(cr, 0x11, &val);
		val |= 1;
		cr_wr_phy16(cr, 0x11, val);
	}

	/* Configure LED */
	if (check_dev_flag(cr, TURN_ON_LED_AT_START))
		cr_turn_on_led(cr);
	else
		cr_turn_off_led(cr);

	/* LED shine disabled, set initial shine cycle period */
	cr_wr_reg(cr, REG_OLT_LED_CTL, LED_SHINE_EN | LED_SPEED_MASK, 0x02);

	/* Reset delink mode */
	cr_wr_reg(cr, REG_CHANGE_LINK_STATE, 0x0A, 0);

	/* Card driving select */
	cr_wr_reg(cr, REG_CARD_DRIVE_SEL, 0xFF,
			option->card_drive_sel);
	retval = sd_sd30_drive_sel(cr, SD_IO_3V3);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, STATUS_FAIL);

	if (cr->asic_code) {
		/* Enable SSC Clock */
		cr_wr_reg(cr, REG_SSC_CTL1, 0xFF, SSC_8X_EN | SSC_SEL_4M);
		cr_wr_reg(cr, REG_SSC_CTL2, 0xFF, 0x12);
	}

	/* Disable cd_pwr_save (u_force_rst_core_en=0, u_cd_rst_core_en=0)
	 *    0xFE5B
	 *    bit[1]    u_cd_rst_core_en	rst_value = 0
	 *    bit[2]    u_force_rst_core_en	rst_value = 0
	 *    bit[5]    u_mac_phy_rst_n_dbg	rst_value = 1
	 *    bit[4]	u_non_sticky_rst_n_dbg	rst_value = 0 */
	cr_wr_reg(cr, REG_CHANGE_LINK_STATE, 0x16, 0x10);

	/* u_force_clkreq_0
	 * If 1, CLKREQ# PIN will be forced to drive 0 to request clock */
	if (option->force_clkreq_0) {
		cr_wr_reg(cr, REG_PETXCFG, 0x80, 0x80);
		rts_trace(TRACE_CR, "CLRREQ forced to drive 0\n");
	} else {
		cr_wr_reg(cr, REG_PETXCFG, 0x80, 0x00);
		rts_trace(TRACE_CR, "CLEAR CLRREQ forced\n");
	}

	if (DEV_ASPM_STATIC == option->dev_aspm_mode) {
		u8 val = 0;
		retval = cr_read_config8(cr, lctlr, &val);
		if (retval != STATUS_SUCCESS)
			TRACE_RET(cr, STATUS_FAIL);
		val &= 0xFC;
		val |= (option->dev_aspm_val & 0x03);
		retval = cr_write_config8(cr, lctlr, val);
		if (retval != STATUS_SUCCESS)
			TRACE_RET(cr, STATUS_FAIL);
		cr->aspm_level[0] = option->dev_aspm_val;
		cr->aspm_enabled = 1;
		if (option->config_host_aspm)
			cr_set_host_aspm(cr, cr->host_aspm_val);
	} else if (DEV_ASPM_DISABLE == option->dev_aspm_mode) {
		u8 val = 0;
		retval = cr_read_config8(cr, lctlr, &val);
		if (retval != STATUS_SUCCESS)
			TRACE_RET(cr, STATUS_FAIL);
		val &= 0xFC;
		retval = cr_write_config8(cr, lctlr, val);
		if (retval != STATUS_SUCCESS)
			TRACE_RET(cr, STATUS_FAIL);
	}

	/* wwang, 2010-04-19
	 * Enable clk_request_n to enable clock power management */
	retval = cr_write_config8(cr, 0x81, 1);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, STATUS_FAIL);

	/* wwang, 2010-06-22
	 * Enter L1 when host tx idle
	 * Set 0x70F to 0x5B using backdoor */
	retval = cr_wr_cfg32(cr, 0x70C, 0xFF000000, 0x5B000000);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, STATUS_FAIL);

	/* Clear Link Ready Interrupt
	 * Write 1 to clear */
	cr_wr_reg(cr, REG_IRQSTAT0, LINK_RDY_INT,
			LINK_RDY_INT);

	/* Enlarge the estimation window of PERST# glitch
	 * to reduce the chance of invalid card interrupt */
	cr_wr_reg(cr, REG_PERST_GLITCH_WIDTH, 0xFF, 0x80);

	/* Enable power gating for autodelink */
	cr_wr_reg(cr, REG_PWD_SUSPEND_EN, 0xFF, option->reg_0xfe76);
	cr_wr_reg(cr, REG_PWR_GATE_CTRL, PWR_GATE_EN, PWR_GATE_EN);
	if (CHECK_PID(cr, 0x522A)) {
		cr_wr_reg(cr, RTS522A_REG_VREF, RTS522A_PWD_SUSPND_EN,
			RTS522A_PWD_SUSPND_EN);
	}

	if (check_dev_flag(cr, SD_CD_WP_REVERSE_EN)) {
		cr_wr_reg(cr, 0xFF03, 0x30, 0x30);
		msleep(20);
	}

	cr_enable_bus_int(cr, cr->hw_param.interrupt_en);

	if (option->write_clear_interrupt)
		cr_wr_reg(cr, REG_NFTS_TX_CTRL, 0x02, 0);

	cr_check_card_in_init(cr);

	/* Update RC oscillator to 400k
	 * bit[0] F_HIGH: for RC oscillator, Rst_value is 1'b1
	 *		1: 2M  0: 400k */
	cr_wr_reg(cr, REG_RCCTL, 0x01, 0x00);

	/* Use config space 0x81 and 0xc1 instead of PM_CLK_FORCE_CTL
	 * to enable clock power management, so we mark it here.
	 * wwang, 2010-04-19, (suggested by LaoJing)
	 * Enable clock power management and force PM clock for all functions */

	option->aux_pwr_exist = 1;
	rts_trace(TRACE_CR, "optioin->aux_pwr_exist = %d\n",
		option->aux_pwr_exist);

	if ((option->remote_wakeup_en ||
			(option->rtd3_en && cr->d3cold_supported)) &&
		!option->auto_delink_en) {

		if (CHECK_PID(cr, 0x522A))
			cr_wr_reg(cr, RTS522A_AUTOLOAD_CFG3, 0x01, 0x01);
		else if (CHECK_PID(cr, 0x5227))
			cr_wr_reg(cr, AUTOLOAD_CFG3, 0x01, 0x01);

		if (option->aux_pwr_exist) {
			if (CHECK_PID(cr, 0x522A))
				cr_wr_reg(cr, RTS522A_REG_PME_FORCE_CTL, 0x30, 0x30);
			else if (CHECK_PID(cr, 0x5227))
				cr_wr_reg(cr, REG_PME_FORCE_CTL, 0xFF, 0x33);
		}
	} else {
		if (CHECK_PID(cr, 0x522A)) {
			cr_wr_reg(cr, RTS522A_AUTOLOAD_CFG3, 0x01, 0x00);
			cr_wr_reg(cr, RTS522A_REG_PME_FORCE_CTL, 0x30, 0x20);
		} else if (CHECK_PID(cr, 0x5227)) {
			cr_wr_reg(cr, AUTOLOAD_CFG3, 0x01, 0x00);
			cr_wr_reg(cr, REG_PME_FORCE_CTL, 0xFF, 0x30);
		}
	}

	if (option->ltr_en) {
		u16 val;
		int devctl2 = cr->dev_pci_cap_offset + PCI_EXP_DEVCTL2;
		if (cr_read_config16(cr, devctl2, &val) < 0)
			TRACE_RET(cr, STATUS_FAIL);
		rts_trace(TRACE_CR, "LTR 0x%04x: 0x%04x\n", devctl2, val);
		if (val & PCI_EXP_LTR_EN) {
			option->ltr_enabled = 1;
			rts_trace(TRACE_PM, "Set active latency%#x\n",
					option->ltr_active_latency);
			option->ltr_active = 1;
			cr_set_ltr_latency(cr, option->ltr_active_latency);
		} else {
			option->ltr_enabled = 0;
		}
	}

	if (option->obff_en)
		cr_wr_reg(cr, 0xFE4C, 0x03, 0x03);
	else
		cr_wr_reg(cr, 0xFE4C, 0x03, 0x00);

	/*cr_rd_reg(cr, 0xFE1C, &val);
	cr_wr_reg(cr, REG_PERSTLB_DEGLITCH, 0xFF, val);
	cr_rd_reg(cr, 0xFE1D, &val);
	cr_wr_reg(cr, 0xFF42, 0xFF, val);*/

	if (CHECK_PID(cr, 0x522A))
		cr_wr_reg(cr, RTS522A_AUTOLOAD_CFG3, U_PM_D3_EN, 0);
	else if (CHECK_PID(cr, 0x5227))
		cr_wr_reg(cr, AUTOLOAD_CFG3, U_PM_D3_EN, 0);

	/* Switch LDO3318 source from DV33 to card_3v3 */
	if (cr->asic_code) {
		cr_wr_reg(cr, 0xFE78, 0x03, 0x00);
		/* msleep(200); */
		cr_wr_reg(cr, 0xFE78, 0x03, 0x01);
	} else {
		cr_wr_reg(cr, 0xFE78, 0x01, 0x01);
	}

	/* Init Pull control */
	retval = rts5227_init_pull_ctl(cr);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, STATUS_FAIL);

	if (check_dev_flag(cr, EXTRA_ACTION_FOR_ASPM_EN))
		rts5227_extra_action_4_aspm(cr);
	cr->driver_first_load = 0;

	return STATUS_SUCCESS;
}

static inline int rts5227_set_cfg_epcore(struct card_reader *cr,
		u16 autoload_addr, u16 cfg_addr, u8 func, u8 mode, u8 data)
{
	cr_wr_reg(cr, autoload_addr, 0xFF, data);
	cr_wr_reg(cr, autoload_addr + 1, 0xFF,
			(mode & 0x0F) | ((func & 0x03) << 4)
			| (((u8)cfg_addr & 0x03) << 6));
	cr_wr_reg(cr, autoload_addr + 2, 0xFF, (u8)(cfg_addr >> 2));
	cr_wr_reg(cr, autoload_addr + 3, 0xFF,
				((u8)(cfg_addr >> 10) & 0x3F) | 0x80);

	return STATUS_SUCCESS;
}

static inline int rts5227_auto_load(struct card_reader *cr,
			u32 relink_time, struct auto_load_map *cfg_map,
								u8 cfg_cnt)
{
	int retval;
	u8 i;

	/* RTS5227 uses a bit (bit 7 of cfg address 0x817)
	 * to lock/unlock BIOS config function, which must be
	 * enabled before setting autoload structure
	 */
	retval = cr_wr_cfg32(cr, 0x814, 0xFF000000, 0x80000000);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, STATUS_FAIL);

	if (CHECK_PID(cr, 0x5227))
		cr_wr_reg(cr, 0xFF00, 0xFF, ((cfg_cnt + 1) & 0x7F) | 0x80);
	else
		cr_wr_reg(cr, 0xFF00, 0xFF, 0x80);

	if (cr->option.auto_delink_en)
		relink_time >>= 4;
	else
		relink_time = 0;
	cr_wr_reg(cr, 0xFF01, 0xFF, (u8)relink_time);
	cr_wr_reg(cr, 0xFF02, 0xFF, (u8)(relink_time >> 8));
	cr_wr_reg(cr, 0xFF03, 0x01, (u8)(relink_time >> 16));

	cr_wr_reg(cr, 0xFF04, 0xFF, (u8)cr->ssvid);
	cr_wr_reg(cr, 0xFF05, 0xFF, (u8)(cr->ssvid >> 8));
	cr_wr_reg(cr, 0xFF06, 0xFF, (u8)cr->ssdid);
	cr_wr_reg(cr, 0xFF07, 0xFF, (u8)(cr->ssdid >> 8));

	if (CHECK_PID(cr, 0x5227)) {
		cr_wr_reg(cr, 0xFF08, 0xFF, 0x10);
		cr_wr_reg(cr, 0xFF09, 0xFF, 0x13);
		cr_wr_reg(cr, 0xFF0A, 0xFF, 0x57);
		cr_wr_reg(cr, 0xFF0B, 0xFF, 0xFE);

		for (i = 0; i < cfg_cnt; i++) {
			retval = rts5227_set_cfg_epcore(cr, 0xFF0C + i * 4,
					cfg_map[i].cfg_addr, cfg_map[i].func,
					cfg_map[i].mode, cfg_map[i].data);
			if (retval != STATUS_SUCCESS)
				TRACE_RET(cr, STATUS_FAIL);
		}
	}

	return STATUS_SUCCESS;
}

static inline void rts5227_init_cfg_map(struct auto_load_map *cfg_map,
						u16 cfg_addr, u8 func, u8 data)
{
	cfg_map->cfg_addr = cfg_addr & 0xFFFC;
	cfg_map->func = func;
	cfg_map->mode = (u8)0x01 << (cfg_addr & 0x03);
	cfg_map->data = data;
}

static int rts5227_init_from_cfg(struct card_reader *cr)
{
	u32 lval;
	struct cr_option * option = &cr->option;

	cr_read_config32(cr, 0x160, &lval);

	if (CHECK_PID(cr, 0x522A)) {
		if ((0 == (lval & 0x0F)) && option->polling_oobs_en)
			cr_enable_oobs_polling(cr);
		else if (lval & 0x0F)
			cr_disable_oobs_polling(cr);
	}
	if (lval & (1 << 3))
		set_dev_flag(cr, ASPM_L1_1_EN);
	else
		clear_dev_flag(cr, ASPM_L1_1_EN);
	if (lval & (1 << 2))
		set_dev_flag(cr, ASPM_L1_2_EN);
	else
		clear_dev_flag(cr, ASPM_L1_2_EN);

	if (lval & (1 << 1))
		set_dev_flag(cr, PM_L1_1_EN);
	else
		clear_dev_flag(cr, PM_L1_1_EN);
	if (lval & (1 << 0))
		set_dev_flag(cr, PM_L1_2_EN);
	else
		clear_dev_flag(cr, PM_L1_2_EN);

	return 0;
}

static int rts5227_init_from_hw(struct card_reader *cr)
{
	struct cr_option *option = &(cr->option);
	struct auto_load_map cfg_map[8];
	int retval;
	u32 lval, lval2;
	u8 val;
	u8 cap = 0;
	u32 sd_capability = cr_get_sd_capability_sw(cr);
	u8 card_spt_map = cr_get_card_spt_map_sw(cr);

	if (0 == option->use_hw_setting) {
		retval = rts5227_auto_load(cr, option->relink_time, cfg_map, 0);
		if (retval != STATUS_SUCCESS)
			TRACE_RET(cr, STATUS_FAIL);
		return STATUS_SUCCESS;
	}

	retval = cr_rd_cfg32(cr, 0x724, &lval);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, STATUS_FAIL);
	rts_trace(TRACE_CR, "dw in 0x724: 0x%x\n", lval);

	retval = cr_rd_cfg32(cr, 0x814, &lval2);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, STATUS_FAIL);

	if (0xFFFFFFFF == lval && 0 == (lval2 & 0xFF)) {
		if (check_dev_flag(cr, ASPM_L1_1_EN | ASPM_L1_2_EN
					| PM_L1_1_EN | PM_L1_2_EN))
			option->force_clkreq_0 = 0;
		retval = rts5227_auto_load(cr, option->relink_time, cfg_map, 0);
		if (retval != STATUS_SUCCESS)
			TRACE_RET(cr, STATUS_FAIL);
		return STATUS_SUCCESS;
	}

	rts5227_init_cfg_map(&cfg_map[0], 0x817, 0, 0x80);
	rts5227_init_cfg_map(&cfg_map[1], 0x724, 0, 0xFF);
	rts5227_init_cfg_map(&cfg_map[2], 0x725, 0, 0xFF);
	rts5227_init_cfg_map(&cfg_map[3], 0x726, 0, 0xFF);
	rts5227_init_cfg_map(&cfg_map[4], 0x727, 0, 0xFF);
	rts5227_init_cfg_map(&cfg_map[5], 0x814, 0, 0x00);
	rts5227_init_cfg_map(&cfg_map[6], 0x815, 0, 0x00);
#if 1
	/* 0x727 */
	val = (u8)(lval >> 24);
	if ((val & 0x01) == 0) {
		u8 lun_mode[4] = {
			0xFF,
			MS_LUN,
			SD_LUN,
			DEFAULT_SINGLE,
		};
		u8 sd_drive[4] = {
			0x01,	/* Type D */
			0x02,	/* Type C */
			0x05,	/* Type A */
			0x03	/* Type B */
		};
		u8 ssc_depth1[4] = {
			SSC_DEPTH_512K,
			SSC_DEPTH_1M,
			SSC_DEPTH_2M,
			SSC_DEPTH_4M,
		};
		u8 ssc_depth2[4] = {
			SSC_DEPTH_256K,
			SSC_DEPTH_512K,
			SSC_DEPTH_1M,
			SSC_DEPTH_2M,
		};

		rts5227_init_cfg_map(&cfg_map[4], 0x727, 0, val);

		option->lun_mode = lun_mode[(val >> 6) & 0x03];
		cap = (val >> 6) & 0x03;
		option->dev_aspm_val = (val >> 4) & 0x03;
		option->force_host_aspm_val = (val >> 4) & 0x03;
		if (check_dev_flag(cr, FORCE_HOST_ASPM_EN))
			cr->host_aspm_val = option->force_host_aspm_val;
		option->sd30_drive_sel_1v8 = sd_drive[(val >> 2) & 0x03];
		option->card_drive_sel &= 0x3F;
		option->card_drive_sel |= ((val >> 1) & 0x01) << 6;
		switch (option->sd30_drive_sel_1v8) {
		case 0x01:	/* type D */
			option->sd30_clk_drive_sel_1v8 = 0x99;
			option->sd30_cmd_drive_sel_1v8 = 0x99;
			option->sd30_dat_drive_sel_1v8 = 0x99;
			break;
		case 0x02:	/* type C */
			option->sd30_clk_drive_sel_1v8 = 0xAA;
			option->sd30_cmd_drive_sel_1v8 = 0xAA;
			option->sd30_dat_drive_sel_1v8 = 0xAA;
			break;
		case 0x05:	/* type A */
			option->sd30_clk_drive_sel_1v8 = 0xFE;
			option->sd30_cmd_drive_sel_1v8 = 0xFE;
			option->sd30_dat_drive_sel_1v8 = 0xFE;
			break;
		case 0x03:	/* type B */
			option->sd30_clk_drive_sel_1v8 = 0xB3;
			option->sd30_cmd_drive_sel_1v8 = 0xB3;
			option->sd30_dat_drive_sel_1v8 = 0xB3;
			break;
		default:
			break;
		}

		/* 0x726 */
		val = (u8)(lval >> 16);
		rts5227_init_cfg_map(&cfg_map[3], 0x726, 0, val);

		if ((val & 0xC0) != 0xC0) {
			option->asic_sd_hs_clk = (49 -
						((val >> 6) & 0x03) * 2) * 2;
			option->asic_mmc_52m_clk = option->asic_sd_hs_clk;
		}

		/* WARNING: keep option->sd_capability  constant
		 * when merge with branch rts5250
		 */
		if (val & (1 << 5))
			sd_capability |= SD_CAP_SDR50;
		else
			sd_capability &= ~SD_CAP_SDR50;
		if (val & (1 << 4))
			sd_capability |= SD_CAP_DDR50;
		else
			sd_capability &= ~SD_CAP_DDR50;
		if (val & (1 << 3))
			sd_capability |= SD_CAP_SDR104;
		else
			sd_capability &= ~SD_CAP_SDR104;
		if (sd_capability & SD_CAP_SD)
			card_spt_map |= 1 << SD_ID;
		else if (0 == sd_capability)
			card_spt_map &= ~(1 << SD_ID);

		if ((val & 0x07) != 0x07)
			option->asic_ms_hg_clk = (59 - (val & 0x07)) * 2;

		/* 0x725 */
		val = (u8)(lval >> 8);
		rts5227_init_cfg_map(&cfg_map[2], 0x725, 0, val);

		if ((val & 0xE0) != 0xE0)
			option->asic_sd_sdr104_clk = 206 -
						((val >> 5) & 0x07) * 3;
		if ((val & 0x1C) != 0x1C)
			option->asic_sd_sdr50_clk = 98 -
						((val >> 2) & 0x07) * 2;
		if ((val & 0x03) != 0x03)
			option->asic_sd_ddr50_clk = (48 -
						(val & 0x03) * 2) * 2;

		/* 0x724 */
		val = (u8)lval;
		rts5227_init_cfg_map(&cfg_map[1], 0x724, 0, val);

		option->ssc_depth_sd_sdr104 = ssc_depth1[(val >> 6) & 0x03];
		option->ssc_depth_sd_sdr50 = option->ssc_depth_sd_sdr104;
		option->ssc_depth_sd_ddr50 = ssc_depth1[(val >> 4) & 0x03];
		option->ssc_depth_sd_hs = ssc_depth2[(val >> 2) & 0x03];
		option->ssc_depth_mmc_52m = option->ssc_depth_sd_hs;
		option->ssc_depth_ms_hg = ssc_depth2[val & 0x03];

		/* 0x814 */
		retval = cr_rd_cfg32(cr, 0x814, &lval);
		if (retval != STATUS_SUCCESS)
			TRACE_RET(cr, STATUS_FAIL);
		rts_trace(TRACE_CR, "dw in 0x814: 0x%x\n", lval);

		val = (u8)lval;
		rts5227_init_cfg_map(&cfg_map[5], 0x814, 0, val);

		option->rtd3_en = (val & 0x02) ? 1 : 0;
		option->auto_delink_en = (val & 0x80) ? 1 : 0;
		option->sd30_drive_sel_3v3 = sd_drive[(val >> 5) & 0x03];
		if (val	& 0x10) {
			sd_capability &= ~SD_CAP_MMC;
			if (0 == sd_capability)
				card_spt_map &= ~(1 << SD_ID);
		} else {
			card_spt_map |= 1 << SD_ID;
			sd_capability |= SD_CAP_MMC;
		}

		if (0x03 == cap)
			card_spt_map = (1 << SD_ID) | (1 << MS_ID);
		else if (0x02 == cap)
			card_spt_map = 1 << SD_ID;
		else if (0x01 == cap)
			card_spt_map = 1 << MS_ID;
		else
			card_spt_map = 0;

		cr_set_card_spt_map_sw(cr, card_spt_map);
		cr_set_sd_capability_sw(cr, sd_capability);
		option->led_always_on = !!(val & 0x08);

		switch (option->sd30_drive_sel_3v3) {
		case 0x01:	/* type D */
			option->sd30_clk_drive_sel_3v3 = 0x13;
			option->sd30_cmd_drive_sel_3v3 = 0x13;
			option->sd30_dat_drive_sel_3v3 = 0x13;
			break;
		case 0x02:	/* type C */
			option->sd30_clk_drive_sel_3v3 = 0x96;
			option->sd30_cmd_drive_sel_3v3 = 0x96;
			option->sd30_dat_drive_sel_3v3 = 0x96;
			break;
		case 0x05:	/* type A */
			option->sd30_clk_drive_sel_3v3 = 0x7F;
			option->sd30_cmd_drive_sel_3v3 = 0x7F;
			option->sd30_dat_drive_sel_3v3 = 0x7F;
			break;
		case 0x03:	/* type B */
			option->sd30_clk_drive_sel_3v3 = 0x96;
			option->sd30_cmd_drive_sel_3v3 = 0x96;
			option->sd30_dat_drive_sel_3v3 = 0x96;
			break;
		default:
			break;
		}

		/* option->obff_en = !!(val & 0x04); */

		/* 0x815[6]: indicate SD_CD/WP if reverse */
		val = (u8) (lval >> 8);
		if ((1 << 6) & val)
			set_dev_flag(cr, SD_CD_WP_REVERSE_EN);
		rts5227_init_cfg_map(&cfg_map[6], 0x815, 0, val);

		/* 0x816[2]: indicate free clkreq at L1/RTD3/Delink */
		val = (u8) (lval >> 16);
		if (CHECK_PID(cr, 0x522A)) {
			option->force_clkreq_0 = !((1 << 2) & val);

			cr_read_config32(cr, 0x160, &lval);
			if (lval & 0x0F)
				option->force_clkreq_0 = 0;
		}
		rts5227_init_cfg_map(&cfg_map[7], 0x816, 0, val);
	}

	if (check_dev_flag(cr, ASPM_L1_1_EN | ASPM_L1_2_EN
				| PM_L1_1_EN | PM_L1_2_EN))
		option->force_clkreq_0 = 0;

	if (option->hp_watch_bios_hotplug && option->auto_delink_en) {
		u8 reg58, reg5b;
		int ret = 0;
		ret = cr_pci_rd_cfg_byte(cr, 0x00, 0x1C, 0x02, 0x58, &reg58);
		ret |= cr_pci_rd_cfg_byte(cr, 0x00, 0x1C, 0x02, 0x5B, &reg5b);

		if (!ret) {
			rts_trace(TRACE_CR, "reg58 = 0x%x, reg5b = 0x%x\n",
								reg58, reg5b);
			/* BIOS don't support hotplug, so we
			 * disable auto_delink here
			 */
			if ((reg58 == 0x00) && (reg5b == 0x01))
				option->auto_delink_en = 0;
		}
	}
#endif

	retval = rts5227_auto_load(cr, option->relink_time, cfg_map, 8);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, STATUS_FAIL);

	cr_patch_hw_setting(cr);

	return STATUS_SUCCESS;
}

static void rts5227_save_cfg_to_reg(struct card_reader *cr)
{
	u8 val;

	if (check_dev_flag(cr, LTR_OBFF_SETBACK_EN)) {
		cr_read_config8(cr, 0x99, &val);
		cr_wr_reg(cr, 0xFF30, 0xFF, val);
	}

	if (check_dev_flag(cr, LTR_EXT_SETBACK_EN)) {
		u32 lval;
		cr_read_config32(cr, 0x154, &lval);
		cr_wr_reg(cr, 0xFF31, 0xFF, (u8)lval);
		cr_wr_reg(cr, 0xFF32, 0xFF, (u8)(lval >> 8));
		cr_wr_reg(cr, 0xFF33, 0xFF, (u8)(lval >> 16));
		cr_wr_reg(cr, 0xFF34, 0xFF, (u8)(lval >> 24));
	}

	if (check_dev_flag(cr, L1SS_SETBACK_EN)) {
		cr_read_config8(cr, 0x164, &val);
		cr_wr_reg(cr, 0xFF35, 0xFF, val);

		cr_read_config8(cr, 0x163, &val);
		cr_wr_reg(cr, 0xFF36, 0xFF, val);

		cr_read_config8(cr, 0x162, &val);
		cr_wr_reg(cr, 0xFF37, 0xFF, val);

		if (CHECK_PID(cr, 0x522A)) {
			cr_read_config8(cr, 0x161, &val);
			cr_wr_reg(cr, 0xFF2E, 0xFF, val);

			cr_read_config8(cr, 0x160, &val);
			cr_wr_reg(cr, 0xFF2F, 0xFF, val);
		} else if (CHECK_PID(cr, 0x5227)) {
			cr_read_config8(cr, 0x161, &val);
			cr_wr_reg(cr, 0xFF38, 0xFF, val);

			cr_read_config8(cr, 0x160, &val);
			cr_wr_reg(cr, 0xFF39, 0xFF, val);
		}
	}
}

static void rts5227_store_cfg_from_reg(struct card_reader *cr)
{
	u8 val;

	if (check_dev_flag(cr, LTR_OBFF_SETBACK_EN)) {
		cr_rd_reg(cr, 0xFF30, &val);
		cr_write_config8(cr, 0x99, val);
	}

	if (check_dev_flag(cr, LTR_EXT_SETBACK_EN)) {
		cr_rd_reg(cr, 0xFF31, &val);
		cr_write_config8(cr, 0x154, val);
		cr_rd_reg(cr, 0xFF32, &val);
		cr_write_config8(cr, 0x155, val);
		cr_rd_reg(cr, 0xFF33, &val);
		cr_write_config8(cr, 0x156, val);
		cr_rd_reg(cr, 0xFF34, &val);
		cr_write_config8(cr, 0x157, val);
	}

	if (check_dev_flag(cr, L1SS_SETBACK_EN)) {
		cr_rd_reg(cr, 0xFF35, &val);
		cr_write_config8(cr, 0x164, val);

		cr_rd_reg(cr, 0xFF36, &val);
		cr_write_config8(cr, 0x163, val);

		cr_rd_reg(cr, 0xFF37, &val);
		cr_write_config8(cr, 0x162, val);

		if (CHECK_PID(cr, 0x522A)) {
			cr_rd_reg(cr, 0xFF2E, &val);
			cr_write_config8(cr, 0x161, val);

			cr_rd_reg(cr, 0xFF2F, &val);
			cr_write_config8(cr, 0x160, val);
		} else if (CHECK_PID(cr, 0x5227)) {
			cr_rd_reg(cr, 0xFF38, &val);
			cr_write_config8(cr, 0x161, val);

			cr_rd_reg(cr, 0xFF39, &val);
			cr_write_config8(cr, 0x160, val);
		}
	}

}

static void rts5227_swap_cfg_with_reg_4_D3(struct card_reader *cr)
{
	u8 had_delinked;

	if (CHECK_PID(cr, 0x522A)) {
		cr_rd_reg(cr, 0xFF2C, &had_delinked);
		had_delinked &= (1 << 0);

		/* set 0xFF3C[0] to indicate device had been probed, sometime
		 * device will auto delink or system enter S3
		 */
		cr_wr_reg(cr, 0xFF2C, 1 << 0, 1 << 0);
	} else if (CHECK_PID(cr, 0x5227)) {
		cr_rd_reg(cr, 0xFF3C, &had_delinked);
		had_delinked &= (1 << 0);

		/* set 0xFF3C[0] to indicate device had been probed, sometime
		 * device will auto delink or system enter S3
		 */
		cr_wr_reg(cr, 0xFF3C, 1 << 0, 1 << 0);
	}

	if (had_delinked)
		rts5227_store_cfg_from_reg(cr);
	else
		rts5227_save_cfg_to_reg(cr);
}

static void rts5227_save_cfg_to_reg_4_RTD3(struct card_reader *cr)
{
	u8 val = 0;

	cr_read_config8(cr, 0x164, &val);
	cr_wr_reg(cr, 0xFF35, 0xFF, val);

	cr_read_config8(cr, 0x163, &val);
	cr_wr_reg(cr, 0xFF36, 0xFF, val);

	cr_read_config8(cr, 0x162, &val);
	cr_wr_reg(cr, 0xFF37, 0xFF, val);

	if (CHECK_PID(cr, 0x522A)) {
		cr_read_config8(cr, 0x161, &val);
		cr_wr_reg(cr, 0xFF2E, 0xFF, val);

		cr_read_config8(cr, 0x160, &val);
		cr_wr_reg(cr, 0xFF2F, 0xFF, val);
	} else if (CHECK_PID(cr, 0x5227)) {
		cr_read_config8(cr, 0x161, &val);
		cr_wr_reg(cr, 0xFF38, 0xFF, val);

		cr_read_config8(cr, 0x160, &val);
		cr_wr_reg(cr, 0xFF39, 0xFF, val);
	}
}

static void rts5227_store_cfg_from_reg_4_RTD3(struct card_reader *cr)
{
	u8 val = 0;

	rts_trace(TRACE_CR, "Resume From RTD3\n");
	rts_trace(TRACE_CR, "L1SS_SETBACK_FOR_INTEL_EN is set,"
			" restore CFG from register\n");

	cr_rd_reg(cr, 0xFF35, &val);
	cr_write_config8(cr, 0x164, val);

	cr_rd_reg(cr, 0xFF36, &val);
	cr_write_config8(cr, 0x163, val);

	cr_rd_reg(cr, 0xFF37, &val);
	cr_write_config8(cr, 0x162, val);

	if (CHECK_PID(cr, 0x522A)) {
		cr_rd_reg(cr, 0xFF2E, &val);
		cr_write_config8(cr, 0x161, val);

		cr_rd_reg(cr, 0xFF2F, &val);
		cr_write_config8(cr, 0x160, val);
	} else if (CHECK_PID(cr, 0x5227)) {
		cr_rd_reg(cr, 0xFF38, &val);
		cr_write_config8(cr, 0x161, val);

		cr_rd_reg(cr, 0xFF39, &val);
		cr_write_config8(cr, 0x160, val);
	}
}

int rts5227_init_cr(struct card_reader *cr)
{
	struct cr_option *option = &(cr->option);
	struct rts5227_info *info = cr->extra;
	int retval;
	u8 val;

	rts_trace(TRACE_CR, "VID: 0x%04x, PID: 0x%04x, SSVID: 0x%04x,"
			"SSDID: 0x%04x\n", cr->vendor_id, cr->product_id,
						cr->ssvid, cr->ssdid);
	val = cr_ioread8(cr, 0x1C);
	if ((val & 0x10) == 0)
		cr->asic_code = 1;
	else
		cr->asic_code = 0;

	retval = cr_rd_reg(cr, 0xFE90, &val);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, STATUS_FAIL);
	rts_trace(TRACE_CR, "0xFE90: 0x%x\n", val);
	cr->ic_version = val & 0x0F;

	cr->dev_pci_cap_offset = cr_find_pci_capability(cr, PCI_CAP_ID_EXP);

	cr_set_stat(cr, RTSX_STAT_INIT);

	/* FIXME: edwin: is this place appropriate to initialise this variable?
	 * or maybe the only choice */
	cr->sd_max_phase = MAX_PHASE;

	option->ltr_enabled = 0;
	option->ltr_active = 0;

	option->auto_delink_allowed = 1;

	if (option->thermal_sensor_en) {
		u8 gpio_input = 0;
		rts_trace(TRACE_CR, "enable thermal sensor\n");
		info->thermal_state = 0;
		info->thermal_time = 0;
		cr_wr_reg(cr, RTS522A_PAD_H3L1, PAD_GPIO_H3L1, PAD_GPIO_H3L1);
		SET_STATUS_FLAG(cr, LED_TEST_MODE);
		cr_disable_auto_blink(cr);
		cr_turn_off_led(cr);
		cr_rd_reg(cr, REG_GPIO_CTL, &gpio_input);
		info->gpio_input = gpio_input & 0x04;
	}

	if (!check_sd_speed_prior(option->sd_speed_prior))
		option->sd_speed_prior = 0x01040203;
	rts_trace(TRACE_CR, "sd_speed_prior = 0x%08x\n",
		option->sd_speed_prior);

	if (!check_sd_current_prior(option->sd_current_prior))
		option->sd_current_prior = 0x00010203;
	rts_trace(TRACE_CR, "sd_current_prior = 0x%08x\n",
		option->sd_current_prior);

	if ((option->mmc_ddr_tx_phase > 31) || (option->mmc_ddr_tx_phase < 0))
		option->mmc_ddr_tx_phase = 0;

	/* Power on SSC */
	cr_wr_reg(cr, REG_FPDCTL, SSC_POWER_DOWN, 0);
	udelay(200);

	/* Set mcu_cnt to 7 to ensure data can be sampled properly */
	cr_wr_reg(cr, REG_CLK_DIV, 0x07, 0x07);
	cr_rd_reg(cr, REG_CLK_DIV, &val);
	rts_trace(TRACE_CR, "CLK_DIV = 0x%x\n", val);

	rts5227_swap_cfg_with_reg_4_D3(cr);

	if (CHECK_STATUS_FLAG(cr, ENTERED_RTD3)
			&& check_dev_flag(cr, L1SS_SETBACK_FOR_INTEL_EN))
		rts5227_store_cfg_from_reg_4_RTD3(cr);

	if (!CHECK_STATUS_FLAG(cr, ENTERED_RTD3)) {
		if (CHECK_PID(cr, 0x522A))
			cr_wr_reg(cr, RTS522A_AUTOLOAD_CFG1,CD_RESUME_EN_MASK, CD_RESUME_EN_MASK);
		else if (CHECK_PID(cr, 0x5227))
			cr_wr_reg(cr, AUTOLOAD_CFG1,CD_RESUME_EN_MASK, CD_RESUME_EN_MASK);
	}

	rts5227_init_from_cfg(cr);
	retval = rts5227_init_from_hw(cr);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, STATUS_FAIL);
	if (option->ss_en == 2)
		/* If ss_en is set to 2, disable ss */
		option->ss_en = 0;

	if (check_dev_flag(cr, CLEAR_0xFE8D_BIT0_EN)) {
		cr_wr_reg(cr, REG_L1SUB_CFG1,
				AUX_CLK_ACTIVE_SEL_MASK, MAC_CKSW_DONE);
	}
	cr_wr_reg(cr, REG_L1SUB_CFG3, 0xFF, 0);

	if (option->l1sub_fe49)
		cr_wr_reg(cr, 0xFE49, 0xFF, 0xEE);

	/* make ltr_l1off_latency contain fixed value 0x9003 (3ms) */
	/* if ASPM L1.2 enabled(bit2 of 0x160), configure ltr_l1off_latency */
	/* && check_dev_flag(cr, LTR_L1SS_PWR_GATE_EN)) { */
	/*if (option->ltr_en) {
		u32 value;
		cr_read_config32(cr, 0x160, &value);
		if (value & (1 << 2)) {
			u32 threshold = (value >> 16) & 0x3FF;
			u32 scale = (value >> 29) & 0x7;
			option->ltr_l1off_latency =
				(1 << 15) | (scale << 10) | threshold;
		}
	}*/

	if (option->aux_clk_freq) {
		if (CHECK_PID(cr, 0x522A))
			cr_wr_reg(cr, RTS522A_AUTOLOAD_CFG4, 1 << 4, 1 << 4);
		else if (CHECK_PID(cr, 0x5227))
			cr_wr_reg(cr, AUTOLOAD_CFG4, (1 << 4), (1 << 4));
	} else {
		if (CHECK_PID(cr, 0x522A))
			cr_wr_reg(cr, RTS522A_AUTOLOAD_CFG4, (1 << 4), 0);
		else if (CHECK_PID(cr, 0x5227))
			cr_wr_reg(cr, AUTOLOAD_CFG4, (1 << 4), 0);
	}

	if (CHECK_PID(cr, 0x522A)) {
		cr_wr_reg(cr, REG_FUNC1_FORCE_CTL, 0x02, 0x02);
		cr_wr_reg(cr, REG_PCLK_CTL, 0x04, 0x04);
		cr_wr_reg(cr, REG_PM_EVENT_DBG, REG_PM_EVENT_DBG0, REG_PM_EVENT_DBG0);
		cr_wr_reg(cr, REG_PM_CLK_FORCE_CTL, 0xFF, 0x11);
	}

	cr_print_option(cr);

	retval = rts5227_reset_cr(cr);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, STATUS_FAIL);

	return STATUS_SUCCESS;
}

void rts5227_init_sw_param(struct cr_option *option)
{
	option->dev_flags |= CLEAR_0xFE8D_BIT0_EN;
	option->dev_flags |= EXTRA_ACTION_FOR_ASPM_EN;
	option->dev_flags |= FORCE_HOST_ASPM_EN;
	option->dev_flags |= L1SS_SETBACK_EN;
	option->dev_flags |= LTR_L1SS_PWR_GATE_EN;
	option->dev_flags |= S3_CLEAR_SETBACK_EN;
	option->dev_flags |= WRITE_CFG_DIFF_RTD3_S3_EN;
	option->dev_flags |= RESET_CARD_AFTER_RESUME_EN;

	option->sd30_clk_drive_sel_1v8 = 0xB3;
	option->sd30_cmd_drive_sel_1v8 = 0xB3;
	option->sd30_dat_drive_sel_1v8 = 0xB3;
	option->sd30_clk_drive_sel_3v3 = 0x96;
	option->sd30_cmd_drive_sel_3v3 = 0x96;
	option->sd30_dat_drive_sel_3v3 = 0x96;
	option->sdr_tx_tuning_en = 1;
	option->support_ms_8bit = 1;
	option->pre_read_th = 0xF000; /*PRE_READ_30M; */
	option->sd40_initial_retry_enable = 1;

	option->phy_optimize = 0;

	/* init latency of active, idle, L1OFF to 60us, 300us, 3ms */
	/* 60us:0x883C, 300us:0x892C, 3ms:0x9003 */
	option->ltr_active_latency = 0x883C;
	option->ltr_idle_latency = 0x892C;
	option->ltr_l1off_latency = 0x9003;
	option->ltr_l1off_snooze_sspwrgate = 0xAC;
	option->ltr_l1off_sspwrgate = 0xAF;
	option->obff_en = 0;

	/* tmp: for rts5227 */
	option->l1sub_fe49 = 0;
	option->phy03_bit5 = 0;

	option->aux_clk_freq = 0;
	option->clk_req_bypass_short = 0;
	option->l1_snooze_delay = 1;
	option->reg_0xfe76 = 0xFF;
	option->sd40_ctl = 0x91821000;
	option->sd_sdr50_default_tx_phase = 27;
}

void rts522a_init_sw_param(struct cr_option *option)
{
	option->dev_flags |= CLEAR_0xFE8D_BIT0_EN;
	option->dev_flags |= EXTRA_ACTION_FOR_ASPM_EN;
	option->dev_flags |= FORCE_HOST_ASPM_EN;
	option->dev_flags |= L1SS_SETBACK_EN;
	option->dev_flags |= LTR_L1SS_PWR_GATE_EN;
	option->dev_flags |= S3_CLEAR_SETBACK_EN;
	option->dev_flags |= WRITE_CFG_DIFF_RTD3_S3_EN;
	option->sd30_clk_drive_sel_1v8 = 0xB3;
	option->sd30_cmd_drive_sel_1v8 = 0xB3;
	option->sd30_dat_drive_sel_1v8 = 0xB3;
	option->sd30_clk_drive_sel_3v3 = 0x96;
	option->sd30_cmd_drive_sel_3v3 = 0x96;
	option->sd30_dat_drive_sel_3v3 = 0x96;
	option->sdr_tx_tuning_en = 1;
	option->support_ms_8bit = 1;
	option->pre_read_th = 0xF000; /* PRE_READ_30M; */
	option->sd40_initial_retry_enable = 1;

	/* init latency of active, idle, L1OFF to 60us, 300us, 3ms */
	/* 60us:0x883C, 300us:0x892C, 3ms:0x9003 */
	option->ltr_active_latency = 0x883C;
	option->ltr_idle_latency = 0x892C;
	option->ltr_l1off_latency = 0x9003;
	option->ltr_l1off_snooze_sspwrgate = 0xF8;
	option->ltr_l1off_sspwrgate = 0xFF;
	option->obff_en = 0;

	option->l1sub_fe49 = 0;
	option->phy03_bit5 = 0;

	option->aux_clk_freq = 0;
	option->clk_req_bypass_short = 0;
	option->l1_snooze_delay = 1;
	option->reg_0xfe76 = 0xFF;
	option->sd40_ctl = 0x91821000;
	option->sd_sdr50_default_tx_phase = 27;
	option->sd_sdr104_default_tx_phase = 20;
	option->phy_optimize = 0;
	option->dev_flags |= RESET_CARD_AFTER_RESUME_EN;
	option->dev_flags |= LTR_L1SS_PWR_GATE_CHECK_CARD_EN;
	option->sd_400mA_ocp_thd = 2;
	option->sd_800mA_ocp_thd = 6;
	option->ltr_l1off_sspwrgate = 0xFF;
	option->polling_oobs_en = 1;
	option->thermal_time_step3 = 0;
	option->thermal_time_recovery = 360000;
}

void rts5227_init_hw_param(struct hw_param *hw_param)
{
	hw_param->interrupt_en = TRANS_OK_INT_EN | TRANS_FAIL_INT_EN;
	hw_param->interrupt_en |= DELINK_INT_EN | SD_INT_EN | MS_INT_EN;
	hw_param->ocp_glitch = SD_OCP_GLITCH_800U;
	hw_param->phy0x00_val = 0xBA43;
	hw_param->phy0x01_val = 0x713F;
	hw_param->phy0x02_val = 0x041F;
	hw_param->phy0x03_val = 0xC152;
	hw_param->phy0x05_val = 0x4600;
	hw_param->phy0x19_val = 0xFE6C;
	hw_param->phy0x1d_val = 0x0824;
	hw_param->phy0x1e_val = 0x78EB;
	hw_param->phy0x1e_val_l1ss_off = 0x58EB;
}

void rts522a_init_hw_param(struct hw_param *hw_param)
{
	hw_param->interrupt_en = TRANS_OK_INT_EN | TRANS_FAIL_INT_EN;
	hw_param->interrupt_en |= DELINK_INT_EN | SD_INT_EN | MS_INT_EN;
	hw_param->ocp_glitch = SD_OCP_GLITCH_10M;
	hw_param->phy0x00_val = 0xBA43;
	hw_param->phy0x01_val = 0x713F;
	hw_param->phy0x02_val = 0x0A1F;
	hw_param->phy0x03_val = 0xC152;
	hw_param->phy0x1a_val = 0x2546;
	hw_param->phy0x1d_val = 0x0004;
	hw_param->phy0x1e_val = 0x5C7F;
}

void rts5227_init_options(struct card_reader *cr)
{
	struct cr_option *option = &(cr->option);
	struct hw_param *hw_param = &(cr->hw_param);

	cr_init_generic_options(cr);
	rts5227_init_sw_param(option);
	rts5227_init_hw_param(hw_param);

	if (option->ocp_en)
		hw_param->interrupt_en |= SD_OC_INT_EN;

	if (!option->adma_mode)
		hw_param->interrupt_en |= DATA_DONE_INT_EN;

}

int rts5227_init(struct card_reader *cr)
{
	if ((cr_request_irq(cr) < 0))
		return -1;

	if (rts5227_init_cr(cr) != STATUS_SUCCESS) {
		rts_trace(TRACE_CR, "rts5227_init_cr() Failed\n");
		return -EIO;
	}

	return 0;
}
int rts5227_sd_current_over_limit400_handler(struct card_reader *cr)
{
	struct cr_option *option = &(cr->option);

	cr_wr_reg(cr, REG_OCPPARA2, SD_OCP_THD_MASK, option->sd_800mA_ocp_thd);

	return 0;
}

void rts5227_process_ocp(struct card_reader *cr)
{
	struct list_head *pos;

	if (!cr->option.ocp_en)
		return;

	/* process ocp in release card for RTS5227A */
	if (CHECK_VERSION(cr, 0x5227, IC_VER_A))
		return;

	list_for_each(pos, &cr->cm->card_list) {
		struct card *c;
		c = list_entry(pos, struct card, link);
		if (cr->ocp_int) {
			cr_get_ocpstat(cr, &(cr->ocp_stat));
			if (SD_ID == c->id && check_card_flag(c, EXIST))
				sd_power_off_card3v3(cr);
			else if (MS_ID == c->id && check_card_flag(c, EXIST))
				ms_power_off_card3v3(cr);
			cr->ocp_int = 0;
		}
	}

	if (cr->release_map) {
		if (cr->ocp_stat & (SD_OC_NOW | SD_OC_EVER))
			cr_clear_ocpstat(cr);
		cr->ocp_stat = 0;
	}

	if (cr->ocp_stat & (SD_OC_NOW | SD_OC_EVER)) {
		rts_trace(TRACE_CR, "Over current, OCPSTAT is 0x%x\n",
				cr->ocp_stat);
		list_for_each(pos, &cr->cm->card_list) {
			struct card *c;
			c = list_entry(pos, struct card, link);
			if (SD_ID == c->id && check_card_flag(c, EXIST)) {
				cr_wr_reg(cr, REG_CARD_OE, SD_OUTPUT_EN, 0);
				set_card_flags(c, FAIL);
			} else if (MS_ID == c->id &&
					check_card_flag(c, EXIST)) {
				cr_wr_reg(cr, REG_CARD_OE, MS_OUTPUT_EN, 0);
				set_card_flags(c, FAIL);
			}
		}
		cr_poweroff_card(cr, SD_ID);
	}
}

static int rts5227A_init_ocp(struct card_reader *cr)
{
	struct cr_option *option = &(cr->option);

	if (option->ocp_en
		&& (check_patch_flag(cr, RTS5227_ENABLE_OCP_4_A))) {
		u8 val = option->sd_400mA_ocp_thd;

		cr_wr_reg(cr, REG_FPDCTL, OC_POWER_DOWN, 0);
		cr_wr_reg(cr, REG_OCPPARA1, SD_OCP_TIME_MASK,
				SD_OCP_TIME_800);
		cr_wr_reg(cr, REG_OCPPARA2, SD_OCP_THD_MASK, val);
		cr_wr_reg(cr, REG_OCPGLITCH, SD_OCP_GLITCH_MASK,
				cr->hw_param.ocp_glitch);
		cr_enable_ocp(cr);
	} else {
		/* OC power down */
		cr_wr_reg(cr, REG_FPDCTL, OC_POWER_DOWN, OC_POWER_DOWN);
	}

	return 0;
}

static int rts5227X_init_ocp(struct card_reader *cr)
{
	struct cr_option *option = &(cr->option);

	if (option->ocp_en) {
		u8 val = option->sd_400mA_ocp_thd;

		cr_wr_reg(cr, REG_FPDCTL, OC_POWER_DOWN, 0);
		cr_wr_reg(cr, REG_OCPPARA1, SD_OCP_TIME_MASK,
				SD_OCP_TIME_800);
		cr_wr_reg(cr, REG_OCPPARA2, SD_OCP_THD_MASK, val);
		cr_wr_reg(cr, REG_OCPGLITCH, SD_OCP_GLITCH_MASK,
				cr->hw_param.ocp_glitch);
		cr_enable_ocp(cr);
	} else {
		/* OC power down */
		cr_wr_reg(cr, REG_FPDCTL, OC_POWER_DOWN, OC_POWER_DOWN);
	}

	return 0;
}

int rts5227_init_ocp(struct card_reader *cr)
{
	if (CHECK_VERSION(cr, 0x5227, IC_VER_A))
		return rts5227A_init_ocp(cr);
	else
		return rts5227X_init_ocp(cr);
}

/* for SD */

int rts5227_sd_pull_ctl_disable(struct card_reader *cr)
{
	/* This function would be called in enter_ss
	 * which is mutual exclusive to interrupt handler.
	 * So we can't use command buffer to access internal register */
	cr_wr_reg(cr, REG_CARD_PULL_CTL2, 0xFF, 0x55);

	cr_wr_reg(cr, REG_CARD_PULL_CTL3, 0xFF, 0xD5);

	return STATUS_SUCCESS;
}

int rts5227_sd_pull_ctl_enable(struct card_reader *cr)
{
	int retval;

	/* SD Data0~3: pull up
	 * SD CD: pull up
	 * SD WP: pull up
	 * SD CMD: pull up
	 * SD CLK: pull down
	 */
	cr_init_cmd(cr);
	cr_add_cmd(cr, WRITE_REG_CMD, REG_CARD_PULL_CTL2, 0xFF, 0xAA);
	cr_add_cmd(cr, WRITE_REG_CMD, REG_CARD_PULL_CTL3, 0xFF, 0xE9);
	retval = cr_send_cmd(cr, SD_CARD, 100);
	if (retval < 0)
		TRACE_RET(cr, STATUS_FAIL);

	return STATUS_SUCCESS;
}

int rts5227_sd_init_power(struct card_reader *cr)
{
	struct cr_option *option = &(cr->option);
	int retval;

	/* Power off LDO3318 */
	cr_wr_reg(cr, REG_PWR_GATE_CTRL, LDO3318_PWR_MASK, LDO_OFF);

	if (CHECK_VERSION(cr, 0x5227, IC_VER_A)) {
		if (option->ocp_en
			&& check_patch_flag(cr, RTS5227_ENABLE_OCP_4_A)) {
			retval = cr_enable_ocp(cr);
			if (retval != STATUS_SUCCESS)
				TRACE_RET(cr, STATUS_FAIL);

			msleep(10);
			cr_clear_ocpstat(cr);
		}
	}

	if ((sd_power_off_card3v3(cr)))
		TRACE_RET(cr, STATUS_FAIL);

	if (sd_change_bank_voltage(cr, SD_COMM_IO_3V3))
		TRACE_RET(cr, STATUS_FAIL);

	retval = sd_sd30_drive_sel(cr, SD_IO_3V3);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, STATUS_FAIL);

	if (!option->ft2_fast_mode)
		msleep(250);
	retval = cr_enable_card_clk(cr, SD_ID);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, STATUS_FAIL);
	if (cr->asic_code) {
		retval = rts5227_sd_pull_ctl_enable(cr);
		if (retval != STATUS_SUCCESS)
			TRACE_RET(cr, STATUS_FAIL);
	} else {
		cr_wr_reg(cr, REG_FPGA_PULL_CTL,
				FPGA_SD_PULL_CTL_BIT | 0x20, 0);
	}

	if (CHECK_VERSION(cr, 0x5227, IC_VER_A)) {
		if (option->ocp_en
			&& check_patch_flag(cr, RTS5227_ENABLE_OCP_4_A)) {
			cr_wr_reg(cr, REG_PWD_SUSPEND_EN, 0x08, 0x00);
			cr_enable_ocp(cr);
		}
	}

	retval = cr_poweron_card(cr, SD_ID);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, STATUS_FAIL);
	msleep(260);

	if (option->ocp_en) {
		u8 ocp;
		cr_get_ocpstat(cr, &ocp);
		if (ocp & (SD_OC_NOW | SD_OC_EVER)) {
			struct card *card = cm_get_card(cr->cm, SD_ID);

			if (card)
				card->reset_flags |= RESET_MEET_OCP;
			rts_trace(TRACE_SD, "OCP! OCPSTAT:%#x\n", ocp);
			TRACE_RET(cr, STATUS_FAIL);
		}
	}

	if (CHECK_VERSION(cr, 0x5227, IC_VER_A)) {
		if (option->ocp_en
			&& check_patch_flag(cr, RTS5227_ENABLE_OCP_4_A)) {
			retval = cr_disable_ocp(cr);
			if (retval != STATUS_SUCCESS)
				TRACE_RET(cr, STATUS_FAIL);
			cr_clear_ocpstat(cr);

			cr_wr_reg(cr, REG_PWD_SUSPEND_EN, 0x08, 0x08);
		}
	}

	cr_wr_reg(cr, REG_CARD_OE, SD_OUTPUT_EN, SD_OUTPUT_EN);

	return STATUS_SUCCESS;
}

int rts5227_release_sd_card(struct card_reader *cr, struct card *card)
{
	struct sd_info *sd_card = card->extra;
	struct cr_option *option = &(cr->option);
	struct rts5227_info *info = cr->extra;
	int retval;

	rts_trace(TRACE_SD, "rts5227_release_sd_card\n");

	clr_card_flags(card, (READY | FAIL | WP));
	card->reset_flags = 0;
	if (option->thermal_sensor_en) {
		info->thermal_state = 0;
		info->thermal_time = 0;
		cr->card_status = 0;
	}

	sd_card->sd_lock_status = 0;
	sd_card->sd_erase_status = 0;
	memset(sd_card->raw_csd, 0, 16);
	memset(sd_card->raw_scr, 0, 8);
	memset(sd_card->raw_cid, 0, 16);
	memset(&sd_card->ge, 0, sizeof(struct sd_ge));

	if (CHECK_VERSION(cr, 0x5227, IC_VER_A)) {
		if (option->ocp_en
			&& check_patch_flag(cr, RTS5227_ENABLE_OCP_4_A)) {
			retval = cr_enable_ocp(cr);
			if (retval != STATUS_SUCCESS)
				TRACE_RET(cr, STATUS_FAIL);

			msleep(10);
			cr_clear_ocpstat(cr);
		}
	}

	if ((sd_power_off_card3v3(cr)))
		TRACE_RET(cr, STATUS_FAIL);
	/* Switch SD bus to 3V3 signal */
	retval = sd_change_bank_voltage(cr, SD_IO_3V3);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, STATUS_FAIL);
	if (CHK_SD30_SPEED(sd_card))
		retval = sd_sd30_drive_sel(cr, SD_IO_3V3);
		if (retval != STATUS_SUCCESS)
			TRACE_RET(cr, STATUS_FAIL);
		cr_wr_reg(cr, REG_OCPPARA2, SD_OCP_THD_MASK,
				option->sd_400mA_ocp_thd);
	/* Suspend LDO3318 */
	/* cr_wr_reg(cr, REG_PWR_GATE_CTRL,
			LDO3318_PWR_MASK, LDO_SUSPEND); */

	return STATUS_SUCCESS;
}

int rts5227_sd_disable_card(struct card_reader *cr, struct card *card)
{
	return STATUS_SUCCESS;
}

int rts5227_sd_update_report_status(struct card_reader *cr, struct card *card)
{
	return 0;
}


/* for MS */

int rts5227_ms_prepare_reset(struct card_reader *cr, struct card *card)
{
	struct cr_option *option = &(cr->option);
	struct ms_info *ms_card = card->extra;
	int retval;

	ms_card->ms_type = 0;
	ms_card->check_ms_flow = 0;
	ms_card->switch_8bit_fail = 0;
	ms_card->delay_write.delay_write_flag = 0;
	ms_card->pro_under_formatting = 0;

	if (CHECK_VERSION(cr, 0x5227, IC_VER_A)) {
		if (option->ocp_en
			&& check_patch_flag(cr, RTS5227_ENABLE_OCP_4_A)) {
			retval = cr_enable_ocp(cr);
			if (retval != STATUS_SUCCESS)
				TRACE_RET(cr, STATUS_FAIL);

			msleep(10);
			cr_clear_ocpstat(cr);
		}
	}

	if (ms_power_off_card3v3(cr))
		TRACE_RET(cr, STATUS_FAIL);

	msleep(250);

	if (cr_enable_card_clk(cr, MS_ID))
		TRACE_RET(cr, STATUS_FAIL);

	if (cr->asic_code) {
		if (ms_pull_ctl_enable(cr))
			TRACE_RET(cr, STATUS_FAIL);
	} else {
		cr_wr_reg(cr, REG_FPGA_PULL_CTL,
				FPGA_MS_PULL_CTL_BIT | 0x20, 0);
	}

	if (CHECK_VERSION(cr, 0x5227, IC_VER_A)) {
		if (option->ocp_en
			&& check_patch_flag(cr, RTS5227_ENABLE_OCP_4_A)) {
			cr_wr_reg(cr, REG_PWD_SUSPEND_EN, 0x08, 0x00);
			cr_enable_ocp(cr);
		}
	}

	if (cr_poweron_card(cr, MS_ID))
		TRACE_RET(cr, STATUS_FAIL);
	msleep(150);

	if (option->ocp_en) {
		u8 ocp;
		cr_get_ocpstat(cr, &ocp);
		if (ocp & (SD_OC_NOW | SD_OC_EVER)) {
			struct card *card = cm_get_card(cr->cm, MS_ID);

			if (card)
				card->reset_flags |= RESET_MEET_OCP;
			rts_trace(TRACE_SD, "OCP! OCPSTAT:%#x\n", ocp);
			TRACE_RET(cr, STATUS_FAIL);
		}
	}

	if (CHECK_VERSION(cr, 0x5227, IC_VER_A)) {
		if (option->ocp_en
			&& check_patch_flag(cr, RTS5227_ENABLE_OCP_4_A)) {
			if (cr_disable_ocp(cr))
				TRACE_RET(cr, STATUS_FAIL);
			cr_clear_ocpstat(cr);
			cr_wr_reg(cr, REG_PWD_SUSPEND_EN, 0x08, 0x08);
		}
	}

	cr_wr_reg(cr, REG_CARD_OE, MS_OUTPUT_EN, MS_OUTPUT_EN);

	/* Reset Registers */
	if (cr->asic_code) {
		cr_wr_reg(cr, REG_MS_CFG, 0xFF, SAMPLE_TIME_RISING
				| PUSH_TIME_DEFAULT | NO_EXTEND_TOGGLE
				| MS_BUS_WIDTH_1);
	} else {
		cr_wr_reg(cr, REG_MS_CFG, 0xFF, SAMPLE_TIME_FALLING
				| PUSH_TIME_DEFAULT | NO_EXTEND_TOGGLE
				| MS_BUS_WIDTH_1);
	}

	cr_wr_reg(cr, REG_MS_TRANS_CFG, 0xFF, NO_WAIT_INT |
			NO_AUTO_READ_INT_REG);
	cr_wr_reg(cr, REG_CARD_STOP, MS_STOP | MS_CLR_ERR,
			MS_STOP | MS_CLR_ERR);
	if (ms_set_init_para(cr, card))
		TRACE_RET(cr, STATUS_FAIL);

	return 0;
}

int rts5227_ms_pull_ctl_disable(struct card_reader *cr)
{
	/* This function would be called in enter_ss
	 * which is mutual exclusive to interrupt handler.
	 * So we can't use command buffer to access internal register */
	cr_wr_reg(cr, REG_CARD_PULL_CTL5, 0xFF, 0x55);
	cr_wr_reg(cr, REG_CARD_PULL_CTL6, 0xFF, 0x15);

	return STATUS_SUCCESS;
}

int rts5227_ms_pull_ctl_enable(struct card_reader *cr)
{
	int retval;

	/*
	 * MS CD: pull up
	 * others: pull down
	 */
	cr_init_cmd(cr);
	cr_add_cmd(cr, WRITE_REG_CMD, REG_CARD_PULL_CTL5, 0xFF, 0x55);
	cr_add_cmd(cr, WRITE_REG_CMD, REG_CARD_PULL_CTL6, 0xFF, 0x15);

	retval = cr_send_cmd(cr, MS_CARD, 100);
	if (retval < 0)
		TRACE_RET(cr, STATUS_FAIL);

	return STATUS_SUCCESS;
}

int rts5227_release_ms_card(struct card_reader *cr, struct card *card)
{
	struct cr_option *option = &(cr->option);
	struct ms_info *ms_card = card->extra;

	rts_trace(TRACE_MS, "%s\n", __func__);

#ifdef MS_DELAY_WRITE
	ms_card->delay_write.delay_write_flag = 0;
#endif
	ms_card->pro_under_formatting = 0;
	clr_card_flags(card, (READY | FAIL | WP));
	ms_free_l2p_tbl(cr, card);
	memset(ms_card->raw_sys_info, 0, sizeof(ms_card->raw_sys_info));
	memset(ms_card->raw_ms_id, 0, sizeof(ms_card->raw_ms_id));
#ifdef SUPPORT_PCGL_1P18
	memset(ms_card->raw_model_name, 0, sizeof(ms_card->raw_model_name));
#endif

	if (CHECK_VERSION(cr, 0x5227, IC_VER_A)) {
		if (option->ocp_en
			&& check_patch_flag(cr, RTS5227_ENABLE_OCP_4_A)) {
			if ((cr_enable_ocp(cr)))
				TRACE_RET(cr, STATUS_FAIL);

			msleep(10);
			cr_clear_ocpstat(cr);
		}
	}

	if (ms_power_off_card3v3(cr))
		TRACE_RET(cr, STATUS_FAIL);

	return 0;
}

/* for PM */

static int rts5227_force_power_on(struct card_reader *cr, u8 ctl)
{
	int retval;
	u8 mask = 0;

	rts_trace(TRACE_PM, "<--\n");

	if (ctl & SSC_PDCTL)
		mask |= SSC_POWER_DOWN;

	if (cr->option.ocp_en && (ctl & OC_PDCTL))
		mask |= SD_OC_POWER_DOWN;
	if (mask) {
		retval = cr_wr_reg(cr, REG_FPDCTL, mask, 0);
		if (retval != STATUS_SUCCESS)
			TRACE_RET(cr, STATUS_FAIL);
	}

	rts_trace(TRACE_PM, "-->\n");

	return STATUS_SUCCESS;
}

static int rts5227_force_power_down(struct card_reader *cr, u8 ctl)
{
	int retval;
	u8 mask = 0, val = 0;

	rts_trace(TRACE_PM, "<--\n");

	if (ctl & SSC_PDCTL)
		mask |= SSC_POWER_DOWN;
	if (cr->option.ocp_en && (ctl & OC_PDCTL))
		mask |= SD_OC_POWER_DOWN;

	if (mask) {
		val = mask;
		retval = cr_wr_reg(cr, REG_FPDCTL, mask, val);
		if (retval != STATUS_SUCCESS)
			TRACE_RET(cr, STATUS_FAIL);
	}

	rts_trace(TRACE_PM, "-->\n");

	return STATUS_SUCCESS;
}

static void rts5227_disable_bus_int(struct card_reader *cr)
{
	cr_iowrite32(cr, BIER, 0);
}

static void rts5227_enter_s1(struct card_reader *cr)
{
	rts_trace(TRACE_PM, "Host enter S1\n");
	cr_wr_reg(cr, REG_HOST_SLEEP_STATE, 0x03,
		HOST_ENTER_S1);
	/* Set relink_time to 0, to avoid bar missing issue */
	cr_wr_reg(cr, 0xFF01, 0xFF, 0);
	cr_wr_reg(cr, 0xFF02, 0xFF, 0);
	cr_wr_reg(cr, 0xFF03, 0x01, 0);

	if (CHECK_PID(cr, 0x522A)) {
		cr_wr_reg(cr, RTS522A_AUTOLOAD_CFG3, 0x10, 0x10);
		rts_trace(TRACE_PM, "Disable CD_RESUME_ENABLE\n");
		cr_wr_reg(cr, RTS522A_AUTOLOAD_CFG1, CD_RESUME_EN_MASK,
			0x00);
	} else if (CHECK_PID(cr, 0x5227)) {
		cr_wr_reg(cr, AUTOLOAD_CFG3, 0x10, 0x10);
		rts_trace(TRACE_PM, "Disable CD_RESUME_ENABLE\n");
		cr_wr_reg(cr, AUTOLOAD_CFG1, CD_RESUME_EN_MASK, 0x00);
	}
}

static void rts5227_enter_s3(struct card_reader *cr)
{
	struct cr_option *option = &cr->option;

	/* wwang, 2010-12-16
	 * Before entering S3, some card may be in sequential mode,
	 * so here we delay some time to ensure programming finished
	 */
	if (option->s3_pwr_off_delay > 0) {
		msleep(option->s3_pwr_off_delay);
	}
	rts_trace(TRACE_PM, "Host enter S3\n");
	cr_wr_reg(cr, REG_HOST_SLEEP_STATE, 0x03, HOST_ENTER_S3);
	/* Set relink_time to 0, to avoid bar missing issue */
	cr_wr_reg(cr, 0xFF01, 0xFF, 0);
	cr_wr_reg(cr, 0xFF02, 0xFF, 0);
	cr_wr_reg(cr, 0xFF03, 0x01, 0);

	rts_trace(TRACE_PM, "set REG_ASPM_FORCE_CTL to 0x0\n");
	cr_wr_reg(cr, REG_ASPM_FORCE_CTL, 0xFF, 0x00);

	rts_trace(TRACE_PM, "cr->d3cold_supported is %i\n",
		cr->d3cold_supported);

	cr_wr_reg(cr, REG_FUNC1_FORCE_CTL, 0x02, 0x02);

	if (cr->d3cold_supported && cr->option.rtd3_en) {
		if (CHECK_PID(cr, 0x522A)) {
			cr_wr_reg(cr, RTS522A_AUTOLOAD_CFG3, 0x10, 0x10);
		} else if (CHECK_PID(cr, 0x5227)) {
			if (CHECK_NOT_VERSION(cr, 0x5227, IC_VER_A) &&
				CHECK_NOT_VERSION(cr, 0x5227, IC_VER_B)) {
				u8 val;
				cr_rd_reg(cr, 0xFE1C, &val);
				cr_wr_reg(cr, 0xFF41, 0xFF, val);
				cr_rd_reg(cr, 0xFE1D, &val);
				cr_wr_reg(cr, 0xFF42, 0xFF, val);
				cr_wr_reg(cr, AUTOLOAD_CFG3, 0x10, 0x10);
			} else if (check_dev_flag(cr, RTD3_TEST_EN))
				cr_wr_reg(cr, AUTOLOAD_CFG3, 0x10, 0x10);
			else {
				cr_wr_reg(cr, AUTOLOAD_CFG2, 0x80, 0x80);
				cr_wr_reg(cr, AUTOLOAD_CFG3, 0x10, 0x00);
			}
		}
	}

	/* distinguish S3/S4 with RTD3*/
	rts_trace(TRACE_PM, "enter_rtd3:%d\n",
		CHECK_STATUS_FLAG(cr, ENTERED_RTD3));
	if (!(CHECK_STATUS_FLAG(cr, ENTERED_RTD3)) &&
		!cr->option.remote_wakeup_en) {
		rts_trace(TRACE_PM, "Disable WWEnable\n");

		if (CHECK_PID(cr, 0x522A)) {
			cr_wr_reg(cr, RTS522A_AUTOLOAD_CFG1, CD_RESUME_EN_MASK,
				0);
			cr_wr_reg(cr, RTS522A_AUTOLOAD_CFG3, 0x01, 0x00);
		} else if (CHECK_PID(cr, 0x5227)) {
			cr_wr_reg(cr, AUTOLOAD_CFG1, CD_RESUME_EN_MASK, 0x00);
			cr_wr_reg(cr, AUTOLOAD_CFG3, 0x01, 0x00);
		}
		if (option->aux_pwr_exist) {
			if (CHECK_PID(cr, 0x522A))
				cr_wr_reg(cr, RTS522A_REG_PME_FORCE_CTL, 0x30,
					0x20);
			else if (CHECK_PID(cr, 0x5227))
				cr_wr_reg(cr, REG_PME_FORCE_CTL, 0xFF, 0x30);
		}
	}

	if (!CHECK_STATUS_FLAG(cr, ENTERED_RTD3)) {
		if (CHECK_PID(cr, 0x522A))
			cr_wr_reg(cr, RTS522A_AUTOLOAD_CFG3, 0x10, 0x10);
		else if (CHECK_PID(cr, 0x5227))
			cr_wr_reg(cr, AUTOLOAD_CFG3, 0x10, 0x10);
	}
	if (check_dev_flag(cr, WRITE_CFG_DIFF_RTD3_S3_EN)) {
		u32 lval = 0;
		cr_rd_cfg32(cr, 0x814, &lval);
		lval &= 0x00FC0000;
		if (!CHECK_STATUS_FLAG(cr, ENTERED_RTD3)) {
			cr_wr_cfg32(cr, 0x814, 0xFF000000, 0x80000000);
			cr_wr_cfg32(cr, 0x814, 0x00FF0000, 0x00020000 | lval);
		} else {
			cr_wr_cfg32(cr, 0x814, 0xFF000000, 0x80000000);
			cr_wr_cfg32(cr, 0x814, 0x00FF0000, 0x00010000 | lval);
		}
	}
}

void rts5227_before_power_down(struct card_reader *cr, int pm_stat)
{
	struct cr_option *option = &cr->option;
	int retval;

	rts_trace(TRACE_PM, "%s: pm_stat = %d\n", __func__, pm_stat);

	if (CHECK_STATUS_FLAG(cr, ENTERED_RTD3)
			&& check_dev_flag(cr, L1SS_SETBACK_FOR_INTEL_EN))
		rts5227_save_cfg_to_reg_4_RTD3(cr);

	if (check_dev_flag(cr, S3_CLEAR_SETBACK_EN)) {
		if (!(CHECK_STATUS_FLAG(cr, ENTERED_RTD3))) {
			if (CHECK_PID(cr, 0x522A)) {
				cr_wr_reg(cr, 0xFF2C, 0x01, 0x00);
			} else if (CHECK_PID(cr, 0x5227)) {
				cr_wr_reg(cr, 0xFF3C, 0x01, 0x00);
			}
		}
	}

	if (check_dev_flag(cr, LTR_L1SS_PWR_GATE_EN))
		cr_set_l1off_sub_cfg_D3(cr);

	/* cr_set_stat(cr, RTSX_STAT_SUSPEND); // edwin: comment out */

	retval = rts5227_force_power_on(cr, SSC_PDCTL);
	if (retval != STATUS_SUCCESS) {
		return;
	}
#if 1 /* FIXME: edwin: comment out, temporarily */
	cm_release_cards(cr->cm);
	cr->cur_card = INVAL_ID;
	cr->cm->cur_card = NULL;
	cr->release_map = 0;
#endif
	rts5227_disable_bus_int(cr);
	if (option->blink_led)
		cr_turn_off_led(cr);

	/* u_force_clkreq_0 */
	if (CHECK_PID(cr, 0x5227))
		cr_wr_reg(cr, REG_PETXCFG, 0x08, 0x08);
	if (!(CHECK_STATUS_FLAG(cr, ENTERED_RTD3))) {
		rts_trace(TRACE_CR, "CLRREQ forced to drive 0\n");
		cr_wr_reg(cr, REG_PETXCFG, 0x80, 0x80);
	}

	if (pm_stat == PM_S1)
		rts5227_enter_s1(cr);
	else if (pm_stat == PM_S3)
		rts5227_enter_s3(cr);

	if (option->do_delink_before_power_down && option->auto_delink_en) {
		/* Enable autodelink */
		cr_wr_reg(cr, REG_CHANGE_LINK_STATE, 0x02, 2);
	}

	/* 2010-03-02: OS will handle dstate, so we mark it here. */
	rts5227_force_power_down(cr, SSC_PDCTL | OC_PDCTL);

	cr_pm_set_flags(cr, PM_POWR_SAVING_MAX);

	if (CHECK_STATUS_FLAG(cr, ENTERED_RTD3))
		cr_patch_rtd3(cr);
}

int rts5227_do_resume(struct card_reader *cr)
{
	if (cr_request_irq(cr) < 0)
		return -1;

	cr_wr_reg(cr, REG_HOST_SLEEP_STATE, 0x03, 0x00);
	rts5227_init_cr(cr);

	return 0;
}

static void rts5227_init_aspm(struct card_reader *cr)
{
	struct cr_option *option = &cr->option;

	if (check_dev_flag(cr, FORCE_HOST_ASPM_EN)) {
		cr->host_aspm_val = option->force_host_aspm_val;
	} else {
		cr_get_host_aspm(cr, &cr->host_aspm_val);
		cr->host_aspm_val &= 0xCF;
	}
	rts_trace(TRACE_PM, "cr->host_aspm_val = 0x%x\n", cr->host_aspm_val);

	if (option->config_host_aspm)
		cr_disable_host_aspm(cr);

	cr_wr_reg(cr, REG_ASPM_FORCE_CTL, 0xFF, 0);

	if (DEV_ASPM_DYNAMIC == option->dev_aspm_mode) {
		int lctlr = cr->dev_pci_cap_offset + PCI_EXP_LNKCTL;
		u8 val = 0;
		cr_read_config8(cr, lctlr, &val);
		val &= 0xFC;
		cr_write_config8(cr, lctlr, val);
	} else if (DEV_ASPM_BACKDOOR == option->dev_aspm_mode) {
		u8 mask = FORCE_ASPM_CTL0 | FORCE_ASPM_VAL_MASK;
		u8 val = FORCE_ASPM_CTL0;
		cr_wr_reg(cr, REG_ASPM_FORCE_CTL, mask, val);
	}
}

void rts5227_enable_aspm(struct card_reader *cr)
{
	struct cr_option *option = &cr->option;

	rts_trace(TRACE_PM, "dev_aspm_val = %#x\n", option->dev_aspm_val);
	rts_trace(TRACE_PM, "dev_aspm_mode = %#x\n", option->dev_aspm_mode);

	if (!cr->aspm_enabled) {
		rts_trace(TRACE_PM, "Try to enable ASPM\n");

		if (option->dev_aspm_mode != DEV_ASPM_DISABLE) {
			u8 val = 0;

			if (option->dev_aspm_mode == DEV_ASPM_DYNAMIC) {
				int lctlr = cr->dev_pci_cap_offset + PCI_EXP_LNKCTL;
				cr_read_config8(cr, lctlr, &val);
				val &= 0xFC;
				val |= (option->dev_aspm_val & 0x03);
				cr_write_config8(cr, lctlr, val);
			} else if (option->dev_aspm_mode == DEV_ASPM_BACKDOOR) {
				u8 mask = FORCE_ASPM_CTL0 | FORCE_ASPM_VAL_MASK;
				val = 0;
				cr_wr_reg(cr, REG_ASPM_FORCE_CTL, mask, val);
			}
		}

		if (option->config_host_aspm)
			cr_set_host_aspm(cr, cr->host_aspm_val);

		if (!CHECK_VERSION(cr, 0x5227, IC_VER_A)
				&& cm_check_cards_flags(cr->cm, READY)) {
			if (check_dev_flag(cr, POLLING_OCP_EN))
				cr_wr_reg(cr, REG_OCPGLITCH, 0x0F, 0x01);
		}

		cr->aspm_enabled = 1;
	}
}

static void rts5227_do_disable_aspm(struct card_reader *cr)
{
	struct cr_option *option = &cr->option;
	int i = 0;

	do {
		u8 val;

		if (option->config_host_aspm)
			cr_disable_host_aspm(cr);
		if (DEV_ASPM_DISABLE != option->dev_aspm_mode) {
			rts_trace(TRACE_PM, "Try to disable ASPM\n");
			if (DEV_ASPM_DYNAMIC == option->dev_aspm_mode) {
				int lctlr = cr->dev_pci_cap_offset + PCI_EXP_LNKCTL;
				cr_read_config8(cr, lctlr, &val);
				val &= 0xFC;
				cr_write_config8(cr, lctlr, val);
			} else if (DEV_ASPM_BACKDOOR == option->dev_aspm_mode) {
				u8 mask = FORCE_ASPM_CTL0 | FORCE_ASPM_VAL_MASK;
				u8 val = FORCE_ASPM_CTL0;
				cr_wr_reg(cr, REG_ASPM_FORCE_CTL, mask, val);
			}
			msleep(10);
		}

		/* If register can be accessed, consider
		 * disable aspm success
		 */
		if (0 == cr_rd_reg(cr, REG_CARD_EXIST, &val))
			break;
	} while (i++ < 3);

	if (!CHECK_VERSION(cr, 0x5227, IC_VER_A)
			&& cm_check_cards_flags(cr->cm, READY)) {
		if (check_dev_flag(cr, POLLING_OCP_EN))
			cr_wr_reg(cr, REG_OCPGLITCH, 0x0F, 0x05);
	}

}

void rts5227_disable_aspm(struct card_reader *cr)
{
	if (cr->aspm_enabled) {
		rts5227_do_disable_aspm(cr);
		cr->aspm_enabled = 0;
	}

	return;
}

int rts5227_do_before_auto_delink(struct card_reader *cr)
{
	if (CHECK_PID(cr, 0x522A))
		cr_wr_reg(cr, RTS522A_AUTOLOAD_CFG3, U_PM_D3_EN, 0);
	else if (CHECK_PID(cr, 0x5227))
		cr_wr_reg(cr, AUTOLOAD_CFG3, U_PM_D3_EN, 0);

	if (check_dev_flag(cr, FREE_CLKREQ_AT_DELINK))
		cr_wr_reg(cr, 0xFF03, 0x80, 0x00);

	return 0;
}

void rts522a_pm_full_on(struct card_reader *cr)
{
	struct cr_option *option = &cr->option;

	cr_force_power_on(cr, SSC_PDCTL | OC_PDCTL);
	cr_disable_aspm(cr);

	if (option->ltr_enabled) {
		u32 latency = option->ltr_active_latency;

		rts_trace(TRACE_PM, "Set active latency%#x\n", latency);
		cr_set_ltr_latency(cr, latency);
	}

	if (check_dev_flag(cr, LTR_L1SS_PWR_GATE_EN))
		cr_set_l1off_sub_cfg_D0(cr, 1);
}

void rts522a_set_l1off_cfg_sub_D0(struct card_reader *cr, int active)
{
	struct cr_option *option = &(cr->option);
	u32 interrupt = cr_ioread32(cr, BIPR);
	int card_exist = (interrupt & SD_EXIST) | (interrupt & MS_EXIST);
	int aspm_L1_1, aspm_L1_2;

	aspm_L1_1 = check_dev_flag(cr, ASPM_L1_1_EN);
	aspm_L1_2 = check_dev_flag(cr, ASPM_L1_2_EN);

	if (active) {
		/* run, latency: 60us */
		if (aspm_L1_1) {
			u8 val = option->ltr_l1off_snooze_sspwrgate;
			if (check_dev_flag(cr,
				LTR_L1SS_PWR_GATE_CHECK_CARD_EN)) {
				if (card_exist)
					val &= ~(1 << 7);
				else
					val |= (1 << 7);
			}

			cr_set_l1off_sub(cr, val);
			rts_trace(TRACE_PM,
				"ltr active and aspm L1.1, L1SUB:%#x\n", val);
		} else {
			cr_set_l1off_sub(cr, 0);
			rts_trace(TRACE_PM,
				"ltr active and not aspm L1.1, L1SUB:%#x\n",
				0x00);
		}
	} else {
		/* l1off, latency: 300us */
		if (aspm_L1_2) {
			u8 val = option->ltr_l1off_sspwrgate;

			if (check_dev_flag(cr,
				LTR_L1SS_PWR_GATE_CHECK_CARD_EN)) {
				if (card_exist)
					val &= ~(1 << 7);
				else
					val |= (1 << 7);
			}

			cr_set_l1off_sub(cr, val);
			rts_trace(TRACE_PM,
				"ltr idle and aspm L1.2, L1SUB:%#x\n", val);
		} else {
			cr_set_l1off_sub(cr, 0);
			rts_trace(TRACE_PM,
				"ltr idle and not aspm L1.2, L1SUB:%#x\n",
				0x00);
		}
	}
}

void rts522a_init_options(struct card_reader *cr)
{
	struct cr_option *option = &(cr->option);
	struct hw_param *hw_param = &(cr->hw_param);

	cr_init_generic_options(cr);
	rts522a_init_sw_param(option);
	rts522a_init_hw_param(hw_param);

	if (option->ocp_en)
		hw_param->interrupt_en |= SD_OC_INT_EN;

	if (!option->adma_mode)
		hw_param->interrupt_en |= DATA_DONE_INT_EN;

}

void rts522a_power_saving(struct card_reader *cr)
{
	struct cr_option *option = &cr->option;

	if (option->ltr_enabled) {
		u32 latency = option->ltr_l1off_latency;

		if (check_dev_flag(cr, L1_SNOOZE_TEST_EN))
			mdelay(option->l1_snooze_delay);

		rts_trace(TRACE_PM, "Set L1OFF latency:%#x\n", latency);
		cr_set_ltr_latency(cr, latency);
	}

	if (check_dev_flag(cr, LTR_L1SS_PWR_GATE_EN))
		cr_set_l1off_sub_cfg_D0(cr, 0);

	cr_enable_aspm(cr);

}

static void rts522a_thermal_state_to_sdr50(struct card_reader *cr,
		struct card *card)
{
	struct rts5227_info *info = cr->extra;
	struct sd_info * sd_card = card->extra;

	if(info->thermal_state >= THERMAL_STATE_TO_SDR50
		|| !CHK_SD_SDR104(sd_card))
		return;

	if (!rtsx_chk_stat(cr, RTSX_STAT_RUN)) {
		cr_set_stat(cr, RTSX_STAT_RUN);
		return;
	}

	sd_cleanup_work(cr, card);
	cr_set_card_status(cr, CARD_THERMAL_JUMP_TO_SDR50);
	rts_trace(TRACE_SD, "go to SDR50\n");

	if (sd_reset_card(cr, card) == STATUS_SUCCESS)
		info->thermal_state = THERMAL_STATE_TO_SDR50;
}

static void rts522a_thermal_state_to_normal(struct card_reader *cr, struct card *card)
{
	struct rts5227_info *info = cr->extra;

	if (info->thermal_state == THERMAL_STATE_TO_NORMAL)
		return;

	if (!rtsx_chk_stat(cr, RTSX_STAT_RUN)) {
		cr_set_stat(cr, RTSX_STAT_RUN);
		return;
	}

	sd_cleanup_work(cr, card);
	rts_trace(TRACE_SD, "go back to normal(sdr104) mode\n");
	card->reset_flags &= ~GO_SD20_DIRECT;
	cr_clr_card_status(cr, CARD_THERMAL_JUMP_TO_SDR50);

	if (sd_reset_card(cr, card) == STATUS_SUCCESS)
		info->thermal_state = THERMAL_STATE_TO_NORMAL;
}


static void rts522a_thermal_action(struct card_reader *cr, struct card *card)
{
	struct cr_option *option = &cr->option;
	struct rts5227_info *info = cr->extra;
	u64 elapse;
	u8 gpio_input = 0;

	if (!card || !option->thermal_sensor_en)
		return;

	cr_rd_reg(cr, REG_GPIO_CTL, &gpio_input);
	gpio_input &= 0x04;

	if (info->gpio_input != gpio_input || 0 == info->thermal_time)
		info->thermal_time = cr_get_tick();
	info->gpio_input = gpio_input;

	if (!gpio_input && info->thermal_state < THERMAL_STATE_TO_SDR50) {
		elapse = cr_get_tick() - info->thermal_time;
		rts_trace(TRACE_CR, "Gpio trig low: %lldms\n", elapse);

		if(elapse > option->thermal_time_step3)
			rts522a_thermal_state_to_sdr50(cr, card);
	} else if (gpio_input &&
		info->thermal_state > THERMAL_STATE_TO_NORMAL) {
		elapse = cr_get_tick() - info->thermal_time;
		rts_trace(TRACE_CR, "Gpio trig high: %lldms\n", elapse);

		if (elapse > option->thermal_time_recovery)
			rts522a_thermal_state_to_normal(cr, card);
	}
}

void rts522a_deal_generic_extra_action(struct card_reader *cr)
{
	struct card *card = cm_get_ready_card(cr->cm, SD_ID);
	struct cr_option *option = &(cr->option);

	if (card && option->thermal_sensor_en)
		rts522a_thermal_action(cr, card);

	cr_deal_generic_extra_action(cr);
}


