/* Driver for Realtek PCIe card reader
 * Header file
 *
 * Copyright(c) 2009 Realtek Semiconductor Corp. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *   wwang (wei_wang@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 *
 *   edwin.rong(edwin_rong@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 *
 *   Tony.Nie(tony_nie@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 */

#ifndef RTS5227_H
#define RTS5227_H

#include "cardreader.h"
#include "rts_cr.h"
#include "cardmanager.h"
#include "pm.h"

#define SUPPORT_MSXC

#ifdef SUPPORT_MSXC
#define XC_POWERCLASS
#define SUPPORT_PCGL_1P18
#endif

#define RTS522A_AUTOLOAD_CFG0			0xFF7B
#define RTS522A_AUTOLOAD_CFG1			0xFF7C
#define RTS522A_AUTOLOAD_CFG2			0xFF7D
#define RTS522A_AUTOLOAD_CFG3			0xFF7E
#define RTS522A_AUTOLOAD_CFG4			0xFF7F

#define RTS522A_REG_VREF			0xFE97
#define RTS522A_PWD_SUSPND_EN			(1 << 4)

#define RTS522A_PAD_H3L1			0xFF79
#define PAD_GPIO_H3L1			(1 << 3)

/* rts5227 patch flags */
#define RTS5227_ENABLE_OCP_4_A		(1 << 0)



/* Single LUN, support SD/MS */
#define DEFAULT_SINGLE		0
#define SD_LUN			1
#define MS_LUN			2

/* For Change_FPGA_SSCClock Function */
#define MULTIPLY_BY_1    0x00
#define MULTIPLY_BY_2    0x01
#define MULTIPLY_BY_3    0x02
#define MULTIPLY_BY_4    0x03
#define MULTIPLY_BY_5    0x04
#define MULTIPLY_BY_6    0x05
#define MULTIPLY_BY_7    0x06
#define MULTIPLY_BY_8    0x07
#define MULTIPLY_BY_9    0x08
#define MULTIPLY_BY_10   0x09

#define DIVIDE_BY_2      0x01
#define DIVIDE_BY_3      0x02
#define DIVIDE_BY_4      0x03
#define DIVIDE_BY_5      0x04
#define DIVIDE_BY_6      0x05
#define DIVIDE_BY_7      0x06
#define DIVIDE_BY_8      0x07
#define DIVIDE_BY_9      0x08
#define DIVIDE_BY_10     0x09

struct rts5227_info {
	u8 gpio_input;
	int thermal_state;
#define THERMAL_STATE_TO_NORMAL			0
#define THERMAL_STATE_TO_SDR50			1

	u64 thermal_time;
};

void rts5227_trans_dma_enable(enum dma_data_direction dir,
			struct card_reader *cr, u32 byte_cnt, u8 pack_size);
void rts5227_dma_enable(struct card_reader *cr,
		enum dma_data_direction dir, u32 byte_cnt, u8 pack_size);
int rts5227_switch_clock(struct card_reader *cr, struct card *card,
								int clk);
void rts5227_auto_delink(struct card_reader *cr);
void rts5227_process_ocp(struct card_reader *cr);
int rts5227_select_card(struct card_reader *cr, enum CARD_ID id);
int rts5227_request_irq(struct card_reader *cr);
int rts5227_init_cr(struct card_reader *cr);
void rts5227_init_options(struct card_reader *cr);
void rts5227_init_sw_param(struct cr_option *option);
void rts5227_init_hw_param(struct hw_param *hw_param);
void rts522a_init_hw_param(struct hw_param *hw_param);
int rts5227_init(struct card_reader *cr);
void rts5227_cd_debounce(struct card_reader *cr);
int rts5227_pre_handle_interrupt(struct card_reader *cr);
int rts5227_clear_ocp(struct card_reader *cr);
int rts5227_enable_ocp(struct card_reader *cr);
int rts5227_disable_ocp(struct card_reader *cr);
void rts5227_process_ocp(struct card_reader *cr);
int rts5227_init_ocp(struct card_reader *cr);
int rts5227_sd_current_over_limit400_handler(struct card_reader *cr);

int rts5227_sd_pull_ctl_disable(struct card_reader *cr);
int rts5227_sd_pull_ctl_enable(struct card_reader *cr);
int rts5227_sd_set_sd30_drive(struct card_reader *cr, u8 voltage);
int rts5227_sd_power_off_card3v3(struct card_reader *cr);
int rts5227_sd_disable_card(struct card_reader *cr, struct card *c);
int rts5227_release_sd_card(struct card_reader *cr, struct card *c);
int rts5227_sd_update_report_status(struct card_reader *cr, struct card *c);
int rts5227_sd_init_power(struct card_reader *cr);

int rts5227_ms_prepare_reset(struct card_reader *cr, struct card *card);
int rts5227_ms_pull_ctl_disable(struct card_reader *cr);
int rts5227_ms_pull_ctl_enable(struct card_reader *cr);
int rts5227_release_ms_card(struct card_reader *cr, struct card *card);

void rts5227_before_power_down(struct card_reader *cr, int pm_stat);
int rts5227_do_resume(struct card_reader *cr);
void rts5227_enable_aspm(struct card_reader *cr);
void rts5227_disable_aspm(struct card_reader *cr);
int rts5227_do_before_auto_delink(struct card_reader *cr);
void rts522a_set_l1off_cfg_sub_D0(struct card_reader *cr, int active);
void rts522a_pm_full_on(struct card_reader *cr);
void rts522a_init_options(struct card_reader *cr);
void rts522a_power_saving(struct card_reader *cr);
void rts522a_deal_generic_extra_action(struct card_reader *cr);

#endif /* RTS5227_H */
