/* Driver for Realtek PCIe card reader
 *
 * Copyright(c) 2012 Realtek Semiconductor Corp. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *   Tony.Nie(tony_nie@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 */

#include <linux/blkdev.h>
#include <linux/kthread.h>
#include <linux/sched.h>
#include <linux/workqueue.h>
#include <linux/dma-mapping.h>
#include <linux/pci.h>
#include <scsi/scsi.h>
#include <linux/module.h>
#include <linux/init.h>

#include "cardreader.h"
#include "cr_driver.h"

#include "rts5227.h"
#include "cr_regs.h"
#include "pm.h"
#include "trace.h"
#include "trans_pcie.h"
#include "rts_cr.h"
#include "cards.h"
#include "cards/sd.h"

extern struct cr_driver cr_pcie_driver;

struct cr_sd_ops rts5227_sd_ops_prot = {
	.init_power = rts5227_sd_init_power,
	.pull_ctl_enable = rts5227_sd_pull_ctl_enable,
	.pull_ctl_disable = rts5227_sd_pull_ctl_disable,
	.current_over_limit400_handler =
		rts5227_sd_current_over_limit400_handler,
	.disable = rts5227_sd_disable_card,
	.release = rts5227_release_sd_card,
	.update_status = rts5227_sd_update_report_status,
	/* .rw = rts5227_sd_rw, */
	/* .update_status = */
};

struct cr_ms_ops rts5227_ms_ops_prot = {
	.pull_ctl_enable = rts5227_ms_pull_ctl_enable,
	.pull_ctl_disable = rts5227_ms_pull_ctl_disable,
	.prepare_reset = rts5227_ms_prepare_reset,
	.release = rts5227_release_ms_card,
};

struct cr_ops rts5227_ops = {
	.init_options = rts5227_init_options,
	.init = rts5227_init,
	.deal_extra_action = cr_deal_generic_extra_action,
	.process_ocp = rts5227_process_ocp,
	.init_ocp = rts5227_init_ocp,
};

struct cr_ops rts522a_ops = {
	.init_options = rts522a_init_options,
	.init = rts5227_init,
	.deal_extra_action = rts522a_deal_generic_extra_action,
	.process_ocp = rts5227_process_ocp,
	.init_ocp = rts5227_init_ocp,
};

struct cr_pm_ops rts5227_pm_ops = {
	.bef_pow_dwn = rts5227_before_power_down,
	.do_resume = rts5227_do_resume,
	/*.update_pm = rts5227_update_pm,
	.full_on = rts5227_pm_full_on,
	.power_saving = rts5227_pm_power_saving,
	.power_saving_max = rts5227_pm_power_saving_max,*/
	.enable_aspm = rts5227_enable_aspm,
	.disable_aspm = rts5227_disable_aspm,
	.do_before_auto_delink = rts5227_do_before_auto_delink,
};

struct cr_pm_ops rts522a_pm_ops = {
	.bef_pow_dwn = rts5227_before_power_down,
	.do_resume = rts5227_do_resume,
	.enable_aspm = rts5227_enable_aspm,
	.disable_aspm = rts5227_disable_aspm,
	.do_before_auto_delink = rts5227_do_before_auto_delink,
	.set_l1off_cfg_sub_D0 = rts522a_set_l1off_cfg_sub_D0,
	.full_on = rts522a_pm_full_on,
};


struct cr_capability rts5227_capability = {
	.pid = 0x5227,
	.vid = 0x10EC,
	.name = "rts5227",
	.extra_size = sizeof(struct rts5227_info),
	.card_conf = {
		.lun_mod = SIG_LUN,
		.card_spt_map = (1 << SD_ID) | (1 << MS_ID),
		.card_lun_map = (LUN2 << 8) | (LUN1 << 4) | (LUN0),
		.sd_capability = SD_CAP_MMC_1BIT | SD_CAP_MMC_4BIT | SD_CAP_HS |
				SD_CAP_UHSI,
	},
	.callback = {
		.specific_cmd_handlers = NULL,
		.sd_ops_prot = &rts5227_sd_ops_prot,
		.ms_ops_prot = &rts5227_ms_ops_prot,
		.ops = &rts5227_ops,
		.pm_ops = &rts5227_pm_ops,
	},
	.drv_data = &cr_pcie_driver,
};

struct cr_capability rts522A_capability = {
	.pid = 0x522A,
	.vid = 0x10EC,
	.name = "rts522A",
	.extra_size = sizeof(struct rts5227_info),
	.card_conf = {
		.lun_mod = SIG_LUN,
		.card_spt_map = (1 << SD_ID) | (1 << MS_ID),
		.card_lun_map = (LUN2 << 8) | (LUN1 << 4) | (LUN0),
		.sd_capability = SD_CAP_MMC_1BIT | SD_CAP_MMC_4BIT | SD_CAP_HS |
				SD_CAP_UHSI,
	},
	.callback = {
		.specific_cmd_handlers = NULL,
		.sd_ops_prot = &rts5227_sd_ops_prot,
		.ms_ops_prot = &rts5227_ms_ops_prot,
		.ops = &rts522a_ops,
		.pm_ops = &rts522a_pm_ops,
	},
	.drv_data = &cr_pcie_driver,
};
