/* Driver for Realtek PCIe card reader
 *
 * Copyright(c) 2009 Realtek Semiconductor Corp. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *   Tony.Nie(tony_nie@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 */

#include <linux/blkdev.h>
#include <linux/kthread.h>
#include <linux/sched.h>
#include <linux/workqueue.h>
#include <linux/dma-mapping.h>
#include <linux/pci.h>
#include <scsi/scsi.h>
#include <linux/module.h>
#include <linux/init.h>

#include "cr_regs.h"
#include "pm.h"
#include "trace.h"
#include "trans_pcie.h"
#include "rts_cr.h"
#include "rts5227.h"
#include "rts5249.h"
#include "cards/sd.h"
#include "cardreader.h"
#include "cr_driver.h"

extern struct cr_driver cr_pcie_driver;

static struct cr_ms_ops rts5249_ms_ops_prot = {
	.pull_ctl_disable = rts5249_ms_pull_ctl_disable,
	.pull_ctl_enable = rts5249_ms_pull_ctl_enable,
	.release = rts5249_release_ms_card,
};

static struct cr_sd_ops rts5249_sd_ops_prot = {
	.init_power = rts5249_sd_init_power,
	.cleanup = rts5249_sd_cleanup_work,
	.release = rts5249_release_sd_card,
	.current_over_limit400_handler =
		rts5249_sd_current_over_limit400_handler,
	.chg_bank_volt = rts5249_sd_change_bank_voltage,
	.pull_ctl_disable = rts5249_sd_pull_ctl_disable,
	.pull_ctl_enable = rts5249_sd_pull_ctl_enable,
	.init_sd40_hw = rts5249_init_sd40_hw,
	.init_sd40_hw_m = rts5249_init_sd40_hw_m,
	.uninit_sd40_hw = rts5249_uninit_sd40_hw,
	.process_dormannt_state = rts5249_process_sd40_dormant_state,
	.set_power_control_mode = rts5249_sd40_set_power_control_mode,
	.stop_sequential = rts5249_sd_stop_sequential,
	.patch_special_card = rts5249_patch_special_sd,
	.take_sd40_init = rts5249_sd40_take_dev_init_process,
	.prepare_reset = rts5249_sd_prepare_reset,
	.set_sd40_n_fcu = rts5249_sd40_set_n_fcu,
};

static struct cr_specific_cmd_handlers rts5249_specific_cmd_handlers = {
};

static struct cr_pm_ops rts5249_pm_ops = {
	.bef_pow_dwn = rts5249_before_power_down,
	.do_resume =  rts5249_do_resume,
	.enable_aspm = rts5249_enable_aspm,
	.disable_aspm = rts5249_disable_aspm,
	.do_before_auto_delink = rts5249_do_before_auto_delink,
};

static struct cr_pm_ops rts5250_pm_ops = {
	.bef_pow_dwn = rts5249_before_power_down,
	.do_resume =  rts5249_do_resume,
	.enable_aspm = rts5249_enable_aspm,
	.disable_aspm = rts5249_disable_aspm,
	.do_before_auto_delink = rts5249_do_before_auto_delink,
	.set_l1off_cfg_sub_D0 = rts5250_set_l1off_cfg_sub_D0,
	.full_on = rts5250_pm_full_on,
};

static struct cr_sd_ops rts5250_sd_ops_prot = {
	.init_power = rts5250_sd_init_power,
	.cleanup = rts5249_sd_cleanup_work,
	.release = rts5249_release_sd_card,
	.current_over_limit400_handler =
		rts5250_sd_current_over_limit400_handler,
	.chg_bank_volt = rts5250_sd_change_bank_voltage,
	.pull_ctl_disable = rts5249_sd_pull_ctl_disable,
	.pull_ctl_enable = rts5249_sd_pull_ctl_enable,
	.init_sd40_hw = rts5249_init_sd40_hw,
	.init_sd40_hw_m = rts5249_init_sd40_hw_m,
	.uninit_sd40_hw = rts5249_uninit_sd40_hw,
	.process_dormannt_state = rts5249_process_sd40_dormant_state,
	.set_power_control_mode = rts5250_sd40_set_power_control_mode,
	.stop_sequential = rts5249_sd_stop_sequential,
	.patch_special_card = rts5249_patch_special_sd,
	.take_sd40_init = rts5249_sd40_take_dev_init_process,
	.prepare_reset = rts5249_sd_prepare_reset,
};

static struct cr_ops rts5250_ops = {
	.init_options = rts5250_init_options,
	.init = rts5249_init,
	.detect_card = rts5249_detect_card,
	.poweron_card = rts5250_poweron_card,
	.poweroff_card = rts5249_poweroff_card,
	.switch_to_sdio = rts5249_switch_to_sdio_ctrl,
	.clear_ocpstat = rts5249_clear_ocpstat,
	.get_ocpstat = rts5249_get_ocpstat,
	.enable_ocp = rts5249_enable_ocp,
	.disable_ocp = rts5249_disable_ocp,
	.init_ocp = rts5249_init_ocp,
	.deal_extra_action = rts5250_deal_generic_extra_action,
	.enable_dma = rts5249_enable_dma,
	.process_ocp = rts5250_process_ocp,
	.patch_dma = rts5250_patch_dma,
};

static struct cr_ops rts5249_ops = {
	.init_options = rts5249_init_options,
	.init = rts5249_init,
	.detect_card = rts5249_detect_card,
	.poweron_card = rts5249_poweron_card,
	.poweroff_card = rts5249_poweroff_card,
	.switch_to_sdio = rts5249_switch_to_sdio_ctrl,
	.clear_ocpstat = rts5249_clear_ocpstat,
	.get_ocpstat = rts5249_get_ocpstat,
	.enable_ocp = rts5249_enable_ocp,
	.disable_ocp = rts5249_disable_ocp,
	.init_ocp = rts5249_init_ocp,
	.deal_extra_action = cr_deal_generic_extra_action,
	.enable_dma = rts5249_enable_dma,
};

static struct cr_ops rts524a_ops = {
	.init_options = rts524a_init_options,
	.init = rts5249_init,
	.detect_card = rts5249_detect_card,
	.poweron_card = rts5249_poweron_card,
	.poweroff_card = rts5249_poweroff_card,
	.switch_to_sdio = rts5249_switch_to_sdio_ctrl,
	.clear_ocpstat = rts5249_clear_ocpstat,
	.get_ocpstat = rts5249_get_ocpstat,
	.enable_ocp = rts5249_enable_ocp,
	.disable_ocp = rts5249_disable_ocp,
	.init_ocp = rts5249_init_ocp,
	.deal_extra_action = rts524a_deal_generic_extra_action,
	.enable_dma = rts5249_enable_dma,
	.process_ocp = rts5250_process_ocp,
	.rd_phy16 = rts524a_rd_phy16,
	.wr_phy16 = rts524a_wr_phy16,
};

static struct cr_ops rts525a_ops = {
	.init_options = rts525a_init_options,
	.init = rts5249_init,
	.detect_card = rts5249_detect_card,
	.poweron_card = rts5250_poweron_card,
	.poweroff_card = rts5249_poweroff_card,
	.clear_ocpstat = rts5249_clear_ocpstat,
	.get_ocpstat = rts5249_get_ocpstat,
	.enable_ocp = rts5249_enable_ocp,
	.disable_ocp = rts5249_disable_ocp,
	.init_ocp = rts5249_init_ocp,
	.deal_extra_action = rts5250_deal_generic_extra_action,
	.enable_dma = rts5249_enable_dma,
	.process_ocp = rts5250_process_ocp,
	.enable_oobs_polling = rts525a_enable_oobs_polling,
	.disable_oobs_polling = rts525a_disable_oobs_polling,
};


static struct cr_pm_ops rts524a_pm_ops = {
	.bef_pow_dwn = rts5249_before_power_down,
	.do_resume =  rts5249_do_resume,
	.enable_aspm = rts5249_enable_aspm,
	.disable_aspm = rts5249_disable_aspm,
	.do_before_auto_delink = rts5249_do_before_auto_delink,
	.set_l1off_cfg_sub_D0 = rts5250_set_l1off_cfg_sub_D0,
	.power_saving = rts522a_power_saving,
	.full_on = rts5250_pm_full_on,
};


struct cr_capability rts5250_capability = {
	.pid = 0x5250,
	.vid = 0x10EC,
	.name = "rts5250",
	.extra_size = sizeof(struct rts5249_info),
	.card_conf = {
		.lun_mod = SIG_LUN,
		.card_spt_map = (1 << SD_ID) | (1 << MS_ID),
		.card_lun_map = (LUN2 << 8) | (LUN1 << 4) | (LUN0),
		.sd_capability = SD_CAP_MMC_1BIT | SD_CAP_MMC_4BIT |
				SD_CAP_MMC_8BIT | SD_CAP_MMC_SDR200 |
				SD_CAP_HS | SD_CAP_UHSI | SD_CAP_UHSII,
	},
	.callback = {
		.specific_cmd_handlers = &rts5249_specific_cmd_handlers,
		.sd_ops_prot = &rts5250_sd_ops_prot,
		.ms_ops_prot = &rts5249_ms_ops_prot,
		.ops = &rts5250_ops,
		.pm_ops = &rts5250_pm_ops,
	},
	.drv_data = &cr_pcie_driver,
};

struct cr_capability rts5249_capability = {
	.pid = 0x5249,
	.vid = 0x10EC,
	.name = "rts5249",
	.extra_size = sizeof(struct rts5249_info),
	.card_conf = {
		.lun_mod = SIG_LUN,
		.card_spt_map = (1 << SD_ID) | (1 << MS_ID),
		.card_lun_map = (LUN2 << 8) | (LUN1 << 4) | (LUN0),
		.sd_capability = SD_CAP_MMC_1BIT | SD_CAP_MMC_4BIT | SD_CAP_HS |
				SD_CAP_UHSI | SD_CAP_UHSII,
	},
	.callback = {
		.specific_cmd_handlers = &rts5249_specific_cmd_handlers,
		.sd_ops_prot = &rts5249_sd_ops_prot,
		.ms_ops_prot = &rts5249_ms_ops_prot,
		.ops = &rts5249_ops,
		.pm_ops = &rts5249_pm_ops,
	},
	.tr = &realtek_transport_sets[REALTEK_TRANSPORT_PCIE],
	.drv_data = &cr_pcie_driver,
};

struct cr_capability rts524A_capability = {
	.pid = 0x524A,
	.vid = 0x10EC,
	.name = "rts524A",
	.extra_size = sizeof(struct rts5249_info),
	.card_conf = {
		.lun_mod = SIG_LUN,
		.card_spt_map = (1 << SD_ID) | (1 << MS_ID),
		.card_lun_map = (LUN2 << 8) | (LUN1 << 4) | (LUN0),
		.sd_capability = SD_CAP_MMC_1BIT | SD_CAP_MMC_4BIT | SD_CAP_HS |
				SD_CAP_UHSI | SD_CAP_UHSII,
	},
	.callback = {
		.specific_cmd_handlers = &rts5249_specific_cmd_handlers,
		.sd_ops_prot = &rts5249_sd_ops_prot,
		.ms_ops_prot = &rts5249_ms_ops_prot,
		.ops = &rts524a_ops,
		.pm_ops = &rts524a_pm_ops,
	},
	.tr = &realtek_transport_sets[REALTEK_TRANSPORT_PCIE],
	.drv_data = &cr_pcie_driver,
};

struct cr_capability rts525A_capability = {
	.pid = 0x525A,
	.vid = 0x10EC,
	.name = "rts525A",
	.extra_size = sizeof(struct rts5249_info),
	.card_conf = {
		.lun_mod = SIG_LUN,
		.card_spt_map = (1 << SD_ID) | (1 << MS_ID),
		.card_lun_map = (LUN2 << 8) | (LUN1 << 4) | (LUN0),
		.sd_capability = SD_CAP_MMC_1BIT | SD_CAP_MMC_4BIT |
				SD_CAP_MMC_8BIT | SD_CAP_MMC_SDR200 |
				SD_CAP_HS | SD_CAP_UHSI | SD_CAP_UHSII,
	},
	.callback = {
		.specific_cmd_handlers = &rts5249_specific_cmd_handlers,
		.sd_ops_prot = &rts5250_sd_ops_prot,
		.ms_ops_prot = &rts5249_ms_ops_prot,
		.ops = &rts525a_ops,
		.pm_ops = &rts5250_pm_ops,
	},
	.drv_data = &cr_pcie_driver,
};
