#ifndef _RTS_RPM_H_
#define _RTS_RPM_H_

#include <linux/device.h>
#include <linux/pci.h>
#include "cardreader.h"

#ifdef RTSX_RTD3_PM_RUNTIME

#define dev_power_ucount(dev) atomic_read(&(dev)->power.usage_count)

static inline void cr_pci_runtime_pm_forbid(struct card_reader *cr)
{
	struct device *dev = cr_to_pci_device(cr);

	pm_runtime_forbid(dev);
	pm_runtime_get_noresume(dev);
	rts_trace(TRACE_PM, "usage_count = %d\n", dev_power_ucount(dev));
}

static inline void cr_pci_runtime_pm_allow(struct card_reader *cr)
{
	struct device *dev = cr_to_pci_device(cr);

	pm_runtime_put_noidle(dev);
	pm_runtime_set_active(dev);
	pm_suspend_ignore_children(dev, true);
	pm_runtime_use_autosuspend(dev);
	pm_runtime_set_autosuspend_delay(dev, 20000);
	rts_trace(TRACE_PM, "usage_count = %d\n", dev_power_ucount(dev));
}

static inline int cr_runtime_get(struct card_reader *cr)
{
	struct device *dev = cr_to_pci_device(cr);
	rts_trace(TRACE_PM, "origin usage_count = %d\n", dev_power_ucount(dev));
	return pm_runtime_get_sync(dev);
}

static inline int cr_runtime_put(struct card_reader *cr)
{
	struct device *dev = cr_to_pci_device(cr);
	rts_trace(TRACE_PM, "origin usage_count = %d\n", dev_power_ucount(dev));
	return pm_runtime_put_sync(dev);
}

static inline int cr_runtime_idle(struct device *dev)
{
	struct card_reader *cr = pci_get_drvdata(to_pci_dev(dev));

	rts_trace(TRACE_PM, "usage_count = %d, state = %d\n",
		dev_power_ucount(dev), cr->rtsx_stat);

	pm_runtime_mark_last_busy(dev);
	pm_runtime_autosuspend(dev);

	return -EBUSY;
}

#else  /* RTSX_RTD3_PM_RUNTIME */
#define dev_power_ucount(dev) 0
static inline void cr_pci_runtime_pm_forbid(struct card_reader *cr) {}
static inline void cr_pci_runtime_pm_allow(struct card_reader *cr) {}
static inline int cr_runtime_get(struct card_reader *cr) { return 0; }
static inline int cr_runtime_put(struct card_reader *cr) { return 0; }
#define cr_runtime_idle NULL
#endif /* RTSX_RTD3_PM_RUNTIME */

#endif /* _RTS_RPM_H_ */
