/* Driver for Realtek card reader
 * Header file
 *
 * Copyright(c) 2009 Realtek Semiconductor Corp. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *   wwang (wei_wang@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 *
 *   edwin.rong(edwin_rong@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 *
 *   Tony.Nie(tony_nie@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 */

#ifndef RTS_CR_SCSI_H
#define RTS_CR_SCSI_H

#include <linux/usb.h>
#include <linux/usb_usual.h>
#include <linux/blkdev.h>
#include <linux/completion.h>
#include <linux/mutex.h>
#include <linux/device.h>
#include <linux/version.h>
#include <scsi/scsi_host.h>
#include <scsi/scsi_device.h>
#include <scsi/scsi_cmnd.h>

#ifndef SERVICE_ACTION_IN_16
#define   SERVICE_ACTION_IN_16	SERVICE_ACTION_IN
#endif	/* SERVICE_ACTION_IN_16 */

/* #define SCSI_SCAN_DELAY */
#define READ_FORMAT_CAPACITIES	0x23
#define READ_BUFFER_CAPACITY	0x5C
#define SEND_CUE_SHEET		0x5D
#define READ_MASTER_CUE		0x59
#define WRITE_CONTINUE		0xE1

#define MS_SP_CMND		0xFA
#define MS_FORMAT		0xA0
#define GET_MS_INFORMATION	0xB0

#define VENDOR_CMND		0xF0

#define READ_STATUS		0x09
#define SET_LED			0x0A

#define READ_MEM		0x0D
#define WRITE_MEM		0x0E
#define GET_BUS_WIDTH		0x13
#if 0 /* edwin: redefine the cmd */
#define GET_SD_CSD		0x14
#else
#define GET_SD_REG		0x14
#endif
#define TOGGLE_GPIO		0x15
#define TRACE_MSG		0x18

#define SCSI_APP_CMD		0x10

#define PP_READ10		0x1A
#define PP_WRITE10		0x0A
#define READ_HOST_REG		0x1D
#define WRITE_HOST_REG		0x0D
#define SET_VAR			0x05
#define GET_VAR			0x15
#define DMA_READ		0x16
#define DMA_WRITE		0x06
#define GET_DEV_STATUS		0x10
#define GET_CARD_STATUS		0x12
#define SET_CHIP_MODE		0x27
#define SUIT_CMD		0xE0
#define WRITE_PHY		0x07
#define READ_PHY		0x17
#define WRITE_CFG		0x0E
#define READ_CFG		0x1E
#define READ10_BYTE		0x31
#define WRITE10_BYTE		0x32
#define RESET_CARD		0x33
#define RELEASE_CARD		0x34
#define CARD_MODE		0x62
#define   CARD_MODE_CF		0x00
#define   CARD_MODE_SM		0x01
#define   CARD_MODE_SD		0x02
#define   CARD_MODE_MS		0x03
#define   CARD_MODE_XD		0x04
#define   CARD_MODE_MMC		0x08
#define   CARD_MODE_MSPRO	0x09
#define   CARD_MODE_SDHC	0x10

#define READ_CSD		0x70
#define WRITE_CSD		0x71


/* { wwang, 2010-04-27
 * Add batch command */
#define INIT_BATCHCMD		0x41
#define ADD_BATCHCMD		0x42
#define SEND_BATCHCMD		0x43
#define GET_BATCHRSP		0x44
/* } wwang, 2010-04-27 */

/* SD Pass Through Command Extention */
#define SD_PASS_THRU_MODE	0xD0
#define SD_EXECUTE_NO_DATA	0xD1
#define SD_EXECUTE_READ		0xD2
#define SD_EXECUTE_WRITE	0xD3
#define SD_GET_RSP		0xD4
#define SD_HW_RST		0xD6
/* #endif */

/* #ifdef SUPPORT_MAGIC_GATE */
#define CMD_MSPRO_MG_RKEY	0xA4	/* Report Key Command */
#define CMD_MSPRO_MG_SKEY	0xA3	/* Send Key Command */

/* CBWCB field: key class */
#define KC_MG_R_PRO		0xBE	/* MG-R PRO */

/* CBWCB field: key format */
#define KF_SET_LEAF_ID		0x31	/* Set Leaf ID */
#define KF_GET_LOC_EKB		0x32	/* Get Local EKB */
#define KF_CHG_HOST		0x33	/* Challenge (host) */
#define KF_RSP_CHG		0x34	/* Response and Challenge (device)  */
#define KF_RSP_HOST		0x35	/* Response (host) */
#define KF_GET_ICV		0x36	/* Get ICV */
#define KF_SET_ICV		0x37	/* SSet ICV */
/* #endif */

#define CHECK_MEDIA_TYPE		0xEC
#define MEDIA_SPEC_FUNCTION		0xED
#define   MSF_GET_SD_INFO		0x01
#define   MSF_SPEED_CLASS_CONTROL	0x02
#define   MSF_CHANGE_BUS_IF_MODE	0x03

struct card_reader;

/*-----------------------------------
    Start-Stop-Unit
-----------------------------------*/
#define STOP_MEDIUM			0x00	/* access disable */
#define MAKE_MEDIUM_READY		0x01	/* access enable */
#define UNLOAD_MEDIUM			0x02	/* unload */
#define LOAD_MEDIUM			0x03	/* load */

/*-----------------------------------
    STANDARD_INQUIRY
-----------------------------------*/
#define QULIFIRE                0x00
#define AENC_FNC                0x00
#define TRML_IOP                0x00
#define REL_ADR                 0x00
#define WBUS_32                 0x00
#define WBUS_16                 0x00
#define SYNC                    0x00
#define LINKED                  0x00
#define CMD_QUE                 0x00
#define SFT_RE                  0x00

#define VEN_ID_LEN              8	/* Vendor ID Length         */
#define PRDCT_ID_LEN            16	/* Product ID Length        */
#define PRDCT_REV_LEN           4	/* Product LOT Length       */

#define DRCT_ACCESS_DEV         0x00	/* Direct Access Device             */
#define RMB_DISC                0x80	/* The Device is Removable          */
#define ANSI_SCSI2              0x02	/* Based on ANSI-SCSI2              */

#define SCSI                    0x00	/* Interface ID                     */

/* Dynamic bitflag definitions (dflags): used in set_bit() etc. */
#define FLIDX_URB_ACTIVE	0	/* current_urb is in use    */
#define FLIDX_SG_ACTIVE		1	/* current_sg is in use     */
#define FLIDX_ABORTING		2	/* abort is in progress     */
#define FLIDX_DISCONNECTING	3	/* disconnect in progress   */
#define FLIDX_RESETTING		4	/* device reset in progress */
#define FLIDX_TIMED_OUT		5	/* SCSI midlayer timed out  */
#define FLIDX_DONT_SCAN		6	/* don't scan (disconnect)  */

/* Size of the autosense data buffer */
#define SENSE_SIZE		18

/* Sense type */
enum {
	SENSE_TYPE_NO_SENSE = 0,
	SENSE_TYPE_MEDIA_CHANGE,
	SENSE_TYPE_MEDIA_NOT_PRESENT,
	SENSE_TYPE_MEDIA_LBA_OVER_RANGE,
	SENSE_TYPE_MEDIA_LUN_NOT_SUPPORT,
	SENSE_TYPE_MEDIA_WRITE_PROTECT,
	SENSE_TYPE_MEDIA_INVALID_CMD_FIELD,
	SENSE_TYPE_MEDIA_UNRECOVER_READ_ERR,
	SENSE_TYPE_MEDIA_WRITE_ERR,
	SENSE_TYPE_FORMAT_IN_PROGRESS,
	SENSE_TYPE_FORMAT_CMD_FAILED,
	/* COPY PROTECTION KEY EXCHANGE FAILURE - KEY NOT ESTABLISHED */
	SENSE_TYPE_MG_KEY_FAIL_NOT_ESTAB,
	/* COPY PROTECTION KEY EXCHANGE FAILURE - AUTHENTICATION FAILURE */
	SENSE_TYPE_MG_KEY_FAIL_NOT_AUTHEN,
	/* INCOMPATIBLE MEDIUM INSTALLED */
	SENSE_TYPE_MG_INCOMPATIBLE_MEDIUM,
	/* WRITE ERROR */
	SENSE_TYPE_MG_WRITE_ERR,
	SENSE_TYPE_OVERLAPPED_CMD_ATTEMPTED,
	/* FOR Locked SD card */
	SENSE_TYPE_MEDIA_READ_FORBIDDEN,
};

/*---- sense key ----*/
#define ILI                     0x20	/* ILI bit is on                    */

#define NO_SENSE                0x00	/* not exist sense key              */
#define RECOVER_ERR             0x01	/* Target/Logical unit is recoverd  */
#define NOT_READY               0x02	/* Logical unit is not ready        */
#define MEDIA_ERR               0x03	/* medium/data error                */
#define HARDWARE_ERR            0x04	/* hardware error                   */
#define ILGAL_REQ               0x05	/* CDB/parameter/identify msg error */
#define UNIT_ATTENTION          0x06	/* unit attention condition occur   */
#define DAT_PRTCT               0x07	/* read/write is desable            */
#define BLNC_CHK                0x08	/* find blank/DOF in read           */
					/* write to unblank area            */
#define CPY_ABRT                0x0a	/* Copy/Compare/Copy&Verify illgal  */
#define ABRT_CMD                0x0b	/* Target make the command in error */
#define EQUAL                   0x0c	/* Search Data end with Equal       */
#define VLM_OVRFLW              0x0d	/* Some data are left in buffer     */
#define MISCMP                  0x0e	/* find inequality                  */

/*-----------------------------------
    SENSE_DATA
-----------------------------------*/
/*---- valid ----*/
#define SENSE_VALID             0x80	/* Sense data is valid as SCSI2     */
#define SENSE_INVALID           0x00	/* Sense data is invalid as SCSI2   */

/*---- error code ----*/
#define CUR_ERR                 0x70	/* current error                    */
#define DEF_ERR                 0x71	/* specific command error           */

/*---- sense key Infomation ----*/
#define SNSKEYINFO_LEN          3	/* length of sense key infomation   */

#define SKSV                    0x80
#define CDB_ILLEGAL             0x40
#define DAT_ILLEGAL             0x00
#define BPV                     0x08
#define BIT_ILLEGAL0            0	/* bit0 is illegal                  */
#define BIT_ILLEGAL1            1	/* bit1 is illegal                  */
#define BIT_ILLEGAL2            2	/* bit2 is illegal                  */
#define BIT_ILLEGAL3            3	/* bit3 is illegal                  */
#define BIT_ILLEGAL4            4	/* bit4 is illegal                  */
#define BIT_ILLEGAL5            5	/* bit5 is illegal                  */
#define BIT_ILLEGAL6            6	/* bit6 is illegal                  */
#define BIT_ILLEGAL7            7	/* bit7 is illegal                  */

/*---- ASC ----*/
#define ASC_NO_INFO             0x00
#define ASC_MISCMP              0x1d
#define ASC_INVLD_CDB           0x24
#define ASC_INVLD_PARA          0x26
#define ASC_LU_NOT_READY	0x04
#define ASC_WRITE_ERR           0x0c
#define ASC_READ_ERR            0x11
#define ASC_LOAD_EJCT_ERR       0x53
#define	ASC_MEDIA_NOT_PRESENT	0x3A
#define	ASC_MEDIA_CHANGED	0x28
#define	ASC_MEDIA_IN_PROCESS	0x04
#define	ASC_WRITE_PROTECT	0x27
#define ASC_LUN_NOT_SUPPORTED	0x25

/*---- ASQC ----*/
#define ASCQ_NO_INFO            0x00
#define	ASCQ_MEDIA_IN_PROCESS	0x01
#define ASCQ_MISCMP             0x00
#define ASCQ_INVLD_CDB          0x00
#define ASCQ_INVLD_PARA         0x02
#define ASCQ_LU_NOT_READY	0x02
#define ASCQ_WRITE_ERR          0x02
#define ASCQ_READ_ERR           0x00
#define ASCQ_LOAD_EJCT_ERR      0x00
#define	ASCQ_WRITE_PROTECT	0x00

struct sense_data_t {
	unsigned char err_code;	/* error code */
	/* bit7 : valid                    */
	/*   (1 : SCSI2)                    */
	/*   (0 : Vendor specific)          */
	/* bit6-0 : error code             */
	/*  (0x70 : current error)          */
	/*  (0x71 : specific command error) */
	unsigned char seg_no;	/* segment No.                      */
	unsigned char sense_key;	/* byte5 : ILI                      */
	/* bit3-0 : sense key              */
	unsigned char info[4];	/* infomation                       */
	unsigned char ad_sense_len;	/* additional sense data length     */
	unsigned char cmd_info[4];	/* command specific infomation      */
	unsigned char asc;	/* ASC                              */
	unsigned char ascq;	/* ASCQ                             */
	unsigned char rfu;	/* FRU                              */
	unsigned char sns_key_info[3];	/* sense key specific infomation    */
};

/* The scsi_lock() and scsi_unlock() macros protect the sm_state and the
 * single queue element srb for write access */
#define scsi_unlock(host)	spin_unlock_irq(host->host_lock)
#define scsi_lock(host)		spin_lock_irq(host->host_lock)

#define SCSI_LUN(srb)		((srb)->device->lun)


#define MAX_ALLOWED_LUN_CNT	8

struct rts_scsi {
	struct completion cmnd_ready;
	struct scsi_cmnd *srb;
	struct card_reader *cr;

	struct sense_data_t sense_buffer[MAX_ALLOWED_LUN_CNT];

	unsigned long dflags; /* dynamic atomic bitflags */
	struct task_struct *ctl_thread; /* the control thread */
	struct completion control_exit; /* control thread exit */
	struct completion notify; /* thread begin/end */
#ifdef SCSI_SCAN_DELAY
	wait_queue_head_t delay_wait; /* wait during scan, reset */
	struct completion scanning_done; /* wait for scan thread */
#endif
};


struct Scsi_Host;
struct scsi_device;
struct scsi_cmnd;

/* Convert between us_data and the corresponding Scsi_Host */
static inline struct Scsi_Host *scsi_to_host(struct rts_scsi *scsi)
{
	return container_of((void *)scsi, struct Scsi_Host, hostdata);
}

static inline struct rts_scsi *host_to_scsi(struct Scsi_Host *host)
{
	return (struct rts_scsi *)(host->hostdata);
}

const char *host_info(struct Scsi_Host *host);
int slave_alloc(struct scsi_device *sdev);
int slave_configure(struct scsi_device *sdev);

#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 37)
int queuecommand(struct scsi_cmnd *srb, void (*done) (struct scsi_cmnd *));
#else
int queuecommand(struct Scsi_Host *, struct scsi_cmnd *);
#endif

int command_abort(struct scsi_cmnd *srb);
int device_reset(struct scsi_cmnd *srb);
int bus_reset(struct scsi_cmnd *srb);
struct rts_scsi *scsi_alloc(void);
int scsi_init(struct rts_scsi *scsi, struct device *dev);
void scsi_release(struct rts_scsi *scsi);
void quiesce_and_remove_host(struct rts_scsi *scsi);
void scsi_invoke_transport(struct scsi_cmnd *srb, struct card_reader *cr);

extern struct scsi_host_template rts_host_template;

int scsi_pm(struct scsi_cmnd *srb, struct card_reader *cr);
/* struct scsi_cmnd transfer buffer access utilities */

enum xfer_buf_dir { TO_XFER_BUF, FROM_XFER_BUF };

#endif /* RTS_CR_SCSI_H */
