/* Driver for Realtek RTS51xx USB card reader
 * Header file
 *
 * Copyright(c) 2009 Realtek Semiconductor Corp. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *   wwang (wei_wang@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 *
 *   edwin.rong(edwin_rong@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 *
 *   Tony.Nie(tony_nie@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 */

#ifndef RTS_CR_TRACE_H
#define RTS_CR_TRACE_H

#define _MSG_TRACE

#define TRACE_ITEM_CNT		64

struct trace_msg_t {
	u16 line;
#define MSG_FUNC_LEN 64
	char func[MSG_FUNC_LEN];
#define MSG_FILE_LEN 32
	char file[MSG_FILE_LEN];
#define TIME_VAL_LEN 16
	u8 timeval_buf[TIME_VAL_LEN];
	u8 valid;
};

#ifdef _MSG_TRACE
static inline char *filename(char *path)
{
	char *ptr;

	if (path == NULL)
		return NULL;

	ptr = path;

	while (*ptr != '\0') {
		if ((*ptr == '\\') || (*ptr == '/'))
			path = ptr + 1;
		ptr++;
	}

	return path;
}

static inline void get_current_time(u8 *timeval_buf, int buf_len)
{
	struct timeval tv;

	if (!timeval_buf || (buf_len < 8))
		return;

	do_gettimeofday(&tv);

	timeval_buf[0] = (u8) (tv.tv_sec >> 24);
	timeval_buf[1] = (u8) (tv.tv_sec >> 16);
	timeval_buf[2] = (u8) (tv.tv_sec >> 8);
	timeval_buf[3] = (u8) (tv.tv_sec);
	timeval_buf[4] = (u8) (tv.tv_usec >> 24);
	timeval_buf[5] = (u8) (tv.tv_usec >> 16);
	timeval_buf[6] = (u8) (tv.tv_usec >> 8);
	timeval_buf[7] = (u8) (tv.tv_usec);
}

#define TRACE_RET(chip, ret)						\
do {									\
	char *_file = filename((char *)__FILE__);			\
	rts_trace(TRACE_ERROR, "[%s][%s]:[%d]\n", _file, __func__, __LINE__); \
	(chip)->trace_msg[(chip)->msg_idx].line = (u16)(__LINE__);	\
	strncpy((chip)->trace_msg[(chip)->msg_idx].func,		\
			__func__, MSG_FUNC_LEN-1);			\
	strncpy((chip)->trace_msg[(chip)->msg_idx].file,		\
			_file, MSG_FILE_LEN-1);				\
	get_current_time((chip)->trace_msg[(chip)->msg_idx].timeval_buf,\
			TIME_VAL_LEN);	\
	(chip)->trace_msg[(chip)->msg_idx].valid = 1;			\
	(chip)->msg_idx++;						\
	if ((chip)->msg_idx >= TRACE_ITEM_CNT) {			\
		(chip)->msg_idx = 0;					\
	}								\
	return ret;							\
} while (0)

#define trace_ret(chip, ret, fmt, arg...)				\
	do {								\
		rts_trace(TRACE_ERROR, fmt, ##arg);			\
		return ret;						\
	} while (0)

#define TRACE_GOTO(chip, label)						\
do {									\
	char *_file = filename((char *)__FILE__);			\
	rts_trace(TRACE_ERROR, "[%s][%s]:[%d]\n", _file, __func__, __LINE__); \
	(chip)->trace_msg[(chip)->msg_idx].line = (u16)(__LINE__);	\
	strncpy((chip)->trace_msg[(chip)->msg_idx].func,		\
			__func__, MSG_FUNC_LEN-1);			\
	strncpy((chip)->trace_msg[(chip)->msg_idx].file,		\
			_file, MSG_FILE_LEN-1);				\
	get_current_time((chip)->trace_msg[(chip)->msg_idx].timeval_buf,\
			TIME_VAL_LEN);					\
	(chip)->trace_msg[(chip)->msg_idx].valid = 1;			\
	(chip)->msg_idx++;						\
	if ((chip)->msg_idx >= TRACE_ITEM_CNT) {			\
		(chip)->msg_idx = 0;					\
	}								\
	goto label;							\
} while (0)

#else
#define TRACE_RET(chip, ret)	return (ret)
#define TRACE_GOTO(chip, label)	goto label
#define trace_ret TRACE_RET
#endif

#define ENABLE_TRIGGER()
#define DISABLE_TRIGGER()

#define RTSX_STOR "rts: "
#define hex_dump(buf, buf_len) \
	print_hex_dump(KERN_DEBUG, RTSX_STOR, DUMP_PREFIX_NONE,	\
		16, 1, (buf), (buf_len), false)

#ifdef DBG

#define RTS_DUMP(buf, buf_len)	\
	print_hex_dump(KERN_DEBUG, RTSX_STOR, DUMP_PREFIX_NONE,	\
		16, 1, (buf), (buf_len), false)
#if 0
#define CATCH_TRIGGER(chip)					\
do {								\
	rts51x_ep0_write_register((chip), 0xFC31, 0x01, 0x01);	\
	RTS_DBGP("Catch trigger!\n");			\
} while (0)
#else
#define CATCH_TRIGGER(chip)					\
do {								\
	cr_wr_reg_ep0((chip), 0xFC31, 0x01, 0x01);	\
	RTS_DBGP("Catch trigger!\n");			\
} while (0)
#endif

#else
#define RTS_DUMP(buf, buf_len)
#define CATCH_TRIGGER(chip)
#endif

#define catch_trigger(cr)	\
	do {\
		if (rts_trace_mask & TRACE_TRIGGER) {		\
			cr_iowrite8(cr, 0x1F, 0xFF);		\
			cr_iowrite8(cr, 0x1F, 0x00);		\
			rts_trace(TRACE_TRIGGER, "Catch Trigger\n");	\
		}	\
	} while (0)

struct card_reader;
void trace_msg(struct card_reader *cr, unsigned char *buf, int clear);

#define RTS_TIP "rts: "

#ifdef DBG
#define RTS_DBGP(x...) printk(KERN_DEBUG RTS_TIP x)
#define RTS_DBGPN(x...) printk(KERN_DEBUG x)
#define RTS_DBGPX(x...) printk(x)
#define RTS_DBG(x) x
#else
#define RTS_DBGP(x...)
#define RTS_DBGPN(x...)
#define RTS_DBGPX(x...)
#define RTS_DBG(x)
#endif

enum {
	TRACE_CR		= 1U << 0,
	TRACE_CR_PROTOCOL	= 1U << 1,
	TRACE_CM		= 1U << 2,
	TRACE_SD		= 1U << 4,
	TRACE_MS		= 1U << 5,
	TRACE_XD		= 1U << 6,
	TRACE_TR		= 1U << 7,
	TRACE_TR_USB		= 1U << 8,
	TRACE_TR_PCIE		= 1U << 9,
	TRACE_SCSI		= 1U << 10,
	TRACE_PM		= 1U << 11,
	TRACE_ERROR		= 1U << 15, /* errors should always print out */
	TRACE_WARN		= 1U << 16, /* warn should always print out */
	TRACE_INFO		= 1U << 17, /* info should always print out */
	TRACE_PROBE		= 1U << 20,
	TRACE_INTR		= 1U << 21,
	TRACE_TRIGGER		= 1U << 22,
	TRACE_DUMP_ALL		= 1U << 30,
	TRACE_POLL_THRD		= 1U << 31,
};

#define DEBUG_BASE (TRACE_ERROR | TRACE_WARN | TRACE_INFO)
#define DEBUG_NORMAL (DEBUG_BASE | TRACE_CR | TRACE_PROBE | TRACE_PM |	\
		TRACE_CM | TRACE_SD | TRACE_MS | TRACE_XD)
#define DEBUG_HIGH (DEBUG_NORMAL | TRACE_CR_PROTOCOL |			\
		TRACE_TR | TRACE_TR_USB | TRACE_TR_PCIE |		\
		TRACE_SCSI | TRACE_DUMP_ALL)
#define DEBUG_ALL (DEBUG_HIGH | 0xFFFFFFFF)

#ifndef RTSX_DEBUG
  #define RTSX_DEBUG		0
#endif

#if RTSX_DEBUG == 1
  #define RTSX_TRACE_MASK	DEBUG_NORMAL
#elif RTSX_DEBUG == 2
  #define RTSX_TRACE_MASK	DEBUG_HIGH
#elif RTSX_DEBUG == 3
  #define RTSX_TRACE_MASK	DEBUG_ALL
#endif

#ifndef RTSX_TRACE_MASK
  #define RTSX_TRACE_MASK	DEBUG_NORMAL
#endif

extern uint32_t rts_trace_mask;

#define rts_trace(mask, fmt, arg...) \
	do { \
		if ((rts_trace_mask | DEBUG_BASE) & mask)		\
			printk(KERN_INFO "%s: " fmt, __func__, ## arg); \
	} while (0)

#endif /* RTS_CR_TRACE_H */
