/* Driver for Realtek RTS51xx USB card reader
 * Header file
 *
 * Copyright(c) 2009 Realtek Semiconductor Corp. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *   wwang (wei_wang@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 *
 *   edwin.rong(edwin_rong@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 *
 *   Tony.Nie(tony_nie@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 */

#ifndef RTS_CR_TRANS_PCIE_H
#define RTS_CR_TRANS_PCIE_H

#include <linux/kernel.h>
#include <linux/version.h>
#include <linux/pci.h>

#include "transport.h"
#include "rts_scsi.h"

/* pci_intx() wrapper */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 14)
#ifdef CONFIG_PCI
#undef pci_intx
#define pci_intx(pci, x)
#endif
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(3, 13, 1)
static inline struct pci_dev *cr_pci_upstream_bridge(struct pci_dev *dev)
{
	dev = pci_physfn(dev);
	if (pci_is_root_bus(dev->bus))
		return NULL;

	return dev->bus->self;
}
#else	/* KERNEL_VERSION(3, 13, 1) */
static inline struct pci_dev *cr_pci_upstream_bridge(struct pci_dev *dev)
{
	return pci_upstream_bridge(dev);
}
#endif	/* KERNEL_VERSION(3, 13, 1) */


#define RTSX_RESV_BUF_LEN	4096
#define HOST_CMDS_BUF_LEN	1024
#define HOST_SG_TBL_BUF_LEN	(RTSX_RESV_BUF_LEN - HOST_CMDS_BUF_LEN)

#define STATE_TRANS_NONE	0
#define STATE_TRANS_CMD		1
#define STATE_TRANS_BUF		2
#define STATE_TRANS_SG		3

#define TRANS_NOT_READY		0
#define TRANS_RESULT_OK		1
#define TRANS_RESULT_FAIL	2

/* SG descriptor */
#define SG_INT			0x04
#define SG_END			0x02
#define SG_VALID		0x01

#define SG_NO_OP		0x00
#define SG_TRANS_DATA		(0x02 << 4)
#define SG_LINK_DESC		(0x03 << 4)

#define cr_alloc_dma_buf(cr, size, flag)	kmalloc((size), (flag))
#define cr_free_dma_buf(cr, ptr)		kfree((ptr))

struct rts_pcie {
#if 1 /* edwin: moved from struct rts5227_info */
	struct pci_dev		*pci_dev;
	u8			func_no;
	int			irq;

	void			*context;

	unsigned long		pcie_start_addr;
	void __iomem		*pcie_start_remap_addr;

	void			*host_cmds_ptr; /* host commands buffer pointer */
	dma_addr_t		host_cmds_addr;
	unsigned int		cmd_index;
#endif

	/* struct pci_dev		*pci_dev; */

	/* pci resources */
	unsigned long		addr;
	void __iomem		*remap_addr;
	/* int			irq; */

	/* locks */
	spinlock_t		reg_lock;

	/* host reserved buffer */
	void			*rtsx_resv_buf;
	dma_addr_t		rtsx_resv_buf_addr;

	void			*host_sg_tbl_ptr;		/* SG descriptor table */
	dma_addr_t		host_sg_tbl_addr;
	int			sgi;				/* SG entry index */

	struct completion	*done;
	char			trans_result;
	char			trans_state;
};

int tr_pcie_transfer_data_partial(struct transport *tr, u8 card, void *buf,
	size_t len, int use_sg, unsigned int *index, unsigned int *offset,
				enum dma_data_direction dma_dir, int timeout);
int tr_pcie_transfer_data(struct transport *tr, u8 card, void *buf, size_t len,
		int use_sg, enum dma_data_direction dma_dir, int timeout);
u8 *tr_pcie_get_cmd_data(struct transport *tr);
int tr_pci_rd_cfg_byte(struct transport *tr, u8 bus, u8 dev, u8 func,
		u8 offset, u8 *val);
int init_transport_pcie(struct transport *tr, void *extra);
int release_transport_pcie(struct transport *tr);
inline void tr_pcie_init_cmd(struct transport *tr);
void tr_pcie_add_cmd(struct transport *tr, u8 cmd_type, u16 reg_addr,
							u8 mask, u8 data);
void tr_pcie_send_cmd_no_wait(struct transport *tr);
void tr_pcie_stop_cmd(struct transport *tr, int card);
int tr_pcie_send_cmd(struct transport *tr, u8 card, int timeout);
inline int tr_pcie_enable_msi(struct transport *tr);
inline void tr_pcie_synchronize_irq(struct transport *tr);
int tr_pci_find_capablility(struct transport *tr, int cap);
int tr_pci_find_host_capablility(struct transport *tr, int cap);

#endif /* RTS_CR_TRANS_PCIE_H */
