/* Driver for Realtek PCIe card reader
 *
 * Copyright(c) 2009 Realtek Semiconductor Corp. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *   Tony.Nie(tony_nie@realsil.com.cn)
 *   No. 450, Shenhu Road, Suzhou Industry Park, Suzhou, China
 */

#include <linux/blkdev.h>
#include <linux/kthread.h>
#include <linux/sched.h>
#include <linux/workqueue.h>
#include <linux/dma-mapping.h>
#include <linux/pci.h>
#include <scsi/scsi.h>
#include <linux/module.h>
#include <linux/init.h>

#include "cr_system.h"
#include "cr_regs.h"
#include "pm.h"
#include "trace.h"
#include "trans_pcie.h"
#include "rts_cr.h"
#include "cardmanager.h"
#include "cards.h"
#include "cards/sd.h"
#include "cards/sd40.h"
#include "cards/ms.h"
#include "operations.h"
#include "rts5249.h"

void rts5249_switch_to_sdio_ctrl(struct card_reader *cr)
{
	u8 reg1 = 0, reg2 = 0;

	cr_rd_reg(cr, 0xF034, &reg1);
	cr_rd_reg(cr, 0xF038, &reg2);

	if ((reg1 & 0xC0) && (reg2 & 0xC0)) {
		cr->sdio_func_init = 1;
		cr_wr_reg(cr, REG_SDIO_MUX_CTRL, 0x3,
			SDIO_BUS_CTRL | SDIO_CD_CTRL);
		cr_wr_reg(cr, REG_PWR_GATE_CTRL, LDO3318_PWR_MASK, LDO_ON);
	}
}

void rts5250_chk_pcie_speed(struct card_reader *cr)
{
	u8 speed, tmp, temp, i = 0;
	u16 val;

	cr_rd_cfg8(cr, cr->dev_pci_cap_offset+0x12, &speed);
	if (((speed & 0xf) == 0x1) && ((cr->host_pci_speed_cap&0x0f) == 0x2)) {
		rts_trace(TRACE_CR, "switch to Gen-II\n");

		cr_rd_phy16(cr, 0x3, &val);
		tmp = (u8)((val&0x3e) >> 1);
		if (tmp < cr->option.pcie_oobs_max_value) {
			tmp++;
			val &= 0xffc1;
			val |= (u16)(tmp << 1);
			cr_wr_phy16(cr, 0x3, val);

			cr_wr_cfg8(cr, 0x80e, 0x2);
			do {
				cr_rd_cfg8(cr, 0x80e, &temp);
				if ((temp&0x2) == 0)
					break;
				i++;
				msleep(100);
			} while (i < 255);
		}
	}
}

void rts5250_pm_full_on(struct card_reader *cr)
{
	struct cr_option *option = &cr->option;

	if (CHECK_PID(cr, 0x5250))
		cr_force_power_on(cr, SSC_PDCTL | OC_PDCTL);
	else
		cr_force_power_on(cr, SSC_PDCTL);
	cr_disable_aspm(cr);

	if (option->ltr_enabled) {
		u32 latency = option->ltr_active_latency;

		rts_trace(TRACE_PM, "Set active latency%#x\n", latency);
		cr_set_ltr_latency(cr, latency);
	}

	if (check_dev_flag(cr, LTR_L1SS_PWR_GATE_EN))
		cr_set_l1off_sub_cfg_D0(cr, 1);

	if (cr->asic_code && (!option->ft2_fast_mode) && CHECK_PID(cr, 0x5250))
		rts5250_chk_pcie_speed(cr);
}

void rts5249_init_sw_param(struct cr_option *option)
{
	option->dev_flags |= CLEAR_0xFE8D_BIT0_EN;
	option->dev_flags |= CONFIG_EIDLE_DEGLITCH;
	option->dev_flags |= CONFIG_N_LSS_DIR;
	option->dev_flags |= EN_PCIE_MAX_READ_REQ_SIZE;
	option->dev_flags |= EXTRA_ACTION_FOR_ASPM_EN;
	option->dev_flags |= FORCE_HOST_ASPM_EN;
	option->dev_flags |= L1SS_SETBACK_EN;
	option->dev_flags |= LTR_L1SS_PWR_GATE_EN;
	option->dev_flags |= S3_CLEAR_SETBACK_EN;
	option->dev_flags |= RESET_CARD_AFTER_RESUME_EN;
	option->dev_flags |= CONFIG_N_LSS_DIR;
	option->sd40_ctl = 0x91821000;

	option->support_ms_8bit = 1;
	option->sd40_ebnum = 0x30;
	option->ignore_sd = 0;

	option->ltr_en = 1;

	/* init latency of active, idle, L1OFF to 60us, 300us, 3ms */
	/* 60us:0x883C, 300us:0x892C, 3ms:0x9003 */
	option->ltr_active_latency = 0x883C;
	option->ltr_idle_latency = 0x892C;
	option->ltr_l1off_latency = 0x9003;
	option->ltr_l1off_snooze_sspwrgate = 0xAC;
	option->ltr_l1off_sspwrgate = 0xAF;
	option->sd40_initial_retry_enable = 1;

	option->obff_en = 0;
	option->aux_pwr_exist = 1;
	option->sd40_nfcu = 8;
	option->n_lss_dir = 0;

	option->sd30_clk_drive_sel_3v3 = 0x96;
	option->sd30_cmd_drive_sel_3v3 = 0x96;
	option->sd30_dat_drive_sel_3v3 = 0x96;
	option->sd30_clk_drive_sel_1v8 = 0xB3;
	option->sd30_cmd_drive_sel_1v8 = 0xB3;
	option->sd30_dat_drive_sel_1v8 = 0xB3;

	option->l1_snooze_delay = 1;
	option->sd_sdr50_default_tx_phase = 29;
	option->sdr_tx_tuning_en = 1;
	option->clk_req_bypass_short = 0;
	option->l1sub_fe49 = 0;
	option->phy_optimize = 0;
	option->sd_sdr104_default_tx_phase = 1;
	option->reg_0xfe76 = 0xF7;
}

void rts524a_init_sw_param(struct cr_option *option)
{
	option->dev_flags |= CLEAR_0xFE8D_BIT0_EN;
	option->dev_flags |= CONFIG_EIDLE_DEGLITCH;
	option->dev_flags |= CONFIG_N_LSS_DIR;
	option->dev_flags |= EN_PCIE_MAX_READ_REQ_SIZE;
	option->dev_flags |= EXTRA_ACTION_FOR_ASPM_EN;
	option->dev_flags |= FORCE_HOST_ASPM_EN;
	option->dev_flags |= L1SS_SETBACK_EN;
	option->dev_flags |= LTR_L1SS_PWR_GATE_EN;
	option->dev_flags |= S3_CLEAR_SETBACK_EN;
	option->dev_flags |= RESET_CARD_AFTER_RESUME_EN;
	option->dev_flags |= CONFIG_N_LSS_DIR;
	option->dev_flags |= LTR_L1SS_PWR_GATE_CHECK_CARD_EN;
	option->dev_flags |= SD40_WRITE_SPEED_UP_EN;
	option->sd40_ctl = 0x91821000;

	option->support_ms_8bit = 1;
	option->sd40_ebnum = 0x30;
	option->ignore_sd = 0;

	option->ltr_en = 1;

	/* init latency of active, idle, L1OFF to 60us, 300us, 3ms */
	/* 60us:0x883C, 300us:0x892C, 3ms:0x9003 */
	option->ltr_active_latency = 0x883C;
	option->ltr_idle_latency = 0x892C;
	option->ltr_l1off_latency = 0x9003;
	option->ltr_l1off_snooze_sspwrgate = 0xF8;
	option->ltr_l1off_sspwrgate = 0xFF;
	option->sd40_initial_retry_enable = 1;

	option->obff_en = 0;
	option->aux_pwr_exist = 1;
	option->n_lss_dir = 0;

	option->sd30_clk_drive_sel_3v3 = 0x96;
	option->sd30_cmd_drive_sel_3v3 = 0x96;
	option->sd30_dat_drive_sel_3v3 = 0x96;
	option->sd30_clk_drive_sel_1v8 = 0xB3;
	option->sd30_cmd_drive_sel_1v8 = 0xB3;
	option->sd30_dat_drive_sel_1v8 = 0xB3;

	option->l1_snooze_delay = 1;
	option->sd_sdr50_default_tx_phase = 29;
	option->sdr_tx_tuning_en = 1;
	option->clk_req_bypass_short = 0;
	option->l1sub_fe49 = 0;
	option->phy_optimize = 0;
	option->sd_400mA_ocp_thd = 0;
	option->sd_800mA_ocp_thd = 4;
	option->reg_0xfe76 = 0xFF;
	option->sd40_nfcu = 8;
	option->sd40_oobs_autoK_en = 1;
	option->polling_oobs_en = 1;
}

void rts5249_init_hw_param(struct hw_param *hw_param)
{
	hw_param->interrupt_en = TRANS_OK_INT_EN | TRANS_FAIL_INT_EN;
	hw_param->interrupt_en |= SD_INT_EN | MS_INT_EN | XD_INT_EN;
	hw_param->ocp_glitch = SD_OCP_GLITCH_800U | SDVIO_OCP_GLITCH_800U;

	hw_param->stb_lh_time = 0xFF;

	hw_param->eidle_num = 7;

	/* extract from ini_PCIE5249_RTSPER.INI */
	hw_param->phy0x08_val = 0x4FE4;
	hw_param->phy0x00_val = 0xBA43;
	hw_param->phy0x01_val = 0x713F;
	hw_param->phy0x03_val = 0xC152;
	hw_param->phy0x82_val = 0x2608;
	hw_param->phy0x92_val = 0xE21D;
	hw_param->phy0x82_val_patch = 0x2408;
	hw_param->phy0x85_val = 0x2497;
	hw_param->phy0x87_val = 0x7C69;
	hw_param->phy0x88_val = 0x4820;
	hw_param->phy0x90_val = 0x09C2;
	hw_param->phy0x05_val = 0x4600;
	hw_param->phy0x19_val = 0xFE6C;
	hw_param->phy0x1e_val = 0x78EB;
	hw_param->phy0x02_val = 0x041F;
	hw_param->phy0x1d_val = 0x0824;
}

void rts524a_init_hw_param(struct hw_param *hw_param)
{
	hw_param->interrupt_en = TRANS_OK_INT_EN | TRANS_FAIL_INT_EN;
	hw_param->interrupt_en |= SD_INT_EN | MS_INT_EN | XD_INT_EN;
	hw_param->ocp_glitch = SD_OCP_GLITCH_10M | SDVIO_OCP_GLITCH_800U;

	hw_param->phy0x03_val = 0xC4EB;

	hw_param->stb_lh_time = 0x7F;

	hw_param->eidle_num = 1;

	/* extract from ini_PCIE524A_RTSPER.INI */
	hw_param->phy0x08_val = 0x57E4;
	hw_param->phy0x00_val = 0xBA42;
	hw_param->phy0x01_val = 0x713F;
	hw_param->phy0x03_val = 0xC4EB;
	hw_param->phy0x82_val = 0x2608;
	hw_param->phy0x92_val = 0xE21D;
	hw_param->phy0x82_val_patch = 0x2408;
	hw_param->phy0x85_val = 0x04DF;
	hw_param->phy0x87_val = 0xEC7B;
	hw_param->phy0x88_val = 0x0820;
	hw_param->phy0x90_val = 0x09F2;
	hw_param->phy0x05_val = 0x4600;
	hw_param->phy0x19_val = 0xFE6C;
	hw_param->phy0x1e_val = 0x5C7F;
	hw_param->phy0x02_val = 0x0A1F;
	hw_param->phy0x1d_val = 0x0004;
	hw_param->phy0x1a_val = 0x2546;
	hw_param->av12s = 0x04;
}

void rts525a_init_hw_param(struct hw_param *hw_param)
{
	hw_param->interrupt_en = TRANS_OK_INT_EN | TRANS_FAIL_INT_EN;
	hw_param->interrupt_en |= SD_INT_EN | MS_INT_EN | XD_INT_EN;
	hw_param->ocp_glitch = SD_OCP_GLITCH_10M | SDVIO_OCP_GLITCH_800U;

	hw_param->phy0x85_val = 0x04DF;
	hw_param->phy0x87_val = 0xEC7B;

	hw_param->stb_lh_time = 0x7F;

	hw_param->eidle_num = 1;

	/* extract from ini_PCIE525A_RTSPER.INI */
	hw_param->phy0x01_val = 0x713F;
	hw_param->phy0x03_val = 0x2748;
	hw_param->phy0x82_val = 0x2608;
	hw_param->phy0x92_val = 0xE21D;
	hw_param->phy0x82_val_patch = 0x2408;
	hw_param->phy0x85_val = 0x2497;
	hw_param->phy0x87_val = 0x6C7B;
	hw_param->phy0x88_val = 0x0820;
	hw_param->phy0x90_val = 0x09F2;
	hw_param->phy0x05_val = 0x4600;
	hw_param->phy0x19_val = 0x3902;
	hw_param->phy0x1e_val = 0x78EB;
	hw_param->phy0x02_val = 0x041F;
	hw_param->phy0x1d_val = 0x99FF;
	hw_param->phy0x1a_val = 0x2546;
}

void rts5250_init_hw_param(struct hw_param *hw_param)
{
	hw_param->interrupt_en = TRANS_OK_INT_EN | TRANS_FAIL_INT_EN;
	hw_param->interrupt_en |= SD_INT_EN | MS_INT_EN | XD_INT_EN;
	hw_param->ocp_glitch = SD_OCP_GLITCH_10M | SDVIO_OCP_GLITCH_800U;

	hw_param->phy0x1d_val = 0x99FF;
	hw_param->eidle_num = 1;
	hw_param->stb_lh_time = 0x7F;
	hw_param->phy0x03_val = 0x2748;
	hw_param->phy0x05_val = 0x4600;
	hw_param->phy0x1d_val = 0x99FF;
	hw_param->phy0x85_val = 0x0497;
	hw_param->phy0x87_val = 0x6C7B;
	hw_param->phy0x88_val = 0x0820;
	hw_param->phy0x90_val = 0x09F2;
	hw_param->phy0x92_val = 0xE21D;
	hw_param->phy0x82_val = 0x2608;
	hw_param->phy0x19_val = 0xFE6C;
	hw_param->phy0x82_val_patch = 0x2408;
}

void rts5250_init_options(struct card_reader *cr)
{
	struct cr_option *option = &(cr->option);
	struct hw_param *hw_param = &(cr->hw_param);

	cr_init_generic_options(cr);
	rts5249_init_sw_param(option);
	rts5250_init_hw_param(hw_param);

	option->reg_0xfe76 = 0xFF;
	option->dev_flags |= LTR_L1SS_PWR_GATE_CHECK_CARD_EN;
	option->sd40_nfcu = 64;
	option->dev_aspm_val = 3;
	option->ltr_l1off_snooze_sspwrgate = 0xF8;
	option->ltr_l1off_sspwrgate = 0xFF;
	option->sd40_oobs_autoK_en = 1;
	option->sd40_initial_retry_enable = 1;

	option->sd30_clk_drive_sel_3v3 = 0x94;
	option->sd30_cmd_drive_sel_3v3 = 0x94;
	option->sd30_dat_drive_sel_3v3 = 0x94;
	hw_param->stb_lh_time = 0x7F;
	option->sd_400mA_ocp_thd = 2;
	option->sd_800mA_ocp_thd = 6;
	option->sd_sdr50_default_tx_phase = 29;
	option->dev_flags |= SD40_WRITE_SPEED_UP_EN;
	option->dev_flags |= CLEAR_0xFD57_EN;
	option->sd40_ctl = 0x91821000;

	if (option->ocp_en)
		hw_param->interrupt_en |= SD_OC_INT_EN;

	if (!option->adma_mode)
		hw_param->interrupt_en |= DATA_DONE_INT_EN;
}

void rts525a_init_options(struct card_reader *cr)
{
	struct cr_option *option = &(cr->option);
	struct hw_param *hw_param = &(cr->hw_param);

	cr_init_generic_options(cr);
	rts5249_init_sw_param(option);
	rts525a_init_hw_param(hw_param);

	option->reg_0xfe76 = 0xFF;
	option->dev_flags |= LTR_L1SS_PWR_GATE_CHECK_CARD_EN;
	option->sd40_nfcu = 64;
	option->dev_aspm_val = 3;
	option->ltr_l1off_snooze_sspwrgate = 0xF8;
	option->ltr_l1off_sspwrgate = 0xFF;
	option->sd40_oobs_autoK_en = 1;
	option->sd40_initial_retry_enable = 1;

	option->sd30_clk_drive_sel_3v3 = 0x94;
	option->sd30_cmd_drive_sel_3v3 = 0x94;
	option->sd30_dat_drive_sel_3v3 = 0x94;
	hw_param->stb_lh_time = 0x7F;
	option->sd_400mA_ocp_thd = 0;
	option->sd_800mA_ocp_thd = 5;
	option->sd_sdr50_default_tx_phase = 29;
	option->dev_flags |= SD40_WRITE_SPEED_UP_EN;
	option->dev_flags |= CLEAR_0xFD57_EN;
	option->sd40_ctl = 0x91821000;
	option->polling_oobs_en = 1;
	option->mem_pwr_cut_en = 1;

	if (option->ocp_en)
		hw_param->interrupt_en |= SD_OC_INT_EN;

	if (!option->adma_mode)
		hw_param->interrupt_en |= DATA_DONE_INT_EN;
}


void rts5249_init_options(struct card_reader *cr)
{
	struct cr_option *option = &(cr->option);
	struct hw_param *hw_param = &(cr->hw_param);

	cr_init_generic_options(cr);
	rts5249_init_sw_param(option);
	rts5249_init_hw_param(hw_param);

	if (option->ocp_en)
		hw_param->interrupt_en |= SD_OC_INT_EN;

	if (!option->adma_mode)
		hw_param->interrupt_en |= DATA_DONE_INT_EN;
}

void rts524a_init_options(struct card_reader *cr)
{
	struct cr_option *option = &(cr->option);
	struct hw_param *hw_param = &(cr->hw_param);

	cr_init_generic_options(cr);
	rts524a_init_sw_param(option);
	rts524a_init_hw_param(hw_param);

	if (option->ocp_en)
		hw_param->interrupt_en |= SD_OC_INT_EN;

	if (!option->adma_mode)
		hw_param->interrupt_en |= DATA_DONE_INT_EN;
}


void rts5249_enable_aspm(struct card_reader *cr)
{
	struct cr_option *option = &cr->option;
	struct card *card = cm_get_ready_card(cr->cm, SD_ID);
	struct sd_info *sd_info = NULL;

	if (card)
		sd_info = card->extra;

	if (check_sd40_flag(cr, SD40_IGNORE_ASPM) ||
		CHECK_VERSION_LOWER_THAN(cr, 0x5249, IC_VER_C)) {
		if ((sd_info) && CHK_SD_UHSII(sd_info)) {
			rts_trace(TRACE_PM,
				"SD40_IGNORE_ASPM is enabled\n");
			return;
		}
	}

	rts_trace(TRACE_PM, "Try to enable ASPM\n");
	rts_trace(TRACE_PM, "dev_aspm_val = %#x\n", option->dev_aspm_val);
	rts_trace(TRACE_PM, "dev_aspm_mode = %#x\n", option->dev_aspm_mode);

	if (cr->aspm_enabled)
		return;
	if (DEV_ASPM_BACKDOOR != option->dev_aspm_mode
		&& DEV_ASPM_DYNAMIC != option->dev_aspm_mode)
		goto host;

	if ((sd_info) && CHK_SD_UHSII(sd_info)) {
		int ret = 0;
		int s_hibernate = check_sd40_flag(cr, SD40_SUPPORT_HIBERNATE);
		if (sd_info->cap.dev.hibernate && s_hibernate) {
			ret = rts5249_process_sd40_dormant_state(cr, card,
					ENTRY_DORMANT, DOR_HIBERNATE_MODE);
			if (!ret)
				sd_info->dormant = DOR_HIBERNATE_STATE;
		} else {
			ret = rts5249_process_sd40_dormant_state(cr, card,
					ENTRY_DORMANT, DOR_NORMAL_MODE);
			if (!ret)
				sd_info->dormant = DOR_DORMANT_STATE;
		}

		if (!ret && !CHECK_VERSION(cr, 0x5249, IC_VER_A)
			&& !CHECK_PID(cr,0x524A) && !CHECK_PID(cr,0x525A))
			cr_wr_reg(cr, REG_PWD_SUSPEND_EN, U_REG_SUSPEND_EN, 0);

		if (ret) {
			sd_info->dormant = DOR_ACTIVE_STATE;
			rts_trace(TRACE_ERROR, "entry dormant fail, return\n");
			goto host;
		}
	}

	/* Enable ASPM */
	if (DEV_ASPM_DYNAMIC == option->dev_aspm_mode) {
		int lctlr = cr->dev_pci_cap_offset + PCI_EXP_LNKCTL;
		u8 val = 0;
		cr_read_config8(cr, lctlr, &val);
		val &= 0xFC;
		val |= (option->dev_aspm_val & 0x03);
		cr_write_config8(cr, lctlr, val);
	} else if (DEV_ASPM_BACKDOOR == option->dev_aspm_mode) {
		u8 mask = FORCE_ASPM_CTL0 | FORCE_ASPM_VAL_MASK;
		u8 val = 0;
		cr_wr_reg(cr, REG_ASPM_FORCE_CTL, mask, val);
	}
host:
	if (option->config_host_aspm)
		cr_set_host_aspm(cr, cr->host_aspm_val);

	cr->aspm_enabled = 1;

	return;
}

void rts5249_disable_aspm(struct card_reader *cr)
{
	struct cr_option *option = &cr->option;
	struct card *card = cm_get_ready_card(cr->cm, SD_ID);
	struct sd_info *sd_info = NULL;

	if (card)
		sd_info = card->extra;

	if (check_sd40_flag(cr, SD40_IGNORE_ASPM) ||
		CHECK_VERSION_LOWER_THAN(cr, 0x5249, IC_VER_C)) {
		if ((sd_info) && CHK_SD_UHSII(sd_info)) {
			rts_trace(TRACE_PM,
				"SD40_IGNORE_ASPM is enabled\n");
			return;
		}
	}

	rts_trace(TRACE_PM, "Try to disable ASPM\n");

	if (!(cr->aspm_enabled))
		return;

	cr->aspm_enabled = 0;

	if (option->config_host_aspm)
		cr_disable_host_aspm(cr);

	if (DEV_ASPM_BACKDOOR != option->dev_aspm_mode
		&& DEV_ASPM_DYNAMIC != option->dev_aspm_mode)
		return;

	if (DEV_ASPM_DYNAMIC == option->dev_aspm_mode) {
		int lctlr = cr->dev_pci_cap_offset + PCI_EXP_LNKCTL;
		u8 val = 0;
		cr_read_config8(cr, lctlr, &val);
		val &= 0xFC;
		cr_write_config8(cr, lctlr, val);
	} else if (DEV_ASPM_BACKDOOR == option->dev_aspm_mode) {
		u8 mask = FORCE_ASPM_CTL0 | FORCE_ASPM_VAL_MASK;
		u8 val = FORCE_ASPM_CTL0;
		cr_wr_reg(cr, REG_ASPM_FORCE_CTL, mask, val);
	}

	msleep(10);

	if ((sd_info) && CHK_SD_UHSII(sd_info)) {
		if (DOR_HIBERNATE_STATE == sd_info->dormant) {
			rts5249_process_sd40_dormant_state(cr, card,
					EXIT_DORMANT, DOR_HIBERNATE_MODE);
		} else if (DOR_DORMANT_STATE == sd_info->dormant) {
			rts5249_process_sd40_dormant_state(cr, card,
					EXIT_DORMANT, DOR_NORMAL_MODE);
		} else {
			/* TODO */
		}

		if (!CHECK_VERSION(cr, 0x5249, IC_VER_A)
			&& !CHECK_PID(cr,0x524A) && !CHECK_PID(cr,0x525A)) {
			u8 mask = U_REG_SUSPEND_EN;
			u8 val = cr->option.reg_0xfe76 & U_REG_SUSPEND_EN;
			cr_wr_reg(cr, REG_PWD_SUSPEND_EN, mask, val);
		}

		sd_info->dormant = DOR_ACTIVE_STATE;
	}

	return;
}

static void config_nec_sd30_drive_from_hw(struct card_reader *cr)
{
	struct cr_option *option = &(cr->option);
	u32 lval = 0;
	u8 val;
	u8 sd30_drive_sel_1v8;
	u8 sd30_drive_sel_3v3;
	u8 sd_drive[4] = {
		0x01,	/* Type D */
		0x02,	/* Type C */
		0x05,	/* Type A */
		0x03	/* Type B */
	};

	cr_rd_cfg32(cr, 0x724, &lval);
	if (lval & (1 << 24))
		return;

	/* 0x727 */
	val = (u8)(lval >> 24);
	sd30_drive_sel_1v8 = sd_drive[(val >> 2) & 0x03];

	/* 0x814 */
	cr_rd_cfg32(cr, 0x814, &lval);
	val = (u8)lval;
	sd30_drive_sel_3v3 = sd_drive[(val >> 5) & 0x03];

	switch (sd30_drive_sel_1v8) {
	case 0x01:
		option->sd30_clk_drive_sel_1v8 = 0x3C;
		option->sd30_cmd_drive_sel_1v8 = 0x3C;
		option->sd30_dat_drive_sel_1v8 = 0x3C;
		break;
	case 0x02:
		option->sd30_clk_drive_sel_1v8 = 0xB3;
		option->sd30_cmd_drive_sel_1v8 = 0xB3;
		option->sd30_dat_drive_sel_1v8 = 0xB3;
		break;
	case 0x03:
		option->sd30_clk_drive_sel_1v8 = 0xC4;
		option->sd30_cmd_drive_sel_1v8 = 0xC4;
		option->sd30_dat_drive_sel_1v8 = 0xC4;
		break;
	case 0x05:
		option->sd30_clk_drive_sel_1v8 = 0xFE;
		option->sd30_cmd_drive_sel_1v8 = 0xFE;
		option->sd30_dat_drive_sel_1v8 = 0xFE;
		break;
	default:
		option->sd30_clk_drive_sel_1v8 = 0xB3;
		option->sd30_cmd_drive_sel_1v8 = 0xB3;
		option->sd30_dat_drive_sel_1v8 = 0xB3;
		break;
	}

	switch (sd30_drive_sel_3v3) {
	case 0x01:
		option->sd30_clk_drive_sel_3v3 = 0x11;
		option->sd30_cmd_drive_sel_3v3 = 0x11;
		option->sd30_dat_drive_sel_3v3 = 0x18;
		break;
	case 0x02:
		option->sd30_clk_drive_sel_3v3 = 0x55;
		option->sd30_cmd_drive_sel_3v3 = 0x55;
		option->sd30_dat_drive_sel_3v3 = 0x5C;
		break;
	case 0x03:
		option->sd30_clk_drive_sel_3v3 = 0x99;
		option->sd30_cmd_drive_sel_3v3 = 0x99;
		option->sd30_dat_drive_sel_3v3 = 0x92;
		break;
	default:
		/* x1.0 */
		option->sd30_clk_drive_sel_3v3 = 0x99;
		option->sd30_cmd_drive_sel_3v3 = 0x99;
		option->sd30_dat_drive_sel_3v3 = 0x92;
		break;
	}

}

static void config_generic_sd30_drive_from_hw(struct card_reader *cr)
{
	struct cr_option *option = &(cr->option);
	u32 lval = 0;
	u8 val;
	u8 sd30_drive_sel_1v8;
	u8 sd30_drive_sel_3v3;
	u8 sd_drive[4] = {
		0x01,	/* Type D */
		0x02,	/* Type C */
		0x05,	/* Type A */
		0x03	/* Type B */
	};

	cr_rd_cfg32(cr, 0x724, &lval);
	if (lval & (1 << 24))
		return;

	/* 0x727 */
	val = (u8)(lval >> 24);
	sd30_drive_sel_1v8 = sd_drive[(val >> 2) & 0x03];

	/* 0x814 */
	cr_rd_cfg32(cr, 0x814, &lval);
	val = (u8)lval;
	sd30_drive_sel_3v3 = sd_drive[(val >> 5) & 0x03];

	switch (sd30_drive_sel_1v8) {
	case 0x01:
		option->sd30_clk_drive_sel_1v8 = 0xC4;
		option->sd30_cmd_drive_sel_1v8 = 0xC4;
		option->sd30_dat_drive_sel_1v8 = 0xC4;
		break;
	case 0x02:
		option->sd30_clk_drive_sel_1v8 = 0x3C;
		option->sd30_cmd_drive_sel_1v8 = 0x3C;
		option->sd30_dat_drive_sel_1v8 = 0x3C;
		break;
	case 0x03:
		option->sd30_clk_drive_sel_1v8 = 0xB3;
		option->sd30_cmd_drive_sel_1v8 = 0xB3;
		option->sd30_dat_drive_sel_1v8 = 0xB3;
		break;
	case 0x05:
		option->sd30_clk_drive_sel_1v8 = 0xFE;
		option->sd30_cmd_drive_sel_1v8 = 0xFE;
		option->sd30_dat_drive_sel_1v8 = 0xFE;
		break;
	default:
		option->sd30_clk_drive_sel_1v8 = 0xB3;
		option->sd30_cmd_drive_sel_1v8 = 0xB3;
		option->sd30_dat_drive_sel_1v8 = 0xB3;
		break;
	}

	if (CHECK_PID(cr, 0x5250)) {
		switch (sd30_drive_sel_3v3) {
		case 0x01:
			option->sd30_clk_drive_sel_3v3 = 0x11;
			option->sd30_cmd_drive_sel_3v3 = 0x11;
			option->sd30_dat_drive_sel_3v3 = 0x18;
			break;
		case 0x02:
			option->sd30_clk_drive_sel_3v3 = 0x55;
			option->sd30_cmd_drive_sel_3v3 = 0x55;
			option->sd30_dat_drive_sel_3v3 = 0x5c;
			break;
		case 0x03:
			option->sd30_clk_drive_sel_3v3 = 0x94;
			option->sd30_cmd_drive_sel_3v3 = 0x94;
			option->sd30_dat_drive_sel_3v3 = 0x94;
			break;
		case 0x05:
			option->sd30_clk_drive_sel_3v3 = 0xff;
			option->sd30_cmd_drive_sel_3v3 = 0xff;
			option->sd30_dat_drive_sel_3v3 = 0xff;
			break;
		default:
			/* x1.0 */
			option->sd30_clk_drive_sel_3v3 = 0x94;
			option->sd30_cmd_drive_sel_3v3 = 0x94;
			option->sd30_dat_drive_sel_3v3 = 0x94;
			break;
		}
	} else {
		switch (sd30_drive_sel_3v3) {
		case 0x01:
			option->sd30_clk_drive_sel_3v3 = 0x11;
			option->sd30_cmd_drive_sel_3v3 = 0x11;
			option->sd30_dat_drive_sel_3v3 = 0x18;
			break;
		case 0x02:
			option->sd30_clk_drive_sel_3v3 = 0x55;
			option->sd30_cmd_drive_sel_3v3 = 0x55;
			option->sd30_dat_drive_sel_3v3 = 0x5c;
			break;
		case 0x03:
			option->sd30_clk_drive_sel_3v3 = 0x96;
			option->sd30_cmd_drive_sel_3v3 = 0x96;
			option->sd30_dat_drive_sel_3v3 = 0x96;
			break;
		case 0x05:
			option->sd30_clk_drive_sel_3v3 = 0xff;
			option->sd30_cmd_drive_sel_3v3 = 0xff;
			option->sd30_dat_drive_sel_3v3 = 0xff;
			break;
		default:
			/* x1.0 */
			option->sd30_clk_drive_sel_3v3 = 0x96;
			option->sd30_cmd_drive_sel_3v3 = 0x96;
			option->sd30_dat_drive_sel_3v3 = 0x96;
			break;
		}
	}

}

static void config_sd30_drive_from_hw(struct card_reader *cr)
{
	/* NEC:	   17AA 604     17AA 6061
	 * LENOVO: 17AA 3800    17AA 3801
	 */
	if (CHECK_PID(cr, 0x5249) &&
		(0x17AA == cr->ssvid &&
			(0x6045 == cr->ssdid
				|| 0x6061 == cr->ssdid
				|| 0x3800 == cr->ssdid
				|| 0x3801 == cr->ssdid)))
		config_nec_sd30_drive_from_hw(cr);
	else
		config_generic_sd30_drive_from_hw(cr);
}

static int rts5249_init_from_cfg(struct card_reader *cr)
{
	struct cr_option *option = &(cr->option);
	u32 lval;

	if (CHECK_PID(cr, 0x524A))
		cr_read_config32(cr, 0x160, &lval);
	else
		cr_read_config32(cr, 0x168, &lval);

	if (CHECK_PID(cr, 0x524A) || CHECK_PID(cr, 0x525A)) {
		if ((0 == (lval & 0x0F)) && option->polling_oobs_en)
			cr_enable_oobs_polling(cr);
		else
			cr_disable_oobs_polling(cr);
	}
	if (lval & (1 << 3))
		set_dev_flag(cr, ASPM_L1_1_EN);
	else
		clear_dev_flag(cr, ASPM_L1_1_EN);
	if (lval & (1 << 2))
		set_dev_flag(cr, ASPM_L1_2_EN);
	else
		clear_dev_flag(cr, ASPM_L1_2_EN);

	if (lval & (1 << 1))
		set_dev_flag(cr, PM_L1_1_EN);
	else
		clear_dev_flag(cr, PM_L1_1_EN);
	if (lval & (1 << 0))
		set_dev_flag(cr, PM_L1_2_EN);
	else
		clear_dev_flag(cr, PM_L1_2_EN);

	/* make ltr_l1off_latency contain fixed value 0x9003 (3ms) */
	/* if (check_dev_flag(cr, ASPM_L1_2_EN) {
	  u32 threshold = (lval >> 16) & 0x3FF;
	  u32 scale = (lval >> 29) & 0x7;
	  option->ltr_l1off_latency =
	  (1 << 15) | (scale << 10) | threshold;
	  }*/

	if (option->ltr_en) {
		u16 val;
		int devctl2 = cr->dev_pci_cap_offset + PCI_EXP_DEVCTL2;
		if (cr_read_config16(cr, devctl2, &val) < 0)
			TRACE_RET(cr, STATUS_FAIL);
		rts_trace(TRACE_CR, "LTR 0x%04x: 0x%04x\n", devctl2, val);
		if (val & PCI_EXP_LTR_EN) {
			option->ltr_enabled = 1;
			rts_trace(TRACE_PM, "Set active latency%#x\n",
					option->ltr_active_latency);
			option->ltr_active = 1;
			cr_set_ltr_latency(cr, option->ltr_active_latency);
		} else {
			option->ltr_enabled = 0;
		}
	}

	return 0;
}

static void rts5250_save_content_to_autoload_space(struct card_reader *cr)
{
	u32 relink_time = 0;
	u16 cfg[] = {0x9, 0xA, 0xB};
	int i = 0;
	u8 cfg_cnt = sizeof(cfg) / sizeof(*cfg);

	cr_wr_cfg32(cr, 0x814, 0xFF000000, 0x80000000);

	cr_wr_reg(cr, 0xFF00, 0xFF, ((cfg_cnt + 2) & 0x7F) | 0x80);

	if (cr->option.auto_delink_en)
		relink_time = (cr->option.relink_time >> 4);
	else
		relink_time = 0;
	cr_wr_reg(cr, 0xFF01, 0xFF, (u8)relink_time);
	cr_wr_reg(cr, 0xFF02, 0xFF, (u8)(relink_time >> 8));
	cr_wr_reg(cr, 0xFF03, 0x01, (u8)(relink_time >> 16));

	cr_wr_reg(cr, 0xFF04, 0xFF, (u8)cr->ssvid);
	cr_wr_reg(cr, 0xFF05, 0xFF, (u8)(cr->ssvid >> 8));
	cr_wr_reg(cr, 0xFF06, 0xFF, (u8)cr->ssdid);
	cr_wr_reg(cr, 0xFF07, 0xFF, (u8)(cr->ssdid >> 8));

	/* save 0xFE57 */
	cr_save_reg_to_autoload(cr, REG_ASPM_FORCE_CTL, 0xFF, 0x30, 0xFF08);
	/* save 0xFE58 */
	cr_save_reg_to_autoload(cr, REG_PM_CLK_FORCE_CTL, 0xFF, 0x12, 0xFF0C);

	if (CHECK_PID(cr, 0x5250)) {
		for (i = 0; i < cfg_cnt; i++)
			cr_save_cfg_to_epcore(cr, 0xFF10 + i * 4, cfg[i], 0);
	}
}

static void rts5249_save_content_to_autoload_space(struct card_reader *cr)
{
	u32 relink_time = 0;
	u16 cfg[] = {0x817, 0x724, 0x725, 0x726, 0x727, 0x814, 0x815, 0x816};
	u8 cfg_cnt = sizeof(cfg) / sizeof(*cfg);


	cr_wr_cfg32(cr, 0x814, 0xFF000000, 0x80000000);

	cr_wr_reg(cr, 0xFF00, 0xFF, ((cfg_cnt + 2) & 0x7F) | 0x80);

	if (cr->option.auto_delink_en)
		relink_time = (cr->option.relink_time >> 4);
	else
		relink_time = 0;
	cr_wr_reg(cr, 0xFF01, 0xFF, (u8)relink_time);
	cr_wr_reg(cr, 0xFF02, 0xFF, (u8)(relink_time >> 8));
	cr_wr_reg(cr, 0xFF03, 0x01, (u8)(relink_time >> 16));

	cr_wr_reg(cr, 0xFF04, 0xFF, (u8)cr->ssvid);
	cr_wr_reg(cr, 0xFF05, 0xFF, (u8)(cr->ssvid >> 8));
	cr_wr_reg(cr, 0xFF06, 0xFF, (u8)cr->ssdid);
	cr_wr_reg(cr, 0xFF07, 0xFF, (u8)(cr->ssdid >> 8));

	/* save 0xFE57 */
	cr_save_reg_to_autoload(cr, REG_ASPM_FORCE_CTL, 0xFF, 0x10, 0xFF08);

	/* save 0xFE8D */
	cr_save_reg_to_autoload(cr, REG_L1SUB_CFG1, 0xFF, 0x00, 0xFF0C);

	if (CHECK_PID(cr, 0x5249)) {
		if (cr->option.use_hw_setting) {
			int i = 0;
			for (i = 0; i < cfg_cnt; i++)
				cr_save_cfg_to_epcore(cr, 0xFF10 + i * 4,
						cfg[i], 0);
		}
	}
}

static void rts52xa_save_content_from_efuse(struct card_reader *cr)
{
	u8 cnt,sv,cv;
	u8 j = 0;
	u8 tmp;
	u8 val;
	int i;

	cr_wr_reg(cr, RTS525A_PME_FORCE_CTL,
		REG_EFUSE_BYPASS | REG_EFUSE_POR, REG_EFUSE_POR);
	udelay(1);

	rts_trace(TRACE_CR, "Enable efuse por!");
	rts_trace(TRACE_CR, "save efuse to autoload");

	cr_wr_reg(cr, RTS525A_EFUSE_ADD, REG_EFUSE_ADD_MASK, 0x00);
	cr_wr_reg(cr, RTS525A_EFUSE_CTL,
		REG_EFUSE_ENABLE | REG_EFUSE_MODE, REG_EFUSE_ENABLE);
	/* Wait transfer end */
	for (j = 0; j < MAX_RW_REG_CNT; j++) {
		cr_rd_reg(cr, RTS525A_EFUSE_CTL, &tmp);
		if ((tmp & 0x80) == 0)
			break;
	}
	cr_rd_reg(cr, RTS525A_EFUSE_DATA, &val);
	cnt = val & 0x0F;
	sv = val & 0x10;
	cv = val & 0x20;

	if (sv) {
		for (i = 0; i < 4; i++) {
			cr_wr_reg(cr, RTS525A_EFUSE_ADD, REG_EFUSE_ADD_MASK, 0x04 + i);
			cr_wr_reg(cr, RTS525A_EFUSE_CTL,
				REG_EFUSE_ENABLE | REG_EFUSE_MODE, REG_EFUSE_ENABLE);
			/* Wait transfer end */
			for (j = 0; j < MAX_RW_REG_CNT; j++) {
				cr_rd_reg(cr, RTS525A_EFUSE_CTL, &tmp);
				if ((tmp & 0x80) == 0)
					break;
			}
			cr_rd_reg(cr, RTS525A_EFUSE_DATA, &val);
			cr_wr_reg(cr, 0xFF04 + i , 0xFF , val);
		}
	} else {
		cr_wr_reg(cr, 0xFF04, 0xFF, (u8)cr->ssvid);
		cr_wr_reg(cr, 0xFF05, 0xFF, (u8)(cr->ssvid >> 8));
		cr_wr_reg(cr, 0xFF06, 0xFF, (u8)cr->ssdid);
		cr_wr_reg(cr, 0xFF07, 0xFF, (u8)(cr->ssdid >> 8));
	}

	for (i = 0; i < cnt*4; i++) {
		if (sv)
			cr_wr_reg(cr, RTS525A_EFUSE_ADD, REG_EFUSE_ADD_MASK, 0x08 + i);
		else
			cr_wr_reg(cr, RTS525A_EFUSE_ADD, REG_EFUSE_ADD_MASK, 0x04 + i);
		cr_wr_reg(cr, RTS525A_EFUSE_CTL,
			REG_EFUSE_ENABLE | REG_EFUSE_MODE, REG_EFUSE_ENABLE);

		/* Wait transfer end */
		for (j = 0; j < MAX_RW_REG_CNT; j++) {
			cr_rd_reg(cr, RTS525A_EFUSE_CTL, &tmp);
			if ((tmp & 0x80) == 0)
				break;
		}
		cr_rd_reg(cr, RTS525A_EFUSE_DATA, &val);
		cr_wr_reg(cr, 0xFF08 + i , 0xFF , val);
	}
	cr_wr_reg(cr, 0xFF00, 0xFF, (cnt & 0x7F) | 0x80);
	cr_wr_reg(cr, RTS525A_PME_FORCE_CTL,
		REG_EFUSE_BYPASS | REG_EFUSE_POR, REG_EFUSE_BYPASS);
	rts_trace(TRACE_CR, "Disable efuse por!");
}

static void rts52xa_save_content_to_autoload_space(struct card_reader *cr)
{
	u32 relink_time = 0;
	u8 val;

	cr_rd_reg(cr, MODE_CONFIG_REG, &val);
	if (val & LOAD_EFUSE_MODE) {
		cr_rd_reg(cr, RTS525A_BIOS_CFG, &val);
		if (val & RTS525A_LOAD_BIOS_FLAG) {
			cr_wr_reg(cr, RTS525A_BIOS_CFG,
				RTS525A_LOAD_BIOS_FLAG, RTS525A_CLEAR_BIOS_FLAG);
			cr_wr_reg(cr, RTS525A_PME_FORCE_CTL,
			REG_EFUSE_POWER_MASK, REG_EFUSE_POWERON);
			rts_trace(TRACE_CR, "Power ON efuse!");
			mdelay(1);
			rts52xa_save_content_from_efuse(cr);
			} else {
			cr_rd_reg(cr, RTS525A_PME_FORCE_CTL, &val);
			if (!(val & 0x08))
				rts52xa_save_content_from_efuse(cr);
			}
	} else {
		rts_trace(TRACE_CR, "Load from autoload");
		cr_wr_reg(cr, 0xFF00, 0xFF, 0x80);
		cr_wr_reg(cr, 0xFF04, 0xFF, (u8)cr->ssvid);
		cr_wr_reg(cr, 0xFF05, 0xFF, (u8)(cr->ssvid >> 8));
		cr_wr_reg(cr, 0xFF06, 0xFF, (u8)cr->ssdid);
		cr_wr_reg(cr, 0xFF07, 0xFF, (u8)(cr->ssdid >> 8));
	}

	cr_wr_cfg32(cr, 0x814, 0xFF000000, 0x80000000);

	if (cr->option.auto_delink_en)
		relink_time = (cr->option.relink_time >> 4);
	else
		relink_time = 0;
	cr_wr_reg(cr, 0xFF01, 0xFF, (u8)relink_time);
	cr_wr_reg(cr, 0xFF02, 0xFF, (u8)(relink_time >> 8));
	cr_wr_reg(cr, 0xFF03, 0x01, (u8)(relink_time >> 16));

}


static void rts52xx_save_content_to_autoload_space(struct card_reader *cr)
{
	if (CHECK_PID(cr, 0x5249))
		rts5249_save_content_to_autoload_space(cr);
	else if (CHECK_PID(cr, 0x5250))
		rts5250_save_content_to_autoload_space(cr);
	else
		rts52xa_save_content_to_autoload_space(cr);
}

static int rts5249_init_from_hw(struct card_reader *cr)
{
	struct cr_option *option = &(cr->option);
	u32 lval;
	u8 val;

	if (CHECK_VERSION(cr, 0x5249, IC_VER_A))
		clear_dev_flag(cr, LTR_L1SS_PWR_GATE_EN);

	if (option->hp_watch_bios_hotplug && option->auto_delink_en) {
		u8 reg58, reg5b;
		if (cr_pci_rd_cfg_byte(cr, 0x00, 0x1C, 0x02, 0x58, &reg58) < 0)
			return 0;
		if (cr_pci_rd_cfg_byte(cr, 0x00, 0x1C, 0x02, 0x5B, &reg5b) < 0)
			return 0;

		rts_trace(TRACE_PROBE, "reg58 = 0x%x, reg5b = 0x%x\n", reg58,
			reg5b);

		/* BIOS don't support hotplug, so we disable auto_delink here */
		if ((reg58 == 0x00) && (reg5b == 0x01))
			option->auto_delink_en = 0;
	}

	if (!(option->use_hw_setting))
		return 0;

	cr_rd_cfg32(cr, 0x724, &lval);
	if (lval & (1 << 24)) {
		if (!CHECK_VERSION(cr, 0x5249, IC_VER_A)
			&& (check_dev_flag(cr, ASPM_L1_1_EN | ASPM_L1_2_EN
						| PM_L1_1_EN | PM_L1_2_EN)))
			option->force_clkreq_0 = 0;

		return 0;
	}
	cr_init_generic_options_from_hw(cr);
	config_sd30_drive_from_hw(cr);

	cr_rd_cfg32(cr, 0x814, &lval);

	option->rtd3_en = !!(lval & 0x04);

	if (!(check_dev_flag(cr, IGNORE_HW_SETTING_SD40_CAP))) {
		u32 sd_capability = cr_get_sd_capability_sw(cr);
		u8 card_spt_map = cr_get_card_spt_map_sw(cr);

		val = (u8) lval;
		if (val & 0x01) {
			card_spt_map |= 1 << SD_ID;
			sd_capability |= SD_CAP_UHSII;
		} else {
			sd_capability &= ~SD_CAP_UHSII;
			if (0 == sd_capability)
				card_spt_map &= ~(1 << SD_ID);
		}

		cr_set_card_spt_map_sw(cr, card_spt_map);
		cr_set_sd_capability_sw(cr, sd_capability);
	}


	/* 0x815[6]: indicate SD_CD/WP if reverse */
	val = (u8) (lval >> 8);
	if ((1 << 6) & val)
		set_dev_flag(cr, SD_CD_WP_REVERSE_EN);

	/* 0x816*/
	val = (u8) (lval >> 16);
	if ((1 << 5) & val)
		set_sd40_flag(cr, SD40_LOW_PWR_MODE);
	if ((val & 0xC0) == 0x00) {
		set_sd40_flag(cr, SD40_SUPPORT_HIBERNATE);
		clear_sd40_flag(cr, SD40_IGNORE_ASPM);
	} else if ((val & 0xC0) == 0x40) {
		clear_sd40_flag(cr, SD40_SUPPORT_HIBERNATE);
		set_sd40_flag(cr, SD40_IGNORE_ASPM);
	} else if (((val & 0xC0) == 0x80) && (!option->rtd3_en))
		set_dev_flag(cr, POWEROFF_UHSII_AT_IDLE);

	/* 0x816[2]: indicate free clkreq at L1/RTD3/Delink */
	if (CHECK_PID(cr, 0x524A)
		|| CHECK_PID(cr, 0x525A)) {
		option->force_clkreq_0 = !((1 << 2) & val);
	}

	if (!CHECK_VERSION(cr, 0x5249, IC_VER_A)
		&& (check_dev_flag(cr, ASPM_L1_1_EN | ASPM_L1_2_EN
					| PM_L1_1_EN | PM_L1_2_EN)))
		option->force_clkreq_0 = 0;

	return 0;
}

static int rts5250_init_phy(struct card_reader *cr)
{
	cr_wr_phy16(cr, 0x1D, cr->hw_param.phy0x1d_val);
	cr_wr_phy16(cr, 0x03, cr->hw_param.phy0x03_val);
	if (CHECK_VERSION(cr, 0x525A, IC_VER_A))
		cr_wr_phy16(cr, 0x19, cr->hw_param.phy0x19_val);
	return 0;
}

static int rts5249_init_phy(struct card_reader *cr)
{
	if (cr_wr_phy16(cr, 0x19, cr->hw_param.phy0x19_val))
		TRACE_RET(cr, STATUS_FAIL);
	msleep(1);

	if (cr_wr_phy16(cr, 0x0A, 0x05C0))
		TRACE_RET(cr, STATUS_FAIL);

	if (cr->option.phy_optimize) {
		if (cr_wr_phy16(cr, 0x00, cr->hw_param.phy0x00_val))
			TRACE_RET(cr, STATUS_FAIL);

		if (cr_wr_phy16(cr, 0x03, cr->hw_param.phy0x03_val))
			TRACE_RET(cr, STATUS_FAIL);
	}

	cr_wr_phy16(cr, 0x1E, cr->hw_param.phy0x1e_val);
	cr_wr_phy16(cr, 0x05, cr->hw_param.phy0x05_val);
	cr_wr_phy16(cr, 0x02, cr->hw_param.phy0x02_val);
	cr_wr_phy16(cr, 0x1D, cr->hw_param.phy0x1d_val);
	cr_wr_phy16(cr, 0x08, cr->hw_param.phy0x08_val);

	return 0;
}

static int rts524a_init_phy(struct card_reader *cr)
{

	if (cr->option.phy_optimize) {
		if (cr_wr_phy16(cr, 0x00, cr->hw_param.phy0x00_val))
			TRACE_RET(cr, STATUS_FAIL);

		if (cr_wr_phy16(cr, 0x03, cr->hw_param.phy0x03_val))
			TRACE_RET(cr, STATUS_FAIL);
	}
	if (CHECK_VERSION(cr, 0x524A, IC_VER_A)) {
		if (cr_wr_phy16(cr, 0x03, cr->hw_param.phy0x03_val))
			TRACE_RET(cr, STATUS_FAIL);

		/*For FLD*/
		cr_wr_phy16(cr, 0x02, cr->hw_param.phy0x02_val);
		cr_wr_phy16(cr, 0x1A, cr->hw_param.phy0x1a_val);
		cr_wr_phy16(cr, 0x1D, cr->hw_param.phy0x1d_val);

		/*For BER*/
		cr_wr_phy16(cr, 0x1E, cr->hw_param.phy0x1e_val);
	}
	cr_wr_phy16(cr, 0x08, cr->hw_param.phy0x08_val);

	return 0;
}

static int rts52xx_init_phy(struct card_reader *cr)
{
	if (CHECK_PID(cr, 0x5249))
		return rts5249_init_phy(cr);
	else if (CHECK_PID(cr, 0x524A))
		return rts524a_init_phy(cr);
	else if (CHECK_PID(cr, 0x5250) || CHECK_PID(cr, 0x525A))
		return rts5250_init_phy(cr);
	else
		return -1;
}

static inline int rts5249_init_led(struct card_reader *cr)
{
	/* Configure LED */
	if (check_dev_flag(cr, TURN_ON_LED_AT_START))
		cr_turn_on_led(cr);
	else
		cr_turn_off_led(cr);

	/* LED shine disabled, set initial shine cycle period */
	cr_wr_reg(cr, REG_OLT_LED_CTL, LED_SHINE_EN | LED_SPEED_MASK, 0x02);

	return 0;
}

static int rts5249_init_aspm(struct card_reader *cr)
{
	struct cr_option *option = &(cr->option);

	if (check_dev_flag(cr, FORCE_HOST_ASPM_EN)) {
		cr->host_aspm_val = option->force_host_aspm_val;
	} else {
		cr_get_host_aspm(cr, &cr->host_aspm_val);
		cr->host_aspm_val &= 0xCF;
	}

	if (option->config_host_aspm)
		cr_disable_host_aspm(cr);

	cr_wr_reg(cr, REG_ASPM_FORCE_CTL, 0xFF, 0);

	if (DEV_ASPM_DYNAMIC == option->dev_aspm_mode) {
		int lctlr = cr->dev_pci_cap_offset + PCI_EXP_LNKCTL;
		u8 val = 0;
		rts_trace(TRACE_CR, "Disable Device dynamic ASPM\n");
		cr_read_config8(cr, lctlr, &val);
		val &= 0xFC;
		cr_write_config8(cr, lctlr, val);
	} else if (DEV_ASPM_BACKDOOR == option->dev_aspm_mode) {
		u8 mask = FORCE_ASPM_CTL0 | FORCE_ASPM_VAL_MASK;
		u8 val = FORCE_ASPM_CTL0;
		rts_trace(TRACE_CR, "Disable Device backdoor ASPM\n");
		cr_wr_reg(cr, REG_ASPM_FORCE_CTL, mask, val);
	}

	return 0;
}

static int rts5249_force_power_on(struct card_reader *cr, u8 ctl)
{
	u8 mask = 0;

	if (ctl & SSC_PDCTL)
		mask |= SSC_POWER_DOWN;

	if (cr->option.ocp_en) {
		if (ctl & OC_PDCTL)
			mask |= SD_OC_POWER_DOWN;
	}

	if (mask) {
		if (cr_wr_reg(cr, REG_FPDCTL, mask, 0))
			TRACE_RET(cr, STATUS_FAIL);
	}

	return 0;
}

static int rts5249_force_power_down(struct card_reader *cr, u8 ctl)
{
	u8 mask = 0;

	if (ctl & SSC_PDCTL)
		mask |= SSC_POWER_DOWN;

	if (cr->option.ocp_en) {
		if (ctl & OC_PDCTL)
			mask |= SD_OC_POWER_DOWN;
	}

	if (mask) {
		if (cr_wr_reg(cr, REG_FPDCTL, mask, mask))
			TRACE_RET(cr, STATUS_FAIL);
	}

	return 0;
}

static void rts5249_extra_action_4_aspm(struct card_reader *cr)
{
	struct cr_option *option = &(cr->option);

	if (DEV_ASPM_BACKDOOR == option->dev_aspm_mode) {
		int lctlr = cr->dev_pci_cap_offset + PCI_EXP_LNKCTL;
		u8 val = 0;
		cr_read_config8(cr, lctlr, &val);
		val &= 0xFC;
		val |= 0x02;
		cr_write_config8(cr, lctlr, val);
	}

	if ((!option->config_host_aspm)
		&& check_dev_flag(cr, FORCE_HOST_ASPM_EN))
			cr_set_host_aspm(cr, cr->host_aspm_val);
}

static int rts5249_init_hw(struct card_reader *cr)
{
	struct cr_option *option = &(cr->option);
	int lctlr = cr->dev_pci_cap_offset + PCI_EXP_LNKCTL;

	if (cr->asic_code)
		rts52xx_init_phy(cr);

	cr_wr_reg(cr, REG_HOST_SLEEP_STATE, 0x03, 0x00);
	cr_wr_reg(cr, REG_CARD_CLK_EN, 0x1E, 0);

	if (option->mem_pwr_cut_en && CHECK_PID(cr, 0x525A))
		cr_wr_reg(cr, RTS5250_LDO_CFG3, RTS525A_CFG_MEM_PD, RTS525A_CFG_MEM_PD);

	cr_init_ocp(cr);

	if (cr->asic_code) {
		if (CHECK_NOT_VERSION(cr, 0x5249, IC_VER_A) ||
			CHECK_PID(cr, 0x524A)) {
			u16 val;
			cr_rd_phy16(cr, 0x11, &val);
			val |= 1;
			cr_wr_phy16(cr, 0x11, val);
		}
	}

	if (check_dev_flag(cr, CLEAR_0xFE8D_BIT0_EN)) {
		cr_wr_reg(cr, REG_L1SUB_CFG1,
			AUX_CLK_ACTIVE_SEL_MASK, MAC_CKSW_DONE);
	}

	rts5249_init_led(cr);

	cr_wr_reg(cr, REG_L1SUB_CFG3, 0xFF, 0);

	/* Reset delink mode */
	cr_wr_reg(cr, REG_CHANGE_LINK_STATE, 0x0A, 0);

	/* Card driving select */
	cr_wr_reg(cr, REG_CARD_DRIVE_SEL, 0xFF, option->card_drive_sel);
	if (sd_sd30_drive_sel(cr, SD_IO_3V3))
		TRACE_RET(cr, STATUS_FAIL);

	if (cr->asic_code) {
		/* Enable SSC Clock */
		cr_wr_reg(cr, REG_SSC_CTL1, 0xFF, SSC_8X_EN | SSC_SEL_4M);
		cr_wr_reg(cr, REG_SSC_CTL2, 0xFF, 0x12);
	}

	cr_wr_reg(cr, REG_CHANGE_LINK_STATE, 0x16, 0x10);

	if (CHECK_PID(cr, 0x5250) ||
		CHECK_PID(cr, 0x524A) ||
		CHECK_PID(cr, 0x525A))
		cr_wr_reg(cr, REG_PM_CLK_FORCE_CTL, CLK_PM_EN, CLK_PM_EN);

	/* Enable clk_request_n to enable clock power management */
	if (cr_write_config8(cr, lctlr + 1, 1))
		TRACE_RET(cr, STATUS_FAIL);

	/* Enter L1 when host tx idle
	 * Set 0x70F to 0x5B using backdoor
	 */
	if (cr_wr_cfg32(cr, 0x70C, 0xFF000000, 0x5B000000))
		TRACE_RET(cr, STATUS_FAIL);

	/* Clear Link Ready Interrupt
	 * Write 1 to clear */
	cr_wr_reg(cr, REG_IRQSTAT0, LINK_RDY_INT, LINK_RDY_INT);

	cr_wr_reg(cr, REG_PERST_GLITCH_WIDTH, 0xFF, 0x80);

	/* Enable power gating for autodelink */
	/* disable then 100MHz clock cann't be closed when L1 */
	cr_wr_reg(cr, REG_PWD_SUSPEND_EN, 0xFF, cr->option.reg_0xfe76);
	cr_wr_reg(cr, REG_PWR_GATE_CTRL, PWR_GATE_EN, PWR_GATE_EN);
	if (CHECK_PID(cr, 0x5250) || CHECK_PID(cr, 0x525A) ||
		CHECK_PID(cr, 0x524A)) {
		cr_wr_reg(cr, REG_VREF, PWD_SUSPND_EN, PWD_SUSPND_EN);
	}

	if (check_dev_flag(cr, SD_CD_WP_REVERSE_EN)) {
		cr_wr_reg(cr, 0xFF03, 0x30, 0x30);
		msleep(20);
	}

	cr_enable_bus_int(cr, cr->hw_param.interrupt_en);

	/* Set interrupt write clear
	 * bit 1: U_elbi_if_rd_clr_en
	 *	1: Enable ELBI interrupt[31:22] & [7:0] flag read clear
	 *	0: ELBI interrupt flag[31:22] & [7:0] only can be write clear */
	if (option->write_clear_interrupt)
		cr_wr_reg(cr, REG_NFTS_TX_CTRL, 0x02, 0);

	cr_wr_reg(cr, REG_RBCTL, U_AUTO_DMA_EN, 0);

	cr_check_card_in_init(cr);

	/* Update RC oscillator to 400k
	 * bit[0] F_HIGH: for RC oscillator, Rst_value is 1'b1
	 *		1: 2M  0: 400k */
	cr_wr_reg(cr, REG_RCCTL, 0x01, 0x00);

	if ((option->remote_wakeup_en ||
			(option->rtd3_en && cr->d3cold_supported)) &&
		!option->auto_delink_en) {
		if (CHECK_PID(cr, 0x5250) ||
			CHECK_PID(cr, 0x524A) ||
			CHECK_PID(cr, 0x525A))
			cr_wr_reg(cr, RTS5250_AUTOLOAD_CFG3, 0x01, 0x01);
		else
			cr_wr_reg(cr, AUTOLOAD_CFG3, 0x01, 0x01);
		if (option->aux_pwr_exist) {
			if (CHECK_PID(cr, 0x5250)) {
				cr_wr_reg(cr, RTS5250_PME_FORCE_CTL, 0x30,
					0x30);
			} else {
				if (CHECK_PID(cr, 0x524A) || CHECK_PID(cr, 0x525A))
					cr_wr_reg(cr, RTS525A_PME_FORCE_CTL,
						0x30, 0x30);
				else
					cr_wr_reg(cr, REG_PME_FORCE_CTL, 0xFF,
						0x33);
			}
		}
	} else {
		if (CHECK_PID(cr, 0x5250)) {
			cr_wr_reg(cr, RTS5250_PME_FORCE_CTL, 0x30, 0x20);
			cr_wr_reg(cr, RTS5250_AUTOLOAD_CFG3, 0x01, 0x00);
		} else if (CHECK_PID(cr, 0x524A) || CHECK_PID(cr, 0x525A)) {
			cr_wr_reg(cr, RTS5250_AUTOLOAD_CFG3, 0x01, 0x00);
			cr_wr_reg(cr, RTS525A_PME_FORCE_CTL, 0x30, 0x20);
		} else {
			cr_wr_reg(cr, REG_PME_FORCE_CTL, 0xFF, 0x30);
			cr_wr_reg(cr, AUTOLOAD_CFG3, 0x01, 0x00);
		}
	}

	if (CHECK_PID(cr, 0x5250) || CHECK_PID(cr, 0x524A)
			|| CHECK_PID(cr, 0x525A))
		cr_wr_reg(cr, RTS5250_AUTOLOAD_CFG3, U_PM_D3_EN, 0x00);
	else
		cr_wr_reg(cr, AUTOLOAD_CFG3, U_PM_D3_EN, 0x00);


	/* u_force_clkreq_0
	 * If 1, CLKREQ# PIN will be forced to drive 0 to request clock */
	if (option->force_clkreq_0)
		cr_wr_reg(cr, REG_PETXCFG, 0x80, 0x80);
	else
		cr_wr_reg(cr, REG_PETXCFG, 0x80, 0x00);

	if (DEV_ASPM_STATIC == option->dev_aspm_mode) {
		u8 val = 0;
		if (cr_read_config8(cr, lctlr, &val))
			TRACE_RET(cr, STATUS_FAIL);
		val &= 0xFC;
		val |= (option->dev_aspm_val & 0x03);
		if ((cr_write_config8(cr, lctlr, val)))
			TRACE_RET(cr, STATUS_FAIL);
		cr->aspm_level[0] = option->dev_aspm_val;
		cr->aspm_enabled = 1;
		if (option->config_host_aspm)
			cr_set_host_aspm(cr, cr->host_aspm_val);
	} else if (DEV_ASPM_DISABLE == option->dev_aspm_mode) {
		u8 val = 0;
		if (cr_read_config8(cr, lctlr, &val))
			TRACE_RET(cr, STATUS_FAIL);
		val &= 0xFC;
		if ((cr_write_config8(cr, lctlr, val)))
			TRACE_RET(cr, STATUS_FAIL);
	}

	if (option->obff_en)
		cr_wr_reg(cr, REG_OBFF_CFG, 0x03, 0x03);
	else
		cr_wr_reg(cr, REG_OBFF_CFG, 0x03, 0x00);

	/* Switch LDO3318 source from DV33 to card_3v3 */
	if (cr->asic_code) {
		cr_wr_reg(cr, REG_LDO_SEL, LDO3318_SRC_SEL_MASK,
			LDO3318_SRC_NONE);
		cr_wr_reg(cr, REG_LDO_SEL, LDO3318_SRC_SEL_MASK,
			LDO3318_SRC_PMOS);
	} else {
		cr_wr_reg(cr, REG_LDO_SEL, 0x01, 0x00);
	}

	/* HW will modify default value in VerB */
	if (CHECK_PID(cr, 0x5250)) {
		cr_wr_reg(cr, RTS5249_SD40_LDO_CTL, SD40_LDO12_MASK,
			SD40_LDO_AV12S_1_15V);
		cr_wr_reg(cr, RTS5250_LDO_CFG3, REG_TUNE_DV12_MASK,
			REG_TUNE_DV12_1_15V);

	}

	if (check_dev_flag(cr, EXTRA_ACTION_FOR_ASPM_EN))
		rts5249_extra_action_4_aspm(cr);

	if (cr->asic_code &&
		(CHECK_PID(cr, 0x5250) || CHECK_PID(cr, 0x524A)
		|| CHECK_PID(cr, 0x525A))) {
		u16 val = 0;

		cr_rd_phy16(cr, 0x8B, &val);
		val &= ~(1 << 3);
		cr_wr_phy16(cr, 0x8B, val);
	}

	cr->driver_first_load = 0;

	return 0;
}

static int rts5249_sd40_phy_test(struct card_reader *cr)
{

	rts_trace(TRACE_PROBE, "*****SD40 PHY Test Mode**********\n");
	rts_trace(TRACE_PROBE, "******Patch PHY ***********\n");

	cr_wr_phy16(cr, 0x86, 0x554f);
	cr_wr_phy16(cr, 0x90, 0x09A8);
	cr_wr_phy16(cr, 0x91, 0x0041);

	cr_wr_phy16(cr, 0x87, 0x7C63);
	if (cr_wr_phy16(cr, 0x08, 0x4FE4)) {
		rts_trace(TRACE_PROBE, "******PHY config fail***********\n");
		TRACE_RET(cr, STATUS_FAIL);
	}

	rts5249_force_power_on(cr, SSC_PDCTL | OC_PDCTL);

	if (!(CHECK_PID(cr, 0x525A)))
		cr_wr_reg(cr, RTS5249_SD40_LDO_CTL, 0xFF, 0xEB);
	cr_wr_reg(cr, REG_CARD_PULL_CTL2, 0xFF, 0x50);
	cr_wr_reg(cr, REG_CARD_PWR_CTL, RTS5249_SD40_PAD_ISO,
					RTS5249_SD40_PAD_ISO);
	cr_wr_reg(cr, REG_CARD_CLK_EN, (SD40_CLK_EN | SD_CLK_EN), SD40_CLK_EN);
	cr_wr_reg(cr, REG_CARD_SELECT, CARD_MOD_MASK, SD40_MOD_SEL);
	cr_wr_reg(cr, REG_SD40_CTRL, 0xFF, (SD40_PHY_RST | SD40_PHY_ISO));
	cr_wr_reg(cr, RTS5249_SD40_MISC, SCRAM_DIS_EN, SCRAM_DIS_EN);
	cr_wr_reg(cr, REG_SD40_TMOUT_1, 0xFF, 0);

	msleep(100);
	cr_wr_reg(cr, REG_CLK_CTL, 0xFF, RTS5249_CHANGE_SD40_PCLK);
	msleep(10);

	rts_trace(TRACE_PROBE, "******PHY Test mode***********\n");
	cr_wr_reg(cr, REG_SD40_CTRL, 0xFF, (SD40_PHY_RST | SD40_PHY_ISO));

	return 0;
}

static void rts5249_save_cfg_to_reg(struct card_reader *cr)
{
	u8 val;

	if (check_dev_flag(cr, LTR_OBFF_SETBACK_EN)) {
		cr_read_config8(cr, 0x99, &val);
		cr_wr_reg(cr, 0xFF30, 0xFF, val);
	}

	if (check_dev_flag(cr, LTR_EXT_SETBACK_EN)) {
		u32 lval;
		cr_read_config32(cr, 0x15C, &lval);
		cr_wr_reg(cr, 0xFF31, 0xFF, (u8)lval);
		cr_wr_reg(cr, 0xFF32, 0xFF, (u8)(lval >> 8));
		cr_wr_reg(cr, 0xFF33, 0xFF, (u8)(lval >> 16));
		cr_wr_reg(cr, 0xFF34, 0xFF, (u8)(lval >> 24));
	}

	if (check_dev_flag(cr, L1SS_SETBACK_EN)) {
		cr_read_config8(cr, 0x16C, &val);
		cr_wr_reg(cr, 0xFF35, 0xFF, val);

		cr_read_config8(cr, 0x16B, &val);
		cr_wr_reg(cr, 0xFF36, 0xFF, val);

		cr_read_config8(cr, 0x16A, &val);
		cr_wr_reg(cr, 0xFF37, 0xFF, val);

		cr_read_config8(cr, 0x169, &val);
		cr_wr_reg(cr, 0xFF38, 0xFF, val);

		cr_read_config8(cr, 0x168, &val);
		cr_wr_reg(cr, 0xFF39, 0xFF, val);
	}
}

static void rts5249_store_cfg_from_reg(struct card_reader *cr)
{
	u8 val;

	if (check_dev_flag(cr, LTR_OBFF_SETBACK_EN)) {
		cr_rd_reg(cr, 0xFF30, &val);
		cr_write_config8(cr, 0x99, val);
	}

	if (check_dev_flag(cr, LTR_EXT_SETBACK_EN)) {
		cr_rd_reg(cr, 0xFF31, &val);
		cr_write_config8(cr, 0x15C, val);
		cr_rd_reg(cr, 0xFF32, &val);
		cr_write_config8(cr, 0x15D, val);
		cr_rd_reg(cr, 0xFF33, &val);
		cr_write_config8(cr, 0x15E, val);
		cr_rd_reg(cr, 0xFF34, &val);
		cr_write_config8(cr, 0x15F, val);
	}

	if (check_dev_flag(cr, L1SS_SETBACK_EN)) {
		cr_rd_reg(cr, 0xFF35, &val);
		cr_write_config8(cr, 0x16C, val);

		cr_rd_reg(cr, 0xFF36, &val);
		cr_write_config8(cr, 0x16B, val);

		cr_rd_reg(cr, 0xFF37, &val);
		cr_write_config8(cr, 0x16A, val);

		cr_rd_reg(cr, 0xFF38, &val);
		cr_write_config8(cr, 0x169, val);

		cr_rd_reg(cr, 0xFF39, &val);
		cr_write_config8(cr, 0x168, val);
	}

}

static void rts524A_save_cfg_to_reg(struct card_reader *cr)
{
	u8 val;

	if (check_dev_flag(cr, LTR_OBFF_SETBACK_EN)) {
		cr_read_config8(cr, 0x99, &val);
		cr_wr_reg(cr, 0xFF30, 0xFF, val);
	}

	if (check_dev_flag(cr, LTR_EXT_SETBACK_EN)) {
		u32 lval;
		cr_read_config32(cr, 0x154, &lval);
		cr_wr_reg(cr, 0xFF31, 0xFF, (u8)lval);
		cr_wr_reg(cr, 0xFF32, 0xFF, (u8)(lval >> 8));
		cr_wr_reg(cr, 0xFF33, 0xFF, (u8)(lval >> 16));
		cr_wr_reg(cr, 0xFF34, 0xFF, (u8)(lval >> 24));
	}

	if (check_dev_flag(cr, L1SS_SETBACK_EN)) {
		cr_read_config8(cr, 0x164, &val);
		cr_wr_reg(cr, 0xFF35, 0xFF, val);

		cr_read_config8(cr, 0x163, &val);
		cr_wr_reg(cr, 0xFF36, 0xFF, val);

		cr_read_config8(cr, 0x162, &val);
		cr_wr_reg(cr, 0xFF37, 0xFF, val);

		cr_read_config8(cr, 0x161, &val);
		cr_wr_reg(cr, 0xFF2E, 0xFF, val);

		cr_read_config8(cr, 0x160, &val);
		cr_wr_reg(cr, 0xFF2F, 0xFF, val);
	}
}

static void rts524A_store_cfg_from_reg(struct card_reader *cr)
{
	u8 val;

	if (check_dev_flag(cr, LTR_OBFF_SETBACK_EN)) {
		cr_rd_reg(cr, 0xFF30, &val);
		cr_write_config8(cr, 0x99, val);
	}

	if (check_dev_flag(cr, LTR_EXT_SETBACK_EN)) {
		cr_rd_reg(cr, 0xFF31, &val);
		cr_write_config8(cr, 0x154, val);
		cr_rd_reg(cr, 0xFF32, &val);
		cr_write_config8(cr, 0x155, val);
		cr_rd_reg(cr, 0xFF33, &val);
		cr_write_config8(cr, 0x156, val);
		cr_rd_reg(cr, 0xFF34, &val);
		cr_write_config8(cr, 0x157, val);
	}

	if (check_dev_flag(cr, L1SS_SETBACK_EN)) {
		cr_rd_reg(cr, 0xFF35, &val);
		cr_write_config8(cr, 0x164, val);

		cr_rd_reg(cr, 0xFF36, &val);
		cr_write_config8(cr, 0x163, val);

		cr_rd_reg(cr, 0xFF37, &val);
		cr_write_config8(cr, 0x162, val);

		cr_rd_reg(cr, 0xFF2E, &val);
		cr_write_config8(cr, 0x161, val);

		cr_rd_reg(cr, 0xFF2F, &val);
		cr_write_config8(cr, 0x160, val);
	}

}


static void rts5249_swap_cfg_with_reg_4_D3(struct card_reader *cr)
{
	u8 had_delinked;

	if (CHECK_PID(cr, 0x524A)) {
		cr_rd_reg(cr, 0xFF2C, &had_delinked);
		had_delinked &= (1 << 0);

		/* set 0xFF2C[0] to indicate device had been probed, sometime
		 * device will auto delink or system enter S3
		 */
		cr_wr_reg(cr, 0xFF2C, 1 << 0, 1 << 0);

		if (had_delinked)
			rts524A_store_cfg_from_reg(cr);
		else
			rts524A_save_cfg_to_reg(cr);
	} else {
		cr_rd_reg(cr, 0xFF3C, &had_delinked);
		had_delinked &= (1 << 0);

		/* set 0xFF3C[0] to indicate device had been probed, sometime
		 * device will auto delink or system enter S3
		 */
		cr_wr_reg(cr, 0xFF3C, 1 << 0, 1 << 0);
		if (had_delinked)
			rts5249_store_cfg_from_reg(cr);
		else
			rts5249_save_cfg_to_reg(cr);
	}
}

static void rts5249_save_cfg_to_reg_4_RTD3(struct card_reader *cr)
{
	u8 val = 0;

	cr_read_config8(cr, 0x16C, &val);
	cr_wr_reg(cr, 0xFF35, 0xFF, val);

	cr_read_config8(cr, 0x16B, &val);
	cr_wr_reg(cr, 0xFF36, 0xFF, val);

	cr_read_config8(cr, 0x16A, &val);
	cr_wr_reg(cr, 0xFF37, 0xFF, val);

	cr_read_config8(cr, 0x169, &val);
	cr_wr_reg(cr, 0xFF38, 0xFF, val);

	cr_read_config8(cr, 0x168, &val);
	cr_wr_reg(cr, 0xFF39, 0xFF, val);
}

static void rts5249_store_cfg_from_reg_4_RTD3(struct card_reader *cr)
{
	u8 val = 0;

	rts_trace(TRACE_CR, "Resume From RTD3\n");
	rts_trace(TRACE_CR, "L1SS_SETBACK_FOR_INTEL_EN is set,"
			" Restore to CFG\n");

	cr_rd_reg(cr, 0xFF35, &val);
	cr_write_config8(cr, 0x16C, val);

	cr_rd_reg(cr, 0xFF36, &val);
	cr_write_config8(cr, 0x16B, val);

	cr_rd_reg(cr, 0xFF37, &val);
	cr_write_config8(cr, 0x16A, val);

	cr_rd_reg(cr, 0xFF38, &val);
	cr_write_config8(cr, 0x169, val);

	cr_rd_reg(cr, 0xFF39, &val);
	cr_write_config8(cr, 0x168, val);
}

static void rts524A_save_cfg_to_reg_4_RTD3(struct card_reader *cr)
{
	u8 val = 0;

	cr_read_config8(cr, 0x164, &val);
	cr_wr_reg(cr, 0xFF35, 0xFF, val);

	cr_read_config8(cr, 0x163, &val);
	cr_wr_reg(cr, 0xFF36, 0xFF, val);

	cr_read_config8(cr, 0x162, &val);
	cr_wr_reg(cr, 0xFF37, 0xFF, val);

	cr_read_config8(cr, 0x161, &val);
	cr_wr_reg(cr, 0xFF2E, 0xFF, val);

	cr_read_config8(cr, 0x160, &val);
	cr_wr_reg(cr, 0xFF2F, 0xFF, val);
}

static void rts524A_store_cfg_from_reg_4_RTD3(struct card_reader *cr)
{
	u8 val = 0;

	rts_trace(TRACE_CR, "Resume From RTD3\n");
	rts_trace(TRACE_CR,
		"L1SS_SETBACK_FOR_INTEL_EN is set, restore CFG from register\n");

	cr_rd_reg(cr, 0xFF35, &val);
	cr_write_config8(cr, 0x164, val);

	cr_rd_reg(cr, 0xFF36, &val);
	cr_write_config8(cr, 0x163, val);

	cr_rd_reg(cr, 0xFF37, &val);
	cr_write_config8(cr, 0x162, val);

	cr_rd_reg(cr, 0xFF2E, &val);
	cr_write_config8(cr, 0x161, val);

	cr_rd_reg(cr, 0xFF2F, &val);
	cr_write_config8(cr, 0x160, val);
}

static void rts5249_config_packet_cap(struct card_reader *cr)
{
	if (CHECK_VERSION_LOWER_THAN(cr, 0x5250, IC_VER_C)) {
		u8 val;

		cr_rd_reg(cr, MODE_CONFIG_REG, &val);

		/* MS_8BIT and MMC_8BIT  was not supported by 32pin packet */
		if (!(val & (1 << 3))) {
			u32 sd_capability = cr_get_sd_capability_sw(cr);

			cr->option.support_ms_8bit = 0;
			sd_capability &= ~(SD_CAP_MMC_8BIT);
			cr_set_sd_capability_sw(cr, sd_capability);
			if (!sd_capability) {
				u8 card_spt_map = cr_get_card_spt_map_sw(cr);

				card_spt_map &= ~(1 << SD_ID);
				cr_set_card_spt_map_sw(cr, card_spt_map);
			}
		}
	}
}

static void rts5249_set_ic_version(struct card_reader *cr)
{
	u8 val;

	cr_rd_reg(cr, 0xFE90, &val);
	if ((val & 0x10) == 0)
		cr->asic_code = 1;
	else
		cr->asic_code = 0;

	cr->ic_version = val & 0x0F;
}

static void rts525a_update_max_payload(struct card_reader *cr)
{
	u8 host_devctl = 0;
	u8 dev_devctl = 0;

	cr_get_host_devctl(cr, &host_devctl);
	cr_get_dev_devctl(cr, &dev_devctl);
	rts_trace(TRACE_CR, "Host DevCtl=%#x\n", host_devctl);
	rts_trace(TRACE_CR, "Device DevCtl=%#x\n", dev_devctl);
	if ((dev_devctl & PCI_EXP_MAX_PAYLOAD) !=
		(host_devctl & PCI_EXP_MAX_PAYLOAD)) {
		rts_trace(TRACE_CR, "reset dev max payload to %#x\n",
			(host_devctl & PCI_EXP_MAX_PAYLOAD)>>5);
		cr_set_dev_devctl(cr, (dev_devctl & ~PCI_EXP_MAX_PAYLOAD)
				| (host_devctl & PCI_EXP_MAX_PAYLOAD));
	}
}

static int rts5249_init_cr(struct card_reader *cr)
{
	struct cr_option *option = &(cr->option);
	struct rts5249_info *info = cr->extra;

	rts_trace(TRACE_PROBE, "VID: 0x%04x, PID: 0x%04x, SSVID: 0x%04x,"
			"SSDID: 0x%04x\n", cr->vendor_id, cr->product_id,
						cr->ssvid, cr->ssdid);
	rts5249_set_ic_version(cr);

	cr->dev_pci_cap_offset = cr_find_pci_capability(cr, PCI_CAP_ID_EXP);
	cr->host_pci_cap_offset =
		cr_find_host_pci_capability(cr, PCI_CAP_ID_EXP);
	cr_read_host_config8(cr->tr, (u16)(cr->host_pci_cap_offset + 0x30),
			&cr->host_pci_speed_cap);
	cr_write_config8(cr, 0x70c, cr->option.ack_every_tlp);

	if(CHECK_PID(cr, 0x524A)) {
		cr_wr_reg(cr, REG_L1SUB_CFG0, 0x0F, 0x06);
		cr_update_phy16(cr, 0x01, 0xFFDF, 0);
	}

	rts5249_init_aspm(cr);

	if (CHECK_PID(cr, 0x5250)) {

		rts_trace(TRACE_CR, "switch to SD BUS\n");
		cr_wr_reg(cr, RTS5250_AUTOLOAD_CFG4, 0x60, 0x00);
	}

	if (CHECK_PID(cr, 0x525A)) {
		cr_update_phy16(cr, 0x25, 0x0FFF, 0x7000);
		cr_update_phy16(cr, 0x1B, 0xE3FF, 0);
		rts525a_update_max_payload(cr);
	}

	if (CHECK_VERSION_HIGHER_THAN(cr, 0x5250, IC_VER_D)) {
		u8 kcfg = 0;

		cr_rd_reg(cr, RTS5250_PCIE_OFFSET_K_CFG, &kcfg);
		if (!(kcfg & RTS5250_PCIE_OFFSET_K_DIS)) {
			u8 mode_cfg = 0;

			cr_rd_reg(cr, MODE_CONFIG_REG, &mode_cfg);
			if (mode_cfg & RX_AMP_OFFSET_AUTOK_EN) {
				u16 phy_val = 0;

				cr_rd_phy16(cr, 0x1F, &phy_val);
				kcfg = (u8) ((phy_val & 0x01E0) >> 5);
				rts_trace(TRACE_CR,
					"PCIE auto_k value = %#x\n", kcfg);

				cr_wr_reg(cr, RTS5250_PCIE_OFFSET_K_CFG,
					RTS5250_PCIE_OFFSET_K_MASK, kcfg);
				cr_wr_reg(cr, RTS5250_PCIE_OFFSET_K_CFG,
					RTS5250_PCIE_OFFSET_K_DIS,
					RTS5250_PCIE_OFFSET_K_DIS);
			}
		}
	}

	if (option->save_oobs_autoK_en &&
		CHECK_VERSION(cr, 0x524A, IC_VER_A)) {
		u16 cfg;
		u8 val;

		cr_rd_phy16(cr, 0x00, &cfg);
		rts_trace(TRACE_CR,"phy00 default value = %#x\n", cfg);

		if (!(cfg & 0x0004)) {
			cr_wr_reg(cr, 0xfd57,0x02, 0x02);
			cr_update_phy16(cr, 0x04, 0xFFFF, 0x8000);
			cr_wr_phy16(cr, 0x10, 0x03A4);
			udelay(20);
			cr_rd_reg(cr, 0xFE92, &val);
			rts_trace(TRACE_CR,"PCIE OOBS auto_k value = %#x\n", val);

			cfg = (u16) ((val & 0x1E) >> 1);
			cr_update_phy16(cr, 0x00, 0x0FFF, (cfg << 12));
			cr_update_phy16(cr, 0x00, 0xFFFB, 0x0004);
			cr_rd_phy16(cr, 0x00, &cfg);
			rts_trace(TRACE_CR,"PCIE OOBS set to phy00 = %#x\n", cfg);

			cr_wr_reg(cr, 0xfd57,0x02, 0x00);
			cr_update_phy16(cr, 0x04, 0x7FFF, 0);
			cr_wr_phy16(cr, 0x10, 0x0000);
		}
	}

	if (option->eios_patch_en) {
		cr_update_phy16(cr, 0x19, 0xEFFF, 0);
		cr_update_phy16(cr, 0x1F, 0xFFFD, 0x2);
	}

	cr_set_stat(cr, RTSX_STAT_INIT);

	option->auto_delink_allowed = 1;

	cr->sd_max_phase = MAX_PHASE;
	CLR_SDIO_EXIST(cr);
	cr->sdio_func_init = 0;

	option->ltr_enabled = 0;
	option->ltr_active = 0;

	if (option->thermal_sensor_en) {
		u8 gpio_input = 0;
		rts_trace(TRACE_CR, "enable thermal sensor\n");
		info->thermal_state = 0;
		info->thermal_time = 0;
		SET_STATUS_FLAG(cr, LED_TEST_MODE);
		cr_disable_auto_blink(cr);
		cr_turn_off_led(cr);
		cr_rd_reg(cr, REG_GPIO_CTL, &gpio_input);
		info->gpio_input = gpio_input & 0x10;
	}

	if (!check_sd_speed_prior(option->sd_speed_prior))
		option->sd_speed_prior = 0x01040203;
	rts_trace(TRACE_PROBE, "sd_speed_prior = 0x%08x\n",
		option->sd_speed_prior);

	if (!check_sd_current_prior(option->sd_current_prior))
		option->sd_current_prior = 0x00010203;
	rts_trace(TRACE_PROBE, "sd_current_prior = 0x%08x\n",
						option->sd_current_prior);

	if ((option->mmc_ddr_tx_phase > 31) || (option->mmc_ddr_tx_phase < 0))
		option->mmc_ddr_tx_phase = 0;

	/* Power on SSC */
	cr_wr_reg(cr, REG_FPDCTL, SSC_POWER_DOWN, 0);
	msleep(200);
	if (CHECK_PID(cr, 0x5250)
		&& check_dev_flag(cr, CLEAR_0xFD57_EN)) {
		rts_trace(TRACE_CR, "Clear 0xFD57 to 0\n");
		cr_wr_reg(cr, REG_DBG_OE, 0xFF, 0);
	}

	cr_patch_dma(cr);

	/* Set mcu_cnt to 7 to ensure data can be sampled properly */
	cr_wr_reg(cr, REG_CLK_DIV, 0x7F, 0x07);

	cr_wr_reg(cr, REG_SSC_DIV_N_0, 0xFF, 0x5D);

	rts5249_swap_cfg_with_reg_4_D3(cr);

	if (CHECK_STATUS_FLAG(cr, ENTERED_RTD3)
		&& check_dev_flag(cr, L1SS_SETBACK_FOR_INTEL_EN)) {
		if (CHECK_PID(cr, 0x524A))
			rts524A_store_cfg_from_reg_4_RTD3(cr);
		else
			rts5249_store_cfg_from_reg_4_RTD3(cr);
		}
	if (!CHECK_STATUS_FLAG(cr, ENTERED_RTD3)) {
		if (CHECK_PID(cr, 0x5250) || CHECK_PID(cr, 0x524A)
			|| CHECK_PID(cr, 0x525A))
			cr_wr_reg(cr, RTS5250_AUTOLOAD_CFG1,
					CD_RESUME_EN_MASK, CD_RESUME_EN_MASK);
		else
			cr_wr_reg(cr, AUTOLOAD_CFG1,
					CD_RESUME_EN_MASK, CD_RESUME_EN_MASK);
	}

	rts5249_config_packet_cap(cr);
	rts5249_init_from_cfg(cr);
	if (rts5249_init_from_hw(cr))
		TRACE_RET(cr, STATUS_FAIL);

	rts52xx_save_content_to_autoload_space(cr);

	if (CHECK_PID(cr, 0x524A)) {
		cr_wr_reg(cr, REG_FUNC1_FORCE_CTL, 0x02, 0x02);
		cr_wr_reg(cr, REG_PM_EVENT_DBG, REG_PM_EVENT_DBG0, REG_PM_EVENT_DBG0);
		cr_wr_reg(cr, RTS525A_LDO_VCC_CFG1, REG_LMT_EN, REG_LMT_EN);
		cr_wr_reg(cr, RTS524A_DELINK_CFG, REG_DOWN_RELINK_INRUSH, REG_DOWN_RELINK_INRUSH);
	}

	if (CHECK_VERSION(cr, 0x524A, IC_VER_A)) {
		cr_wr_reg(cr, RTS525A_LDO_DV18_CFG, REG_DV18_SR_MASK, REG_DV18_SR_DF);
		cr_wr_reg(cr, RTS525A_LDO_VCC_CFG1, REG_TUNEREF_VCC_MASK, REG_TUNEREF_VCC_1V2);
		cr_wr_reg(cr, RTS525A_LDO_VIO_CFG, REG_TUNEREF_VIO_MASK, REG_TUNEREF_VIO_1V2);
		cr_wr_reg(cr, RTS525A_LDO_VIO_CFG, REG_SDVIO_SR_MASK, REG_SDVIO_SR_DF);
		cr_wr_reg(cr, RTS525A_LDO_DV12S_CFG, REG_TUNERFF12_MASK, REG_TUNERFF12_DF);
		cr_wr_reg(cr, RTS5250_SD40_LDO_CTL1, SD40_VIO_TUNE_MASK, SD40_VIO_TUNE_1V7);
	}

	if (CHECK_VERSION(cr, 0x525A, IC_VER_A)) {
		cr_wr_reg(cr, REG_L1SUB_CFG2, 0x02, 0x02);
		cr_wr_reg(cr, RTS525A_RREF_CFG, REG_VBGSEL_MASK, REG_VBGSEL_1V25);
		cr_wr_reg(cr, RTS525A_LDO_VIO_CFG, REG_TUNE_VIO_MASK, REG_LDO_VIO_17V);
		cr_wr_reg(cr, RTS525A_LDO_DV12S_CFG, REG_TUNED12_MASK, REG_TUNED12_DF);
		cr_wr_reg(cr, RTS525A_LDO_AV12S_CFG, REG_TUNE_AV12S_MASK, REG_TUNE_AV12S_DF);
		cr_wr_reg(cr, RTS525A_LDO_VCC_CFG0, REG_SDVCC_LMTVTH_MASK, REG_SDVCC_LMTVTH_2A);
		if (!option->ft2_fast_mode)
			cr_wr_reg(cr, RTS525A_OOBS_CFG, REG_OOBS_AUTOK_DIS | REG_OOBS_VAL_MASK, 0x89);
	}

	if (CHECK_PID(cr, 0x524A) || CHECK_PID(cr, 0x525A)) {
		if (option->dma_len_ext_en)
			cr_wr_reg(cr, REG_PCLK_CTL,
			CFG_DMA_LEN_EXT | PCLK_MODE_SEL, CFG_DMA_LEN_EXT | PCLK_MODE_SEL);
		else
			cr_wr_reg(cr, REG_PCLK_CTL, PCLK_MODE_SEL, PCLK_MODE_SEL);
	}

	if (CHECK_PID(cr, 0x525A)) {
		cr_wr_reg(cr, RTS525A_LDO_VCC_CFG1, RTS525A_SD_VCC_PD_MASK, option->sdvcc_pd<<6);
	} else {
		cr_wr_reg(cr, RTS5250_SD40_LDO_CTL1, SD_VCC_PD_MASK, option->sdvcc_pd);
	}

	if (option->ss_en == 2)
		option->ss_en = 0;

	if (CHECK_VERSION(cr, 0x5249, IC_VER_A))
		option->force_clkreq_0 = 1;

	if (rts5249_init_hw(cr))
		TRACE_RET(cr, STATUS_FAIL);

	if (check_sd40_flag(cr, SD40_PHY_TEST_MODE))
		rts5249_sd40_phy_test(cr);

	if (option->sd_dat3_cd_en &&
		(CHECK_PID(cr, 0x524A) || CHECK_PID(cr, 0x525A)))
		cr_enable_sd_dat3_poll(cr);

	if (CHECK_PID(cr, 0x524A) || CHECK_PID(cr, 0x525A)) {
		cr_wr_reg(cr, RTS525A_PME_FORCE_CTL,
			REG_EFUSE_POWER_MASK, REG_EFUSE_POWEROFF);
		rts_trace(TRACE_CR, "Power off efuse!");
	}

	if (RTSX_TRACE_MASK & TRACE_DUMP_ALL) {
		cr_dump_phy(cr, 0x00, 0x1F);
		cr_dump_phy(cr, 0x80, 0x9F);
		cr_dump_register(cr, 0xFC00, 0xFF7F);
	}

	return 0;
}

int rts5249_init(struct card_reader *cr)
{
	if (cr_request_irq(cr) < 0)
		return -1;

	if (rts5249_init_cr(cr)) {
		rts_trace(TRACE_ERROR, "Init 5249 Failed\n");
		return -EIO;
	}

	return 0;
}

int rts5249_detect_card(struct card_reader *cr, enum CARD_ID id)
{
	u32 card_cd, status;
	u8 val;

	if (cr->option.sd_dat3_cd_en && SD_ID == id) {
		rts_trace(TRACE_CR, "SDD3CDEnable=1, return STATUS_SUCCESS \n");
		return STATUS_SUCCESS;
	}

	if (id == SD_ID) {
		card_cd = SD_EXIST;
	} else if (id == MS_ID) {
		card_cd = MS_EXIST;
	} else if (id == XD_ID) {
		card_cd = XD_EXIST;
	} else {
		rts_trace(TRACE_ERROR, "Wrong card type: 0x%x\n", id);
		TRACE_RET(cr, STATUS_FAIL);
	}

	if (id == XD_ID) {
		cr_rd_reg(cr, REG_CARD_EXIST, &val);
		if (!(val & XD_CARD))
			TRACE_RET(cr, STATUS_FAIL);
	} else {
		status = cr_ioread32(cr, BIPR);
		if (!(status & card_cd))
			TRACE_RET(cr, STATUS_FAIL);
	}

	return 0;
}


static void rtsx_enter_d3cold(struct card_reader *cr)
{
	if (CHECK_PID(cr, 0x5249)) {
		if (CHECK_VERSION(cr, 0x5249, IC_VER_A)) {
			cr_wr_reg(cr, AUTOLOAD_CFG2, 0x80, 0x80);
			cr_wr_reg(cr, AUTOLOAD_CFG3, 0x10, 0x00);
		} else {
			u8 val;
			cr_rd_reg(cr, 0xFE1D, &val);
			cr_wr_reg(cr, 0xFF40, 0xFF, val);
			cr_rd_reg(cr, 0xFE1C, &val);
			cr_wr_reg(cr, 0xFF41, 0xFF, val);
			cr_wr_reg(cr, AUTOLOAD_CFG3, 0x10, 0x10);
		}
	} else if (CHECK_PID(cr, 0x5250) ||
		CHECK_PID(cr, 0x524A) || CHECK_PID(cr, 0x525A)) {
		cr_wr_reg(cr, RTS5250_AUTOLOAD_CFG3, U_PM_D3_EN, U_PM_D3_EN);
	}
}

void rts5249_before_power_down(struct card_reader *cr, int pm_stat)
{
	struct cr_option *option = &cr->option;
	int retval;

	rts_trace(TRACE_PM, "%s: pm_stat = %d\n", __func__, pm_stat);

	if (CHECK_STATUS_FLAG(cr, ENTERED_RTD3)
		&& check_dev_flag(cr, L1SS_SETBACK_FOR_INTEL_EN)) {
		if (CHECK_PID(cr,0x524A))
			rts524A_save_cfg_to_reg_4_RTD3(cr);
		else
			rts5249_save_cfg_to_reg_4_RTD3(cr);
		}

	if (check_dev_flag(cr, S3_CLEAR_SETBACK_EN)) {
		if (!(CHECK_STATUS_FLAG(cr, ENTERED_RTD3))) {
			if (CHECK_PID(cr, 0x524A))
				cr_wr_reg(cr, 0xFF2C, 0x01, 0x00);
			else
				cr_wr_reg(cr, 0xFF3C, 0x01, 0x00);
			}
	}

	if (check_dev_flag(cr, LTR_L1SS_PWR_GATE_EN))
		cr_set_l1off_sub_cfg_D3(cr);

	retval = rts5249_force_power_on(cr, SSC_PDCTL);
	if (retval != STATUS_SUCCESS) {
		return;
	}

	/* fix the issue that cannot find card after resume */
	cm_release_cards(cr->cm);
	cr->cur_card = INVAL_ID;
	cr->cm->cur_card = NULL;
	cr->release_map = 0;

	cr_disable_bus_int(cr);
	if (option->blink_led)
		cr_turn_off_led(cr);

	/* u_force_clkreq_0 */
	if (CHECK_PID(cr, 0x5249) || CHECK_PID(cr, 0x5250))
		cr_wr_reg(cr, REG_PETXCFG, 0x08, 0x08);

	if (pm_stat == PM_S1) {
		rts_trace(TRACE_PM, "Host enter S1\n");
		cr_wr_reg(cr, REG_HOST_SLEEP_STATE, 0x03,
						HOST_ENTER_S1);
		cr_wr_reg(cr, 0xFF01, 0xFF, 0);
		cr_wr_reg(cr, 0xFF02, 0xFF, 0);
		cr_wr_reg(cr, 0xFF03, 0x01, 0);
		if (CHECK_PID(cr, 0x5250) || CHECK_PID(cr, 0x524A)
			|| CHECK_PID(cr, 0x525A)) {
			cr_wr_reg(cr, RTS5250_AUTOLOAD_CFG1,
					CD_RESUME_EN_MASK, 0x00);
			cr_wr_reg(cr, RTS5250_AUTOLOAD_CFG3,
							U_PM_D3_EN, U_PM_D3_EN);
		} else {
			cr_wr_reg(cr, AUTOLOAD_CFG1,
					CD_RESUME_EN_MASK, 0x00);
			cr_wr_reg(cr, AUTOLOAD_CFG3, U_PM_D3_EN, U_PM_D3_EN);
		}
	} else if (pm_stat == PM_S3) {
		/* wwang, 2010-12-16
		 * Before entering S3, some card may be in sequential mode,
		 * so here we delay some time to ensure programming finished
		 */
		if (option->s3_pwr_off_delay > 0) {
			msleep(option->s3_pwr_off_delay);
		}
		rts_trace(TRACE_PM, "Host enter S3\n");
		cr_wr_reg(cr, REG_HOST_SLEEP_STATE, 0x03,
				HOST_ENTER_S3);
		/* Set relink_time to 0, to avoid bar missing issue */
		cr_wr_reg(cr, 0xFF01, 0xFF, 0);
		cr_wr_reg(cr, 0xFF02, 0xFF, 0);
		cr_wr_reg(cr, 0xFF03, 0x01, 0);

		rts_trace(TRACE_PM, "cr->d3cold_supported is %i\n",
			cr->d3cold_supported);
		cr_wr_reg(cr, REG_FUNC1_FORCE_CTL, 0x02, 0x02);
		if (cr->d3cold_supported && cr->option.rtd3_en)
			rtsx_enter_d3cold(cr);

		/* distinguish S3/S4 with RTD3*/
		rts_trace(TRACE_PM, "enter_rtd3:%d\n",
				CHECK_STATUS_FLAG(cr, ENTERED_RTD3));
		if (!(CHECK_STATUS_FLAG(cr, ENTERED_RTD3))
				&& !cr->option.remote_wakeup_en) {
			rts_trace(TRACE_PM, "Disable WWEnable\n");
			if (CHECK_PID(cr, 0x5250)) {
				cr_wr_reg(cr, RTS5250_AUTOLOAD_CFG1,
						CD_RESUME_EN_MASK, 0x00);
				cr_wr_reg(cr, RTS5250_AUTOLOAD_CFG3,
								0x01, 0x00);
				cr_wr_reg(cr, RTS5250_PME_FORCE_CTL,
								0x30, 0x20);
			} else if (CHECK_PID(cr, 0x524A) || CHECK_PID(cr, 0x525A)) {
				cr_wr_reg(cr, RTS5250_AUTOLOAD_CFG1,
						CD_RESUME_EN_MASK, 0x00);
				cr_wr_reg(cr, RTS5250_AUTOLOAD_CFG3,
								0x01, 0x00);
				cr_wr_reg(cr, RTS525A_PME_FORCE_CTL,
								0x30, 0x20);
			} else {
				cr_wr_reg(cr, AUTOLOAD_CFG1,
						CD_RESUME_EN_MASK, 0x00);
				cr_wr_reg(cr, AUTOLOAD_CFG3, 0x01, 0x00);
				cr_wr_reg(cr, REG_PME_FORCE_CTL, 0xFF, 0x30);
			}
		}

		if (!CHECK_STATUS_FLAG(cr, ENTERED_RTD3)) {
			if (CHECK_PID(cr, 0x5250) || CHECK_PID(cr, 0x524A)
			|| CHECK_PID(cr, 0x525A)) {
				cr_wr_reg(cr, RTS5250_AUTOLOAD_CFG3,
						U_PM_D3_EN, U_PM_D3_EN);
			} else {
				cr_wr_reg(cr, AUTOLOAD_CFG3,
						U_PM_D3_EN, U_PM_D3_EN);
			}
		}
	}

	if (CHECK_PID(cr, 0x5250) && (PM_S1 == pm_stat)) {
		rts_trace(TRACE_CR, "switch to SDIO BUS\n");
		cr_wr_reg(cr, LTR_CTL, LTR_LATENCY_MODE_MASK,
						LTR_LATENCY_MODE_HW);
		cr_wr_reg(cr, RTS5250_AUTOLOAD_CFG4, 0x60, 0x60);

		/* restore CRC Clock for SDIO bus */
		cr_wr_reg(cr, REG_CLK_CTL, 0xFF, 0);
		cr_wr_reg(cr, REG_CLK_DIV, CRC_CLK_SEL_MASK | MCU_CLK_DIV_MASK,
			CRC_CLK_SSC);
		cr_wr_reg(cr, REG_SSC_DIV_N_0, 0xFF, 0xC4);

		/*restore OCP control for SDIO bus*/
		cr_wr_reg(cr, REG_OCPCTL, 0xFF, SD_DETECT_EN | SDVIO_DETECT_EN);
	}

	/* Enable autodelink */
	if (option->do_delink_before_power_down && option->auto_delink_en)
		cr_wr_reg(cr, REG_CHANGE_LINK_STATE, 0x02, 2);

	/* 2010-03-02: OS will handle dstate, so we mark it here. */
	/* rtsx_handle_pm_dstate(cr, 0x03); */

	if (CHECK_PID(cr, 0x5250) && (PM_S1 == pm_stat))
		rts5249_force_power_on(cr, SSC_PDCTL);
	else
		rts5249_force_power_down(cr, SSC_PDCTL | OC_PDCTL);

	cr_pm_set_flags(cr, PM_POWR_SAVING_MAX);

	if (CHECK_STATUS_FLAG(cr, ENTERED_RTD3))
		cr_patch_rtd3(cr);
}


int rts5249_do_resume(struct card_reader *cr)
{
	if (cr_request_irq(cr) < 0)
		return -1;

	cr_wr_reg(cr, REG_HOST_SLEEP_STATE, 0x03, 0x00);
	rts5249_init_cr(cr);

	return 0;
}

int rts5249_poweron_card(struct card_reader *cr, enum CARD_ID id)
{
	struct cr_option *option = &(cr->option);
	u8 mask, val;
	int uhsii = 0;

	if (option->ocp_en && CHECK_PID(cr, 0x524A))
		cr_enable_ocp(cr);

	if ((id != SD_ID)) {
		uhsii = 0;
	} else {
		struct card *card = cm_get_card(cr->cm, SD_ID);
		if (card)
			uhsii = !!(card->reset_flags & RESET_UHSII_NOW);
		else
			uhsii = 0;
	}

	cr_init_cmd(cr);
	if (uhsii) {
		mask = RTS5249_SD_VIO_PWR_MASK | RTS5249_SD_POWER_MASK;
		val = RTS5249_SD_VIO_PWR14_ON | RTS5249_SD_PWR14_ON;
		cr_add_cmd(cr, WRITE_REG_CMD, REG_CARD_PWR_CTL, mask, val);
		cr_add_cmd(cr, WRITE_REG_CMD, REG_PWR_GATE_CTRL,
				LDO3318_PWR_MASK, LDO_SUSPEND);
	} else {
		mask = RTS5249_SD_POWER_MASK;
		val = RTS5249_SD_PWR14_ON;
		cr_add_cmd(cr, WRITE_REG_CMD, REG_CARD_PWR_CTL, mask, val);
		cr_add_cmd(cr, WRITE_REG_CMD, REG_PWR_GATE_CTRL,
				LDO3318_PWR_MASK, LDO_SUSPEND);
	}

	if (cr_send_cmd(cr, 0, 100))
		TRACE_RET(cr, STATUS_FAIL);

	udelay(cr->option.pmos_pwr_on_interval);

	cr_init_cmd(cr);
	if (uhsii) {
		mask = RTS5249_SD_VIO_PWR_MASK | RTS5249_SD_POWER_MASK;
		val = RTS5249_SD_VIO_PWR_ON | RTS5249_SD_POWER_ON;
		cr_add_cmd(cr, WRITE_REG_CMD, REG_CARD_PWR_CTL, mask, val);
	} else {
		mask = RTS5249_SD_POWER_MASK;
		val = RTS5249_SD_POWER_ON;
		cr_add_cmd(cr, WRITE_REG_CMD, REG_CARD_PWR_CTL, mask, val);
		cr_add_cmd(cr, WRITE_REG_CMD, REG_PWR_GATE_CTRL,
				LDO3318_PWR_MASK, LDO_ON);
	}

	if (cr_send_cmd(cr, 0, 100))
		TRACE_RET(cr, STATUS_FAIL);

	return STATUS_SUCCESS;
}

int rts5249_poweroff_card(struct card_reader *cr, enum CARD_ID id)
{
	u8 mask = 0;
	u8 val = 0;

	mask = RTS5249_SD_VIO_PWR_MASK | RTS5249_SD_POWER_MASK;
	mask |= RTS5249_SD40_PAD_ISO;
	val = RTS5249_SD_VIO_PWR_OFF | RTS5249_SD_POWER_OFF;
	val |= RTS5249_SD40_PAD_ISO_DEFAULT;
	cr_wr_reg(cr, REG_CARD_PWR_CTL, mask, val);

	cr_wr_reg(cr, REG_PWR_GATE_CTRL, LDO3318_PWR_MASK, LDO_OFF);

	return STATUS_SUCCESS;
}

int rts5249_get_ocpstat(struct card_reader *cr, u8 *val)
{
	return cr_rd_reg(cr, REG_OCPSTAT, val);
}

int rts5249_clear_ocpstat(struct card_reader *cr)
{
	u8 mask = 0;
	u8 val = 0;

	mask = SD_OCP_INT_CLR | SD_OC_CLR;
	mask |= SDVIO_OCP_INT_CLR | SDVIO_OC_CLR;
	val = SD_OCP_INT_CLR | SD_OC_CLR;
	val |= SDVIO_OCP_INT_CLR | SDVIO_OC_CLR;

	cr_wr_reg(cr, REG_OCPCTL, mask, val);
	udelay(10);
	cr_wr_reg(cr, REG_OCPCTL, mask, 0);

	return 0;
}

int rts5249_enable_ocp(struct card_reader *cr)
{
	u8 val = 0;

	cr_wr_reg(cr, REG_FPDCTL, OC_POWER_DOWN, 0);

	val = SD_OCP_INT_EN | SD_DETECT_EN;
	val |= SDVIO_OCP_INT_EN | SDVIO_DETECT_EN;
	cr_wr_reg(cr, REG_OCPCTL, 0xFF, val);

	return 0;
}

int rts5249_disable_ocp(struct card_reader *cr)
{
	u8 mask = 0;

	mask = SD_OCP_INT_EN | SD_DETECT_EN;
	mask |= SDVIO_OCP_INT_EN | SDVIO_DETECT_EN;
	cr_wr_reg(cr, REG_OCPCTL, mask, 0);

	cr_wr_reg(cr, REG_FPDCTL, OC_POWER_DOWN, OC_POWER_DOWN);
	return 0;
}

int rts5249_init_ocp(struct card_reader *cr)
{
	struct cr_option *option = &(cr->option);

	if (option->ocp_en) {
		u8 mask, val;
		/* SSC power on, OCD power on */
		cr_wr_reg(cr, REG_FPDCTL, OC_POWER_DOWN, 0);

		/* mask = SD_OCP_TIME_MASK;
		val = SD_OCP_TIME_800;
		cr_wr_reg(cr, REG_OCPPARA1, mask, val);*/

		/* REG_OCPPARA2: no wirte mask for rts5249 */
		mask = SD_OCP_THD_MASK;
		if (CHECK_VERSION(cr, 0x524A, IC_VER_A)
			|| CHECK_VERSION(cr, 0x525A, IC_VER_A))
			mask |= SDVIO_OCP_THD_MASK;

		val = option->sd_400mA_ocp_thd;
		if (CHECK_VERSION(cr, 0x524A, IC_VER_A))
			val |= SDVIO_OCP_THD_380;
		else if (CHECK_VERSION(cr, 0x525A, IC_VER_A))
			val |= SDVIO_OCP_THD_500;
		cr_wr_reg(cr, REG_OCPPARA2, mask, val);

		mask = SD_OCP_GLITCH_MASK | SDVIO_OCP_GLITCH_MASK;
		val = cr->hw_param.ocp_glitch;
		cr_wr_reg(cr, REG_OCPGLITCH, mask, val);

		if (CHECK_PID(cr, 0x5249))
			cr_enable_ocp(cr);
		else
			cr_disable_ocp(cr);
	} else {
		/* OC power down */
		cr_wr_reg(cr, REG_FPDCTL, OC_POWER_DOWN, OC_POWER_DOWN);
	}

	return 0;
}

void rts5249_enable_dma(struct card_reader *cr, u32 byte_cnt,
		u8 pack_size, enum dma_data_direction dir)
{
	u8 mask, val;

	cr_add_cmd(cr, WRITE_REG_CMD, REG_IRQSTAT0, DMA_DONE_INT, DMA_DONE_INT);

	cr_add_cmd(cr, WRITE_REG_CMD, REG_DMATC3, 0xFF, (u8)(byte_cnt >> 24));
	cr_add_cmd(cr, WRITE_REG_CMD, REG_DMATC2, 0xFF, (u8)(byte_cnt >> 16));
	cr_add_cmd(cr, WRITE_REG_CMD, REG_DMATC1, 0xFF, (u8)(byte_cnt >> 8));
	cr_add_cmd(cr, WRITE_REG_CMD, REG_DMATC0, 0xFF, (u8)byte_cnt);

	mask = 0x03 | RTS5249_DMA_PACK_SIZE_MASK;
	if (dir == DMA_FROM_DEVICE)
		val = DMA_DIR_FROM_CARD | DMA_EN | pack_size;
	else
		val = DMA_DIR_TO_CARD | DMA_EN | pack_size;
	cr_add_cmd(cr, WRITE_REG_CMD, REG_DMACTL, mask, val);

	cr_add_cmd(cr, WRITE_REG_CMD, REG_CARD_DATA_SOURCE, 0x01, RING_BUFFER);
}

/* for SD */

int rts5249_sd_pull_ctl_enable(struct card_reader *cr)
{
	struct card *card = cm_get_card(cr->cm, SD_ID);
	int uhsii = 0;

	if (card)
		uhsii = !!(card->reset_flags & RESET_UHSII_NOW);

	cr_init_cmd(cr);

	if (uhsii) {
		cr_add_cmd(cr, WRITE_REG_CMD, REG_CARD_PULL_CTL1, 0xFF, 0x66);
		/* SD_D3 no pull */
		cr_add_cmd(cr, WRITE_REG_CMD, REG_CARD_PULL_CTL2, 0xFF, 0x10);
		cr_add_cmd(cr, WRITE_REG_CMD, REG_CARD_PULL_CTL3, 0xFF, 0xE5);
	} else {
		cr_add_cmd(cr, WRITE_REG_CMD, REG_CARD_PULL_CTL1, 0xFF, 0x66);
		cr_add_cmd(cr, WRITE_REG_CMD, REG_CARD_PULL_CTL2, 0xFF, 0xAA);
		cr_add_cmd(cr, WRITE_REG_CMD, REG_CARD_PULL_CTL3, 0xFF, 0xE9);
		cr_add_cmd(cr, WRITE_REG_CMD, REG_CARD_PULL_CTL4, 0xFF, 0xAA);
	}

	if (cr_send_cmd(cr, SD_ID, 100))
		TRACE_RET(cr, STATUS_FAIL);

	return 0;
}

int rts5249_sd_pull_ctl_disable(struct card_reader *cr)
{
	cr_wr_reg(cr, REG_CARD_PULL_CTL1, 0xFF, 0x66);
	cr_wr_reg(cr, REG_CARD_PULL_CTL2, 0xFF, 0x55);
	cr_wr_reg(cr, REG_CARD_PULL_CTL3, 0xFF, 0xD5);
	cr_wr_reg(cr, REG_CARD_PULL_CTL4, 0xFF, 0x55);

	return 0;
}

int rts5249_sd_init_power(struct card_reader *cr)
{
	struct cr_option *option = &(cr->option);

	cr_wr_reg(cr, REG_PWR_GATE_CTRL, LDO3318_PWR_MASK, LDO_OFF);
	if (sd_power_off_card3v3(cr))
		TRACE_RET(cr, STATUS_FAIL);

	if (sd_change_bank_voltage(cr, SD_COMM_IO_3V3))
		TRACE_RET(cr, STATUS_FAIL);

	if ((sd_sd30_drive_sel(cr, SD_IO_3V3)))
		TRACE_RET(cr, STATUS_FAIL);

	if (!option->ft2_fast_mode)
		msleep(250);

	if (cr_enable_card_clk(cr, SD_ID))
		TRACE_RET(cr, STATUS_FAIL);

	if (cr->asic_code) {
		if (sd_pull_ctl_enable(cr))
			TRACE_RET(cr, STATUS_FAIL);
	} else {
		cr_wr_reg(cr, REG_FPGA_PULL_CTL,
				FPGA_SD_PULL_CTL_BIT | 0x20, 0);
	}

	if (cr_poweron_card(cr, SD_ID))
		TRACE_RET(cr, STATUS_FAIL);

	msleep(260);

	if (cr->option.ocp_en) {
		u8 ocp;
		cr_get_ocpstat(cr, &ocp);
		if (ocp & (SD_OC_NOW | SD_OC_EVER | SDVIO_OC_NOW
							| SDVIO_OC_EVER)) {
			struct card *card = cm_get_card(cr->cm, SD_ID);

			if (card)
				card->reset_flags |= RESET_MEET_OCP;
			rts_trace(TRACE_SD, "OCP! OCPSTAT:%#x\n", ocp);
			TRACE_RET(cr, STATUS_FAIL);
		}
	}

	cr_wr_reg(cr, REG_CARD_OE, SD_OUTPUT_EN, SD_OUTPUT_EN);

	return 0;
}

int rts5250_sd_init_power(struct card_reader *cr)
{
	struct cr_option *option = &(cr->option);
	struct card *card = cm_get_card(cr->cm, SD_ID);
	int mmc_sdr200 = !!(card->reset_flags & RESET_E_MMC45_NOW);

	cr_wr_reg(cr, REG_PWR_GATE_CTRL, LDO3318_PWR_MASK, LDO_OFF);
	if (sd_power_off_card3v3(cr))
		TRACE_RET(cr, STATUS_FAIL);

	if (cr_enable_card_clk(cr, SD_ID))
		TRACE_RET(cr, STATUS_FAIL);

	if (check_sd_flag(cr, EMMC45_DONT_SWITCH_POWER))
		mmc_sdr200 = 1;

	if (mmc_sdr200) {
		rts_trace(TRACE_CR, "%s change to 1.8v for mmc4.5\n", __func__);
		if (sd_change_bank_voltage(cr, SD_COMM_IO_1V8))
			TRACE_RET(cr, STATUS_FAIL);
		if ((sd_sd30_drive_sel(cr, SD_IO_1V8)))
			TRACE_RET(cr, STATUS_FAIL);
	} else {
		rts_trace(TRACE_CR, "%s change to 3.3v for mmc4.5\n", __func__);
		if (sd_change_bank_voltage(cr, SD_COMM_IO_3V3))
			TRACE_RET(cr, STATUS_FAIL);
		if ((sd_sd30_drive_sel(cr, SD_IO_3V3)))
			TRACE_RET(cr, STATUS_FAIL);
	}

	if (!option->ft2_fast_mode)
		msleep(250);

	if (cr->asic_code) {
		if (sd_pull_ctl_enable(cr))
			TRACE_RET(cr, STATUS_FAIL);
	} else {
		cr_wr_reg(cr, REG_FPGA_PULL_CTL,
				FPGA_SD_PULL_CTL_BIT | 0x20, 0);
	}

	if (cr_poweron_card(cr, SD_ID))
		TRACE_RET(cr, STATUS_FAIL);

	msleep(260);

	if (cr->option.ocp_en) {
		u8 ocp;
		cr_get_ocpstat(cr, &ocp);
		if (ocp & (SD_OC_NOW | SD_OC_EVER | SDVIO_OC_NOW
							| SDVIO_OC_EVER)) {
			struct card *card = cm_get_card(cr->cm, SD_ID);

			if (card)
				card->reset_flags |= RESET_MEET_OCP;
			rts_trace(TRACE_SD, "OCP! OCPSTAT:%#x\n", ocp);
			TRACE_RET(cr, STATUS_FAIL);
		}
	}

	cr_wr_reg(cr, REG_CARD_OE, SD_OUTPUT_EN, SD_OUTPUT_EN);

	return 0;
}

int rts5249_release_sd_card(struct card_reader *cr, struct card *c)
{
	struct sd_info *sd_card = c->extra;
	struct cr_option *option = &(cr->option);
	struct rts5249_info *info = cr->extra;
	u8 val = 0;

	rts_trace(TRACE_SD, "release_sd_card\n");
	clr_card_flags(c, READY | FAIL | WP);
	c->reset_flags = 0;
	if (option->thermal_sensor_en) {
		info->thermal_state = 0;
		info->thermal_time = 0;
		cr->card_status = 0;
	}
	sd_card->sd_lock_status = 0;
	sd_card->sd_erase_status = 0;
	sd_card->sd40_seq_sectors = 0;

	memset(sd_card->raw_csd, 0, 16);
	memset(sd_card->raw_scr, 0, 8);
	memset(sd_card->raw_cid, 0, 16);
	memset(&sd_card->ge, 0, sizeof(struct sd_ge));

	if (CHK_SD_UHSII(sd_card))
		sd40_uninit_hw(cr, c);

	if (!cr->option.ignore_sd) {
		CLR_SDIO_EXIST(cr);
		cr_wr_reg(cr, REG_SDIO_MUX_CTRL, 0x3, 0x00);
	}

	if (sd_power_off_card3v3(cr))
		TRACE_RET(cr, STATUS_FAIL);

	/* Switch SD bus to 3V3 signal */
	if (sd_change_bank_voltage(cr, SD_COMM_IO_3V3))
		TRACE_RET(cr, STATUS_FAIL);

	if (CHK_SD30_SPEED(sd_card))
		sd_sd30_drive_sel(cr, SD_IO_3V3);

	/* REG_OCPPARA2: no wirte mask for rts5249 */
	val = option->sd_400mA_ocp_thd | SDVIO_OCP_THD_380;
	cr_wr_reg(cr, REG_OCPPARA2, SD_OCP_THD_MASK, val);

	if (option->ocp_en && (!CHECK_PID(cr, 0x5249)))
		cr_disable_ocp(cr);

	if (option->sd_dat3_cd_en &&
		(CHECK_PID(cr, 0x524A) || CHECK_PID(cr, 0x525A)))
		cr_enable_sd_dat3_poll(cr);

	return 0;
}

int rts5249_sd_cleanup_work(struct card_reader *cr, struct card *c)
{
	struct sd_info *sd_card = c->extra;

	if (SEQ_RW == sd_card->rw_mode) {
		rts_trace(TRACE_SD, "SD: stop transmission for SEQ_RW\n");
		sd_stop_seq_mode(cr, c);
	} else if ((PRE_READ == sd_card->rw_mode)
			|| (PRE_WRITE == sd_card->rw_mode)) {
		rts_trace(TRACE_SD, "SD: stop transmission for PRE_RW\n");
		sd_stop_sequential(c);
	}

	return 0;
}

int rts5249_sd_current_over_limit400_handler(struct card_reader *cr)
{
	struct cr_option *option = &(cr->option);
	u8 val = option->sd_800mA_ocp_thd | SDVIO_OCP_THD_380;

	/* REG_OCPPARA2: no wirte mask for rts5249 */
	cr_wr_reg(cr, REG_OCPPARA2, SD_OCP_THD_MASK, val);

	return 0;
}

int rts5250_sd_current_over_limit400_handler(struct card_reader *cr)
{
	struct cr_option *option = &(cr->option);

	cr_wr_reg(cr, REG_OCPPARA2, SD_OCP_THD_MASK, option->sd_800mA_ocp_thd);

	return 0;
}

int rts5249_sd_change_bank_voltage(struct card_reader *cr, u8 voltage)
{
	u16 val = 0;

	if (voltage == SD_IO_3V3) {
		if (cr->asic_code) {
			cr_rd_phy16(cr, 0x08, &val);
			val &= 0xFC3F;
			val |= 0x03C0;
			if ((cr_wr_phy16(cr, 0x08, val)))
				TRACE_RET(cr, STATUS_FAIL);
		} else {
			cr_wr_reg(cr, REG_SD_PAD_CTL, SD_IO_USING_1V8, 0);
		}
	} else if (voltage == SD_IO_1V8) {
		if (cr->asic_code) {
			u16 phy;
			if (!(CHECK_PID(cr, 0x524A))) {
				cr_rd_phy16(cr, 0x11, &phy);
				phy &= 0xFFF3;
				if ((cr_wr_phy16(cr, 0x11, phy)))
					TRACE_RET(cr, STATUS_FAIL);
			}

			cr_rd_phy16(cr, 0x08, &val);
			val &= 0xFC3F;
			if (CHECK_VERSION(cr, 0x5249, IC_VER_A))
				val |= 0x0040;
			else if ((CHECK_PID(cr, 0x524A)))
				val |= 0x0100;
			else
				val |= 0x0080;
			if ((cr_wr_phy16(cr, 0x08, val)))
				TRACE_RET(cr, STATUS_FAIL);
		} else {
			cr_wr_reg(cr, REG_SD_PAD_CTL,
					SD_IO_USING_1V8, SD_IO_USING_1V8);
		}
	} else if (voltage == SD_IO_1V9) {
		if (cr->asic_code) {
			if (!(CHECK_PID(cr, 0x524A))) {
				if ((cr_update_phy16(cr, 0x11, 0xFFF3, 0)))
					TRACE_RET(cr, STATUS_FAIL);
			}

			if (CHECK_VERSION(cr, 0x5249, IC_VER_A))
				val = 0x0080;
			else if ((CHECK_PID(cr, 0x524A)))
				val = 0x0140;
			else
				val = 0x0100;
			if ((cr_update_phy16(cr, 0x08, 0xFC3F, val)))
				TRACE_RET(cr, STATUS_FAIL);
		} else {
			cr_wr_reg(cr, REG_SD_PAD_CTL,
					SD_IO_USING_1V8, SD_IO_USING_1V8);
		}
	} else {
		TRACE_RET(cr, STATUS_FAIL);
	}

	return STATUS_SUCCESS;
}

int rts5250_sd_change_bank_voltage(struct card_reader *cr, u8 voltage)
{
	if (voltage == SD_IO_3V3) {
		if (cr->asic_code) {
			u8 val = RTS5250_LDO_D3318_33V;
			u8 mask = RTS5250_LDO_D3318_MASK;
			if (CHECK_PID(cr,0x5250))
				cr_wr_reg(cr, RTS5250_LDO_CFG2, mask, val);
			else
				cr_wr_reg(cr, RTS525A_LDO_CFG2, mask, val);
		}
		cr_wr_reg(cr, REG_SD_PAD_CTL, SD_IO_USING_1V8, 0);
	} else if (voltage == SD_IO_1V8) {
		if (cr->asic_code) {
			u8 val = RTS5250_LDO_D3318_18V;
			u8 mask = RTS5250_LDO_D3318_MASK;
			if (CHECK_PID(cr,0x5250))
				cr_wr_reg(cr, RTS5250_LDO_CFG2, mask, val);
			else
				cr_wr_reg(cr, RTS525A_LDO_CFG2,
					RTS525A_LDO_D3318_MASK,
					RTS525A_LDO_D3318_18V);
		}
		cr_wr_reg(cr, REG_SD_PAD_CTL,
				SD_IO_USING_1V8, SD_IO_USING_1V8);
	} else if (voltage == SD_IO_1V9) {
		if (cr->asic_code) {
			if (CHECK_PID(cr,0x5250))
				cr_wr_reg(cr, RTS5250_LDO_CFG2,
					RTS5250_LDO_D3318_MASK,
					RTS5250_LDO_D3318_19V);
			else
				cr_wr_reg(cr, RTS525A_LDO_CFG2,
					RTS525A_LDO_D3318_MASK,
					RTS525A_LDO_D3318_19V);
		}
		cr_wr_reg(cr, REG_SD_PAD_CTL,
				SD_IO_USING_1V8, SD_IO_USING_1V8);
	} else {
		TRACE_RET(cr, STATUS_FAIL);
	}

	return STATUS_SUCCESS;
}

static int rts5249_wait_dlsm_idle(struct card_reader *cr)
{
	u8 status = 0;
	int i = 20;

	rts_trace(TRACE_SD, "Check DLSM_IDLE status\n");
	do {
		if (cr_rd_reg(cr, REG_SD40_STATUS3, &status))
			TRACE_RET(cr, STATUS_FAIL);
		if((status & DLSM_STATE_MASK) == DLSM_STATE_IDLE)
			break;
		msleep(10);
	} while (i--);

	if (i <= 0) {
		rts_trace(TRACE_ERROR, "Wait DLSM_IDLE fail\n");
		return -ETIMEDOUT;
	}

	rts_trace(TRACE_SD, "Now in DLSM_IDLE\n");
	return 0;
}

int rts5249_process_sd40_dormant_state(struct card_reader *cr, struct card *c,
		int dir, int dor_mode)
{
	u8 mask, val;
	struct cr_option *option = &cr->option;

	if (dir == ENTRY_DORMANT) {
		if (sd40_go_dormant_state(c, dor_mode)) {
			rts_trace(TRACE_SD,
				"Send GO_DORMANT_STATE CCMD Fail\n");
			TRACE_RET(cr, STATUS_FAIL);
		}

		cr_wr_reg(cr, REG_SD40_CTRL,
			SD40_GO_DORMANT, SD40_GO_DORMANT);
	} else  if (dir == EXIT_DORMANT) {
		rts_trace(TRACE_SD, "Power on SD40\n");

		if (CHECK_VERSION_HIGHER_THAN(cr, 0x5249, IC_VER_D)) {
			u16 value;
			cr_rd_phy16(cr, 0x9F, &value);
			value |= 0xF04;
			cr_wr_phy16(cr, 0x9F, value);

			cr_rd_phy16(cr, 0x86, &value);
			value |= 0x0C;
			cr_wr_phy16(cr, 0x86, value);
		}

		cr_rd_reg(cr, REG_CARD_PWR_CTL, &val);
		rts_trace(TRACE_SD, "CARD_PWR_CTL: 0x%X\n", val);
		if ((val & RTS5249_SD_POWER_MASK) == RTS5249_SD_POWER_OFF) {
			rts_trace(TRACE_SD, "SD40 Power on\n");
			mask = RTS5249_SD_POWER_MASK;
			val = RTS5249_SD_PWR14_ON;
			cr_wr_reg(cr, REG_CARD_PWR_CTL, mask, val);
			mdelay(5);
			mask = RTS5249_SD_POWER_MASK;
			val = RTS5249_SD_POWER_ON;
			cr_wr_reg(cr, REG_CARD_PWR_CTL, mask, val);
			mdelay(1);
			/* cr_poweron_card(cr, SD_ID); */
		}

		/* don't generate timeout error when leaving dormant state */
		cr_wr_reg(cr, REG_SD40_TMOUT_1, SD40_TIME_OUT_ERR_EN, 0x00);

		mask = SD40_GO_DORMANT | SD40_EXIT_DORMANT;
		val = SD40_EXIT_DORMANT;
		cr_wr_reg(cr, REG_SD40_CTRL, mask, val);

		cr_wr_reg(cr, REG_SD40_TMOUT_1, SD40_TIME_OUT_ERR_EN,
							SD40_TIME_OUT_ERR_EN);
	}

	msleep(5);

	if (dir == ENTRY_DORMANT) {
		if (dor_mode == DOR_HIBERNATE_MODE) {
			rts_trace(TRACE_SD, "SD40 Power off\n");
			mask = RTS5249_SD_POWER_MASK;
			val = RTS5249_SD_POWER_OFF;
			cr_wr_reg(cr, REG_CARD_PWR_CTL, mask, val);
			msleep(option->sd40_hiber_pwr_off_delay);
		}

		cr_rd_reg(cr, REG_SD40_STATUS3, &val);
		rts_trace(TRACE_SD, "SD40_STATUS3(DLSM): 0x%X\n", val);
		if(val & SD40_DET) {
			rts_trace(TRACE_SD, "***Entry dormant fail.\n");
			TRACE_RET(cr, STATUS_FAIL);
		} else {
			if(val & DLSM_STATE_MASK) {
				cr_wr_reg(cr, REG_SD40_CTRL,
					SD40_DLSM_RST, SD40_DLSM_RST);
				cr_wr_reg(cr, REG_CLK_CTL,
					RTS5249_CHANGE_SD40_PCLK | CLK_LOW_FREQ,
					CLK_LOW_FREQ);
				cr_wr_reg(cr, REG_SD40_CTRL, SD40_PHY_RST,0);
				cr_wr_reg(cr, REG_SD40_CTRL,
					SD40_PHY_RST, SD40_PHY_RST);
				cr_wr_reg(cr, REG_CLK_CTL, 0xFF,
					RTS5249_CHANGE_SD40_PCLK);
			}

			if (CHECK_VERSION_HIGHER_THAN(cr, 0x5249, IC_VER_D)) {
				u16 value;
				cr_rd_phy16(cr, 0x9F, &value);
				value &= 0xF0FB;
				cr_wr_phy16(cr, 0x9F, value);

				cr_rd_phy16(cr, 0x86, &value);
				value &= 0xFFF3;
				cr_wr_phy16(cr, 0x86, value);
			}
		}

	} else if (dir == EXIT_DORMANT) {
		int ret;

		ret = rts5249_wait_dlsm_idle(cr);
		if (ret)
			return ret;

		cr_init_cmd(cr);
		cr_add_cmd(cr, READ_REG_CMD, REG_SD40_STATUS1, 0xFF, 0);
		cr_add_cmd(cr, READ_REG_CMD, REG_SD40_TMOUT_0, 0xFF, 0);
		cr_add_cmd(cr, READ_REG_CMD, REG_SD40_TMOUT_1, 0xFF, 0);
		ret = cr_send_cmd(cr, SD_ID, 100);
		if (ret) {
			rts_trace(TRACE_SD, "send cmd fail\n");
		} else {
			u8 *ptr;
			ptr = cr_get_cmd_data(cr);

			rts_trace(TRACE_SD, "SD40_TRAN: 0x%X\n", ptr[0]);
			rts_trace(TRACE_SD, "RSP-HDR: 0x%X, 0x%X\n", ptr[1],
				ptr[2]);

			if ((ptr[0] & (SD40_HEAD_ERR | SD40_FRAM_ERR | SD40_CRC_ERR))
				|| (ptr[1]) || (ptr[2] & 0x0F)) {
				rts_trace(TRACE_SD, "****Exit dormant Fail!\n");
				rts_trace(TRACE_SD, "SD40_STATUS1: 0x%X\n", ptr[0]);
				rts_trace(TRACE_SD, "SD40_TMOUT_0: 0x%X\n", ptr[1]);
				rts_trace(TRACE_SD, "SD40_TMOUT_1: 0x%X\n", ptr[2]);
			}
		}
	}

	return 0;
}

static void enable_aphy_debug(struct card_reader *cr)
{
	u16 val = 0;

	cr_rd_phy16(cr, 0x9D, &val);
	if (CHECK_PID(cr, 0x5250) || CHECK_PID(cr, 0x524A)
		|| CHECK_PID(cr, 0x525A))
		val |= (1 << 8);
	else
		val |= (1 << 9);
	cr_wr_phy16(cr, 0x9D, val);
}

static void disable_aphy_debug(struct card_reader *cr)
{
	u16 val;

	cr_rd_phy16(cr, 0x9D, &val);
	if (CHECK_PID(cr, 0x5250) || CHECK_PID(cr, 0x524A)
		|| CHECK_PID(cr, 0x525A))
		val &= ~(1 << 8);
	else
		val &= ~(1 << 9);
	cr_wr_phy16(cr, 0x9D, val);
}

static void save_auto_K(struct card_reader *cr)
{
	u16 val = 0;
	u16 auto_K = 0;

	enable_aphy_debug(cr);

	cr_rd_phy16(cr, 0x80, &auto_K);

	rts_trace(TRACE_SD, "auto_K:%#x\n", auto_K);

	/* write auto_K[9~12] to PHY0x88[6~9] */
	cr_rd_phy16(cr, 0x88, &val);
	val &= 0xFC3F;
	if (CHECK_PID(cr, 0x524A) || CHECK_PID(cr, 0x525A))
		val |= (auto_K & 0x3C00) >> 4;
	else
		val |= (auto_K & 0x1E00) >> 3;
	cr_wr_phy16(cr, 0x88, val);

	/* write auto_K[4~7] to PHY0x90[6~9] */
	cr_rd_phy16(cr, 0x90, &val);
	val &= 0xFC3F;
	val |= (auto_K & 0x00F0) << 2;
	cr_wr_phy16(cr, 0x90, val);

	/* disable auto K, use preset value */
	cr_rd_phy16(cr, 0x85, &val);
	val &= ~(1 << 0);
	cr_wr_phy16(cr, 0x85, val);

	disable_aphy_debug(cr);

	cr_rd_phy16(cr, 0x9E, &val);
	val &= 0xbfff;
	cr_wr_phy16(cr, 0x9E, val);

}

static void sd40_do_auto_K_apm_offset(struct card_reader *cr)
{
	u16 val = 0;
	u16 val1 = 0;

	rts_trace(TRACE_SD, "auto-K amp offset\n");

	if (CHECK_VERSION(cr, 0x5249, IC_VER_A)) {
		val1 = 0x80;
		val1 |= 0x05;
		cr_wr_phy16(cr, 0x9E, val1);

		/* toggle calibration */
		val = 0x244F;
		val &= 0xFFFB;
		cr_wr_phy16(cr, 0x85, val);
		val |= 0x0004;
		cr_wr_phy16(cr, 0x85, val);


		val1 &= ~(0x05);
		cr_wr_phy16(cr, 0x9E, val1);
	} else {
		if (CHECK_PID(cr, 0x5250) || CHECK_PID(cr, 0x524A) ||
			CHECK_PID(cr, 0x525A)) {
			cr_rd_phy16(cr, 0x90, &val);
			val |= 0x20;
			cr_wr_phy16(cr, 0x90, val);

			cr_rd_phy16(cr, 0x87, &val);
			val &= ~(1 << 0);
			cr_wr_phy16(cr, 0x87, val);

			cr_rd_phy16(cr, 0x9E, &val);
			val |= 0x4000;
			cr_wr_phy16(cr, 0x9E, val);
		}

		cr_rd_phy16(cr, 0x9E, &val1);
		if (CHECK_PID(cr, 0x5250) || CHECK_PID(cr, 0x524A) ||
			CHECK_PID(cr, 0x525A))
			val1 |= 0x205;
		else
			val1 |= 0x05;
		cr_wr_phy16(cr, 0x9E, val1);

		cr_rd_phy16(cr, 0x85, &val);

		/* toggle calibration */
		val &= 0xFFFB;
		cr_wr_phy16(cr, 0x85, val);
		val |= 0x0004;
		cr_wr_phy16(cr, 0x85, val);


		if (CHECK_PID(cr, 0x5250) || CHECK_PID(cr, 0x524A) ||
			CHECK_PID(cr, 0x525A))
			val1 &= ~(0x205);
		else
			val1 &= ~(0x05);
		cr_wr_phy16(cr, 0x9E, val1);

		if (CHECK_PID(cr, 0x5250) || CHECK_PID(cr, 0x524A) ||
			CHECK_PID(cr, 0x525A)) {
			cr_rd_phy16(cr, 0x87, &val);
			val |= (1 << 0);
			cr_wr_phy16(cr, 0x87, val);
		}
	}
}


static void sd40_auto_K_apm_offset(struct card_reader *cr)
{
	u8 eq1 = 0;
	u8 eq2 = 0;
	u16 val = 0;

	sd40_do_auto_K_apm_offset(cr);

	enable_aphy_debug(cr);
	cr_rd_phy16(cr, 0x80, &val);
	disable_aphy_debug(cr);

	eq1 = (val & 0x00F0) >> 4;
	if (CHECK_PID(cr, 0x524A) || CHECK_PID(cr, 0x525A))
		eq2 = (val & 0x3C00) >> 10;
	else
		eq2 = (val & 0x1E00) >> 9;

	/* if offset K over threshold, extend the
	 * range of EQOS and auto K again
	 */
	if ((0x00 == eq1 || 0x0F == eq1) || (0x00 == eq2 || 0x0F == eq2)) {
		cr_rd_phy16(cr, 0x86, &val);
		val &= 0x3FFF;
		val |= 0xC000;
		cr_wr_phy16(cr, 0x86, val);
		sd40_do_auto_K_apm_offset(cr);
	}
}

static void sd40_oobs_auto_K(struct card_reader *cr)
{
	u16 val;
	u16 sum;
	u16 final;
	u8 temp;
	u8 i;

	enable_aphy_debug(cr);

	cr_update_phy16(cr, 0x87, 0x7FFF, 0);
	cr_update_phy16(cr, 0x8D, 0xF1FF, 0x0C00);
	cr_update_phy16(cr, 0x86, 0xFCFF, 0x0082);
	cr_update_phy16(cr, 0x9E, 0xFFFF, 0x4000);

	udelay(100);

	cr_rd_phy16(cr, 0xA3, &val);
	temp = (u8) ((val >> 8) & 0x1F);

	if (CHECK_VERSION(cr, 0x524A, IC_VER_A) ||
		CHECK_VERSION(cr, 0x525A, IC_VER_A))
		rts_trace(TRACE_SD, "sd40 oobs result:%#x\n", temp);
	else {
		if((val & 0x8000) == 0x8000)
			rts_trace(TRACE_SD, "sd40 oobs time out\n");
		else
			rts_trace(TRACE_SD, "sd40 oobs result:%#x\n", temp);
	}

	sum = temp;

	for(i = 0; i < 7; i++)
	{
		cr_rd_phy16(cr, 0x86, &val);
		val &= 0xFFFD;
		cr_wr_phy16(cr, 0x86, val);
		val |= 0x0002;
		cr_wr_phy16(cr, 0x86, val);

		udelay(100);
		cr_rd_phy16(cr, 0xA3, &val);
		temp = (u8) ((val >> 8) & 0x1F);

		if (CHECK_VERSION(cr, 0x524A, IC_VER_A) ||
			CHECK_VERSION(cr, 0x525A, IC_VER_A))
			rts_trace(TRACE_SD, "Num %x:sd40 oobs result:%x\n", i, temp);
		else {
			if(!(val & 0x8000))
				rts_trace(TRACE_SD, "Num %x:sd40 oobs result:%x\n", i, temp);
			else
				rts_trace(TRACE_SD, "Num %x:sd40 oobs time out\n", i);
		}

		sum += temp;
	}

	final = ((sum + 0x04) >> 3);
	rts_trace(TRACE_SD, "sd40 oobs final result:%#x\n", final);

	cr_update_phy16(cr, 0x86, 0xFF7D, 0);
	cr_update_phy16(cr, 0x87, 0x03FF, (final << 10) | 0x8000);

	disable_aphy_debug(cr);

	cr_rd_phy16(cr, 0x87, &final);
	rts_trace(TRACE_SD, "sd40 oobs auto_K:%#x\n", final);
}


/* rts5249_patch_sd40_hw - patch HW configuration
 *
 * @cr
 * @c
 *
 */
static int rts5249_patch_sd40_hw(struct card_reader *cr, struct card *c)
{
	u8 mask, val;

	struct cr_option *option = &(cr->option);

	rts_trace(TRACE_SD, "Configure SD40 Reg.\n");
	cr_init_cmd(cr);

	if (cr->asic_code) {
		if (CHECK_PID(cr, 0x525A)) {
			val = cr->hw_param.av12s & 0x7;
			cr_add_cmd(cr, WRITE_REG_CMD, RTS525A_LDO_AV12S_CFG,
				REG_TUNE_AV12S_MASK, val);
			cr_add_cmd(cr, WRITE_REG_CMD, RTS525A_LDO_VCC_CFG1,
				REG_TUNE_VCC_MASK, REG_LDO_VCC_33V);
			cr_add_cmd(cr, WRITE_REG_CMD, RTS5249_SD40_LDO_CTL,
				0xFF, SD40_LD012AS_ON_ALL);
		} else {
			val = 0xE3 | ((cr->hw_param.av12s & 0x7) << 2);
			cr_add_cmd(cr, WRITE_REG_CMD, RTS5249_SD40_LDO_CTL, 0xFF, val);
		}
		mask = RTS5249_SD40_PAD_ISO;
		val = RTS5249_SD40_PAD_ISO;
		cr_add_cmd(cr, WRITE_REG_CMD, REG_CARD_PWR_CTL, mask, val);
	}

	/* Enable SD40_CLK, disable SD_CLK */
	mask = SD40_CLK_EN|SD_CLK_EN;
	val = SD40_CLK_EN;
	cr_add_cmd(cr, WRITE_REG_CMD, REG_CARD_CLK_EN, mask, val);

	mask = CARD_MOD_MASK;
	val = SD40_MOD_SEL;
	cr_add_cmd(cr, WRITE_REG_CMD, REG_CARD_SELECT, mask, val);

	mask = 0xFF;
	val = cr->hw_param.stb_lh_time;
	cr_add_cmd(cr, WRITE_REG_CMD, REG_EIDL_IN_OUT_CNT, mask, val);

	/* modify dir default value */
	cr_add_cmd(cr, WRITE_REG_CMD, REG_SD40_PHY_CAPABILITY_4, 0xFF, 0x50);


	/* sd40 phy reset deactive , should be conducted
	 * before CHANGE_SD40_PCLK
	 */
	val = SD40_PHY_RST|SD40_PHY_ISO;
	cr_add_cmd(cr, WRITE_REG_CMD, REG_SD40_CTRL, 0xFF, val);

	if (check_sd40_flag(cr, SD40_SCARM_DIS)) {
		/* disable scramble, should be set after SD40_CLK_EN */
		mask = SCRAM_DIS_EN;
		val = SCRAM_DIS_EN;
		cr_add_cmd(cr, WRITE_REG_CMD, RTS5249_SD40_MISC, mask, val);
	} else {
		mask = SCRAM_DIS_EN;
		cr_add_cmd(cr, WRITE_REG_CMD, RTS5249_SD40_MISC, mask, 0);
	}


	if (cr_send_cmd(cr, SD_ID, 100) < 0) {
		rts_trace(TRACE_ERROR, "Patch HW config fail\n");
		TRACE_RET(cr, STATUS_FAIL);
	}

	msleep(100);
	cr_wr_reg(cr, REG_CLK_CTL, 0xFF, RTS5249_CHANGE_SD40_PCLK);
	if (cr->asic_code)
		cr_wr_reg(cr, REG_CLK_DIV, 0xf, 0x3);
	msleep(10);

	if (cr->asic_code) {
		if (CHECK_PID(cr, 0x524A) || CHECK_PID(cr, 0x525A)) {
			if (option->sd40_oobs_autoK_en) {
				rts_trace(TRACE_SD, "Enable sd40 oobs auto_K\n");
				sd40_oobs_auto_K(cr);
				}
			else {
				rts_trace(TRACE_SD, "Disable sd40 oobs auto_K\n");
				cr_update_phy16(cr, 0x86, 0xFCFF, 0);
				cr_update_phy16(cr, 0x87, 0x03FF, 0xD800);
				}
		}

		sd40_auto_K_apm_offset(cr);
		if (CHECK_PID(cr, 0x5250) || CHECK_PID(cr, 0x525A) ||
			CHECK_PID(cr, 0x524A) || CHECK_VERSION_HIGHER_THAN(cr, 0x5249, IC_VER_D))
			save_auto_K(cr);

		if (CHECK_PID(cr, 0x524A) || CHECK_PID(cr, 0x525A)) {
			cr_update_phy16(cr, 0x97, 0xFFF7, 0);
			cr_update_phy16(cr, 0x97, 0xFFFF, 0x0008);
		}
	}

	return 0;
}

static int rts52xa_patch_sd40_phy(struct card_reader *cr, struct card *c)
{
	int ret = 0;

	rts_trace(TRACE_SD, "******Patch PHY ***********\n");

	if (cr->asic_code) {
		u16 val;
		cr_rd_phy16(cr, 0x9E, &val);
		val &= 0xbfff;
		cr_wr_phy16(cr, 0x9E, val);

		cr_wr_phy16(cr, 0x85, 0x04DF);
		cr_wr_phy16(cr, 0x90, 0x09F4);
		if (c->reset_flags & PATCH_SPECIAL_SD40_PHY)
			cr_wr_phy16(cr, 0x82, cr->hw_param.phy0x82_val_patch);
		else
			cr_wr_phy16(cr, 0x82, cr->hw_param.phy0x82_val);
		cr_wr_phy16(cr, 0x85, cr->hw_param.phy0x85_val);

		/* change OOBS to max if PATCH_SPECIAL_SD40_PHY enabled */
		if (c->reset_flags & PATCH_SPECIAL_SD40_PHY)
			cr_wr_phy16(cr, 0x87, 0xEC69);
		else
			cr_wr_phy16(cr, 0x87, cr->hw_param.phy0x87_val);

		cr_rd_phy16(cr, 0x86, &val);
		val &= ~(3 << 8);
		cr_wr_phy16(cr, 0x86, val);

		cr_wr_phy16(cr, 0x88, cr->hw_param.phy0x88_val);
		cr_wr_phy16(cr, 0x92, cr->hw_param.phy0x92_val);

		cr_rd_phy16(cr, 0x8B, &val);
		val |= 1 << 3;
		cr_wr_phy16(cr, 0x8B, val);

		cr_rd_phy16(cr, 0x9F, &val);
		val |= 0x0004;
		cr_wr_phy16(cr, 0x9F, val);

		cr_rd_phy16(cr, 0xA2, &val);
		val &= 0xFFE0;
		cr_wr_phy16(cr, 0xA2, val);
	} else {
		if (!(check_sd40_flag(cr, SD40_PHISON_TEST)))
			ret = cr_wr_phy16(cr, 0x86, 0x554D);
	}

	return ret;
}

static int rts5250_patch_sd40_phy(struct card_reader *cr, struct card *c)
{
	int ret = 0;

	rts_trace(TRACE_SD, "******Patch PHY ***********\n");

	if (cr->asic_code) {
		u16 val;
		cr_rd_phy16(cr, 0x9E, &val);
		val &= 0xbfff;
		cr_wr_phy16(cr, 0x9E, val);

		cr_wr_phy16(cr, 0x85, 0x0497);
		cr_wr_phy16(cr, 0x90, 0x09F4);
		if (c->reset_flags & PATCH_SPECIAL_SD40_PHY)
			cr_wr_phy16(cr, 0x82, cr->hw_param.phy0x82_val_patch);
		else
			cr_wr_phy16(cr, 0x82, cr->hw_param.phy0x82_val);
		cr_wr_phy16(cr, 0x85, cr->hw_param.phy0x85_val);

		/* change OOBS to max if PATCH_SPECIAL_SD40_PHY enabled */
		if (c->reset_flags & PATCH_SPECIAL_SD40_PHY)
			cr_wr_phy16(cr, 0x87, 0x6C69);
		else
			cr_wr_phy16(cr, 0x87, cr->hw_param.phy0x87_val);

		cr_rd_phy16(cr, 0x86, &val);
		val &= ~(3 << 8);
		cr_wr_phy16(cr, 0x86, val);

		cr_wr_phy16(cr, 0x88, cr->hw_param.phy0x88_val);
		cr_wr_phy16(cr, 0x92, cr->hw_param.phy0x92_val);

		cr_rd_phy16(cr, 0x8B, &val);
		val |= 1 << 3;
		cr_wr_phy16(cr, 0x8B, val);

		cr_rd_phy16(cr, 0xA1, &val);
		val |= 3 << 2;
		cr_wr_phy16(cr, 0xA1, val);

		cr_rd_phy16(cr, 0x9F, &val);
		val |= 0x0004;
		cr_wr_phy16(cr, 0x9F, val);
	} else {
		if (!(check_sd40_flag(cr, SD40_PHISON_TEST)))
			ret = cr_wr_phy16(cr, 0x86, 0x554D);
	}

	return ret;
}

static int rts5249_patch_sd40_phy(struct card_reader *cr, struct card *c)
{
	int ret = 0;

	rts_trace(TRACE_SD, "******Patch PHY ***********\n");

	if (cr->asic_code) {
		cr_wr_phy16(cr, 0x85, 0x2497);
		cr_wr_phy16(cr, 0x90, 0x09C6);
		if (c->reset_flags & PATCH_SPECIAL_SD40_PHY)
			cr_wr_phy16(cr, 0x82, cr->hw_param.phy0x82_val_patch);
		else
			cr_wr_phy16(cr, 0x82, cr->hw_param.phy0x82_val);
		cr_wr_phy16(cr, 0x85, cr->hw_param.phy0x85_val);

		/* change OOBS to max if PATCH_SPECIAL_SD40_PHY enabled */
		if (c->reset_flags & PATCH_SPECIAL_SD40_PHY)
			cr_wr_phy16(cr, 0x87, 0x7469);
		else if (CHECK_VERSION_HIGHER_THAN(cr, 0x5249, IC_VER_D))
			cr_wr_phy16(cr, 0x87, 0x7469);
		else
			cr_wr_phy16(cr, 0x87, cr->hw_param.phy0x87_val);

		cr_wr_phy16(cr, 0x88, cr->hw_param.phy0x88_val);
		cr_wr_phy16(cr, 0x92, cr->hw_param.phy0x92_val);

		if (CHECK_VERSION_HIGHER_THAN(cr, 0x5249, IC_VER_D)) {
			u16 val = 0;

			cr_wr_phy16(cr, 0x8A, 0x61CF);
			cr_wr_phy16(cr, 0x9F, 0xFF24);

			cr_rd_phy16(cr, 0x86, &val);
			val |= 0x0C;
			cr_wr_phy16(cr, 0x86, val);
		} else {
			cr_wr_phy16(cr, 0x8A, 0x61CD);
			cr_wr_phy16(cr, 0x9F, 0xFF20);
		}

	} else {
		if (!(check_sd40_flag(cr, SD40_PHISON_TEST)))
			ret = cr_wr_phy16(cr, 0x86, 0x554D);
	}

	return ret;
}

static int rts52xx_patch_sd40_phy(struct card_reader *cr, struct card *c)
{
	if (CHECK_PID(cr, 0x5249))
		return rts5249_patch_sd40_phy(cr, c);
	else if (CHECK_PID(cr, 0x5250))
		return rts5250_patch_sd40_phy(cr, c);
	else if (CHECK_PID(cr, 0x524A) || CHECK_PID(cr, 0x525A))
		return rts52xa_patch_sd40_phy(cr, c);
	else
		return -1;

}

static int rts5249_check_sd40_status3(struct card_reader *cr)
{
	u8 val;

	rts_trace(TRACE_SD, "Check SD40_STATUS3\n");

	cr_wr_reg(cr, REG_SD40_TMOUT_1, SD40_TIME_OUT_ERR_EN, 0x00);
	cr_wr_reg(cr, REG_SD40_CTRL, SD40_GO_HIBERNATE, SD40_GO_HIBERNATE);
	udelay(2);

	cr_init_cmd(cr);

	val = SD40_PHY_RST | SD40_PHY_ISO;
	val |= SD40_EXIT_DORMANT | SD40_GO_HIBERNATE;
	cr_add_cmd(cr, WRITE_REG_CMD, REG_SD40_CTRL, 0xFF, val);

	cr_add_cmd(cr, CHECK_REG_CMD, REG_SD40_STATUS3, DLSM_STATE_MASK, 0x20);

	if (cr_send_cmd(cr, SD_ID, 100) < 0) {

		cr_rd_reg(cr, REG_SD40_TRAN, &val);
		if (0x08 == val)
			rts_trace(TRACE_ERROR, "SD40_STATUS3:STB.L Fail!\n");
		else if (0x10 == val)
			rts_trace(TRACE_ERROR, "SD40_STATUS3:SYNC Fail!\n");
		else
			rts_trace(TRACE_ERROR, "Check SD40_STATUS3 Fail!\n");

		cr_wr_reg(cr, REG_SD40_TMOUT_1, SD40_TIME_OUT_ERR_EN,
							SD40_TIME_OUT_ERR_EN);
		TRACE_RET(cr, STATUS_FAIL);
	}

	cr_wr_reg(cr, REG_SD40_TMOUT_1, SD40_TIME_OUT_ERR_EN,
							SD40_TIME_OUT_ERR_EN);

	return 0;
}

static int rts5250_check_sd40_status3(struct card_reader *cr)
{
	u8 val, i;
	u16 tmp;

	rts_trace(TRACE_SD, "Check SD40_STATUS3\n");

	cr_wr_reg(cr, REG_SD40_TMOUT_1, SD40_TIME_OUT_ERR_EN, 0x00);
	cr_wr_reg(cr, REG_SD40_CTRL, SD40_GO_HIBERNATE, SD40_GO_HIBERNATE);
	udelay(2);

	val = SD40_PHY_RST | SD40_PHY_ISO;
	val |= SD40_EXIT_DORMANT | SD40_GO_HIBERNATE;
	cr_wr_reg(cr, REG_SD40_CTRL, 0xFF, val);

	for (i = 0; i < 100; i++) {
		cr_rd_reg(cr, REG_SD40_STATUS3, &val);
		if ((val & 0x04) == 0x04)
			break;
		udelay(5);
	}
	if (50 == i)
		TRACE_RET(cr, STATUS_FAIL);

	if (cr->asic_code) {
		cr_rd_phy16(cr, 0x9E, &tmp);
		tmp |= 0x4000;
		cr_wr_phy16(cr, 0x9E, tmp);
	}

	cr_init_cmd(cr);
	cr_add_cmd(cr, CHECK_REG_CMD, REG_SD40_STATUS3, DLSM_STATE_MASK, 0x20);

	if (cr_send_cmd(cr, SD_ID, 100) < 0) {

		cr_rd_reg(cr, REG_SD40_TRAN, &val);
		if (0x08 == val)
			rts_trace(TRACE_ERROR, "SD40_STATUS3:STB.L Fail!\n");
		else if (0x10 == val)
			rts_trace(TRACE_ERROR, "SD40_STATUS3:SYNC Fail!\n");
		else
			rts_trace(TRACE_ERROR, "Check SD40_STATUS3 Fail!\n");

		if (CHECK_PID(cr, 0x5250))
			cr_wr_reg(cr, REG_SD40_TMOUT_1, SD40_TIME_OUT_ERR_EN,
							SD40_TIME_OUT_ERR_EN);
		TRACE_RET(cr, STATUS_FAIL);
	}

	cr_wr_reg(cr, REG_SD40_TMOUT_1, SD40_TIME_OUT_ERR_EN,
							SD40_TIME_OUT_ERR_EN);

	return 0;
}

static int rts52xx_check_sd40_status3(struct card_reader *cr)
{

	if (CHECK_PID(cr, 0x5249))
		return rts5249_check_sd40_status3(cr);
	else if (CHECK_PID(cr, 0x5250) ||
		CHECK_PID(cr, 0x524A) || CHECK_PID(cr, 0x525A))
		return rts5250_check_sd40_status3(cr);

	return -1;
}

static int rts5249_config_eidle(struct card_reader *cr)
{
	if (check_dev_flag(cr, CONFIG_EIDLE_DEGLITCH)) {
		u16 val = 0;

		if (CHECK_VERSION(cr, 0x5249, IC_VER_A)) {
			val = 0xFF08;
		} else {
			cr_rd_phy16(cr, 0x9F, &val);
			val &= 0xFF0F;
		}

		val |= (cr->hw_param.eidle_num & 0x0F) << 4;
		val |= (1 << 3);
		cr_wr_phy16(cr, 0x9F, val);
	}

	return 0;
}

static int rts5250_config_eidle(struct card_reader *cr)
{
	if (check_dev_flag(cr, CONFIG_EIDLE_DEGLITCH)) {
		u16 val = 0;

		/* phy0x9F[8~9]: config min threshold
		 * of continuous electric idle
		 */
		cr_rd_phy16(cr, 0x9F, &val);

		val &= ~(3 << 8);
		val |= (cr->hw_param.eidle_num & 0x03) << 8;

		cr_wr_phy16(cr, 0x9F, val);
	}

	return 0;
}

static int rts52xx_config_eidle(struct card_reader *cr)
{

	if (CHECK_PID(cr, 0x5249))
		return rts5249_config_eidle(cr);
	else if (CHECK_PID(cr, 0x5250)
		|| CHECK_PID(cr, 0x524A) || CHECK_PID(cr, 0x525A))
		return rts5250_config_eidle(cr);

	return -1;
}

int rts5249_init_sd40_hw(struct card_reader *cr, struct card *c)
{
	u16 phy = 0;

	if (CHECK_VERSION(cr, 0x5249, IC_VER_A)
			|| CHECK_VERSION(cr, 0x5249, IC_VER_B)
			|| CHECK_VERSION(cr, 0x5249, IC_VER_C)) {
		if (cr->option.sd40_ctl & PATCH_SPECIAL_UHSII)
			c->runtime_flags |= PATCH_TOSHIBA_UHSII;
		else
			c->runtime_flags &= ~PATCH_TOSHIBA_UHSII;

		c->runtime_flags |= FORBIT_UHSII_UNSPECIFIED_LEN_MODE;
	} else {
		c->runtime_flags &= ~PATCH_TOSHIBA_UHSII;
	}

	if (rts52xx_patch_sd40_phy(cr, c))
		TRACE_RET(cr, STATUS_FAIL);

	if (rts5249_patch_sd40_hw(cr, c))
		TRACE_RET(cr, STATUS_FAIL);

	if (rts52xx_check_sd40_status3(cr))
		TRACE_RET(cr, STATUS_FAIL);

	if (check_sd40_flag(cr, SD40_PHY_TEST_MODE)) {
		u8 val = SD40_PHY_RST | SD40_PHY_ISO;

		rts_trace(TRACE_SD, "PHY Test mode\n");
		cr_wr_reg(cr, REG_SD40_CTRL, 0xFF, val);
	}

	if (cr->asic_code && check_sd40_flag(cr, SD40_USE_EXT_CLOCK)) {
		rts_trace(TRACE_CR, "SD40 using extra clock\n");
		if (CHECK_VERSION(cr, 0x5249, IC_VER_A))
			phy = 0x80;
		else
			cr_rd_phy16(cr, 0x9E, &phy);

		phy |= 0x0480;
		cr_wr_phy16(cr, 0x9E, phy);
		cr_wr_reg(cr, REG_RCCTL, SD40_EXTCLK_SEL_MASK, SD40_EXTCLK_SEL);
	}

	if (cr->asic_code) {
		if (CHECK_VERSION(cr, 0x5249, IC_VER_A))
			phy = 0x80;
		else
			cr_rd_phy16(cr, 0x9E, &phy);
		phy |= (1 << 13);
		cr_wr_phy16(cr, 0x9E, phy);

		return rts52xx_config_eidle(cr);
	}

	return 0;
}

int rts5249_init_sd40_hw_m(struct card_reader *cr, struct card *card)
{
	struct cr_option *option = &(cr->option);
	struct sd_info *sd_card = card->extra;
	u8 n_fcu = sd_card->setting.host.n_fcu;

	if (cr->asic_code) {
		if (SPEED_SEL_RANGE_B == sd_card->setting.host.speed_range) {
			int ver_a = CHECK_VERSION(cr, 0x5249, IC_VER_A);
			/* is_ABC |= CHECK_VERSION(cr, 0x5249, IC_VER_B); */
			/* is_ABC |= CHECK_VERSION(cr, 0x5249, IC_VER_C); */

			if (card->reset_flags & PATCH_SPECIAL_SD40_PHY) {
				if (ver_a) {
					cr_wr_phy16(cr, 0x90, 0x9C6);
				} else {
					u16 val;
					cr_rd_phy16(cr, 0x90, &val);
					val &= 0xFFF1;
					val |= 0x06;
					cr_wr_phy16(cr, 0x90, val);
				}
			} else {
				if (ver_a) {
					cr_wr_phy16(cr, 0x90, cr->hw_param.phy0x90_val);
				} else {
					u16 val1, val2;
					val1 = cr->hw_param.phy0x90_val & 0x0E;
					cr_rd_phy16(cr, 0x90, &val2);
					val2 &= 0xFFF1;
					val2 |= val1;
					cr_wr_phy16(cr, 0x90, val2);
				}
			}
		}

		if (check_sd40_flag(cr, SD40_EN_EBNUM)) {
			u16 val;

			if (CHECK_VERSION(cr, 0x5249, IC_VER_A))
				val = 0x46;
			else
				cr_rd_phy16(cr, 0x91, &val);

			val &= 0x3f8f;
			val |= option->sd40_ebnum;
			cr_wr_phy16(cr, 0x91, val);
		}

		if (check_sd40_flag(cr, SD40_SSC_EN)) {
			u16 val;

			if (CHECK_VERSION(cr, 0x5249, IC_VER_A))
				val = 0x00;
			else
				cr_rd_phy16(cr, 0x81, &val);
			val |= 0x4000;
			cr_wr_phy16(cr, 0x81, val);
		}
	}

	/* TODO: consider n_fcu > 64 in future */
	if (64 == n_fcu) {
		cr_wr_reg(cr, REG_SD40_MISC, SD40_RETRY_EN, 0);
		if (CHECK_PID(cr, 0x5250)) {
			if(check_dev_flag(cr, SD40_WRITE_SPEED_UP_EN)) {
				cr_wr_reg(cr, SD40_HD_SPEED_UP,
					SD40_WR_SPEED_UP_MASK,
					SD40_WR_SPEED_UP_EN);
				cr_wr_reg(cr, REG_RBCTL,
					U_SD40_WRITE_SPEED_UP,
					U_SD40_WRITE_SPEED_UP);
				cr_wr_reg(cr, REG_PCLK_CTL,
					WRITE_BOUNDARY_BUG_FIX, 0);
			}
		} else if (CHECK_PID(cr, 0x525a)) {
			if(check_dev_flag(cr, SD40_WRITE_SPEED_UP_EN)) {
				cr_wr_reg(cr, SD40_HD_SPEED_UP,
					SD40_WR_SPEED_UP_MASK,
					SD40_WR_SPEED_UP_EN);
			}
		}
	} else {
		if (CHECK_PID(cr, 0x524a) && (16 == n_fcu)) {
			cr_wr_reg(cr, REG_SD40_MISC, SD40_RETRY_EN, 0);
			if(check_dev_flag(cr, SD40_WRITE_SPEED_UP_EN)) {
				cr_wr_reg(cr, SD40_HD_SPEED_UP,
					SD40_WR_SPEED_UP_MASK,
					SD40_WR_SPEED_UP_EN);
			}
		}
		else {
			cr_wr_reg(cr, REG_SD40_MISC, SD40_RETRY_EN, SD40_RETRY_EN);
		}
	}

	/* modify read max payload size */
	if (check_dev_flag(cr, EN_PCIE_MAX_READ_REQ_SIZE)) {
		u16 val;
		int devctl = cr->dev_pci_cap_offset + PCI_EXP_DEVCTL;

		/* TODO: get addr of PCIE DEVICE CONTROL REGISGER by
		 * parsing config space.
		 */
		cr_read_config16(cr, devctl, &val);
		val &= 0x8FFF;
		val |= 0x5000;
		cr_write_config16(cr, devctl, val);
	}

	if (cr->asic_code &&
		(CHECK_PID(cr, 0x5250) || CHECK_PID(cr, 0x524A)
		|| CHECK_PID(cr, 0x525A))) {
		u16 val = 0;

		cr_rd_phy16(cr, 0xA2, &val);
		val |= 0x1F;
		cr_wr_phy16(cr, 0xA2, val);
	}

	return 0;
}

int rts5249_uninit_sd40_hw(struct card_reader *cr, struct card *c)
{
	struct sd_info *sd_card = c->extra;
	int mode = SD40_OFF_FORCE;
	u16 addr;
	u8  val;

	cr_wr_reg(cr, REG_CARD_CLK_EN, (SD40_CLK_EN | SD_CLK_EN), SD40_CLK_EN);
	if (CHECK_PID(cr, 0x5250)) {
		if(check_dev_flag(cr, SD40_WRITE_SPEED_UP_EN)) {
			cr_wr_reg(cr, SD40_HD_SPEED_UP,
				SD40_WR_SPEED_UP_MASK, SD40_WR_SPEED_UP_DIS);
			cr_wr_reg(cr, REG_RBCTL, U_SD40_WRITE_SPEED_UP, 0);
			cr_wr_reg(cr, REG_PCLK_CTL,
				WRITE_BOUNDARY_BUG_FIX, WRITE_BOUNDARY_BUG_FIX);
		}
	}

	if (CHECK_PID(cr, 0x524a) || CHECK_PID(cr, 0x525a)) {
		if(check_dev_flag(cr, SD40_WRITE_SPEED_UP_EN))
			cr_wr_reg(cr, SD40_HD_SPEED_UP,
				SD40_WR_SPEED_UP_MASK, SD40_WR_SPEED_UP_DIS);
	}

	/* Select Speed Range 00b */
	addr = ((SD40_IOADR_CFG_BASE << 8) + SD40_IOADR_PHY_SET_L) << 2;
	cr_read_host_sd40_iospace(cr, addr, &val, 1);
	val &= 0x3F;
	cr_write_host_sd40_iospace(cr, addr, &val, 1);
	sd_card->setting.host.speed_range = SPEED_SEL_RANGE_A;

	if (((mode == SD40_OFF_NORMAL) && (CHK_SD_UHSII(sd_card)))
 || (mode == SD40_OFF_FORCE)) {

		rts_trace(TRACE_SD, "***Reset SD40 Module***\n");


		if (CHECK_VERSION_HIGHER_THAN(cr, 0x5249, IC_VER_D)) {
			u16 val;

			/* clear extend eidle */
			cr_rd_phy16(cr, 0x90, &val);
			val &= 0xFFEF;
			cr_wr_phy16(cr, 0x90, val);
		}

		cr_init_cmd(cr);

		/* step 1: switch clock to CRC clock */
		cr_add_cmd(cr, WRITE_REG_CMD, REG_CLK_CTL,
				(RTS5249_CHANGE_SD40_PCLK | CLK_LOW_FREQ),
				CLK_LOW_FREQ);
		if (CHECK_PID(cr, 0x524a) || CHECK_PID(cr, 0x525a))
			cr_add_cmd(cr, WRITE_REG_CMD, SD40_STOP_CTL , SD40_LINK_RST,
								SD40_LINK_RST);
		cr_add_cmd(cr, WRITE_REG_CMD, REG_SD40_CTRL, SD40_MAC_RST,
								SD40_MAC_RST);

		/* step 2: Disable and reset SD40 PHY. */
		cr_add_cmd(cr, WRITE_REG_CMD, REG_SD40_CTRL,
					(SD40_PHY_RST | SD40_PHY_ISO
					 | SD40_DLSM_RST | SD40_TSSM_RST),
					SD40_DLSM_RST|SD40_TSSM_RST);

		cr_add_cmd(cr, WRITE_REG_CMD, REG_SD40_GEN_SETTING_7,
						SD40_CONFIG_COMPLETE, 0x00);
		cr_add_cmd(cr, WRITE_REG_CMD, REG_CARD_CLK_EN,
					(SD_CLK_EN | SD40_CLK_EN), SD_CLK_EN);
		cr_add_cmd(cr, WRITE_REG_CMD, REG_CARD_SELECT,
						CARD_MOD_MASK, SD_MOD_SEL);
		cr_add_cmd(cr, WRITE_REG_CMD, RTS5249_SD40_LDO_CTL,
					SD40_LDO12AS_MASK, SD40_LD012AS_OFF);

		cr_add_cmd(cr, WRITE_REG_CMD, REG_CARD_PWR_CTL,
			RTS5249_SD40_PAD_ISO, RTS5249_SD40_PAD_ISO_DEFAULT);

		if (cr_send_cmd(cr, SD_ID, 100))
			TRACE_RET(cr, STATUS_FAIL);
	}

	if (cr->asic_code &&
		(CHECK_PID(cr, 0x5250) || CHECK_PID(cr, 0x524A)
		|| CHECK_PID(cr, 0x525A))) {
		u16 val = 0;

		cr_rd_phy16(cr, 0x8B, &val);
		val &= ~(1 << 3);
		cr_wr_phy16(cr, 0x8B, val);

		cr_rd_phy16(cr, 0xA2, &val);
		val &= 0xFFE0;
		cr_wr_phy16(cr, 0xA2, val);
	}

	return 0;
}

static int wait_rb_complete(struct card *card)
{
	struct card_reader *cr = card->cm->cr;
	struct sd_info *sd_card = card->extra;
	int i = 0;
	u8 rb_ctl = 0, status1 = 0, status2 = 0, status3 = 0;

	i = 5000;
	do {
		if (cr_rd_reg(cr, REG_RBCTL, &rb_ctl))
			TRACE_RET(cr, STATUS_FAIL);

		if (cr->option.sd40_seq_speedup_en) {
			if ((PRE_READ == sd_card->rw_mode) && (RB_FULL & rb_ctl)) {
				u8 status3 = 0;
				int dlsm = 0;

				rts_trace(TRACE_SD, "Wait RB full break!\n");

				dlsm = CHECK_VERSION_LOWER_THAN(cr, 0x5249, IC_VER_E);
				dlsm |=  CHECK_VERSION_LOWER_THAN(cr, 0x5250, IC_VER_D);

				if (dlsm) {
					if (cr_rd_reg(cr, REG_SD40_STATUS3, &status3))
						TRACE_RET(cr, STATUS_FAIL);
					if(0x5 == ((status3 & 0xF8) >> 3))
						break;
				} else {
					break;
				}
			} else if (PRE_READ == sd_card->rw_mode
				&& 0 != (sd_card->sd40_seq_sectors%sd_card->n_fcu)) {

				rts_trace(TRACE_SD, "Wait TPSM break!\n");

				if (cr_rd_reg(cr, REG_SD40_STATUS0, &status1))
					TRACE_RET(cr, STATUS_FAIL);

				if (cr_rd_reg(cr, REG_SD40_STATUS0, &status2))
					TRACE_RET(cr, STATUS_FAIL);

				if (cr_rd_reg(cr, REG_SD40_STATUS0, &status3))
					TRACE_RET(cr, STATUS_FAIL);

				if ((0x60 == (status1 & 0xF0))
						&& (0x60 == (status2 & 0xF0))
						&& (0x60 == (status3 & 0xF0)))
					break;
			}
		} else {
			if ((PRE_READ == sd_card->rw_mode) && (RB_FULL & rb_ctl)) {
				u8 status3 = 0;
				int dlsm = 0;

				dlsm = CHECK_VERSION_LOWER_THAN(cr, 0x5249, IC_VER_E);
				dlsm |=  CHECK_VERSION_LOWER_THAN(cr, 0x5250, IC_VER_D);

				if (dlsm) {
					if (cr_rd_reg(cr, REG_SD40_STATUS3, &status3))
						TRACE_RET(cr, STATUS_FAIL);
					if(0x5 == ((status3 & 0xF8) >> 3))
						break;
				} else {
					break;
				}
			}

		}

		if ((PRE_WRITE == sd_card->rw_mode) && (RB_EMPTY & rb_ctl)) {
			if (cr_rd_reg(cr, REG_SD40_STATUS0, &status1))
				TRACE_RET(cr, STATUS_FAIL);

			if (cr_rd_reg(cr, REG_SD40_STATUS0, &status2))
				TRACE_RET(cr, STATUS_FAIL);

			if ((0x60 == (status1 & 0xF0))
					&& (0x60 == (status2 & 0xF0)))
				break;
		}

		mdelay(1);
	} while (--i > 0);

	rts_trace(TRACE_SD, "RBCTL:%#x SD40_STATUS0:%#x", rb_ctl, status1);
	rts_trace(TRACE_SD, "sd_card->sd40_seq_sectors=%#x \n",
		sd_card->sd40_seq_sectors);

	if (i <= 0) {
		rts_trace(TRACE_SD, "wait tpsm fail\n");
		return -ETIMEDOUT;
	} else {
		return 0;
	}
}

int rts5249_sd_stop_sequential(struct card *card)
{
	struct sd_info *sd_card = card->extra;
	struct card_reader *cr = card->cm->cr;
	int ret  = 0;
	int error = 0;

	rts_trace(TRACE_SD, "rw_mode:%#x, prev_dir:%#x\n", sd_card->rw_mode,
			sd_card->prev_dir);
	if ((DMA_FROM_DEVICE == sd_card->prev_dir
			&& PRE_READ == sd_card->rw_mode)
			|| (DMA_TO_DEVICE == sd_card->prev_dir
			&& PRE_WRITE == sd_card->rw_mode)) {
		u32 reg, value;

		error = wait_rb_complete(card);
		sd_card->sd40_seq_sectors = 0;
		cr_wr_reg(cr, REG_PRE_RW_MODE, EN_INFINITE_MODE, 0);

		value = cr_ioread32(cr, HCBCTLR);
		cr_iowrite32(cr, HCBCTLR, value | STOP_CMD);
		value = cr_ioread32(cr, HDBCTLR);
		cr_iowrite32(cr, HDBCTLR, value | STOP_DMA);

		if ((card->runtime_flags & PATCH_TOSHIBA_UHSII)
			&& CHECK_VERSION(cr, 0x5249, IC_VER_C)
			&& (cr->option.sd40_ctl & FIX_CMD12_ISSUE)) {
			u16 val;

			cr_rd_phy16(cr, 0x9E, &val);
			val |= (1 << 1);
			cr_wr_phy16(cr, 0x9E, val);

			cr_rd_phy16(cr, 0x8A, &val);
			val |= (1 << 1);
			cr_wr_phy16(cr, 0x8A, val);

			cr_rd_phy16(cr, 0x9F, &val);
			val |= (1 << 2);
			cr_wr_phy16(cr, 0x9F, val);

			udelay(100);
		}

		if (CHK_SD_UHSII(sd_card)) {
			if (CHECK_PID(cr, 0x524a) || CHECK_PID(cr, 0x525a))
				cr_wr_reg(cr, SD40_STOP_CTL , SD40_LINK_RST, SD40_LINK_RST);
			cr_wr_reg(cr, REG_SD40_CTRL,
					SD40_MAC_RST, SD40_MAC_RST);
		}

		if ((card->runtime_flags & PATCH_TOSHIBA_UHSII)
			&& CHECK_VERSION(cr, 0x5249, IC_VER_C)
			&& ((cr->option.sd40_ctl & FIX_CMD12_ISSUE))) {
			u16 val;

			cr_rd_phy16(cr, 0x9E, &val);
			val &= ~(1 << 1);
			cr_wr_phy16(cr, 0x9E, val);

			cr_rd_phy16(cr, 0x8A, &val);
			val &= ~(1 << 1);
			cr_wr_phy16(cr, 0x8A, val);

			cr_rd_phy16(cr, 0x9F, &val);
			val &= ~(1 << 2);
			cr_wr_phy16(cr, 0x9F, val);

			mdelay(5);
		}

		sd_clear_sd_error(cr);
		cr_wr_reg(cr, REG_DMACTL, DMA_RST, DMA_RST);

		reg = cr->hw_param.interrupt_en & (~DATA_DONE_INT_EN);
		cr_enable_bus_int(cr, reg);
	}

	/*
	 * For PQI MMC 32MB burn in fail:
	 * Use our own burn in tool, random burn in,
	 * write error will occur. The stop of previous read command
	 * is sent too early so that the following write command will
	 * fail. The phenomina is write one page data to card and
	 * no CRC status from card. So we add one dummy cmd13 between
	 * read command and stop command random rw,
	 * so prev_sec_cnt < 0x80
	 */
	if ((sd_card->prev_sec_cnt < 0x80)
		&& (sd_card->prev_dir == DMA_FROM_DEVICE)
		&& !CHK_SD_UHS(sd_card)
		&& !CHK_SD_HS(sd_card)
		&& !CHK_MMC_HS(sd_card))
		sd_clear_status(card);

	ret = sd_stop_transmission(card);
	if (ret) {
		card->rw_need_retry = 1;
		sd_set_error_code(cr, card, SD_STS_ERR);
		sd_set_error_code(cr, card, SD_CMD12_ERR);
		return ret;
	}

	/* For a SDXC card:
	 * Sometimes this card responds cmd12 too quickly, and this would
	 * cause next cmd18 fail. So delay 50us here
	 */

	udelay(50);

	sd_card->rw_mode = NORMAL_RW;
	ret = cr_wr_reg(cr, REG_RBCTL, RB_FLUSH, RB_FLUSH);
	if (ret) {
		sd_set_error_code(cr, card, SD_IO_ERR);
		return ret;
	}
	/*
	 * For PRETEC MMC 32MB card, we shall delay a moment
	 * after "STOP" command before next R/W command.
	 */
	/* random rw, so prev_sec_cnt < 0x80 */
	if ((sd_card->prev_sec_cnt < 0x80) && !CHK_SD30_SPEED(sd_card)
			&& !CHK_SD_HS(sd_card) && !CHK_MMC_HS(sd_card))
		sd_clear_status(card);

	if (error) {
		rts_trace(TRACE_SD, "Wait TPSM and RB complete fail\n");
		return error;
	}

	return 0;
}

static u8 TOSHIBA_special_uhsii[6] = {
	0x02, 0x54, 0x4D, 0x58, 0x41, 0x30
};

static int rts5249_patch_TOSHIBA(struct card_reader *cr, struct card *card)
{
	if (card->runtime_flags & PATCH_TOSHIBA_UHSII) {
		rts_trace(TRACE_SD, "PATCH for TOSHIBA UHSII card\n");

		if (cr->option.sd40_ctl & FIX_CMD12_ISSUE) {
			card->runtime_flags &=
				~FORBIT_UHSII_UNSPECIFIED_LEN_MODE;
		} else {
			card->runtime_flags |=
				FORBIT_UHSII_UNSPECIFIED_LEN_MODE;
		}

	}

	return 0;
}

int rts5249_patch_special_sd(struct card_reader *cr, struct card *card)
{
	struct sd_info *sd_card = card->extra;

	if (!memcmp(sd_card->raw_cid, TOSHIBA_special_uhsii,
				sizeof(TOSHIBA_special_uhsii)))
		return rts5249_patch_TOSHIBA(cr, card);

	return 0;
}

/* end for SD */


/* for MS */

int rts5249_ms_pull_ctl_enable(struct card_reader *cr)
{
	cr_init_cmd(cr);
	cr_add_cmd(cr, WRITE_REG_CMD, REG_CARD_PULL_CTL4, 0xFF, 0x55);
	cr_add_cmd(cr, WRITE_REG_CMD, REG_CARD_PULL_CTL5, 0xFF, 0x55);
	cr_add_cmd(cr, WRITE_REG_CMD, REG_CARD_PULL_CTL6, 0xFF, 0x15);
	if (cr_send_cmd(cr, MS_CARD, 100))
		TRACE_RET(cr, STATUS_FAIL);

	return 0;
}

int rts5249_ms_pull_ctl_disable(struct card_reader *cr)
{
	cr_wr_reg(cr, REG_CARD_PULL_CTL4, 0xFF, 0x55);
	cr_wr_reg(cr, REG_CARD_PULL_CTL5, 0xFF, 0x55);
	cr_wr_reg(cr, REG_CARD_PULL_CTL6, 0xFF, 0x15);

	return 0;
}

/* end for MS */


/* for 5250 */

int rts5250_poweron_card(struct card_reader *cr, enum CARD_ID id)
{
	struct cr_option *option = &(cr->option);
	u8 mask, val;
	int uhsii = 0;
	int mmc_sdr200 = 0;

	if (option->ocp_en)
		cr_enable_ocp(cr);

	if ((id != SD_ID)) {
		uhsii = 0;
	} else {
		struct card *card = cm_get_card(cr->cm, SD_ID);
		if (card) {
			uhsii = !!(card->reset_flags & RESET_UHSII_NOW);
			mmc_sdr200 = !!(card->reset_flags & RESET_E_MMC45_NOW);
		} else {
			uhsii = 0;
			mmc_sdr200 = 0;
		}
	}

	if (CHECK_PID(cr, 0x525A)) {
		if (mmc_sdr200) {
			mask = REG_TUNE_VCC_MASK;
			val = REG_LDO_VCC_18V;
		} else {
			mask = REG_TUNE_VCC_MASK;
			val = REG_LDO_VCC_33V;
		}
		cr_wr_reg(cr, RTS525A_LDO_VCC_CFG1, mask, val);

		if (check_sd_flag(cr, EMMC45_DONT_SWITCH_POWER)) {
			mask = REG_TUNE_VIO_MASK;
			val = REG_LDO_VIO_18V;
			cr_wr_reg(cr, RTS525A_LDO_VIO_CFG, mask, val);
		}
	}
	else {
		if (mmc_sdr200) {
			mask = SD40_VCC_TUNE_MASK;
			val = SD_VCC_LDO_18V;
		} else {
			mask = SD40_VCC_TUNE_MASK;
			val = SD_VCC_LDO_33V;
		}
		cr_wr_reg(cr, RTS5249_SD40_LDO_CTL, mask, val);

		if (check_sd_flag(cr, EMMC45_DONT_SWITCH_POWER)) {
			mask = SD40_VIO_TUNE_MASK;
			val = SD_VIO_LDO_18V;
			cr_wr_reg(cr, RTS5250_SD40_LDO_CTL1, mask, val);
		}
	}

	cr_init_cmd(cr);
	if (uhsii || check_sd_flag(cr, EMMC45_DONT_SWITCH_POWER)) {
		mask = RTS5249_SD_VIO_PWR_MASK | RTS5249_SD_POWER_MASK;
		val = RTS5249_SD_VIO_PWR14_ON | RTS5249_SD_PWR14_ON;
		cr_add_cmd(cr, WRITE_REG_CMD, REG_CARD_PWR_CTL, mask, val);
		cr_add_cmd(cr, WRITE_REG_CMD, REG_PWR_GATE_CTRL,
				LDO3318_PWR_MASK, LDO_SUSPEND);
	} else {
		mask = RTS5249_SD_POWER_MASK;
		val = RTS5249_SD_PWR14_ON;
		cr_add_cmd(cr, WRITE_REG_CMD, REG_CARD_PWR_CTL, mask, val);
		cr_add_cmd(cr, WRITE_REG_CMD, REG_PWR_GATE_CTRL,
				LDO3318_PWR_MASK, LDO_SUSPEND);
	}

	if (cr_send_cmd(cr, 0, 100))
		TRACE_RET(cr, STATUS_FAIL);

	udelay(cr->option.pmos_pwr_on_interval);

	cr_init_cmd(cr);
	if (uhsii || check_sd_flag(cr, EMMC45_DONT_SWITCH_POWER)) {
		mask = RTS5249_SD_VIO_PWR_MASK | RTS5249_SD_POWER_MASK;
		val = RTS5249_SD_VIO_PWR_ON | RTS5249_SD_POWER_ON;
		cr_add_cmd(cr, WRITE_REG_CMD, REG_CARD_PWR_CTL, mask, val);
	} else {
		mask = RTS5249_SD_POWER_MASK;
		val = RTS5249_SD_POWER_ON;
		cr_add_cmd(cr, WRITE_REG_CMD, REG_CARD_PWR_CTL, mask, val);
		cr_add_cmd(cr, WRITE_REG_CMD, REG_PWR_GATE_CTRL,
				LDO3318_PWR_MASK, LDO_ON);
	}

	if (cr_send_cmd(cr, 0, 100))
		TRACE_RET(cr, STATUS_FAIL);

	return STATUS_SUCCESS;
}

int rts5249_do_before_auto_delink(struct card_reader *cr)
{
	if (CHECK_PID(cr, 0x5250) || CHECK_PID(cr, 0x524A))
		cr_wr_reg(cr, RTS5250_AUTOLOAD_CFG3, U_PM_D3_EN, 0);
	else
		cr_wr_reg(cr, AUTOLOAD_CFG3, U_PM_D3_EN, 0);

	if (check_dev_flag(cr, FREE_CLKREQ_AT_DELINK))
		cr_wr_reg(cr, 0xFF03, 0x80, 0x00);

	return 0;
}

int rts5249_sd40_set_power_control_mode(struct card *c, u8 mode)
{
	struct card_reader *cr = c->cm->cr;
	u8 payload[16], val;
	u16 addr;

	rts_trace(TRACE_SD, "SD40 Set Power Mode %#x\n", mode);

	if (CHECK_VERSION_HIGHER_THAN(cr, 0x5249, IC_VER_D)) {
		u16 phy;
		/* extend eidle */
		cr_rd_phy16(cr, 0x90, &phy);
		phy |= 0x10;
		cr_wr_phy16(cr, 0x90, phy);

		cr_rd_phy16(cr, 0x86, &phy);
		phy |= 0x80;
		cr_wr_phy16(cr, 0x86, phy);
	}

	if (!cr->asic_code) {
		if (mode == LOW_PWR_MODE)
			cr_wr_reg(cr, 0xEC44, 0xFF, 0xFF);
	}

	memset(payload, 0, sizeof(payload));
	payload[3] = mode;

	if (sd40_write_cfg_reg(c, SD40_IOADR_GEN_SET_L, payload, 4)) {
		rts_trace(TRACE_SD, "sd40 set power mode Fail\n");
		dump_all_sd40_register(cr);
		TRACE_RET(cr, STATUS_FAIL);
	}

	addr = ((SD40_IOADR_CFG_BASE << 8) + SD40_IOADR_GEN_SET_L) << 2;
	cr_read_host_sd40_iospace(cr, addr, &val, 1);
	val &= 0xFE;
	val |= mode;
	cr_write_host_sd40_iospace(cr, addr, &val, 1);

	return 0;
}

int rts5250_sd40_set_power_control_mode(struct card *c, u8 mode)
{
	struct card_reader *cr = c->cm->cr;
	u8 payload[16], val;
	u16 addr;

	rts_trace(TRACE_SD, "SD40 Set Power Mode %#x\n", mode);

	if (!cr->asic_code) {
		if (mode == LOW_PWR_MODE)
			cr_wr_reg(cr, 0xEC44, 0xFF, 0xFF);
	}

	memset(payload, 0, sizeof(payload));
	payload[3] = mode;

	if (sd40_write_cfg_reg(c, SD40_IOADR_GEN_SET_L, payload, 4)) {
		rts_trace(TRACE_SD, "sd40 set power mode Fail\n");
		dump_all_sd40_register(cr);
		TRACE_RET(cr, STATUS_FAIL);
	}

	addr = ((SD40_IOADR_CFG_BASE << 8) + SD40_IOADR_GEN_SET_L) << 2;
	cr_read_host_sd40_iospace(cr, addr, &val, 1);
	val &= 0xFE;
	val |= mode;
	cr_write_host_sd40_iospace(cr, addr, &val, 1);

	if ((cr->asic_code) && (mode == LOW_PWR_MODE)) {
		u16 phy;
		cr_rd_phy16(cr, 0xA1, &phy);
		phy |= 0x0008;
		cr_wr_phy16(cr, 0xA1, phy);

		cr_rd_phy16(cr, 0x85, &phy);
		phy &= 0xDFFF;
		cr_wr_phy16(cr, 0x85, phy);
	}

	return 0;
}

int rts5249_sd40_take_dev_init_process(struct card *card)
{
	struct card_reader *cr = card->cm->cr;
	int ret = 0;

	if (CHECK_VERSION(cr, 0x5249, IC_VER_E)
			|| CHECK_VERSION_HIGHER_THAN(cr, 0x5250, IC_VER_B)
			|| CHECK_PID(cr, 0x524A) || CHECK_PID(cr, 0x525A)) {
		cr_wr_reg(cr, REG_SD40_Dummy0, SD40_DEV_INIT_TO_EXT_EN,
					SD40_DEV_INIT_TO_EXT_EN);
	}

	ret = sd40_common_take_dev_init_process(card);

	if (CHECK_VERSION(cr, 0x5249, IC_VER_E)
			|| CHECK_VERSION_HIGHER_THAN(cr, 0x5250, IC_VER_B)
			|| CHECK_PID(cr, 0x524A) || CHECK_PID(cr, 0x525A)) {
		cr_wr_reg(cr, REG_SD40_Dummy0, SD40_DEV_INIT_TO_EXT_EN,
					SD40_DEV_INIT_TO_EXT_DIS);
	}

	return ret;
}

void rts5250_set_l1off_cfg_sub_D0(struct card_reader *cr, int active)
{
	struct cr_option *option = &(cr->option);
	u32 interrupt = cr_ioread32(cr, BIPR);
	int card_exist = (interrupt & SD_EXIST) | (interrupt & MS_EXIST);
	int aspm_L1_1, aspm_L1_2;

	aspm_L1_1 = check_dev_flag(cr, ASPM_L1_1_EN);
	aspm_L1_2 = check_dev_flag(cr, ASPM_L1_2_EN);

	if (active) {
		/* run, latency: 60us */
		if (aspm_L1_1) {
			u8 val = option->ltr_l1off_snooze_sspwrgate;
			if (check_dev_flag(cr,
					LTR_L1SS_PWR_GATE_CHECK_CARD_EN)) {
				if (card_exist)
					val &= ~(1 << 7);
				else
					val |= (1 << 7);
			}

			if (CHECK_PID(cr,0x5250)) {
				if (card_exist) {
					struct card *card = cm_get_card(cr->cm, SD_ID);
					struct sd_info *sd_card = NULL;

					if (card)
						sd_card = card->extra;
					if (sd_card && CHK_SD_UHSII(sd_card))
						val &= ~(1 << 3);
				}
			}

			cr_set_l1off_sub(cr, val);
			rts_trace(TRACE_PM, "ltr active and aspm L1.1,"
							"L1SUB:%#x\n", val);
		} else {
			cr_set_l1off_sub(cr, 0);
			rts_trace(TRACE_PM, "ltr active and not aspm L1.1,"
							"L1SUB:%#x\n", 0x00);
		}
	} else {
		/* l1off, latency: 300us */
		if (aspm_L1_2) {
			u8 val = option->ltr_l1off_sspwrgate;
			struct card *card = cm_get_card(cr->cm, SD_ID);

			if (check_dev_flag(cr,
					LTR_L1SS_PWR_GATE_CHECK_CARD_EN)) {
				if (card_exist)
					val &= ~(1 << 7);
				else
					val |= (1 << 7);
			}

			if (CHECK_PID(cr,0x5250)) {
				if (card_exist && card) {
					struct sd_info *sd_card = card->extra;
					if (CHK_SD_UHSII(sd_card))
						val &= ~(1 << 3);
				}
			}

			cr_set_l1off_sub(cr, val);
			rts_trace(TRACE_PM, "ltr idle and aspm L1.2,"
							"L1SUB:%#x\n", val);
		} else {
			cr_set_l1off_sub(cr, 0);
			rts_trace(TRACE_PM, "ltr idle and not aspm L1.2,"
							"L1SUB:%#x\n", 0x00);
		}
	}
}


void rts5250_process_ocp(struct card_reader *cr)
{
	struct list_head *pos;

	if (!cr->option.ocp_en)
		return;

	list_for_each(pos, &cr->cm->card_list) {
		struct card *c;
		c = list_entry(pos, struct card, link);
		if (cr->ocp_int) {
			cr_get_ocpstat(cr, &(cr->ocp_stat));
			if (SD_ID == c->id && check_card_flag(c, EXIST))
				sd_power_off_card3v3(cr);
			else if (MS_ID == c->id && check_card_flag(c, EXIST))
				ms_power_off_card3v3(cr);
			cr->ocp_int = 0;
		}
	}

	if (cr->release_map) {
		if (cr->ocp_stat & (SD_OC_NOW | SD_OC_EVER | SDVIO_OC_NOW |
					SDVIO_OC_EVER))
			cr_clear_ocpstat(cr);
		cr->ocp_stat = 0;
	}

	if (cr->ocp_stat & (SD_OC_NOW | SD_OC_EVER | SDVIO_OC_NOW |
				SDVIO_OC_EVER)) {
		rts_trace(TRACE_CR, "Over current, OCPSTAT is 0x%x\n",
			cr->ocp_stat);
		list_for_each(pos, &cr->cm->card_list) {
			struct card *c;
			c = list_entry(pos, struct card, link);
			if (SD_ID == c->id && check_card_flag(c, EXIST)) {
				cr_wr_reg(cr, REG_CARD_OE, SD_OUTPUT_EN, 0);
				set_card_flags(c, FAIL);
			} else if (MS_ID == c->id &&
					check_card_flag(c, EXIST)) {
				cr_wr_reg(cr, REG_CARD_OE, MS_OUTPUT_EN, 0);
				set_card_flags(c, FAIL);
			}
		}
		cr_poweroff_card(cr, SD_ID);
	}
}

int rts5249_release_ms_card(struct card_reader *cr, struct card *card)
{
	struct ms_info *ms_card = card->extra;

	rts_trace(TRACE_MS, "%s\n", __func__);

#ifdef MS_DELAY_WRITE
	ms_card->delay_write.delay_write_flag = 0;
#endif
	ms_card->pro_under_formatting = 0;
	clr_card_flags(card, (READY | FAIL | WP));
	ms_free_l2p_tbl(cr, card);
	memset(ms_card->raw_sys_info, 0, sizeof(ms_card->raw_sys_info));
	memset(ms_card->raw_ms_id, 0, sizeof(ms_card->raw_ms_id));
#ifdef SUPPORT_PCGL_1P18
	memset(ms_card->raw_model_name, 0, sizeof(ms_card->raw_model_name));
#endif

	if (ms_power_off_card3v3(cr))
		TRACE_RET(cr, STATUS_FAIL);

	if ((!CHECK_PID(cr, 0x5249)) && cr->option.ocp_en)
		cr_disable_ocp(cr);

	return 0;
}

int rts5249_sd_prepare_reset(struct card_reader *cr, struct card *card)
{
	struct sd_info *sd_card = card->extra;
	int retval;

	if (cr->asic_code)
		sd_card->sd_clock = 29;
	else
		sd_card->sd_clock = CLK_30;

	SET_SD(sd_card);
	sd_card->sd_addr = 0;
	sd_card->rw_mode = NORMAL_RW;
	sd_card->sd_data_buf_ready = 0;
	card->capacity = 0;
	sd_card->sd_lock_status = 0;
	sd_card->sd_erase_status = 0;
	card->capacity = 0;

	/* init for UHSII */
	sd_card->sd40_2L_HD = 0;
	sd_card->sd40_did = 0;
	sd_card->sd40_sid_tid = 0;
	sd_card->hibernate_mode_en = 0;
	sd_card->lan_mode = DM_FD;

	retval = sd_set_init_para(cr, card);
	if (retval != STATUS_SUCCESS)
		TRACE_RET(cr, retval);

	/* Initialize SD_CFG1 register */
	cr_wr_reg(cr, REG_SD_CFG1, 0xFF,
		SD_CLK_DIVIDE_128 | SD_20_MODE | SD_BUS_WIDTH_1);
	cr_wr_reg(cr, REG_SD_SAMPLE_POINT_CTL, 0xFF, SD20_RX_POS_EDGE);
	cr_wr_reg(cr, REG_SD_PUSH_POINT_CTL, 0xFF, 0);
	cr_wr_reg(cr, REG_CARD_STOP, SD_STOP | SD_CLR_ERR,
						SD_STOP | SD_CLR_ERR);

	/* Reset SD_CFG3 register */
	cr_wr_reg(cr, REG_SD_CFG3, SD30_CLK_END_EN, 0);
	cr_wr_reg(cr, REG_SD_STOP_SDCLK_CFG,
		SD30_CLK_STOP_CFG_EN | SD30_CLK_STOP_CFG1 | SD30_CLK_STOP_CFG0,
		0);

	/* TODO: REG_PRE_RW_MODE only make sense for 5249/5250, so
	 * implement this handler separately.
	 */
	cr_wr_reg(cr, REG_PRE_RW_MODE, EN_INFINITE_MODE, 0);
	if (cr_select_card(cr, card))
		TRACE_RET(cr, STATUS_FAIL);

	return 0;
}

static void rts5250_thermal_state_to_bypass_current(struct card_reader *cr,
		struct card *card)
{
	struct rts5249_info *info = cr->extra;

	if(info->thermal_state >= THERMAL_STATE_TO_BYPASS_CURRENT)
		return;

	if (!rtsx_chk_stat(cr, RTSX_STAT_RUN)) {
		cr_set_stat(cr, RTSX_STAT_RUN);
		return;
	}

	rts5249_sd_cleanup_work(cr, card);
	rts_trace(TRACE_SD, "switch bypass current\n");
	card->reset_flags |= UHSII_TO_BYPASS_CURRENT;
	sd40_uninit_hw(cr, card);
	if (sd_reset_card(cr, card) == STATUS_SUCCESS)
		info->thermal_state = THERMAL_STATE_TO_BYPASS_CURRENT;
}

static void rts5250_thermal_state_to_range_a(struct card_reader *cr,
		struct card *card)
{
	struct rts5249_info *info = cr->extra;

	if(info->thermal_state >= THERMAL_STATE_TO_RANGE_A)
		return;

	if (!rtsx_chk_stat(cr, RTSX_STAT_RUN)) {
		cr_set_stat(cr, RTSX_STAT_RUN);
		return;
	}

	rts5249_sd_cleanup_work(cr, card);
	rts_trace(TRACE_SD, "go to range A\n");
	card->reset_flags |= UHSII_TO_RANGE_A;
	sd40_uninit_hw(cr, card);
	if (sd_reset_card(cr, card) == STATUS_SUCCESS)
		info->thermal_state = THERMAL_STATE_TO_RANGE_A;
}

static void rts5250_thermal_state_to_uhsi(struct card_reader *cr,
		struct card *card)
{
	struct rts5249_info *info = cr->extra;
	struct sd_info * sd_card = card->extra;

	if(info->thermal_state >= THERMAL_STATE_TO_UHSI)
		return;

	if (!rtsx_chk_stat(cr, RTSX_STAT_RUN)) {
		cr_set_stat(cr, RTSX_STAT_RUN);
		return;
	}

	rts5249_sd_cleanup_work(cr, card);
	cr_set_card_status(cr, CARD_THERMAL_JUMP_TO_SDR50);
	rts_trace(TRACE_SD, "go direct to SDR50\n");

	card->reset_flags &= ~UHSII_TO_BYPASS_CURRENT;
	card->reset_flags &= ~UHSII_TO_RANGE_A;
	card->reset_flags |= GO_SD_LEGACY_DIRECT;
	if ((CHK_SD_UHSII(sd_card)))
		sd40_uninit_hw(cr, card);

	if (sd_reset_card(cr, card) == STATUS_SUCCESS)
		info->thermal_state = THERMAL_STATE_TO_UHSI;
}

static void rts5250_thermal_state_to_normal(struct card_reader *cr, struct card *card)
{
	struct cr_option *option = &cr->option;
	struct rts5249_info *info = cr->extra;

	if (info->thermal_state == THERMAL_STATE_TO_NORMAL)
		return;

	if (!rtsx_chk_stat(cr, RTSX_STAT_RUN)) {
		cr_set_stat(cr, RTSX_STAT_RUN);
		return;
	}

	rts5249_sd_cleanup_work(cr, card);
	rts_trace(TRACE_SD, "go back to normal(sd40) mode\n");
	card->reset_flags &= ~UHSII_TO_BYPASS_CURRENT;
	card->reset_flags &= ~UHSII_TO_RANGE_A;
	if (!cr_chk_card_status(cr, CARD_TYPE_UHSII))
		card->reset_flags |= GO_SD_LEGACY_DIRECT;
	else
		card->reset_flags &= ~GO_SD_LEGACY_DIRECT;
	card->reset_flags &= ~GO_SD20_DIRECT;
	cr_clr_card_status(cr, CARD_THERMAL_JUMP_TO_SDR50);

	if (!option->thermal_direct_to_step3)
		sd40_uninit_hw(cr, card);
	if (sd_reset_card(cr, card) == STATUS_SUCCESS)
		info->thermal_state = THERMAL_STATE_TO_NORMAL;
}

static void rts5250_thermal_action(struct card_reader *cr, struct card *card)
{
	struct cr_option *option = &cr->option;
	struct rts5249_info *info = cr->extra;
	u64 elapse;
	u8 gpio_input = 0;

	if (!card || !option->thermal_sensor_en)
		return;

	cr_rd_reg(cr, REG_GPIO_CTL, &gpio_input);
	gpio_input &= 0x10;

	if (info->gpio_input != gpio_input || 0 == info->thermal_time)
		info->thermal_time = cr_get_tick();
	info->gpio_input = gpio_input;

	if (!gpio_input && info->thermal_state < THERMAL_STATE_TO_UHSI) {
		elapse = cr_get_tick() - info->thermal_time;
		rts_trace(TRACE_CR, "Gpio trig low: %lldms\n", elapse);

		if(elapse > option->thermal_time_step3)
			rts5250_thermal_state_to_uhsi(cr, card);
		else if ((elapse > option->thermal_time_step2) &&
			(!option->thermal_direct_to_step3))
			rts5250_thermal_state_to_range_a(cr, card);
		else if ((elapse > option->thermal_time_step1) &&
			(!option->thermal_direct_to_step3))
			rts5250_thermal_state_to_bypass_current(cr, card);
	} else if (gpio_input &&
		info->thermal_state > THERMAL_STATE_TO_NORMAL) {
		elapse = cr_get_tick() - info->thermal_time;
		rts_trace(TRACE_CR, "Gpio trig high: %lldms\n", elapse);

		if (elapse > option->thermal_time_recovery)
			rts5250_thermal_state_to_normal(cr, card);
	}
}

int rts5249_sd40_set_n_fcu(struct card *c, u8 n_fcu)
{
	struct card_reader *cr = c->cm->cr;
	struct sd_info *sd_card = c->extra;
	u8 payload[16];
	u16 addr = 0;
	u8 retry = 0;
	u8 val = 0;


	rts_trace(TRACE_SD, "rts5249 Set N_FCU CCMD Test\n");

	memset(payload, 0, sizeof(payload));
	if (sd40_read_cfg_reg(c, SD40_IOADR_LINK_SET_L, payload, 4)) {
		rts_trace(TRACE_ERROR, "read CFG register fail!\n");
		TRACE_RET(cr, STATUS_FAIL);
	}
	retry = payload[1];

	memset(payload, 0, sizeof(payload));
	payload[0] = 0x20;
	if (CHECK_VERSION_LOWER_THAN(cr, 0x5249, IC_VER_D))
		payload[1] = 2;
	else
		payload[1] = retry;
	payload[2] = (u8)n_fcu;

	if (sd40_write_cfg_reg(c, SD40_IOADR_LINK_SET_L, payload, 4)) {
		rts_trace(TRACE_ERROR, "set N_FCU fail!!\n");
		TRACE_RET(cr, STATUS_FAIL);
	}

	addr = (((SD40_IOADR_CFG_BASE << 8) + SD40_IOADR_LINK_SET_L) << 2) + 1;
	cr_write_host_sd40_iospace(cr, addr, &n_fcu, 1);

	/* set host MAX_RETRY_NUM of Data Burst Retry */
	addr = (((SD40_IOADR_CFG_BASE << 8) + SD40_IOADR_LINK_SET_L) << 2) + 2;
	cr_read_host_sd40_iospace(cr, addr, &val, 1);
	val &= 0xFC;
	val |= (retry & 0x3);
	cr_write_host_sd40_iospace(cr, addr, &val, 1);

	sd_card->setting.host.n_fcu = n_fcu;
	sd_card->setting.dev.n_fcu = n_fcu;

	return 0;
}

void rts524a_process_sd_remove(struct card_reader *cr)
{
	struct card *c = NULL;
	struct sd_command cmd;
	struct sd_info* sd_card = NULL;
	c = cm_get_ready_card(cr->cm, SD_ID);
	if (NULL == c || 0 == cr->option.sd_dat3_cd_en)
		return;
	sd_card = (struct sd_info*)c->extra;
	INIT_SD_CMD(cmd, SD_STATUS, sd_card->sd_addr, 0, SD_RSP_TYPE_R1, 48);
	if (sd_send_cmd(cr, c, &cmd)) {
		sd_clear_sd_error(cr);
		rts_trace(TRACE_CR, "Remove SD\n");
		set_bit(SD_ID, &cr->need_release);
		clear_bit(SD_ID, &cr->need_reset);
	}
}

void rts524a_deal_generic_extra_action(struct card_reader *cr)
{
	cr_deal_generic_extra_action(cr);
	rts524a_process_sd_remove(cr);
}

void rts5250_deal_generic_extra_action(struct card_reader *cr)
{
	u8 val;
	struct card *card = cm_get_ready_card(cr->cm, SD_ID);
	struct cr_option *option = &(cr->option);

	if (card && option->thermal_sensor_en)
		rts5250_thermal_action(cr, card);

	if (CHECK_PID(cr, 0x525A) && cr->option.sd_dat3_cd_en)
		rts524a_process_sd_remove(cr);

	if (!rtsx_chk_stat(cr, RTSX_STAT_IDLE) &&
		!rtsx_chk_stat(cr, RTSX_STAT_SS))
		return;

	if (cm_check_cards_flags(cr->cm, READY)) {
		if (check_dev_flag(cr, POLLING_OCP_EN)) {
			cr_rd_reg(cr, REG_OCPSTAT, &val);

			/* some chip cannot issue OCP interrupt */
			if (val & (SD_OC_NOW | SD_OC_EVER | SDVIO_OC_NOW |
					SDVIO_OC_EVER))
				cr->ocp_stat = val;

			mdelay(1);
		}
	}
}

int rts524a_wr_phy16(struct card_reader *cr, u8 addr, u16 val)
{
	if (cr->asic_code && (addr & 0x80))
		addr = (addr & 0x7f) | 0x40;

	return cr_common_wr_phy16(cr, addr, val);
}

int rts524a_rd_phy16(struct card_reader *cr, u8 addr, u16 *val)
{
	if (cr->asic_code && (addr & 0x80))
		addr = (addr & 0x7f) | 0x40;

	return cr_common_rd_phy16(cr, addr, val);
}

int rts525a_enable_oobs_polling(struct card_reader *cr)
{
	rts_trace(TRACE_CR, "Enable Polling OOBS\n");
	cr_wr_reg(cr, REG_CFG_OOBS_OFF_TIMER, 0xFF, 0x32);
	cr_wr_reg(cr, REG_CFG_OOBS_ON_TIMER, 0xFF, 0x05);
	cr_wr_reg(cr, REG_CFG_VCM_ON_TIMER, 0xFF, 0x83);
	cr_wr_reg(cr, REG_CFG_OOBS_POLLING, 0xFF, 0xDE);
	return 0;
}

int rts525a_disable_oobs_polling(struct card_reader *cr)
{
	rts_trace(TRACE_CR, "Disable Polling OOBS\n");
	cr_wr_reg(cr, REG_CFG_VCM_ON_TIMER, 0xFF, 0x03);
	cr_wr_reg(cr, REG_CFG_OOBS_POLLING, 0xFF, 0x00);
	return 0;
}

int rts5250_patch_dma(struct card_reader *cr)
{
	struct cr_option *option = &cr->option;
	rts_trace(TRACE_CR, "-> cr_patch_dma \n");
	if (option->sd40_cache_rw_en
		|| check_dev_flag(cr, SD40_WRITE_SPEED_UP_EN)) {
		/* fix dma entry cannot be larger than 64k*/
		u8 value = 0;
		rts_trace(TRACE_CR, "SD40 speed up enabled\n");
		if (CHECK_VERSION_LOWER_THAN(cr, 0x5250, IC_VER_D)) {
			/* Make sure reg(RTS5250_HOST_CTL2) can be wrote properly */
			cr_rd_reg(cr, RTS5250_HOST_CTL2, &value);
			if (value & RTS5250_ADMA2_LENGTH_26BIT) {
				rts_trace(TRACE_CR,
					"RTS5250_HOST_CTL2 value = %#x, do not need patch\n", value);
				rts_trace(TRACE_CR, "<- cr_patch_dma \n");
				return 0;
			}
			cr_wr_reg(cr, REG_CLK_CTL, 0xFF, 0);
			cr_wr_reg(cr, REG_CLK_DIV,
				CRC_CLK_SEL_MASK | MCU_CLK_DIV_MASK,
				CRC_CLK_SSC);
			cr_wr_reg(cr, REG_SSC_DIV_N_0, SSC_DIV_N_0_MASK, 0xC4);
			cr_wr_reg(cr, RTS5250_HOST_CTL2,
					RTS5250_ADMA2_LENGTH_MASK,
					RTS5250_ADMA2_LENGTH_26BIT);
			cr_rd_reg(cr, RTS5250_HOST_CTL2, &value);
			rts_trace(TRACE_CR,
				"RTS5250_HOST_CTL2 value = %#x after patched\n", value);
			if (!(value & RTS5250_ADMA2_LENGTH_26BIT))
				option->sd40_cache_rw_en = 0;
		}
	}
	rts_trace(TRACE_CR, "<- cr_patch_dma \n");
	return 0;
}

